/*

   KEYS.CH

   INKEY() key-code definitions.  Taken from INKEY.CH and supplemented
   with additional codes.

   INKEY.CH Copyright (c) 1990-1993, Computer Associates International,
   Inc.  All rights reserved.

   Additions by Kevin Carmody, September 1997.

   NOTE: Keystroke descriptions marked with an asterisk (*) are only
         available on enhanced keyboards (those with more than 84 keys).

*/

// Cursor movement keys

#DEFINE K_UP                5   //   Up arrow, Ctrl-E
#DEFINE K_DOWN             24   //   Down arrow, Ctrl-X
#DEFINE K_LEFT             19   //   Left arrow, Ctrl-S
#DEFINE K_RIGHT             4   //   Right arrow, Ctrl-D
#DEFINE K_HOME              1   //   Home, Ctrl-A
#DEFINE K_END               6   //   End, Ctrl-F
#DEFINE K_PGUP             18   //   PgUp, Ctrl-R
#DEFINE K_PGDN              3   //   PgDn, Ctrl-C

#DEFINE K_CTRL_UP         397   // * Ctrl-Up arrow
#DEFINE K_CTRL_DOWN       401   // * Ctrl-Down arrow
#DEFINE K_CTRL_LEFT        26   //   Ctrl-Left arrow, Ctrl-Z
#DEFINE K_CTRL_RIGHT        2   //   Ctrl-Right arrow, Ctrl-B
#DEFINE K_CTRL_HOME        29   //   Ctrl-Home, Ctrl-]
#DEFINE K_CTRL_END         23   //   Ctrl-End, Ctrl-W
#DEFINE K_CTRL_PGUP        31   //   Ctrl-PgUp, Ctrl-Hyphen
#DEFINE K_CTRL_PGDN        30   //   Ctrl-PgDn, Ctrl-^

#DEFINE K_ALT_UP          408   // * Alt-Up arrow
#DEFINE K_ALT_DOWN        416   // * Alt-Down arrow
#DEFINE K_ALT_LEFT        411   // * Alt-Left arrow
#DEFINE K_ALT_RIGHT       413   // * Alt-Right arrow
#DEFINE K_ALT_HOME        407   // * Alt-Home
#DEFINE K_ALT_END         415   // * Alt-End
#DEFINE K_ALT_PGUP        409   // * Alt-PgUp
#DEFINE K_ALT_PGDN        417   // * Alt-PgDn

// Misc. keys

#DEFINE K_ENTER            13   //   Enter, Ctrl-M
#DEFINE K_RETURN           13   //   Return, Ctrl-M
#DEFINE K_SPACE            32   //   Space bar
#DEFINE K_ESC              27   //   Esc, Ctrl-[

#DEFINE K_CTRL_ENTER       10   //   Ctrl-Enter
#DEFINE K_CTRL_RETURN      10   //   Ctrl-Return
#DEFINE K_CTRL_PRTSCR     370   // * Ctrl-Print Screen

#DEFINE K_ALT_ENTER       284   // * Alt-Enter
#DEFINE K_ALT_RETURN      284   // * Alt-Return
#DEFINE K_ALT_ESC         257   // * Alt-Esc

// Alt punctuation keys

#DEFINE K_ALT_GRAVE       297   // * Alt-`, Alt-~
#DEFINE K_ALT_HYPHEN      386   // * Alt-Hyphen, Alt-Underscore
#DEFINE K_ALT_EQUALS      387   // * Alt-Equals, Alt-Plus
#DEFINE K_ALT_LBRACKET    282   // * Alt-[, Alt-{
#DEFINE K_ALT_RBRACKET    283   // * Alt-], Alt-}
#DEFINE K_ALT_COLON       295   // * Alt-Semicolon, Alt-Colon
#DEFINE K_ALT_QUOTE       296   // * Alt-', Alt-"
#DEFINE K_ALT_COMMA       307   // * Alt-Comma, Alt-<
#DEFINE K_ALT_PERIOD      308   // * Alt-Period, Alt->
#DEFINE K_ALT_SLASH       309   // * Alt-/, Alt-?
#DEFINE K_ALT_BACKSLASH   299   // * Alt-\, Alt-|

// Keypad keys

#DEFINE KP_5              332   // * Keypad 5

#DEFINE KP_CTRL_5         399   // * Keypad Ctrl-5
#DEFINE KP_CTRL_SLASH     405   // * Keypad Ctrl-/
#DEFINE KP_CTRL_ASTERISK  406   // * Keypad Ctrl-*
#DEFINE KP_CTRL_MINUS     398   // * Keypad Ctrl--
#DEFINE KP_CTRL_PLUS      400   // * Keypad Ctrl-+

#DEFINE KP_ALT_SLASH      420   // * Keypad Alt-/
#DEFINE KP_ALT_ASTERISK   311   // * Keypad Alt-*
#DEFINE KP_ALT_MINUS      330   // * Keypad Alt--
#DEFINE KP_ALT_PLUS       334   // * Keypad Alt-+
#DEFINE KP_ALT_ENTER      422   // * Keypad Alt-Enter

// Editing keys

#DEFINE K_INS              22   //   Ins, Ctrl-V
#DEFINE K_DEL               7   //   Del, Ctrl-G
#DEFINE K_BS                8   //   Backspace, Ctrl-H
#DEFINE K_TAB               9   //   Tab, Ctrl-I
#DEFINE K_SH_TAB          271   //   Shift-Tab

#DEFINE K_CTRL_INS        402   // * Ctrl-Ins
#DEFINE K_CTRL_DEL        403   // * Ctrl-Del
#DEFINE K_CTRL_BS         127   //   Ctrl-Backspace
#DEFINE K_CTRL_TAB        404   // * Ctrl-Tab

#DEFINE K_ALT_INS         418   // * Alt-Ins
#DEFINE K_ALT_DEL         419   // * Alt-Del
#DEFINE K_ALT_BS          270   // * Alt-Backspace
#DEFINE K_ALT_TAB         421   // * Alt-Tab

// Control alphanumeric keys

#DEFINE K_CTRL_A            1   //   Ctrl-A, Home
#DEFINE K_CTRL_B            2   //   Ctrl-B, Ctrl-Right arrow
#DEFINE K_CTRL_C            3   //   Ctrl-C, PgDn, Ctrl-ScrollLock
#DEFINE K_CTRL_D            4   //   Ctrl-D, Right arrow
#DEFINE K_CTRL_E            5   //   Ctrl-E, Up arrow
#DEFINE K_CTRL_F            6   //   Ctrl-F, End
#DEFINE K_CTRL_G            7   //   Ctrl-G, Del
#DEFINE K_CTRL_H            8   //   Ctrl-H, Backspace
#DEFINE K_CTRL_I            9   //   Ctrl-I, Tab
#DEFINE K_CTRL_J           10   //   Ctrl-J
#DEFINE K_CTRL_K           11   //   Ctrl-K
#DEFINE K_CTRL_L           12   //   Ctrl-L
#DEFINE K_CTRL_M           13   //   Ctrl-M, Return
#DEFINE K_CTRL_N           14   //   Ctrl-N
#DEFINE K_CTRL_O           15   //   Ctrl-O
#DEFINE K_CTRL_P           16   //   Ctrl-P
#DEFINE K_CTRL_Q           17   //   Ctrl-Q
#DEFINE K_CTRL_R           18   //   Ctrl-R, PgUp
#DEFINE K_CTRL_S           19   //   Ctrl-S, Left arrow
#DEFINE K_CTRL_T           20   //   Ctrl-T
#DEFINE K_CTRL_U           21   //   Ctrl-U
#DEFINE K_CTRL_V           22   //   Ctrl-V, Ins
#DEFINE K_CTRL_W           23   //   Ctrl-W, Ctrl-End
#DEFINE K_CTRL_X           24   //   Ctrl-X, Down arrow
#DEFINE K_CTRL_Y           25   //   Ctrl-Y
#DEFINE K_CTRL_Z           26   //   Ctrl-Z, Ctrl-Left arrow
#DEFINE K_CTRL_2          259   //   Ctrl-2, Ctrl-@
#DEFINE K_CTRL_6           30   //   Ctrl-6, Ctrl-^

// Alt alphanumeric keys

#DEFINE K_ALT_A           286   //   Alt-A
#DEFINE K_ALT_B           304   //   Alt-B
#DEFINE K_ALT_C           302   //   Alt-C
#DEFINE K_ALT_D           288   //   Alt-D
#DEFINE K_ALT_E           274   //   Alt-E
#DEFINE K_ALT_F           289   //   Alt-F
#DEFINE K_ALT_G           290   //   Alt-G
#DEFINE K_ALT_H           291   //   Alt-H
#DEFINE K_ALT_I           279   //   Alt-I
#DEFINE K_ALT_J           292   //   Alt-J
#DEFINE K_ALT_K           293   //   Alt-K
#DEFINE K_ALT_L           294   //   Alt-L
#DEFINE K_ALT_M           306   //   Alt-M
#DEFINE K_ALT_N           305   //   Alt-N
#DEFINE K_ALT_O           280   //   Alt-O
#DEFINE K_ALT_P           281   //   Alt-P
#DEFINE K_ALT_Q           272   //   Alt-Q
#DEFINE K_ALT_R           275   //   Alt-R
#DEFINE K_ALT_S           287   //   Alt-S
#DEFINE K_ALT_T           276   //   Alt-T
#DEFINE K_ALT_U           278   //   Alt-U
#DEFINE K_ALT_V           303   //   Alt-V
#DEFINE K_ALT_W           273   //   Alt-W
#DEFINE K_ALT_X           301   //   Alt-X
#DEFINE K_ALT_Y           277   //   Alt-Y
#DEFINE K_ALT_Z           300   //   Alt-Z
#DEFINE K_ALT_1           376   //   Alt-1
#DEFINE K_ALT_2           377   //   Alt-2
#DEFINE K_ALT_3           378   //   Alt-3
#DEFINE K_ALT_4           379   //   Alt-4
#DEFINE K_ALT_5           380   //   Alt-5
#DEFINE K_ALT_6           381   //   Alt-6
#DEFINE K_ALT_7           382   //   Alt-7
#DEFINE K_ALT_8           383   //   Alt-8
#DEFINE K_ALT_9           384   //   Alt-9
#DEFINE K_ALT_0           385   //   Alt-0

// Function keys

#DEFINE K_F1               28   //   F1, Ctrl-Backslash
#DEFINE K_F2               -1   //   F2
#DEFINE K_F3               -2   //   F3
#DEFINE K_F4               -3   //   F4
#DEFINE K_F5               -4   //   F5
#DEFINE K_F6               -5   //   F6
#DEFINE K_F7               -6   //   F7
#DEFINE K_F8               -7   //   F8
#DEFINE K_F9               -8   //   F9
#DEFINE K_F10              -9   //   F10
#DEFINE K_F11             -40   // * F11
#DEFINE K_F12             -41   // * F12

// Shift function keys

#DEFINE K_SH_F1           -10   //   Shift-F1
#DEFINE K_SH_F2           -11   //   Shift-F2
#DEFINE K_SH_F3           -12   //   Shift-F3
#DEFINE K_SH_F4           -13   //   Shift-F4
#DEFINE K_SH_F5           -14   //   Shift-F5
#DEFINE K_SH_F6           -15   //   Shift-F6
#DEFINE K_SH_F7           -16   //   Shift-F7
#DEFINE K_SH_F8           -17   //   Shift-F8
#DEFINE K_SH_F9           -18   //   Shift-F9
#DEFINE K_SH_F10          -19   //   Shift-F10
#DEFINE K_SH_F11          -42   // * Shift-F11
#DEFINE K_SH_F12          -43   // * Shift-F12

// Control function keys

#DEFINE K_CTRL_F1         -20   //   Ctrl-F1
#DEFINE K_CTRL_F2         -21   //   Ctrl-F2
#DEFINE K_CTRL_F3         -22   //   Ctrl-F4
#DEFINE K_CTRL_F4         -23   //   Ctrl-F3
#DEFINE K_CTRL_F5         -24   //   Ctrl-F5
#DEFINE K_CTRL_F6         -25   //   Ctrl-F6
#DEFINE K_CTRL_F7         -26   //   Ctrl-F7
#DEFINE K_CTRL_F8         -27   //   Ctrl-F8
#DEFINE K_CTRL_F9         -28   //   Ctrl-F9
#DEFINE K_CTRL_F10        -29   //   Ctrl-F10
#DEFINE K_CTRL_F11        -44   // * Ctrl-F11
#DEFINE K_CTRL_F12        -45   // * Ctrl-F12

// Alt function keys

#DEFINE K_ALT_F1          -30   //   Alt-F1
#DEFINE K_ALT_F2          -31   //   Alt-F2
#DEFINE K_ALT_F3          -32   //   Alt-F3
#DEFINE K_ALT_F4          -33   //   Alt-F4
#DEFINE K_ALT_F5          -34   //   Alt-F5
#DEFINE K_ALT_F6          -35   //   Alt-F6
#DEFINE K_ALT_F7          -36   //   Alt-F7
#DEFINE K_ALT_F8          -37   //   Alt-F8
#DEFINE K_ALT_F9          -38   //   Alt-F9
#DEFINE K_ALT_F10         -39   //   Alt-F10
#DEFINE K_ALT_F11         -46   // * Alt-F11
#DEFINE K_ALT_F12         -47   // * Alt-F12

/*

// Definitions in numeric order

#DEFINE K_ALT_F12        - 47   // * Alt-F12
#DEFINE K_ALT_F11        - 46   // * Alt-F11
#DEFINE K_CTRL_F12       - 45   // * Ctrl-F12
#DEFINE K_CTRL_F11       - 44   // * Ctrl-F11
#DEFINE K_SH_F12         - 43   // * Shift-F12
#DEFINE K_SH_F11         - 42   // * Shift-F11
#DEFINE K_F12            - 41   // * F12
#DEFINE K_F11            - 40   // * F11
#DEFINE K_ALT_F10        - 39   //   Alt-F10
#DEFINE K_ALT_F9         - 38   //   Alt-F9
#DEFINE K_ALT_F8         - 37   //   Alt-F8
#DEFINE K_ALT_F7         - 36   //   Alt-F7
#DEFINE K_ALT_F6         - 35   //   Alt-F6
#DEFINE K_ALT_F5         - 34   //   Alt-F5
#DEFINE K_ALT_F4         - 33   //   Alt-F4
#DEFINE K_ALT_F3         - 32   //   Alt-F3
#DEFINE K_ALT_F2         - 31   //   Alt-F2
#DEFINE K_ALT_F1         - 30   //   Alt-F1
#DEFINE K_CTRL_F10       - 29   //   Ctrl-F10
#DEFINE K_CTRL_F9        - 28   //   Ctrl-F9
#DEFINE K_CTRL_F8        - 27   //   Ctrl-F8
#DEFINE K_CTRL_F7        - 26   //   Ctrl-F7
#DEFINE K_CTRL_F6        - 25   //   Ctrl-F6
#DEFINE K_CTRL_F5        - 24   //   Ctrl-F5
#DEFINE K_CTRL_F4        - 23   //   Ctrl-F3
#DEFINE K_CTRL_F3        - 22   //   Ctrl-F4
#DEFINE K_CTRL_F2        - 21   //   Ctrl-F2
#DEFINE K_CTRL_F1        - 20   //   Ctrl-F1
#DEFINE K_SH_F10         - 19   //   Shift-F10
#DEFINE K_SH_F9          - 18   //   Shift-F9
#DEFINE K_SH_F8          - 17   //   Shift-F8
#DEFINE K_SH_F7          - 16   //   Shift-F7
#DEFINE K_SH_F6          - 15   //   Shift-F6
#DEFINE K_SH_F5          - 14   //   Shift-F5
#DEFINE K_SH_F4          - 13   //   Shift-F4
#DEFINE K_SH_F3          - 12   //   Shift-F3
#DEFINE K_SH_F2          - 11   //   Shift-F2
#DEFINE K_SH_F1          - 10   //   Shift-F1
#DEFINE K_F10            -  9   //   F10
#DEFINE K_F9             -  8   //   F9
#DEFINE K_F8             -  7   //   F8
#DEFINE K_F7             -  6   //   F7
#DEFINE K_F6             -  5   //   F6
#DEFINE K_F5             -  4   //   F5
#DEFINE K_F4             -  3   //   F4
#DEFINE K_F3             -  2   //   F3
#DEFINE K_F2             -  1   //   F2
#DEFINE K_CTRL_A            1   //   Ctrl-A, Home
#DEFINE K_HOME              1   //   Home, Ctrl-A
#DEFINE K_CTRL_B            2   //   Ctrl-B, Ctrl-Right arrow
#DEFINE K_CTRL_RIGHT        2   //   Ctrl-Right arrow, Ctrl-B
#DEFINE K_CTRL_C            3   //   Ctrl-C, PgDn, Ctrl-ScrollLock
#DEFINE K_PGDN              3   //   PgDn, Ctrl-C
#DEFINE K_CTRL_D            4   //   Ctrl-D, Right arrow
#DEFINE K_RIGHT             4   //   Right arrow, Ctrl-D
#DEFINE K_CTRL_E            5   //   Ctrl-E, Up arrow
#DEFINE K_UP                5   //   Up arrow, Ctrl-E
#DEFINE K_CTRL_F            6   //   Ctrl-F, End
#DEFINE K_END               6   //   End, Ctrl-F
#DEFINE K_CTRL_G            7   //   Ctrl-G, Del
#DEFINE K_DEL               7   //   Del, Ctrl-G
#DEFINE K_BS                8   //   Backspace, Ctrl-H
#DEFINE K_CTRL_H            8   //   Ctrl-H, Backspace
#DEFINE K_CTRL_I            9   //   Ctrl-I, Tab
#DEFINE K_TAB               9   //   Tab, Ctrl-I
#DEFINE K_CTRL_ENTER       10   //   Ctrl-Enter
#DEFINE K_CTRL_J           10   //   Ctrl-J
#DEFINE K_CTRL_RETURN      10   //   Ctrl-Return
#DEFINE K_CTRL_K           11   //   Ctrl-K
#DEFINE K_CTRL_L           12   //   Ctrl-L
#DEFINE K_CTRL_M           13   //   Ctrl-M, Return
#DEFINE K_ENTER            13   //   Enter, Ctrl-M
#DEFINE K_RETURN           13   //   Return, Ctrl-M
#DEFINE K_CTRL_N           14   //   Ctrl-N
#DEFINE K_CTRL_O           15   //   Ctrl-O
#DEFINE K_CTRL_P           16   //   Ctrl-P
#DEFINE K_CTRL_Q           17   //   Ctrl-Q
#DEFINE K_CTRL_R           18   //   Ctrl-R, PgUp
#DEFINE K_PGUP             18   //   PgUp, Ctrl-R
#DEFINE K_CTRL_S           19   //   Ctrl-S, Left arrow
#DEFINE K_LEFT             19   //   Left arrow, Ctrl-S
#DEFINE K_CTRL_T           20   //   Ctrl-T
#DEFINE K_CTRL_U           21   //   Ctrl-U
#DEFINE K_CTRL_V           22   //   Ctrl-V, Ins
#DEFINE K_INS              22   //   Ins, Ctrl-V
#DEFINE K_CTRL_END         23   //   Ctrl-End, Ctrl-W
#DEFINE K_CTRL_W           23   //   Ctrl-W, Ctrl-End
#DEFINE K_CTRL_X           24   //   Ctrl-X, Down arrow
#DEFINE K_DOWN             24   //   Down arrow, Ctrl-X
#DEFINE K_CTRL_Y           25   //   Ctrl-Y
#DEFINE K_CTRL_LEFT        26   //   Ctrl-Left arrow, Ctrl-Z
#DEFINE K_CTRL_Z           26   //   Ctrl-Z, Ctrl-Left arrow
#DEFINE K_ESC              27   //   Esc, Ctrl-[
#DEFINE K_F1               28   //   F1, Ctrl-Backslash
#DEFINE K_CTRL_HOME        29   //   Ctrl-Home, Ctrl-]
#DEFINE K_CTRL_6           30   //   Ctrl-6, Ctrl-^
#DEFINE K_CTRL_PGDN        30   //   Ctrl-PgDn, Ctrl-^
#DEFINE K_CTRL_PGUP        31   //   Ctrl-PgUp, Ctrl-Hyphen
#DEFINE K_SPACE            32   //   Space bar
#DEFINE K_CTRL_BS         127   //   Ctrl-Backspace
#DEFINE K_ALT_ESC         257   // * Alt-Esc
#DEFINE K_CTRL_2          259   //   Ctrl-2, Ctrl-@
#DEFINE K_ALT_BS          270   // * Alt-Backspace
#DEFINE K_SH_TAB          271   //   Shift-Tab
#DEFINE K_ALT_Q           272   //   Alt-Q
#DEFINE K_ALT_W           273   //   Alt-W
#DEFINE K_ALT_E           274   //   Alt-E
#DEFINE K_ALT_R           275   //   Alt-R
#DEFINE K_ALT_T           276   //   Alt-T
#DEFINE K_ALT_Y           277   //   Alt-Y
#DEFINE K_ALT_U           278   //   Alt-U
#DEFINE K_ALT_I           279   //   Alt-I
#DEFINE K_ALT_O           280   //   Alt-O
#DEFINE K_ALT_P           281   //   Alt-P
#DEFINE K_ALT_LBRACKET    282   // * Alt-[, Alt-{
#DEFINE K_ALT_RBRACKET    283   // * Alt-], Alt-}
#DEFINE K_ALT_ENTER       284   // * Alt-Enter
#DEFINE K_ALT_RETURN      284   // * Alt-Return
#DEFINE K_ALT_A           286   //   Alt-A
#DEFINE K_ALT_S           287   //   Alt-S
#DEFINE K_ALT_D           288   //   Alt-D
#DEFINE K_ALT_F           289   //   Alt-F
#DEFINE K_ALT_G           290   //   Alt-G
#DEFINE K_ALT_H           291   //   Alt-H
#DEFINE K_ALT_J           292   //   Alt-J
#DEFINE K_ALT_K           293   //   Alt-K
#DEFINE K_ALT_L           294   //   Alt-L
#DEFINE K_ALT_COLON       295   // * Alt-Semicolon, Alt-Colon
#DEFINE K_ALT_QUOTE       296   // * Alt-', Alt-"
#DEFINE K_ALT_GRAVE       297   // * Alt-`, Alt-~
#DEFINE K_ALT_BACKSLASH   299   // * Alt-\, Alt-|
#DEFINE K_ALT_Z           300   //   Alt-Z
#DEFINE K_ALT_X           301   //   Alt-X
#DEFINE K_ALT_C           302   //   Alt-C
#DEFINE K_ALT_V           303   //   Alt-V
#DEFINE K_ALT_B           304   //   Alt-B
#DEFINE K_ALT_N           305   //   Alt-N
#DEFINE K_ALT_M           306   //   Alt-M
#DEFINE K_ALT_COMMA       307   // * Alt-Comma, Alt-<
#DEFINE K_ALT_PERIOD      308   // * Alt-Period, Alt->
#DEFINE K_ALT_SLASH       309   // * Alt-/, Alt-?
#DEFINE KP_ALT_ASTERISK   311   // * Keypad Alt-*
#DEFINE KP_ALT_MINUS      330   // * Keypad Alt--
#DEFINE KP_5              332   // * Keypad 5
#DEFINE KP_ALT_PLUS       334   // * Keypad Alt-+
#DEFINE K_CTRL_PRTSCR     370   // * Ctrl-Print Screen
#DEFINE K_ALT_1           376   //   Alt-1
#DEFINE K_ALT_2           377   //   Alt-2
#DEFINE K_ALT_3           378   //   Alt-3
#DEFINE K_ALT_4           379   //   Alt-4
#DEFINE K_ALT_5           380   //   Alt-5
#DEFINE K_ALT_6           381   //   Alt-6
#DEFINE K_ALT_7           382   //   Alt-7
#DEFINE K_ALT_8           383   //   Alt-8
#DEFINE K_ALT_9           384   //   Alt-9
#DEFINE K_ALT_0           385   //   Alt-0
#DEFINE K_ALT_HYPHEN      386   // * Alt-Hyphen, Alt-Underscore
#DEFINE K_ALT_EQUALS      387   // * Alt-Equals, Alt-Plus
#DEFINE K_CTRL_UP         397   // * Ctrl-Up arrow
#DEFINE KP_CTRL_MINUS     398   // * Keypad Ctrl--
#DEFINE KP_CTRL_5         399   // * Keypad Ctrl-5
#DEFINE KP_CTRL_PLUS      400   // * Keypad Ctrl-+
#DEFINE K_CTRL_DOWN       401   // * Ctrl-Down arrow
#DEFINE K_CTRL_INS        402   // * Ctrl-Ins
#DEFINE K_CTRL_DEL        403   // * Ctrl-Del
#DEFINE K_CTRL_TAB        404   // * Ctrl-Tab
#DEFINE KP_CTRL_SLASH     405   // * Keypad Ctrl-/
#DEFINE KP_CTRL_ASTERISK  406   // * Keypad Ctrl-*
#DEFINE K_ALT_HOME        407   // * Alt-Home
#DEFINE K_ALT_UP          408   // * Alt-Up arrow
#DEFINE K_ALT_PGUP        409   // * Alt-PgUp
#DEFINE K_ALT_LEFT        411   // * Alt-Left arrow
#DEFINE K_ALT_RIGHT       413   // * Alt-Right arrow
#DEFINE K_ALT_END         415   // * Alt-End
#DEFINE K_ALT_DOWN        416   // * Alt-Down arrow
#DEFINE K_ALT_PGDN        417   // * Alt-PgDn
#DEFINE K_ALT_INS         418   // * Alt-Ins
#DEFINE K_ALT_DEL         419   // * Alt-Del
#DEFINE KP_ALT_SLASH      420   // * Keypad Alt-/
#DEFINE K_ALT_TAB         421   // * Alt-Tab
#DEFINE KP_ALT_ENTER      422   // * Keypad Alt-Enter

*/