/*

  DBEBACK.PRG

  Back end routines for DBE.

  This file is part of the source code for DBE, a general purpose database
  utility.

  Copyright (C) 1998 Kevin Carmody
  kcarmody@compuserve.com

  This program is free software.  You can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.

  You should have received a copy of the GNU General Public License along
  with this program, in a file called COPYING.TXT.  If not, write to:

  Free Software Foundation, Inc.
  59 Temple Place, Suite 330,
  Boston, MA 02111-1307
  USA
  http://www.gnu.org

*/

ANNOUNCE DBEBack

#INCLUDE 'DBE.CH'
#INCLUDE 'KEYS.CH'
#INCLUDE 'CHARS.CH'
#INCLUDE 'DBSTRUCT.CH'
#INCLUDE 'FILEIO.CH'
#INCLUDE 'ERROR.CH'
#INCLUDE 'MOUSCLOC.CH'

//***************************************************************************

FUNCTION Sys()

STATIC aSys := {}

RETURN aSys  //  Sys

//***************************************************************************

FUNCTION SysDefault()

STATIC aSys := {}

RETURN aSys  //  SysDefault

//***************************************************************************

FUNCTION SetDefault()

STATIC aSet := {}

RETURN aSet  //  SetDefault

//***************************************************************************

FUNCTION Data()

STATIC aData := {}

/*

Structure of aData array

aData [nArea] [DATA_ALIAS]
              [DATA_TABLEFILE]
              [DATA_TABLEPATH]
              [DATA_AREA]
              [DATA_EXCLUSIVE]
              [DATA_READONLY]
              [DATA_DRIVER]
              [DATA_TABLEEXT]
              [DATA_BAGEXT]
              [DATA_MULTITAG]
              [DATA_NORDER]
              [DATA_NRELATION]
              [DATA_NFILTER]
              [DATA_NFIELD]
              [DATA_BFIELD]
              [DATA_FLDCOLEN]
              [DATA_RECNOLEN]

              [DATA_BAGFILE]  [nBag]

              [DATA_ORDER]    [nOrder]    [DATA_ONAME]
                                          [DATA_OBAGFILE]
                                          [DATA_OBAGPATH]
                                          [DATA_OKEY]
                                          [DATA_OFOR]
                                          [DATA_OUNIQUE]
                                          [DATA_ODESCEND]

              [DATA_RELATION] [nRelation] [DATA_RALIAS]
                                          [DATA_REXP]
                                          [DATA_RBLOCK]

              [DATA_FILTER]   [nFilter]   [DATA_LNAME]
                                          [DATA_LEXP]
                                          [DATA_LBLOCK]

              [DATA_STRUC]    [nField]    [DATA_SNAME]
                                          [DATA_STYPE]
                                          [DATA_SLEN]
                                          [DATA_SDEC]
                                          [DATA_SNUM]
                                          [DATA_SPOS]

              [DATA_FIELD]    [nField]    [DATA_FNAME]
                                          [DATA_FALIAS]
                                          [DATA_FEXP]
                                          [DATA_FBLOCK]
                                          [DATA_FEDIT]
                                          [DATA_FTYPE]
                                          [DATA_FLEN]
                                          [DATA_FDEC]
                                          [DATA_FNUM]

              [DATA_BROWSE]

              [DATA_HORIZ]
              [DATA_CENTER]
              [DATA_FINIT]
              [DATA_FMAX]
              [DATA_FSORT]
              [DATA_FSORT]
              [DATA_SCREEN]

*/

RETURN aData  //  Data

//***************************************************************************

FUNCTION Current(nSetCurrent)

STATIC aCurrent := {}

IF nSetCurrent # NIL
  aCurrent := IF(EMPTY(nSetCurrent), {}, Data()[nSetCurrent])
  Sys()[SYS_SELECT] := nSetCurrent
  IF !EMPTY(nSetCurrent)
    SELECT (Data()[nSetCurrent][DATA_ALIAS])
  END
END

RETURN aCurrent  //  Current

//***************************************************************************

FUNCTION KeyValue()

LOCAL cKey := INDEXKEY(0)
LOCAL bKey := IF(EMPTY(cKey), {|| .Y.}, {|| &cKey})
LOCAL xKey := bKey:EVAL()

RETURN xKey  //  KeyValue

//***************************************************************************

PROCEDURE TableAdd(cDatabase, cAlias, lExclusive, lReadonly, cDriver)

LOCAL cPath    := PathAdj(cDatabase, .Y.)
LOCAL cPathExt := PathAdj(cDatabase, .Y., DBTABLEEXT(), .Y.)
LOCAL nHandle  := FOPEN(cPathExt, IF(lReadonly, FO_READ, FO_READWRITE) + ;
  IF(lExclusive, FO_EXCLUSIVE, FO_SHARED))
LOCAL aCurrent := ARRAY(DATA_TOTAL)
LOCAL aDriver  := { ;
  {'DBFNTX', .N., .Y., .Y., .Y.}, ;
  {'DBFNDX', .N., .N., .N., .N.}, ;
  {'DBFCDX', .Y., .Y., .Y., .N.}, ;
  {'DBFMDX', .Y., .Y., .N., .N.}  }
LOCAL nDriver  := ASCAN(aDriver, {|aRow| aRow[1] == UPPER(ALLTRIM(cDriver))})
LOCAL lMulti   := IF(EMPTY(nDriver), .N., aDriver[nDriver][2])
LOCAL lCond    := IF(EMPTY(nDriver), .N., aDriver[nDriver][3])
LOCAL lEval    := IF(EMPTY(nDriver), .N., aDriver[nDriver][4])
LOCAL lPos     := IF(EMPTY(nDriver), .N., aDriver[nDriver][5])

CheckError(EG_OPEN, 'Open', cPathExt, .N.)
FCLOSE(nHandle)
DBUSEAREA(.Y., ALLTRIM(cDriver), cPath, ;
  IF(EMPTY(cAlias), NIL, cAlias), !lExclusive, lReadonly)

aCurrent[DATA_ALIAS]     := ALIAS()
aCurrent[DATA_TABLEFILE] := ALLTRIM(cDatabase)
aCurrent[DATA_TABLEPATH] := cPath
aCurrent[DATA_AREA]      := SELECT()
aCurrent[DATA_EXCLUSIVE] := lExclusive
aCurrent[DATA_READONLY]  := lReadonly
aCurrent[DATA_DRIVER]    := cDriver
aCurrent[DATA_TABLEEXT]  := DBTABLEEXT()
aCurrent[DATA_BAGEXT]    := ORDBAGEXT()
aCurrent[DATA_MULTITAG]  := lMulti
aCurrent[DATA_CONDORDER] := lCond
aCurrent[DATA_EVALORDER] := lEval
aCurrent[DATA_ORDPOS]    := lPos
aCurrent[DATA_NORDER]    := 1
aCurrent[DATA_NRELATION] := 0
aCurrent[DATA_NFILTER]   := 1
aCurrent[DATA_NFIELD]    := 0
aCurrent[DATA_BFIELD]    := 1
aCurrent[DATA_FLDCOLEN]  := LENNUM(FCOUNT())
aCurrent[DATA_RECNOLEN]  := LENNUM(RECCOUNT())
aCurrent[DATA_BAGFILE]   := {}
aCurrent[DATA_OATTRIB]   := {{.N., .N.}}
aCurrent[DATA_ORDER]     := {{'Natural', '', '', 'RECNO()', '', .N., .N.}}
aCurrent[DATA_RELATION]  := {}
aCurrent[DATA_FILTER]    := {{'All records', '.Y.', {|| .Y.}}}
aCurrent[DATA_STRUC]     := {}
aCurrent[DATA_FIELD]     := {}
aCurrent[DATA_BROWSE]    := NIL
aCurrent[DATA_HORIZ]     := .Y.
aCurrent[DATA_CENTER]    := .N.
aCurrent[DATA_FSORT]     := .Y.
aCurrent[DATA_FINIT]     := .N.
aCurrent[DATA_FMAX]      := 0
aCurrent[DATA_SCREEN]    := ''
aCurrent[DATA_STRUC]     := ACLONE(TableStru())

AADD(Data(), aCurrent)
Current(LEN(Data()))
RefreshSetup()

RETURN  //  TableAdd

//***************************************************************************

PROCEDURE TableReopen()

IF !EMPTY(ALIAS(Current()[DATA_AREA]))
  CLOSE (Current()[DATA_AREA])
END
DBUSEAREA(.Y., Current()[DATA_DRIVER], Current()[DATA_TABLEPATH], ;
  Current()[DATA_ALIAS], !Current()[DATA_EXCLUSIVE], ;
  Current()[DATA_READONLY])
Current()[DATA_AREA]  := SELECT()
Current()[DATA_STRUC] := ACLONE(TableStru())
OrderReopen()
RelationReopen()
FilterReopen()
IF Current()[DATA_FINIT] .OR. YesNo('Reset field list?', .N.)
  FieldInit()
END

RETURN  //  TableReopen

//***************************************************************************

PROCEDURE TableDel()

LOCAL nLTable := Sys()[SYS_RSETUP][SETUP_TABLE]

COMMIT
CLOSE (Data()[nLTable][DATA_AREA])
ADEL(Data(), nLTable)
ASIZE(Data(), LEN(Data()) - 1)
DO CASE
CASE nLTable == Sys()[SYS_SELECT]
  DO CASE
  CASE EMPTY(Data())
    Current(0)
    EditBox()
  CASE nLTable > LEN(Data())
    Current(Sys()[SYS_SELECT] - 1)
    EditInit(.Y.)
  OTHERWISE
    Current(Sys()[SYS_SELECT])
    EditInit(.Y.)
  END
CASE nLTable < Sys()[SYS_SELECT]
  Sys()[SYS_SELECT] --
END
RefreshSetup()

RETURN  //  TableDel

//***************************************************************************

PROCEDURE OrderAdd(cIndex)

LOCAL cIndexPath := ''

IF !EMPTY(cIndex)
  IF EMPTY(Current()[DATA_BAGFILE])
    Current()[DATA_NORDER] := 2
  END
  cIndexPath := UPPER(PathAdj(cIndex, .Y., ORDBAGEXT(), .Y.))
  IF EMPTY(ASCAN(Current()[DATA_BAGFILE], ;
    {|cBag| UPPER(PathAdj(cBag, .Y., ORDBAGEXT(), .Y.)) == cIndexPath}))
    AADD(Current()[DATA_BAGFILE], cIndex)
  END
END

OrderReopen()

RETURN  //  OrderAdd

//***************************************************************************

PROCEDURE OrderReopen()

LOCAL nOrder     := 0
LOCAL cIndex     := ''
LOCAL cIndexPath := ''

Current()[DATA_OATTRIB] := {{.N., .N.}}
ORDLISTCLEAR()
FOR nOrder := 1 TO LEN(Current()[DATA_BAGFILE])
  cIndex     := Current()[DATA_BAGFILE][nOrder]
  cIndexPath := PathAdj(cIndex, .Y., ORDBAGEXT(), .Y.)
  OrdAttrib(cIndexPath)
  ORDLISTADD(cIndexPath)
  IF NETERR()
    ALERT(cIndexPath + ' locked.')
    BREAK
  END
NEXT
OrderInit()

RETURN  //  OrderReopen

//***************************************************************************

PROCEDURE OrderInit()

LOCAL aOrder  := {}
LOCAL aAttrib := {}
LOCAL nOrder  := 0
LOCAL cFile   := ''
LOCAL cPath   := ''

ASIZE(Current()[DATA_ORDER], 1)

WHILE !EMPTY(ORDNAME(++ nOrder))
  aOrder := ARRAY(DATA_OTOTAL)
  cPath := PathAdj(ORDBAGNAME(nOrder),, ORDBAGEXT(), .N.)
  cFile := PathAdj(cPath, .N.)
  aOrder[DATA_ONAME]    := ORDNAME(nOrder)
  aOrder[DATA_OBAGFILE] := cFile
  aOrder[DATA_OBAGPATH] := cPath
  aOrder[DATA_OKEY]     := ORDKEY(nOrder)
  aOrder[DATA_OFOR]     := ORDFOR(nOrder)
  aOrder[DATA_OUNIQUE]  := .N.
  aOrder[DATA_ODESCEND] := .N.
  AADD(Current()[DATA_ORDER], ACLONE(aOrder))
END

IF LEN(Current()[DATA_ORDER]) == LEN(Current()[DATA_OATTRIB])
  FOR nOrder := 1 TO LEN(Current()[DATA_ORDER])
    aOrder  := Current()[DATA_ORDER][nOrder]
    aAttrib := Current()[DATA_OATTRIB][nOrder]
    aOrder[DATA_OUNIQUE]  := aAttrib[1]
    aOrder[DATA_ODESCEND] := aAttrib[2]
  NEXT
END

RefreshSetup()

RETURN  //  OrderInit

//***************************************************************************

PROCEDURE OrdAttrib(cIndexPath)

LOCAL aDirectory := DIRECTORY(cIndexPath, 'H')
LOCAL lFile      := !EMPTY(aDirectory)
LOCAL nHandle    := IF(!lFile, -1, FOPEN(cIndexPath))
LOCAL cBuffer    := ''
LOCAL nSize      := 0
LOCAL bError     := {|| BREAK(NIL)}
LOCAL lUnique    := .N.
LOCAL lDescend   := .N.
LOCAL nTags      := 0
LOCAL nTag       := 1
LOCAL nPos       := 0
LOCAL aTagLoc    := {}
LOCAL nOption    := 0
LOCAL nRBits     := 0
LOCAL nRDiv      := 0
LOCAL nRLen      := 0
LOCAL cRecord    := ''
LOCAL nRecord    := 0
LOCAL cOrdType   := SUBSTR(ORDBAGEXT(), 2)

BEGIN SEQUENCE

  IF !lFile
    BREAK
  END
  IF !EMPTY(FERROR())
    BREAK
  END

  DO CASE
  CASE cOrdType == 'NTX'

    cBuffer  := FileRead(nHandle, .N., 0, 281, bError)
    lUnique  := !EMPTY(ASC(SUBSTR(cBuffer, 279, 1)))
    lDescend := !EMPTY(ASC(SUBSTR(cBuffer, 281, 1)))
    AADD(Current()[DATA_OATTRIB], {lUnique, lDescend})

  CASE cOrdType == 'NDX'

    cBuffer  := FileRead(nHandle, .N., 0, 24, bError)
    lUnique  := !EMPTY(ASC(SUBSTR(cBuffer, 24, 1)))
    AADD(Current()[DATA_OATTRIB], {lUnique, .N.})

  CASE cOrdType == 'MDX'

    cBuffer  := FileRead(nHandle, .N., 0, 48, bError)
    nTags    := BIN2W(SUBSTR(cBuffer, 29, 2))
    FOR nTag := 1 TO nTags
      cBuffer  := FileRead(nHandle, .N., 512 + nTag * 32, 32, bError)
      nPos     := BIN2L(SUBSTR(cBuffer, 1, 4)) * 512
      AADD(aTagLoc, nPos)
    NEXT
    FOR nTag := 1 TO nTags
      cBuffer  := FileRead(nHandle, .N., aTagLoc[nTag], 129, bError)
      nOption  := ASC(SUBSTR(cBuffer, 9, 1))
      lUnique  := INT(nOption/ 8) % 2 == 1
      lDescend := INT(nOption/64) % 2 == 1
      AADD(Current()[DATA_OATTRIB], {lUnique, lDescend})
    NEXT

  CASE cOrdType == 'CDX'

    cBuffer  := FileRead(nHandle, .N., 0, 512, bError)
    nPos     := BIN2L(SUBSTR(cBuffer, 1, 4))
    cBuffer  := FileRead(nHandle, .N., nPos, 512, bError)
    nTags    := BIN2W(SUBSTR(cBuffer, 3, 2))
    nRBits   := ASC(SUBSTR(cBuffer, 21, 1))
    nRDiv    := INT(2^nRBits)
    nRLen    := ASC(SUBSTR(cBuffer, 24, 1))
    FOR nTag := 1 TO nTags
      cRecord := SUBSTR(cBuffer, 25 + (nTag - 1) * nRLen, nRLen)
      nRecord := BIN2W(cRecord)
      nPos    := nRecord % nRDiv
      AADD(aTagLoc, nPos)
    NEXT
    FOR nTag := 1 TO nTags
      cBuffer  := FileRead(nHandle, .N., aTagLoc[nTag], 512, bError)
      nOption  := ASC(SUBSTR(cBuffer, 15, 1))
      lUnique  := nOption % 2 == 1
      nOption  := ASC(SUBSTR(cBuffer, 503, 1))
      lDescend := !EMPTY(nOption)
      AADD(Current()[DATA_OATTRIB], {lUnique, lDescend})
    NEXT

  END

END SEQUENCE

FCLOSE(nHandle)

RETURN  //  OrdAttrib

//***************************************************************************

PROCEDURE OrderRebuild()

LOCAL aOrdList := Current()[DATA_ORDER]
LOCAL aOrder   := {}
LOCAL nOrder   := 0
LOCAL nOrders  := LEN(aOrdList)
LOCAL cIndex   := ''
LOCAL cTag     := ''
LOCAL cIExpr   := ''
LOCAL cIFor    := ''
LOCAL lUnique  := .N.
LOCAL lDescend := .N.
LOCAL lMulti   := Current()[DATA_MULTITAG]
LOCAL lCond    := Current()[DATA_CONDORDER]
LOCAL lEval    := Current()[DATA_EVALORDER]

FOR nOrder := 2 TO nOrders
  BoxStatus('Reindexing order ' + LTRIM(STR(nOrder)) + ' of ' + ;
    LTRIM(STR(nOrders)) + '...',, lEval)
  aOrder   := aOrdList[nOrder]
  cIndex   := aOrder[DATA_OBAGFILE]
  cTag     := aOrder[DATA_ONAME]
  cIExpr   := aOrder[DATA_OKEY]
  cIFor    := aOrder[DATA_OFOR]
  lUnique  := aOrder[DATA_OUNIQUE]
  lDescend := aOrder[DATA_ODESCEND]
  IF lEval
    Sys()[SYS_ODOMETER][ODOM_INC] := Sys()[SYS_ODOMETER][ODOM_ROLL]
  END
  ORDCONDSET(IF(!lCond .OR. EMPTY(cIFor), NIL, cIFor), ;
    IF(!lCond .OR. EMPTY(cIFor), NIL, BlockStr(cIFor)),,, ;
    IF(!lEval, NIL, {|| Odom()}), ;
    IF(!lEval, NIL, Sys()[SYS_ODOMETER][ODOM_ROLL]),,,,, ;
    IF(!lCond, NIL, lDescend))
  ORDCREATE(PathAdj(cIndex, .Y., ORDBAGEXT(), .Y.), ;
    IF(!lMulti, NIL, cTag), cIExpr, BlockStr(cIExpr), lUnique)
  IF lEval
    OdomEnd()
  END
  COMMIT
  BoxStatus()
NEXT

RETURN  //  OrderRebuild

//***************************************************************************

PROCEDURE OrderDel()

LOCAL aOrdList := Current()[DATA_ORDER]
LOCAL nLOrder  := Sys()[SYS_RSETUP][SETUP_ORDER]
LOCAL aLOrder  := aOrdList[nLOrder]
LOCAL cLIndex  := aLOrder[DATA_OBAGFILE]
LOCAL nHandle  := ASCAN(Current()[DATA_BAGFILE], {|cBag| ;
  UPPER(PathAdj(cBag, .Y., ORDBAGEXT(), .Y.)) == ;
  UPPER(PathAdj(cLIndex, .Y., ORDBAGEXT(), .Y.))})

COMMIT
IF !EMPTY(nHandle)
  ADEL(Current()[DATA_BAGFILE], nHandle)
  ASIZE(Current()[DATA_BAGFILE], LEN(Current()[DATA_BAGFILE]) - 1)
  OrderReopen()
  DO CASE
  CASE nLOrder == Current()[DATA_NORDER] .AND. nLOrder > LEN(aOrdList)
    Current()[DATA_NORDER] := LEN(aOrdList)
  CASE nLOrder < Current()[DATA_NORDER]
    Current()[DATA_NORDER] --
  END
  ORDSETFOCUS(Current()[DATA_NORDER] - 1)
  RefreshSetup()
  EditInit(.N.)
END

RETURN  //  OrderDel

//***************************************************************************

PROCEDURE RelationAdd(cRelAlias, cRelExpr)

LOCAL aRelation := ARRAY(DATA_OTOTAL)
LOCAL bRelExpr  := BlockStr(cRelExpr)

DBSETRELATION(cRelAlias, bRelExpr, cRelExpr)
aRelation[DATA_RALIAS] := cRelAlias
aRelation[DATA_REXP]   := cRelExpr
aRelation[DATA_RBLOCK] := bRelExpr
AADD(Current()[DATA_RELATION], ACLONE(aRelation))
RefreshSetup()
RelationInit()

RETURN  //  RelationAdd

//***************************************************************************

PROCEDURE RelationEdit(cRelAlias, cRelExpr, nRelation)

LOCAL aRelation := Current()[DATA_RELATION][nRelation]
LOCAL bRelExpr  := BlockStr(cRelExpr)

aRelation[DATA_RALIAS] := cRelAlias
aRelation[DATA_REXP]   := cRelExpr
aRelation[DATA_RBLOCK] := bRelExpr
RefreshSetup()
RelationReopen()

RETURN  //  RelationEdit

//***************************************************************************

PROCEDURE RelationReopen()

DBCLEARRELATION()
AEVAL(Current()[DATA_RELATION], {|aRelation| ;
  DBSETRELATION(aRelation[DATA_RALIAS], aRelation[DATA_REXP])})
RelationInit()

RETURN  //  RelationReopen

//***************************************************************************

PROCEDURE RelationInit()

LOCAL aRelation := {}
LOCAL nRelation := 0
LOCAL cRelExpr  := ''

ASIZE(Current()[DATA_RELATION], 0)

WHILE !EMPTY(DBRSELECT(++ nRelation))
  aRelation := ARRAY(DATA_RTOTAL)
  aRelation[DATA_RALIAS] := ALIAS(DBRSELECT(nRelation))
  cRelExpr := DBRELATION(nRelation)
  aRelation[DATA_REXP]   := cRelExpr
  aRelation[DATA_RBLOCK] := BlockStr(cRelExpr)
  AADD(Current()[DATA_RELATION], ACLONE(aRelation))
END

Current()[DATA_NRELATION] := IF(EMPTY(nRelation), 0, 1)
RefreshSetup()

RETURN  //  RelationInit

//***************************************************************************

PROCEDURE RelationDel()

LOCAL nLRelation := Sys()[SYS_RSETUP][SETUP_RELATION]

ADEL(Current()[DATA_RELATION], nLRelation)
ASIZE(Current()[DATA_RELATION], LEN(Current()[DATA_RELATION]) - 1)
RelationReopen()
DO CASE
CASE nLRelation == Current()[DATA_NRELATION] .AND. ;
  nLRelation > LEN(Current()[DATA_RELATION])
  Current()[DATA_NRELATION] := LEN(Current()[DATA_RELATION])
CASE nLRelation < Current()[DATA_NRELATION]
  Current()[DATA_NRELATION] --
END
RefreshSetup()

RETURN  //  RelationDel

//***************************************************************************

PROCEDURE FilterAdd(cFiltExpr, cFiltName)

LOCAL aFilter   := ARRAY(DATA_LTOTAL)
LOCAL bFiltExpr := BlockStr(cFiltExpr)

IF EMPTY(cFiltName)
  cFiltName := Dots(cFiltExpr, 26)
END
DBSETFILTER(bFiltExpr, cFiltExpr)
aFilter[DATA_LNAME]     := cFiltName
aFilter[DATA_LEXP]      := cFiltExpr
aFilter[DATA_LBLOCK]    := bFiltExpr
AADD(Current()[DATA_FILTER], ACLONE(aFilter))
Current()[DATA_NFILTER] := LEN(Current()[DATA_FILTER])
Current()[DATA_CENTER]  := .N.
RefreshSetup()
IF !bFiltExpr:EVAL()
  GO TOP
END

RETURN  //  FilterAdd

//***************************************************************************

PROCEDURE FilterEdit(cFiltExpr, cFiltName, nFilter)

LOCAL aFilter   := Current()[DATA_FILTER][nFilter]
LOCAL bFiltExpr := BlockStr(cFiltExpr)

IF EMPTY(cFiltName)
  cFiltName := Dots(cFiltExpr, 26)
END
DBSETFILTER(bFiltExpr, cFiltExpr)
aFilter[DATA_LNAME]    := cFiltName
aFilter[DATA_LEXP]     := cFiltExpr
aFilter[DATA_LBLOCK]   := bFiltExpr
Current()[DATA_CENTER] := .N.
RefreshSetup()
IF !bFiltExpr:EVAL()
  GO TOP
END

RETURN  //  FilterEdit

//***************************************************************************

PROCEDURE FilterReopen()

LOCAL nFilter   := Current()[DATA_NFILTER]
LOCAL aFilter   := Current()[DATA_FILTER][nFilter]
LOCAL cFiltExpr := aFilter[DATA_LEXP]
LOCAL bFiltExpr := aFilter[DATA_LBLOCK]

IF nFilter == 1
  DBCLEARFILTER()
ELSE
  DBSETFILTER(bFiltExpr, cFiltExpr)
  Current()[DATA_CENTER] := .N.
  IF !bFiltExpr:EVAL()
    GO TOP
  END
END

RETURN  //  FilterReopen

//***************************************************************************

PROCEDURE FilterDel()

LOCAL nLFilter := Sys()[SYS_RSETUP][SETUP_FILTER]

ADEL(Current()[DATA_FILTER], nLFilter)
ASIZE(Current()[DATA_FILTER], LEN(Current()[DATA_FILTER]) - 1)
DO CASE
CASE nLFilter == Current()[DATA_NFILTER] .AND. ;
  nLFilter > LEN(Current()[DATA_FILTER])
  Current()[DATA_NFILTER] := LEN(Current()[DATA_FILTER])
CASE nLFilter < Current()[DATA_NFILTER]
  Current()[DATA_NFILTER] --
END
FilterReopen()
RefreshSetup()

RETURN  //  FilterDel

//***************************************************************************

PROCEDURE FieldAdd(cName, cAlias, cExpr, cType, nLen, nDec)

LOCAL aDRow  := ARRAY(DATA_FTOTAL)
LOCAL nField := Current()[DATA_FMAX] + 1
LOCAL aArea  := {}
LOCAL aSRow  := {}
LOCAL nAlias := 0
LOCAL nStruc := 0

BEGIN SEQUENCE

  cName := UPPER(ALLTRIM(cName))
  aDRow[DATA_FNAME] := cName
  aDRow[DATA_FNUM]  := nField

  DO CASE
  CASE EMPTY(cAlias) .AND. EMPTY(cExpr)

    nStruc := ASCAN(Current()[DATA_STRUC], ;
      {|aSRow| aSRow[DATA_SNAME] == cName})
    IF EMPTY(nStruc)
      ALERT('Field name ' + cName + ' not found.')
      BREAK
    END
    aSRow := Current()[DATA_STRUC][nStruc]
    aDRow[DATA_FALIAS] := ''
    aDRow[DATA_FEXP]   := ''
    aDRow[DATA_FBLOCK] := FIELDBLOCK(cName)
    aDRow[DATA_FEDIT]  := .Y.
    aDRow[DATA_FTYPE]  := aSRow[DATA_STYPE]
    aDRow[DATA_FLEN]   := aSRow[DATA_SLEN]
    aDRow[DATA_FDEC]   := aSRow[DATA_SDEC]

  CASE !EMPTY(cAlias)

    cAlias := UPPER(ALLTRIM(cAlias))
    nAlias := ASCAN(Data(), {|aArea| aArea[DATA_ALIAS] == cAlias})
    IF EMPTY(nAlias)
      ALERT('Alias ' + cAlias + ' not found.')
      BREAK
    END
    aArea  := Data()[nAlias]
    nStruc := ASCAN(aArea[DATA_STRUC], ;
      {|aSRow| aSRow[DATA_SNAME] == cName})
    IF EMPTY(nStruc)
      ALERT('Field name ' + cName + ' not found in alias ' + cAlias + '.')
      BREAK
    END
    aSRow := aArea[DATA_STRUC][nStruc]
    aDRow[DATA_FALIAS] := cAlias
    aDRow[DATA_FEXP]   := ''
    aDRow[DATA_FBLOCK] := FIELDWBLOCK(cName, aArea[DATA_AREA])
    aDRow[DATA_FEDIT]  := .Y.
    aDRow[DATA_FTYPE]  := aSRow[DATA_STYPE]
    aDRow[DATA_FLEN]   := aSRow[DATA_SLEN]
    aDRow[DATA_FDEC]   := aSRow[DATA_SDEC]

  OTHERWISE

    cExpr := ALLTRIM(cExpr)
    aDRow[DATA_FALIAS] := ''
    aDRow[DATA_FEXP]   := cExpr
    aDRow[DATA_FBLOCK] := BlockStr(cExpr)
    aDRow[DATA_FEDIT]  := .N.
    aDRow[DATA_FTYPE]  := UPPER(LEFT(cType, 1))
    aDRow[DATA_FLEN]   := nLen
    aDRow[DATA_FDEC]   := nDec

  END

  AADD(Current()[DATA_FIELD], ACLONE(aDRow))
  RefreshSetup()
  IF LEN(Current()[DATA_FIELD]) == 1
    Current()[DATA_NFIELD] := 1
  END
  Current()[DATA_FMAX]  := nField
  Current()[DATA_FINIT] := .N.
  EditInit(.N.)

END SEQUENCE

RETURN  //  FieldAdd

//***************************************************************************

PROCEDURE FieldEdit(cName, cAlias, cExpr, cType, nLen, nDec, nField)

LOCAL aDRow  := Current()[DATA_FIELD][nField]
LOCAL aArea  := {}
LOCAL aSRow  := {}
LOCAL nAlias := 0
LOCAL nStruc := 0

BEGIN SEQUENCE

  cName := UPPER(ALLTRIM(cName))
  aDRow[DATA_FNAME] := cName

  DO CASE
  CASE EMPTY(cAlias) .AND. EMPTY(cExpr)

    nStruc := ASCAN(Current()[DATA_STRUC], ;
      {|aSRow| aSRow[DATA_SNAME] == cName})
    IF EMPTY(nStruc)
      ALERT('Field name ' + cName + ' not found.')
      BREAK
    END
    aSRow := Current()[DATA_STRUC][nStruc]
    aDRow[DATA_FALIAS] := ''
    aDRow[DATA_FEXP]   := ''
    aDRow[DATA_FBLOCK] := FIELDBLOCK(cName)
    aDRow[DATA_FEDIT]  := .Y.
    aDRow[DATA_FTYPE]  := aSRow[DATA_STYPE]
    aDRow[DATA_FLEN]   := aSRow[DATA_SLEN]
    aDRow[DATA_FDEC]   := aSRow[DATA_SDEC]

  CASE !EMPTY(cAlias)

    cAlias := UPPER(ALLTRIM(cAlias))
    nAlias := ASCAN(Data(), {|aArea| aArea[DATA_ALIAS] == cAlias})
    IF EMPTY(nAlias)
      ALERT('Alias ' + cAlias + ' not found.')
      BREAK
    END
    aArea  := Data()[nAlias]
    nStruc := ASCAN(aArea[DATA_STRUC], ;
      {|aSRow| aSRow[DATA_SNAME] == cName})
    IF EMPTY(nStruc)
      ALERT('Field name ' + cName + ' not found in alias ' + cAlias + '.')
      BREAK
    END
    aSRow := aArea[DATA_STRUC][nStruc]
    aDRow[DATA_FALIAS] := cAlias
    aDRow[DATA_FEXP]   := ''
    aDRow[DATA_FBLOCK] := FIELDWBLOCK(cName, aArea[DATA_AREA])
    aDRow[DATA_FEDIT]  := .Y.
    aDRow[DATA_FTYPE]  := aSRow[DATA_STYPE]
    aDRow[DATA_FLEN]   := aSRow[DATA_SLEN]
    aDRow[DATA_FDEC]   := aSRow[DATA_SDEC]

  OTHERWISE

    cExpr := ALLTRIM(cExpr)
    aDRow[DATA_FALIAS] := ''
    aDRow[DATA_FEXP]   := cExpr
    aDRow[DATA_FBLOCK] := BlockStr(cExpr)
    aDRow[DATA_FEDIT]  := .N.
    aDRow[DATA_FTYPE]  := cType
    aDRow[DATA_FLEN]   := nLen
    aDRow[DATA_FDEC]   := nDec

  END

  RefreshSetup()
  Current()[DATA_FINIT] := .N.
  EditInit(.N.)

END SEQUENCE

RETURN  //  FieldEdit

//***************************************************************************

PROCEDURE FieldDel()

LOCAL aFieldList := Current()[DATA_FIELD]
LOCAL nLField    := Sys()[SYS_RSETUP][SETUP_FIELD]
LOCAL aLField    := IF(EMPTY(nLField), {}, aFieldList[nLField])
LOCAL nLFieldNo  := IF(EMPTY(aLField), 0, aLField[DATA_FNUM])

ADEL(Current()[DATA_FIELD], nLField)
ASIZE(Current()[DATA_FIELD], LEN(Current()[DATA_FIELD]) - 1)
AEVAL(Current()[DATA_FIELD], {|aRow, nRow, nFOld, nFNew| ;
  nFOld := aRow[DATA_FNUM], ;
  nFNew := nFOld - IF(nFOld < nLFieldNo, 0, 1), ;
  Current()[DATA_FIELD][nRow][DATA_FNUM] := nFNew})
Current()[DATA_FMAX]  --
Current()[DATA_FINIT] := .N.
DO CASE
CASE nLField == Current()[DATA_NFIELD] .AND. ;
  nLField > LEN(Current()[DATA_FIELD])
  Current()[DATA_NFIELD] := LEN(Current()[DATA_FIELD])
CASE nLField < Current()[DATA_NFIELD]
  Current()[DATA_NFIELD] --
END
RefreshSetup()
EditInit(.N.)

RETURN  //  FieldDel

//***************************************************************************

PROCEDURE FieldInit()

LOCAL aStruc  := Current()[DATA_STRUC]
LOCAL aSRow   := {}
LOCAL aDRow   := {}
LOCAL nField  := 1
LOCAL nFields := LEN(aStruc)

ASIZE(Current()[DATA_FIELD], nFields)
FOR nField := 1 TO nFields
  aSRow := aStruc[nField]
  aDRow := ARRAY(DATA_FTOTAL)
  aDRow[DATA_FNAME]  := aSRow[DATA_SNAME]
  aDRow[DATA_FALIAS] := ''
  aDRow[DATA_FEXP]   := ''
  aDRow[DATA_FBLOCK] := FIELDBLOCK(aSRow[DATA_SNAME])
  aDRow[DATA_FEDIT]  := .Y.
  aDRow[DATA_FTYPE]  := aSRow[DATA_STYPE]
  aDRow[DATA_FLEN]   := aSRow[DATA_SLEN]
  aDRow[DATA_FDEC]   := aSRow[DATA_SDEC]
  aDRow[DATA_FNUM]   := nField
  Current()[DATA_FIELD][nField] := ACLONE(aDRow)
NEXT
Current()[DATA_NFIELD] := 1
Current()[DATA_FMAX]   := nFields
Current()[DATA_FINIT]  := .Y.

RefreshSetup()

RETURN  //  FieldInit

//***************************************************************************

STATIC FUNCTION FileRead(nHandle, lRelative, nPos, nSize, bError)

LOCAL cBuffer := SPACE(nSize)
LOCAL nBytes  := 0

FSEEK(nHandle, nPos, IF(!lRelative, FS_SET, FS_RELATIVE))
IF !EMPTY(FERROR())
  bError:EVAL()
END
nBytes := FREAD(nHandle, @cBuffer, nSize)
IF nBytes < nSize
  bError:EVAL()
END

RETURN cBuffer  //  FileRead

//***************************************************************************

FUNCTION MemoClean(cIn)

LOCAL cTran := STRTRAN(cIn, C_SOFT_NEW_LINE)
LOCAL cOut  := ''
LOCAL cChar := ''
LOCAL nPos  := 1
LOCAL nChar := 0
LOCAL nLen  := LEN(cTran)

FOR nPos := 1 TO nLen
  cChar := SUBSTR(cTran, nPos, 1)
  nChar := ASC(cChar)
  DO CASE
  CASE nChar >= 32
    cOut += cChar
  CASE nChar == K_ENTER .AND. SUBSTR(cTran, nPos, 2) == CRLF
    cOut += CRLF
    nPos ++
  END
NEXT

RETURN cOut  //  MemoClean

//***************************************************************************

FUNCTION PrintCodes(lCodes, lInit)

LOCAL cCodes  := ''
LOCAL aCodes  := Sys()[SYS_PRINTCODE]
LOCAL cEsc    := CHR(K_ESC)
LOCAL aReset  := {'', cEsc + 'E'}
LOCAL aPitch  := {'', cEsc + '(s0p10H', cEsc + '(s0p12H', ;
  cEsc + '(s0p16.67H'}
LOCAL aLPI    := {'', cEsc + '&l6D', cEsc + '&l8D'}
LOCAL aOrient := {'', cEsc + '&l0O', cEsc + '&l1O'}
LOCAL aPaper  := {'', cEsc + '&l2A', cEsc + '&l3A'}
LOCAL aTray   := {'', cEsc + '&l8H', cEsc + '&l1H', cEsc + '&l4H', ;
  cEsc + '&l5H', cEsc + '&l2H'}
LOCAL aBin    := {'', cEsc + '&l1G', cEsc + '&l2G'}
LOCAL aDuplex := {'', cEsc + '&l0S', cEsc + '&l1S', cEsc + '&l2S'}
LOCAL aSymbol := {'', cEsc + '(10U', cEsc + '(0U'   }

IF lCodes
  cCodes += aReset[aCodes[PC_RESET ]]
  IF lInit
    cCodes += aBin   [aCodes[PC_BIN   ]]
    cCodes += aTray  [aCodes[PC_TRAY  ]]
    cCodes += aDuplex[aCodes[PC_DUPLEX]]
    cCodes += aPaper [aCodes[PC_PAPER ]]
    cCodes += aOrient[aCodes[PC_ORIENT]]
    cCodes += aSymbol[aCodes[PC_SYMBOL]]
    cCodes += aPitch [aCodes[PC_PITCH ]]
    cCodes += aLPI   [aCodes[PC_LPI   ]]
  END
END

RETURN cCodes  //  PrintCodes

//***************************************************************************

PROCEDURE PrintSet(lSet, cFile, lAdditive)

LOCAL cTrimFile := ''
LOCAL lPrinter  := .N.
LOCAL cSetFile  := ''
LOCAL aPrinter := { ;
  'PRN', 'PRN:', 'LPT1', 'LPT1:', 'LPT2', 'LPT2:', 'LPT3', 'LPT3:', ;
  'AUX', 'AUX:', 'COM1', 'COM1:', 'COM2', 'COM2:', 'COM3', 'COM3:'  }

IF lSet
  cTrimFile := UPPER(ALLTRIM(cFile))
  lPrinter  := !EMPTY(ASCAN(aPrinter, {|cRow| cRow == cTrimFile}))
  cSetFile  := PathAdj(cTrimFile, .Y., '.TXT', .Y.)
  SET(_SET_PRINTFILE, cSetFile, lAdditive)
END
SET(_SET_PRINTER, lSet)
IF !lSet
  SET(_SET_PRINTFILE, '')
  COMMIT
END

RETURN  //  PrintSet

//***************************************************************************

STATIC FUNCTION KSTrans()

LOCAL aSKTrans := {}

AEVAL({ ;
  {'[LB]',    ASC('[')        }, ;
  {'[RB]',    ASC(']')        }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[UP]',    K_UP            }, ;
  {'[DN]',    K_DOWN          }, ;
  {'[LF]',    K_LEFT          }, ;
  {'[RT]',    K_RIGHT         }, ;
  {'[HOME]',  K_HOME          }, ;
  {'[END]',   K_END           }, ;
  {'[PU]',    K_PGUP          }, ;
  {'[PD]',    K_PGDN          }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[^UP]',   K_CTRL_UP       }, ;
  {'[^DN]',   K_CTRL_DOWN     }, ;
  {'[^LF]',   K_CTRL_LEFT     }, ;
  {'[^RT]',   K_CTRL_RIGHT    }, ;
  {'[^HOME]', K_CTRL_HOME     }, ;
  {'[^END]',  K_CTRL_END      }, ;
  {'[^PU]',   K_CTRL_PGUP     }, ;
  {'[^PD]',   K_CTRL_PGDN     }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[*UP]',   K_ALT_UP        }, ;
  {'[*DN]',   K_ALT_DOWN      }, ;
  {'[*LF]',   K_ALT_LEFT      }, ;
  {'[*RT]',   K_ALT_RIGHT     }, ;
  {'[*HOME]', K_ALT_HOME      }, ;
  {'[*END]',  K_ALT_END       }, ;
  {'[*PU]',   K_ALT_PGUP      }, ;
  {'[*PD]',   K_ALT_PGDN      }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[EN]',    K_ENTER         }, ;
  {'[ESC]',   K_ESC           }, ;
  {'[^EN]',   K_CTRL_ENTER    }, ;
  {'[^PS]',   K_CTRL_PRTSCR   }, ;
  {'[*END]',  K_ALT_ENTER     }, ;
  {'[*ESC]',  K_ALT_ESC       }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[*`]',    K_ALT_GRAVE     }, ;
  {'[*-]',    K_ALT_HYPHEN    }, ;
  {'[*=]',    K_ALT_EQUALS    }, ;
  {'[*[]',    K_ALT_LBRACKET  }, ;
  {'[*]]',    K_ALT_RBRACKET  }, ;
  {'[*:]',    K_ALT_COLON     }, ;
  {'[*"]',    K_ALT_QUOTE     }, ;
  {'[*,]',    K_ALT_COMMA     }, ;
  {'[*.]',    K_ALT_PERIOD    }, ;
  {'[*/]',    K_ALT_SLASH     }, ;
  {'[*\]',    K_ALT_BACKSLASH }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[K5]',    KP_5            }, ;
  {'[^K5]',   KP_CTRL_5       }, ;
  {'[^K/]',   KP_CTRL_SLASH   }, ;
  {'[^K*]',   KP_CTRL_ASTERISK}, ;
  {'[^K-]',   KP_CTRL_MINUS   }, ;
  {'[^K+]',   KP_CTRL_PLUS    }, ;
  {'[*K/]',   KP_ALT_SLASH    }, ;
  {'[*K*]',   KP_ALT_ASTERISK }, ;
  {'[*K-]',   KP_ALT_MINUS    }, ;
  {'[*K+]',   KP_ALT_PLUS     }, ;
  {'[*KEN]',  KP_ALT_ENTER    }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[INS]',   K_INS           }, ;
  {'[DEL]',   K_DEL           }, ;
  {'[BS]',    K_BS            }, ;
  {'[TAB]',   K_TAB           }, ;
  {'[/TAB]',  K_SH_TAB        }, ;
  {'[^INS]',  K_CTRL_INS      }, ;
  {'[^DEL]',  K_CTRL_DEL      }, ;
  {'[^BS]',   K_CTRL_BS       }, ;
  {'[^TAB]',  K_CTRL_TAB      }, ;
  {'[*INS]',  K_ALT_INS       }, ;
  {'[*DEL]',  K_ALT_DEL       }, ;
  {'[*BS]',   K_ALT_BS        }, ;
  {'[*TAB]',  K_ALT_TAB       }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[^K]',    K_CTRL_K        }, ;
  {'[^L]',    K_CTRL_L        }, ;
  {'[^N]',    K_CTRL_N        }, ;
  {'[^O]',    K_CTRL_O        }, ;
  {'[^P]',    K_CTRL_P        }, ;
  {'[^Q]',    K_CTRL_Q        }, ;
  {'[^T]',    K_CTRL_T        }, ;
  {'[^U]',    K_CTRL_U        }, ;
  {'[^Y]',    K_CTRL_Y        }, ;
  {'[^2]',    K_CTRL_2        }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[*A]',    K_ALT_A         }, ;
  {'[*B]',    K_ALT_B         }, ;
  {'[*C]',    K_ALT_C         }, ;
  {'[*D]',    K_ALT_D         }, ;
  {'[*E]',    K_ALT_E         }, ;
  {'[*F]',    K_ALT_F         }, ;
  {'[*G]',    K_ALT_G         }, ;
  {'[*H]',    K_ALT_H         }, ;
  {'[*I]',    K_ALT_I         }, ;
  {'[*J]',    K_ALT_J         }, ;
  {'[*K]',    K_ALT_K         }, ;
  {'[*L]',    K_ALT_L         }, ;
  {'[*M]',    K_ALT_M         }, ;
  {'[*N]',    K_ALT_N         }, ;
  {'[*O]',    K_ALT_O         }, ;
  {'[*P]',    K_ALT_P         }, ;
  {'[*Q]',    K_ALT_Q         }, ;
  {'[*R]',    K_ALT_R         }, ;
  {'[*S]',    K_ALT_S         }, ;
  {'[*T]',    K_ALT_T         }, ;
  {'[*U]',    K_ALT_U         }, ;
  {'[*V]',    K_ALT_V         }, ;
  {'[*W]',    K_ALT_W         }, ;
  {'[*X]',    K_ALT_X         }, ;
  {'[*Y]',    K_ALT_Y         }, ;
  {'[*Z]',    K_ALT_Z         }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[*1]',    K_ALT_1         }, ;
  {'[*2]',    K_ALT_2         }, ;
  {'[*3]',    K_ALT_3         }, ;
  {'[*4]',    K_ALT_4         }, ;
  {'[*5]',    K_ALT_5         }, ;
  {'[*6]',    K_ALT_6         }, ;
  {'[*7]',    K_ALT_7         }, ;
  {'[*8]',    K_ALT_8         }, ;
  {'[*9]',    K_ALT_9         }, ;
  {'[*0]',    K_ALT_0         }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[F1]',    K_F1            }, ;
  {'[F2]',    K_F2            }, ;
  {'[F3]',    K_F3            }, ;
  {'[F4]',    K_F4            }, ;
  {'[F5]',    K_F5            }, ;
  {'[F6]',    K_F6            }, ;
  {'[F7]',    K_F7            }, ;
  {'[F8]',    K_F8            }, ;
  {'[F9]',    K_F9            }, ;
  {'[F10]',   K_F10           }, ;
  {'[F11]',   K_F11           }, ;
  {'[F12]',   K_F12           }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[/F1]',   K_SH_F1         }, ;
  {'[/F2]',   K_SH_F2         }, ;
  {'[/F3]',   K_SH_F3         }, ;
  {'[/F4]',   K_SH_F4         }, ;
  {'[/F5]',   K_SH_F5         }, ;
  {'[/F6]',   K_SH_F6         }, ;
  {'[/F7]',   K_SH_F7         }, ;
  {'[/F8]',   K_SH_F8         }, ;
  {'[/F9]',   K_SH_F9         }, ;
  {'[/F10]',  K_SH_F10        }, ;
  {'[/F11]',  K_SH_F11        }, ;
  {'[/F12]',  K_SH_F12        }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[^F1]',   K_CTRL_F1       }, ;
  {'[^F2]',   K_CTRL_F2       }, ;
  {'[^F3]',   K_CTRL_F3       }, ;
  {'[^F4]',   K_CTRL_F4       }, ;
  {'[^F5]',   K_CTRL_F5       }, ;
  {'[^F6]',   K_CTRL_F6       }, ;
  {'[^F7]',   K_CTRL_F7       }, ;
  {'[^F8]',   K_CTRL_F8       }, ;
  {'[^F9]',   K_CTRL_F9       }, ;
  {'[^F10]',  K_CTRL_F10      }, ;
  {'[^F11]',  K_CTRL_F11      }, ;
  {'[^F12]',  K_CTRL_F12      }  }, {|aRow| AADD(aSKTrans, aRow)})

AEVAL({ ;
  {'[*F1]',   K_ALT_F1        }, ;
  {'[*F2]',   K_ALT_F2        }, ;
  {'[*F3]',   K_ALT_F3        }, ;
  {'[*F4]',   K_ALT_F4        }, ;
  {'[*F5]',   K_ALT_F5        }, ;
  {'[*F6]',   K_ALT_F6        }, ;
  {'[*F7]',   K_ALT_F7        }, ;
  {'[*F8]',   K_ALT_F8        }, ;
  {'[*F9]',   K_ALT_F9        }, ;
  {'[*F10]',  K_ALT_F10       }, ;
  {'[*F11]',  K_ALT_F11       }, ;
  {'[*F12]',  K_ALT_F12       }  }, {|aRow| AADD(aSKTrans, aRow)})

RETURN aSKTrans  //  KSTrans

//***************************************************************************

FUNCTION StrKey(aKeyRecord)

LOCAL aSKTrans   := KSTrans()
LOCAL cKeyRecord := ''
LOCAL nKeyRecord := 0
LOCAL nKey       := 0
LOCAL nLB        := ASC('[')
LOCAL nRB        := ASC(']')
LOCAL nSKTrans   := 0

FOR nKeyRecord := 1 TO LEN(aKeyRecord)
  nKey := aKeyRecord[nKeyRecord]
  IF !(nKey < 32 .OR. nKey == 127 .OR. ;
    nKey == nLB .OR. nKey == nRB .OR. nKey > 255)
    cKeyRecord += CHR(nKey)
  ELSE
    nSKTrans := ASCAN(aSKTrans, {|aRow| aRow[2] == nKey})
    IF !EMPTY(nSKTrans)
      cKeyRecord += aSKTrans[nSKTrans][1]
    ELSE
      cKeyRecord += '[#' + ALLTRIM(STR(nKey)) + ']'
    END
  END
NEXT

RETURN cKeyRecord  //  StrKey

//***************************************************************************

FUNCTION KeyStr(cKeyRecord)

LOCAL aSKTrans   := KSTrans()
LOCAL aKeyRecord := {}
LOCAL nKeyRecord := 1
LOCAL nKeyGroup  := 0
LOCAL nKey       := 0
LOCAL nSKTrans   := 0
LOCAL cKeySearch := cKeyRecord + ']'
LOCAL cKeyGroup  := ''
LOCAL cKey       := ''

FOR nKeyRecord := 1 TO LEN(cKeyRecord)
  cKey := SUBSTR(cKeyRecord, nKeyRecord, 1)
  IF !(cKey == '[')
    AADD(aKeyRecord, ASC(cKey))
  ELSE
    nKeyGroup := AT(']', SUBSTR(cKeySearch, nKeyRecord + 1))
    cKeyGroup := UPPER(SUBSTR(cKeySearch, nKeyRecord, nKeyGroup + 1))
    IF !(SUBSTR(cKeyGroup, 2, 1) == '#')
      nSKTrans := ASCAN(aSKTrans, {|aRow| aRow[1] == cKeyGroup})
      IF !EMPTY(nSKTrans)
        AADD(aKeyRecord, aSKTrans[nSKTrans][2])
      END
    ELSE
      nKey := VAL(SUBSTR(cKeyGroup, 3, nKeyGroup - 3))
      AADD(aKeyRecord, nKey)
    END
    nKeyRecord += nKeyGroup
  END
NEXT

RETURN aKeyRecord  //  KeyStr

//***************************************************************************

FUNCTION OrdRecNo(nSetOrder)

LOCAL lData     := !EMPTY(Current())
LOCAL lOrderPos := lData .AND. Current()[DATA_ORDPOS]
LOCAL nOrder    := IF(nSetOrder # NIL, nSetOrder, ORDNUMBER())
LOCAL nRecNo    := RECNO()
LOCAL nOrderPos := IF(!lOrderPos, 0, NTXPOS(nOrder, nRecNo))
LOCAL nOrdRecNo := IF(EMPTY(nOrder), nRecNo, nOrderPos)

RETURN nOrdRecNo  //  OrdRecNo

//***************************************************************************

FUNCTION TableStru()

LOCAL aSStru  := DBSTRUCT()
LOCAL nFields := LEN(aSStru)
LOCAL aDStru  := ARRAY(nFields)
LOCAL nField  := 1
LOCAL nPos    := 1
LOCAL aSRow   := {}
LOCAL aDRow   := {}

FOR nField := 1 TO nFields
  aSRow := aSStru[nField]
  aDRow := ARRAY(DATA_STOTAL)
  aDRow[DATA_SNAME]  := aSRow[DBS_NAME]
  aDRow[DATA_STYPE]  := aSRow[DBS_TYPE]
  aDRow[DATA_SLEN]   := aSRow[DBS_LEN]
  aDRow[DATA_SDEC]   := aSRow[DBS_DEC]
  aDRow[DATA_SNUM]   := nField
  aDRow[DATA_SPOS]   := nPos
  nPos += aSRow[DBS_LEN]
  aDStru[nField] := ACLONE(aDRow)
NEXT

RETURN aDStru  //  TableStru

//***************************************************************************

FUNCTION FreeSpace(cDrive)

LOCAL nDrive  := 0
LOCAL nSpace  := 0
LOCAL cLocal  := Sys()[SYS_LOCAL]
LOCAL bLetter := {|cPath| ;
  !EMPTY(cPath) .AND. (LEN(cPath) == 1 .OR. SUBSTR(cPath, 2, 1) == ':')}
LOCAL lDrive  := bLetter:EVAL(cDrive)
LOCAL lLocal  := bLetter:EVAL(cLocal)

IF !lDrive .AND. lLocal
  cDrive := LEFT(cLocal, 1)
  lDrive := bLetter:EVAL(cDrive)
END
IF lDrive
  IF !EMPTY(DIRECTORY(cDrive + ':\*.*', 'DH'))
    nDrive := ASC(cDrive) - ASC("A") + 1
  ELSE
    ALERT('Drive ' + cDrive + ': not found.')
  END
END
nSpace := DISKSPACE(nDrive)

RETURN nSpace  //  FreeSpace

//***************************************************************************

FUNCTION ExePath()

LOCAL cPath := UPPER(EXE_PATH())

RETURN cPath  //  ExePath

//***************************************************************************

FUNCTION ExeArgs()

LOCAL aDBERef := Sys()[SYS_DARGS]
LOCAL cArgs   := ''

AEVAL(aDBERef, {|cDBERef| IF(EMPTY(cDBERef), NIL, ;
  cArgs := IF(EMPTY(cArgs), '', ' ') + cArgs + cDBERef)})

RETURN cArgs  //  ExeArgs

//***************************************************************************

PROCEDURE CheckError(nCode, cDesc, cFile, lForce)

LOCAL oErr  := ERRORNEW()
LOCAL nDErr := DOSERROR()
LOCAL nFErr := FERROR()
LOCAL nErr  := IF(EMPTY(nDErr), nFErr, nDErr)

IF !EMPTY(nErr) .OR. lForce
  oErr:DESCRIPTION := cDesc + ' error'
  oErr:FILENAME    := cFile
  oErr:OSCODE      := nErr
  oErr:SEVERITY    := 2
  oErr:SUBSYSTEM   := 'DBE'
  oErr:SUBCODE     := nCode
  ERRORBLOCK():EVAL(oErr)
END

RETURN  //  CheckError

//***************************************************************************

FUNCTION PlainDisp(xVal, lTrim)

LOCAL cType  := VALTYPE(xVal)
LOCAL cDisp  := ''

DO CASE
CASE cType == 'C' .OR. cType == 'M'
  IF !lTrim
    cDisp := xVal
  ELSE
    cDisp := RTRIM(xVal)
  END
  IF cType == 'M'
    cDisp := MemoClean(cDisp)
  END
CASE cType == 'N'
  IF !lTrim
    cDisp := STR(xVal)
  ELSE
    cDisp := LTRIM(STR(xVal))
  END
CASE cType == 'D'
  cDisp := DTOC(xVal)
CASE cType == 'L'
  cDisp := TRANSFORM(xVal, 'Y')
END

RETURN cDisp  //  PlainDisp

//***************************************************************************

FUNCTION EvalDisp(xVal, nMaxWidth)

LOCAL cType  := VALTYPE(xVal)
LOCAL cDisp  := '[Unknown]'
LOCAL lFixed := SET(_SET_FIXED)
LOCAL cDelim := [""]

DO CASE
CASE cType == 'A'
  cDisp := '[Array]'
CASE cType == 'B'
  cDisp := '[Block]'
CASE cType == 'C'
  cDelim := IF(["] $ xVal, IF(['] $ xVal, '[]', ['']), [""])
  cDisp := IF(EMPTY(nMaxWidth) .OR. LEN(xVal) <= nMaxWidth - 3, ;
    LEFT(cDelim,1) + xVal + RIGHT(cDelim,1), ;
    LEFT(cDelim,1) + PADR(xVal, nMaxWidth - 3) + RIGHT(cDelim,1) + '...')
CASE cType == 'D'
  cDisp := [CTOD("] + DTOC(xVal) + [")]
CASE cType == 'L'
  cDisp := IF(xVal, '.Y.', '.N.')
CASE cType == 'M'
  cDelim := IF(["] $ xVal, IF(['] $ xVal, '[]', ['']), [""])
  cDisp := IF(EMPTY(nMaxWidth) .OR. LEN(xVal) <= nMaxWidth - 3, ;
    LEFT(cDelim,1) + xVal + RIGHT(cDelim,1), ;
    LEFT(cDelim,1) + PADR(xVal, nMaxWidth - 3) + RIGHT(cDelim,1) + '...')
CASE cType == 'N'
  SET(_SET_FIXED, .N.)
  cDisp := ALLTRIM(STR(xVal))
  SET(_SET_FIXED, lFixed)
CASE cType == 'O'
  cDisp := '[Object type + ' + xVal:CLASSNAME + ']'
CASE cType == 'U'
  cDisp := 'NIL'
CASE cType == 'UE'
  cDisp := 'Syntax error'
END

RETURN cDisp  //  EvalDisp

//***************************************************************************

FUNCTION StrEval(cExpr, nMaxWidth)

LOCAL cBlock := 'EVAL({||' + ALLTRIM(cExpr) + '})'
LOCAL xVal   := &cBlock
LOCAL cDisp  := EvalDispCom(xVal, nMaxWidth)

RETURN cDisp  //  StrEval

//***************************************************************************

FUNCTION EvalDispExp(xVal, nMaxWidth)

LOCAL cType   := VALTYPE(xVal)
LOCAL nIndent := 79 - nMaxWidth
LOCAL nLines  := 0
LOCAL nLine   := 0
LOCAL cIndent := SPACE(nIndent)
LOCAL cDelim  := [""]
LOCAL cDisp   := ''

IF (cType == 'C' .OR. cType == 'M') .AND. ;
  !(EMPTY(nMaxWidth) .OR. LEN(xVal) <= nMaxWidth)
  cDelim := IF(["] $ xVal, IF(['] $ xVal, '[]', ['']), [""])
  nLines := MLCOUNT(xVal, nMaxWidth)
  cDisp  := LEFT(cDelim, 1) + MEMOLINE(xVal, nMaxWidth, 1)
  FOR nLine := 2 TO nLines
    cDisp += CRLF + cIndent + MEMOLINE(xVal, nMaxWidth, nLine)
  NEXT
  cDisp += RIGHT(cDelim, 1)
ELSE
  cDisp := EvalDisp(xVal, nMaxWidth)
END

RETURN cDisp  //  EvalDispExp

//***************************************************************************

FUNCTION EvalDispCom(xVal, nMaxWidth)

LOCAL cType := VALTYPE(xVal)
LOCAL cDisp := IF(cType == 'N' .AND. xVal == INT(xVal), ;
  Commas(xVal), EvalDisp(xVal, nMaxWidth))

RETURN cDisp  //  EvalDispCom

//***************************************************************************

FUNCTION FieldDisp(nField, nMaxWidth)

LOCAL aField    := Current()[DATA_FIELD][nField]
LOCAL nFieldPos := aField[DATA_FNUM]
LOCAL cType     := aField[DATA_FTYPE]
LOCAL nLen      := aField[DATA_FLEN]
LOCAL nDec      := aField[DATA_FDEC]
LOCAL xVal      := aField[DATA_FBLOCK]:EVAL()
LOCAL cDisp     := '[Unknown]'

DO CASE
CASE cType == 'C'
  cDisp := IF(nLen <= nMaxWidth, xVal, LEFT(xVal, nMaxWidth))
CASE cType == 'N'
  cDisp := STR(xVal, nLen, nDec)
CASE cType == 'D'
  cDisp := DTOC(xVal)
CASE cType == 'L'
  cDisp := IF(xVal, 'Y', 'N')
CASE cType == 'M'
  cDisp := IF(EMPTY(xVal), PADR('[memo]', nMaxWidth), ;
    '[Memo] ' + PADR(xVal, nMaxWidth - 10) + '...')
END

RETURN cDisp  //  FieldDisp

//***************************************************************************

FUNCTION BlockStr(cExpr)

LOCAL cTest := '{||' + IF(EMPTY(cExpr), '.Y.', ALLTRIM(cExpr)) + '}'
LOCAL bTest := &cTest

RETURN bTest  //  BlockStr

//***************************************************************************

FUNCTION BlockOdomStr(cExpr)

STATIC bTest := {|| .Y.}

LOCAL cTest := '{||' + IF(EMPTY(cExpr), '.Y.', ALLTRIM(cExpr)) + '}'
LOCAL bOdom := {|| .Y.}

IF EMPTY(cExpr)
  bTest := {|| .Y.}
  bOdom := {|| Odom()}
ELSE
  bTest := &cTest
  bOdom := {|| Odom() .AND. bTest:EVAL()}
END

RETURN bOdom  //  BlockOdomStr

//***************************************************************************

FUNCTION ValBlock(xIn)

LOCAL xOut := IF(VALTYPE(xIn) == 'B', xIn:EVAL(), xIn)

RETURN xOut  //  ValBlock

//***************************************************************************

PROCEDURE QLine(cStr)

QQOUT(IF(cStr == NIL, '', cStr))
QOUT()

RETURN

//***************************************************************************

FUNCTION PathAdj(cInPath, lAddLocal, cExt, lAddExt, lPad)

LOCAL cLocal   := Sys()[SYS_LOCAL]
LOCAL cTLocal  := STRTRAN(cLocal, '/', '\')
LOCAL cOutPath := ALLTRIM(cInPath)
LOCAL cTPath   := STRTRAN(cOutPath, '/', '\')
LOCAL nPos     := 0
LOCAL cFile    := cOutPath

nPos := MAX(MAX(RAT(':', cOutPath), RAT('/', cOutPath)), RAT('\', cOutPath))
IF !EMPTY(nPos)
  cFile := SUBSTR(cOutPath, nPos + 1)
END

IF lAddLocal # NIL
  DO CASE
  CASE lAddLocal .AND. EMPTY(nPos)
    cOutPath := cLocal + cOutPath
  CASE !lAddLocal .AND. !EMPTY(cTLocal) .AND. ;
    UPPER(LEFT(cTPath, LEN(cTLocal))) == UPPER(cTLocal)
    cOutPath := cFile
  END
END

nPos := AT('.', cFile)

IF lAddExt # NIL
  DO CASE
  CASE lAddExt .AND. EMPTY(nPos)
    cOutPath += cExt
  CASE !lAddExt .AND. UPPER(RIGHT(cOutPath, LEN(cExt))) == UPPER(cExt)
    cOutPath := LEFT(cOutPath, LEN(cOutPath) - LEN(cExt))
  END
END

IF !EMPTY(lPad)
  cOutPath := PADR(cOutPath, LEN(cInPath))
END

RETURN cOutPath  //  PathAdj

//***************************************************************************

FUNCTION Transfer(xSetTrans, xSaveItem)

STATIC xCurTrans := NIL

LOCAL xOldTrans := xCurTrans

IF xSetTrans # NIL
  xCurTrans := xSetTrans
  IF xSaveItem # NIL
    McSaveItem(xSaveItem)
  END
END

RETURN xOldTrans  //  Transfer

//***************************************************************************

PROCEDURE SaveHistory(xSave)

LOCAL cType   := VALTYPE(xSave)
LOCAL cSave   := ''
LOCAL nLocate := 0

DO CASE
CASE !EMPTY(ASCAN({'C', 'N', 'D'}, cType))
  cSave    := PlainDisp(xSave, .Y.)
  nLocate  := ASCAN(Sys()[SYS_HISTORY], ;
    {|cLine| ALLTRIM(cLine) == ALLTRIM(cSave)})
  IF !EMPTY(nLocate)
    ADEL(Sys()[SYS_HISTORY], nLocate)
    ASIZE(Sys()[SYS_HISTORY], LEN(Sys()[SYS_HISTORY]) - 1)
  END
  AADD(Sys()[SYS_HISTORY], cSave)
CASE cType == 'A'
  AEVAL(xSave, {|xElement| SaveHistory(xElement)})
END

RETURN  //  SaveHistory

//***************************************************************************

FUNCTION DefaultExt(xSetExt)

STATIC xCurExt := ''

LOCAL xOldExt := xCurExt

IF xSetExt # NIL
  xCurExt := xSetExt
END

RETURN xOldExt  //  DefaultExt

//***************************************************************************

FUNCTION ArrayList(cList, cDelim, nMax)

LOCAL nPos  := 0
LOCAL aList := {}

WHILE !EMPTY(cList)
  IF EMPTY(nMax) .OR. LEN(aList) + 1 < nMax
    nPos := AT(cDelim, cList + cDelim)
  ELSE
    nPos := LEN(cList + cDelim)
  END
  AADD(aList, LEFT(cList, nPos - 1))
  cList := SUBSTR(cList, nPos + 1)
END

IF RIGHT(cList, 1) == cDelim
  AADD(aList, '')
END

RETURN aList  //  ArrayList

//***************************************************************************

FUNCTION ListArray(aList, cDelim)

LOCAL nList := 1
LOCAL cList := ''

FOR nList := 1 TO LEN(aList) - 1
  cList += aList[nList] + cDelim
NEXT
cList += ATAIL(aList)

RETURN cList  //  ListArray

//***************************************************************************

FUNCTION Dots(cInStr, nLen, lLeft)

LOCAL cOutStr := cInStr

IF LEN(cOutStr) > nLen
  IF EMPTY(lLeft)
    cOutStr := LEFT(cOutStr, nLen - 2) + '..'
  ELSE
    cOutStr := '..' + RIGHT(cOutStr, nLen - 2)
  END
END

RETURN cOutStr  //  Dots

//***************************************************************************

FUNCTION Commas(nOrg)

LOCAL nInt  := INT(nOrg)
LOCAL nFrac := nOrg - nInt
LOCAL lFrac := !EMPTY(nFrac)
LOCAL nLen  := LEN(ALLTRIM(STR(nInt)))
LOCAL nDec  := IF(!lFrac, 0, LEN(ALLTRIM(STR(nFrac)))-2)
LOCAL nPic  := 1
LOCAL cPic  := ''
LOCAL cOut  := ''

FOR nPic := 1 TO nLen
  cPic := '9' + cPic
  IF EMPTY(nPic % 3) .AND. nPic < nLen
    cPic := ',' + cPic
  END
NEXT
IF lFrac
  cPic += '.' + REPLICATE('9', nDec)
END

cOut := TRANSFORM(nOrg, cPic)

RETURN cOut  //  Commas

//***************************************************************************

STATIC FUNCTION Occurs(cSearch, cSource)

LOCAL nPos    := 0
LOCAL nCount  := 0
LOCAL cTarget := cSource

DO WHILE !EMPTY(cTarget)
  IF !EMPTY(nPos := AT(cSearch, cTarget))
    nCount  ++
    cTarget := SUBSTR(cTarget, nPos + 1)
  ELSE
    cTarget := ''
  END
ENDDO

RETURN nCount  //  Occurs

//***************************************************************************

STATIC FUNCTION XMLCount(cString, nLength, nTab, lWrap)

LOCAL nLines := 0

nLength := IF(nLength == NIL, 79 , nLength)
nTab    := MIN(IF(nTab == NIL, 4, nTab), nLength - 1)
lWrap   := IF(lWrap == NIL, .Y., lWrap)
nLines  := MLCOUNT(RTRIM(cString), nLength, nTab, lWrap)

RETURN nLines  //  XMLCount

//***************************************************************************

STATIC FUNCTION XMemoLine(cString, nLength, nLine, nTab, lWrap)

LOCAL cLine := ''

nLength := IF(nLength == NIL, 79 , nLength)
nLine   := IF(nLine == NIL, 1, nLine)
nTab    := MIN(IF(nTab == NIL, 4, nTab), nLength - 1)
lWrap   := IF(lWrap == NIL, .Y., lWrap)
cLine   := MEMOLINE(cString, nLength, nLine, nTab, lWrap)

RETURN cLine  //  XMemoLine

//***************************************************************************

STATIC FUNCTION W2Bin(nIn)

LOCAL nBound := 32768
LOCAL nOut   := IF(nIn < nBound, nIn, nIn - 2 * nBound)
LOCAL cOut   := I2BIN(nOut)

RETURN cOut  //  W2Bin

//***************************************************************************

FUNCTION ListBrowse(oBrowse, nOrig, bIndex, nTop, nBottom)

LOCAL cText    := ''
LOCAL nLine    := 0
LOCAL nIndex   := nTop
LOCAL nCols    := oBrowse:COLCOUNT
LOCAL nCol     := 1
LOCAL oCol     := NIL
LOCAL cCol     := ''
LOCAL nMaxLen  := 0
LOCAL nColLen  := 0
LOCAL nColSep  := LEN(oBrowse:COLSEP)
LOCAL aBroHead := {}
LOCAL aColHead := {}
LOCAL aColSep  := {}

FOR nCol := 1 TO nCols
  oCol     := oBrowse:GETCOLUMN(nCol)
  aColHead := ArrayList(oCol:HEADING, ';')
  nMaxLen  := MAX(nMaxLen, LEN(aColHead))
  AADD(aBroHead, ACLONE(aColHead))
  AADD(aColSep, IF(nCol == 1, 0, ;
    IF(oCol:COLSEP == NIL, nColSep, LEN(oCol:COLSEP))))
NEXT
FOR nCol := 1 TO nCols
  aColHead := aBroHead[nCol]
  nColLen  := LEN(aColHead)
  FOR nLine := nColLen + 1 TO nMaxLen
    AADD(aColHead, NIL)
    AINS(aColHead, 1)
    aColHead[1] := ''
  NEXT
NEXT

FOR nLine := 1 TO nMaxLen
  FOR nCol := 1 TO nCols
    oCol  := oBrowse:GETCOLUMN(nCol)
    cCol  := aBroHead[nCol][nLine]
    cText += SPACE(aColSep[nCol]) + PADR(cCol, oCol:WIDTH)
  NEXT
  cText += CRLF
NEXT

cText += CRLF

FOR nIndex := nTop TO nBottom
  bIndex:EVAL(nIndex)
  FOR nCol := 1 TO nCols
    oCol  := oBrowse:GETCOLUMN(nCol)
    cCol  := oCol:BLOCK:EVAL()
    cText += SPACE(aColSep[nCol]) + PADR(cCol, oCol:WIDTH)
  NEXT
  cText += CRLF
NEXT

bIndex:EVAL(nOrig)

RETURN cText  //  ListBrowse

//***************************************************************************

FUNCTION ParseDBE( ;
  cText, nDatabaseNo, lMaxLine, lDraw, nKeyRecord, cKeyRecord)

LOCAL aData       := {}
LOCAL aCurrent    := ARRAY(DATA_TOTAL)
LOCAL lFiltScan   := .N.
LOCAL nLen        := 120
LOCAL nLines      := MLCOUNT(cText, nLen)
LOCAL nLine       := 0
LOCAL cLine       := ''
LOCAL nEqual      := 0
LOCAL cKeyword    := ''
LOCAL cValueL     := ''
LOCAL cValueU     := ''
LOCAL nValue      := 0
LOCAL lValue      := .N.
LOCAL aValue      := {}
LOCAL lHistory    := .N.
LOCAL bScan       := {|aSearch, cSearch, nDefault, nSearch| ;
  nSearch := ASCAN(aSearch, cSearch), IF(EMPTY(nSearch), nDefault, nSearch)}
LOCAL aReset      := {'N', 'Y'}
LOCAL aPitch      := {'N', '10', '12', '16'}
LOCAL aLPI        := {'N', '6', '8'}
LOCAL aOrient     := {'N', 'PORTRAIT', 'LANDSCAPE'}
LOCAL aPaper      := {'N', 'LETTER', 'LEGAL'}
LOCAL aTray       := {'N', '1', '2', '3', '4', 'MANUAL'}
LOCAL aBin        := {'N', 'TOP', 'REAR'}
LOCAL aDuplex     := {'N', 'SIMPLEX', 'DUPLEX LONG', 'DUPLEX SHORT'}
LOCAL aSymbol     := {'N', 'PC', 'ASCII'}

FOR nLine := 1 TO nLines

  cLine  := MEMOLINE(cText, nLen, nLine)
  nEqual := AT('=', cLine)

  DO CASE
  CASE lHistory

    IF !EMPTY(cLine)
      SaveHistory(cLine)
    END

  CASE !EMPTY(nEqual)

    cKeyword := UPPER(STRTRAN(LEFT(cLine, nEqual-1), ' '))
    cValueL  := ALLTRIM(SUBSTR(cLine, nEqual+1))
    cValueU  := UPPER(cValueL)
    nValue   := VAL(cValueU)
    lValue   := 'Y' $ cValueU .OR. 'T' $ cValueU

    IF lFiltScan .AND. cKeyword # 'FILTERNAME'
      lFiltScan := .N.
    END

    DO CASE
    CASE cKeyWord == 'DATABASE'
      AADD(aData, ARRAY(DATA_TOTAL))
      aCurrent := ATAIL(aData)
      aCurrent[DATA_ALIAS]     := NIL
      aCurrent[DATA_TABLEFILE] := cValueU
      aCurrent[DATA_TABLEPATH] := PathAdj(cValueU, .Y.)
      aCurrent[DATA_EXCLUSIVE] := .N.
      aCurrent[DATA_READONLY]  := .N.
      aCurrent[DATA_DRIVER]    := 'DBFNTX'
      aCurrent[DATA_HORIZ]     := .Y.
      aCurrent[DATA_CENTER]    := .N.
      aCurrent[DATA_FSORT]     := .Y.
      aCurrent[DATA_NORDER]    := 1
      aCurrent[DATA_NRELATION] := 0
      aCurrent[DATA_NFILTER]   := 1
      aCurrent[DATA_NFIELD]    := 0
      aCurrent[DATA_RECNOLEN]  := 0
      aCurrent[DATA_BAGFILE]   := {}
      aCurrent[DATA_OATTRIB]   := {{.N., .N.}}
      aCurrent[DATA_RELATION]  := {}
      aCurrent[DATA_FILTER]    := {}
      aCurrent[DATA_FIELD]     := {}
      lFiltScan := .N.
    CASE cKeyWord == 'ALIAS'
      aCurrent[DATA_ALIAS] := cValueU
    CASE cKeyWord == 'EXCLUSIVE'
      aCurrent[DATA_EXCLUSIVE] := lValue
    CASE cKeyWord == 'READONLY'
      aCurrent[DATA_READONLY] := lValue
    CASE cKeyWord == 'DRIVER'
      aCurrent[DATA_DRIVER] := cValueU
    CASE cKeyWord == 'HORIZONTAL'
      aCurrent[DATA_HORIZ] := lValue
    CASE cKeyWord == 'CENTER'
      aCurrent[DATA_CENTER] := lValue
    CASE cKeyWord == 'FIELDSORT'
      aCurrent[DATA_FSORT] := lValue
    CASE cKeyWord == 'INDEX'
      AADD(aCurrent[DATA_BAGFILE], ;
        PathAdj(cValueU, .Y., ORDBAGEXT(), .N.))
      aCurrent[DATA_NORDER] := 2
    CASE cKeyWord == 'RELATION'
      aValue := ArrayList(cValueL, ',', 2)
      IF LEN(aValue) >= 2
        AADD(aCurrent[DATA_RELATION], ;
          {UPPER(ALLTRIM(aValue[1])), ALLTRIM(aValue[2])})
      END
    CASE cKeyWord == 'FILTER'
      AADD(aCurrent[DATA_FILTER], {'', cValueU})
      lFiltScan := .Y.
    CASE cKeyWord == 'FILTERNAME' .AND. lFiltScan
      ATAIL(aCurrent[DATA_FILTER])[1] := cValueU
      lFiltScan := .N.
    CASE cKeyWord == 'FIELD'
      AADD(aCurrent[DATA_FIELD], {cValueU,,,,,})
    CASE cKeyWord == 'FIELDALIASED'
      aValue := ArrayList(cValueL, ',', 2)
      IF LEN(aValue) >= 2
        AADD(aCurrent[DATA_FIELD], ;
          {UPPER(ALLTRIM(aValue[2])), UPPER(ALLTRIM(aValue[1])),,,,})
      END
    CASE cKeyWord == 'FIELDCALCULATED'
      aValue := ArrayList(cValueL, ',', 5)
      IF LEN(aValue) >= 5
        AADD(aCurrent[DATA_FIELD], {UPPER(ALLTRIM(aValue[1])),, aValue[5], ;
          UPPER(ALLTRIM(aValue[2])), VAL(aValue[3]), VAL(aValue[4])})
      END
    CASE cKeyWord == 'RECORDNO'
      aCurrent[DATA_RECNOLEN] := nValue
    CASE cKeyWord == 'INDEXNO'
      aCurrent[DATA_NORDER] := nValue
    CASE cKeyWord == 'RELATIONNO'
      aCurrent[DATA_NRELATION] := nValue
    CASE cKeyWord == 'FILTERNO'
      aCurrent[DATA_NFILTER] := nValue
    CASE cKeyWord == 'FIELDNO'
      aCurrent[DATA_NFIELD] := nValue
    CASE cKeyWord == 'DATABASENO'
      nDatabaseNo := nValue
    CASE cKeyWord == 'MAXLINES'
      lMaxLine := lValue
      lDraw := .Y.
    CASE cKeyWord == 'LOCAL'
      Sys()[SYS_LOCAL] := cValueU + IF(RIGHT(cValueU, 1) == '\', '', '\')
    CASE cKeyWord == 'KEYSTROKERECORDING'
      nKeyRecord := IF(!lValue, KR_OFF, KR_RECORD)
    CASE cKeyWord == 'HOTKEYSHIFTF1'
      Sys()[SYS_HOTKEY][HK_SH_F1 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF2'
      Sys()[SYS_HOTKEY][HK_SH_F2 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF3'
      Sys()[SYS_HOTKEY][HK_SH_F3 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF4'
      Sys()[SYS_HOTKEY][HK_SH_F4 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF5'
      Sys()[SYS_HOTKEY][HK_SH_F5 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF6'
      Sys()[SYS_HOTKEY][HK_SH_F6 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF7'
      Sys()[SYS_HOTKEY][HK_SH_F7 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF8'
      Sys()[SYS_HOTKEY][HK_SH_F8 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF9'
      Sys()[SYS_HOTKEY][HK_SH_F9 ] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF10'
      Sys()[SYS_HOTKEY][HK_SH_F10] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF11'
      Sys()[SYS_HOTKEY][HK_SH_F11] := cValueL
    CASE cKeyWord == 'HOTKEYSHIFTF12'
      Sys()[SYS_HOTKEY][HK_SH_F12] := cValueL
    CASE cKeyWord == 'COLORMAINSCREEN'
      Sys()[SYS_COLOR][DC_MAIN    ] := cValueU
      lDraw := .Y.
    CASE cKeyWord == 'COLOREDITCELL'
      Sys()[SYS_COLOR][DC_EDIT    ] := cValueU
    CASE cKeyWord == 'COLORMEMOVIEW'
      Sys()[SYS_COLOR][DC_MEMOVIEW] := cValueU
    CASE cKeyWord == 'COLORMENUS'
      Sys()[SYS_COLOR][DC_MENU    ] := cValueU
    CASE cKeyWord == 'COLORHOTKEYMENUS'
      Sys()[SYS_COLOR][DC_HOTMENU ] := cValueU
    CASE cKeyWord == 'COLORGETBOXES'
      Sys()[SYS_COLOR][DC_GETBOX  ] := cValueU
    CASE cKeyWord == 'COLORTABLES'
      Sys()[SYS_COLOR][DC_TABLE   ] := cValueU
    CASE cKeyWord == 'COLORPICKLISTS'
      Sys()[SYS_COLOR][DC_LIST    ] := cValueU
    CASE cKeyWord == 'COLORSTATUSBOXES'
      Sys()[SYS_COLOR][DC_STATUS  ] := cValueU
    CASE cKeyWord == 'COLORFILEVIEW'
      Sys()[SYS_COLOR][DC_FILEVIEW] := cValueU
    CASE cKeyWord == 'COLORMEMVARBROWSE'
      Sys()[SYS_COLOR][DC_MVBROWSE] := cValueU
    CASE cKeyWord == 'COLORMEMVAREDIT'
      Sys()[SYS_COLOR][DC_MVEDIT  ] := cValueU
    CASE cKeyWord == 'COLORMEMVARSTATUS'
      Sys()[SYS_COLOR][DC_MVSTATUS] := cValueU
    CASE cKeyWord == 'COLORHELPSCREEN'
      Sys()[SYS_COLOR][DC_HELP    ] := cValueU
    CASE cKeyWord == 'COLORQUITBOX'
      Sys()[SYS_COLOR][DC_QUIT    ] := cValueU
    CASE cKeyWord == 'CURSORINSERT'
      Sys()[SYS_ICURSOR] := nValue
    CASE cKeyWord == 'CURSOROVERSTRIKE'
      Sys()[SYS_OCURSOR] := nValue
    CASE cKeyWord == 'SMALLSETUPLENGTH'
      Sys()[SYS_SSLINES] := MIN(MAX(nValue, 1), 16)
      lDraw := .Y.
    CASE cKeyWord == 'LARGESETUPLENGTH'
      Sys()[SYS_LSLINES] := MIN(MAX(nValue, 1), 41)
      lDraw := .Y.
    CASE cKeyWord == 'PRINTRESET'
      Sys()[SYS_PRINTCODE][PC_RESET ] := bScan:EVAL(aReset , cValueU, 2)
    CASE cKeyWord == 'PRINTPITCH'
      Sys()[SYS_PRINTCODE][PC_PITCH ] := bScan:EVAL(aPitch , cValueU, 1)
    CASE cKeyWord == 'PRINTLPI'
      Sys()[SYS_PRINTCODE][PC_LPI   ] := bScan:EVAL(aLPI   , cValueU, 1)
    CASE cKeyWord == 'PRINTORIENT'
      Sys()[SYS_PRINTCODE][PC_ORIENT] := bScan:EVAL(aOrient, cValueU, 1)
    CASE cKeyWord == 'PRINTPAPER'
      Sys()[SYS_PRINTCODE][PC_PAPER ] := bScan:EVAL(aPaper , cValueU, 1)
    CASE cKeyWord == 'PRINTTRAY'
      Sys()[SYS_PRINTCODE][PC_TRAY  ] := bScan:EVAL(aTray  , cValueU, 1)
    CASE cKeyWord == 'PRINTBIN'
      Sys()[SYS_PRINTCODE][PC_BIN   ] := bScan:EVAL(aBin   , cValueU, 1)
    CASE cKeyWord == 'PRINTDUPLEX'
      Sys()[SYS_PRINTCODE][PC_DUPLEX] := bScan:EVAL(aDuplex, cValueU, 1)
    CASE cKeyWord == 'PRINTSYMBOL'
      Sys()[SYS_PRINTCODE][PC_SYMBOL] := bScan:EVAL(aSymbol, cValueU, 1)
    CASE cKeyWord == 'DATEFORMAT'
      SET(_SET_DATEFORMAT, cValueU)
    CASE cKeyWord == 'EPOCH'
      SET(_SET_EPOCH, nValue)
    CASE cKeyWord == 'DECIMALS'
      SET(_SET_DECIMALS, nValue)
    CASE cKeyWord == 'MEMOWIDTH'
      Sys()[SYS_MEMOWIDTH] := nValue
    CASE cKeyWord == 'MEMOTAB'
      Sys()[SYS_MEMOTAB] := nValue
    CASE cKeyWord == 'MARGIN'
      SET(_SET_MARGIN, nValue)
    CASE cKeyWord == 'DELETED'
      SET(_SET_DELETED, lValue)
    CASE cKeyWord == 'SOFTSEEK'
      SET(_SET_SOFTSEEK, lValue)
    CASE cKeyWord == 'ODOMETER'
      Sys()[SYS_ODOMETER][ODOM_ROLL] := nValue
    CASE cKeyWord == 'PLAY'
      cKeyRecord := cValueL
    CASE cKeyWord == 'HISTORY'
      lHistory := .Y.
    END

  END

NEXT

RETURN aData  //  ParseDBE

//***************************************************************************

FUNCTION GenDBE(lDBEData, lDBEPos, lDBESet, lDBEDef, lDBEHist, cKeyRecord)

LOCAL cText    := ''
LOCAL nArea    := 1
LOCAL aCurrent := {}
LOCAL aList    := {}
LOCAL aItem    := {}
LOCAL nItem    := 1
LOCAL aHistory := Sys()[SYS_HISTORY]
LOCAL lData    := !EMPTY(Current())
LOCAL aReset   := {'N', 'Y'}
LOCAL aPitch   := {'N', '10', '12', '16'}
LOCAL aLPI     := {'N', '6', '8'}
LOCAL aOrient  := {'N', 'Portrait', 'Landscape'}
LOCAL aPaper   := {'N', 'Letter', 'Legal'}
LOCAL aTray    := {'N', '1', '2', '3', '4', 'Manual'}
LOCAL aBin     := {'N', 'Top', 'Rear'}
LOCAL aDuplex  := {'N', 'Simplex', 'Duplex Long', 'Duplex Short'}
LOCAL aSymbol  := {'N', 'PC', 'ASCII'}
LOCAL bCon     := {|cKeyWord, xValue| ;
  cKeyword + ' = ' + PlainDisp(xValue, .Y.) + CRLF}
LOCAL bDef     := {|cKeyWord, xValue, xDefault| ;
  IF(!lDBEDef .AND. xValue == xDefault, '', ;
  cKeyword + ' = ' + PlainDisp(xValue, .Y.) + CRLF)}
LOCAL bDefArr  := {|cKeyWord, aValue, nValue, nDefault| ;
  IF(!lDBEDef .AND. nValue == nDefault, '', ;
  cKeyword + ' = ' + aValue[nValue] + CRLF)}

IF lDBEData

  FOR nArea := 1 TO LEN(Data())
    aCurrent := Data()[nArea]
    cText += ;
      bCon:EVAL('Database   ', aCurrent[DATA_TABLEFILE]) + ;
      bDef:EVAL('Exclusive  ', aCurrent[DATA_EXCLUSIVE], .N.) + ;
      bDef:EVAL('Readonly   ', aCurrent[DATA_READONLY], .N.) + ;
      bDef:EVAL('Driver     ', aCurrent[DATA_DRIVER], 'DBFNTX')
    aList := aCurrent[DATA_BAGFILE]
    FOR nItem := 1 TO LEN(aList)
      cText += bCon:EVAL('Index      ', aList[nItem])
    NEXT
    aList := aCurrent[DATA_RELATION]
    FOR nItem := 1 TO LEN(aList)
      aItem := aList[nItem]
      cText += bCon:EVAL('Relation   ', ;
        aItem[DATA_RALIAS] + ', ' + aItem[DATA_REXP])
    NEXT
    aList := aCurrent[DATA_FILTER]
    FOR nItem := 2 TO LEN(aList)
      aItem := aList[nItem]
      cText += bCon:EVAL('Filter     ', aItem[DATA_LEXP])
      IF !(ALLTRIM(aItem[DATA_LEXP]) == ALLTRIM(aItem[DATA_LNAME]))
        cText += bCon:EVAL('Filter Name', aItem[DATA_LNAME])
      END
    NEXT
    IF !aCurrent[DATA_FINIT]
      aList := aCurrent[DATA_FIELD]
      FOR nItem := 1 TO LEN(aList)
        aItem := aList[nItem]
        DO CASE
        CASE EMPTY(aItem[DATA_FALIAS]) .AND. EMPTY(aItem[DATA_FEXP])
          cText += bCon:EVAL('Field     ', aItem[DATA_FNAME])
        CASE !EMPTY(aItem[DATA_FALIAS])
          cText += bCon:EVAL('Field Aliased   ', ;
            aItem[DATA_FALIAS] + ', ' + aItem[DATA_FNAME])
        OTHERWISE
          cText += bCon:EVAL('Field Calculated', ;
            aItem[DATA_FNAME] + ', ' + ;
            aItem[DATA_FTYPE] + ', ' + ;
            ALLTRIM(STR(aItem[DATA_FLEN])) + ', ' + ;
            ALLTRIM(STR(aItem[DATA_FDEC])) + ', ' + ;
            aItem[DATA_FEXP])
        END
      NEXT
    END
    IF lDBEPos
      cText += ;
        bDef:EVAL('Horizontal ', aCurrent[DATA_HORIZ ], .Y.) + ;
        bDef:EVAL('Center     ', aCurrent[DATA_CENTER], .N.) + ;
        bDef:EVAL('Field Sort ', aCurrent[DATA_FSORT ], .Y.) + ;
        bCon:EVAL('Record No  ', (aCurrent[DATA_AREA])->(RECNO())) + ;
        bCon:EVAL('Index No   ', aCurrent[DATA_NORDER   ]) + ;
        bCon:EVAL('Relation No', aCurrent[DATA_NRELATION]) + ;
        bCon:EVAL('Filter No  ', aCurrent[DATA_NFILTER  ]) + ;
        bCon:EVAL('Field No   ', aCurrent[DATA_NFIELD   ])
    END
    cText += CRLF
  NEXT

  IF !lDBESet
    cText += ;
        bDef:EVAL('Local      ', Sys()[SYS_LOCAL], SysDefault()[SYS_LOCAL])
  END

END

IF lDBESet

  cText += ;
    bCon:EVAL('Database No        ', IF(!lData, 0, Sys()[SYS_SELECT])) + ;
    bCon:EVAL('Max Lines          ', MAXROW() > 24) + ;
    bDef:EVAL('Local              ', Sys()[SYS_LOCAL], ;
      SysDefault()[SYS_LOCAL]) + ;
    bDef:EVAL('Keystroke Recording', McSet(MCSET_KEYRECORD) == KR_RECORD, ;
      .N.)
  cText += ;
    bDef:EVAL('Hot Key Shift F1   ', Sys()[SYS_HOTKEY][HK_SH_F1 ], '') + ;
    bDef:EVAL('Hot Key Shift F2   ', Sys()[SYS_HOTKEY][HK_SH_F2 ], '') + ;
    bDef:EVAL('Hot Key Shift F3   ', Sys()[SYS_HOTKEY][HK_SH_F3 ], '') + ;
    bDef:EVAL('Hot Key Shift F4   ', Sys()[SYS_HOTKEY][HK_SH_F4 ], '') + ;
    bDef:EVAL('Hot Key Shift F5   ', Sys()[SYS_HOTKEY][HK_SH_F5 ], '') + ;
    bDef:EVAL('Hot Key Shift F6   ', Sys()[SYS_HOTKEY][HK_SH_F6 ], '') + ;
    bDef:EVAL('Hot Key Shift F7   ', Sys()[SYS_HOTKEY][HK_SH_F7 ], '') + ;
    bDef:EVAL('Hot Key Shift F8   ', Sys()[SYS_HOTKEY][HK_SH_F8 ], '') + ;
    bDef:EVAL('Hot Key Shift F9   ', Sys()[SYS_HOTKEY][HK_SH_F9 ], '') + ;
    bDef:EVAL('Hot Key Shift F10  ', Sys()[SYS_HOTKEY][HK_SH_F10], '') + ;
    bDef:EVAL('Hot Key Shift F11  ', Sys()[SYS_HOTKEY][HK_SH_F11], '') + ;
    bDef:EVAL('Hot Key Shift F12  ', Sys()[SYS_HOTKEY][HK_SH_F12], '')
  cText += ;
    bDef:EVAL('Color Main Screen  ', Sys()[SYS_COLOR][DC_MAIN    ], ;
      SysDefault()[SYS_COLOR][DC_MAIN    ]) + ;
    bDef:EVAL('Color Edit Cell    ', Sys()[SYS_COLOR][DC_EDIT    ], ;
      SysDefault()[SYS_COLOR][DC_EDIT    ]) + ;
    bDef:EVAL('Color Memo View    ', Sys()[SYS_COLOR][DC_MEMOVIEW], ;
      SysDefault()[SYS_COLOR][DC_MEMOVIEW]) + ;
    bDef:EVAL('Color Menus        ', Sys()[SYS_COLOR][DC_MENU    ], ;
      SysDefault()[SYS_COLOR][DC_MENU    ]) + ;
    bDef:EVAL('Color Hotkey Menus ', Sys()[SYS_COLOR][DC_HOTMENU ], ;
      SysDefault()[SYS_COLOR][DC_HOTMENU ]) + ;
    bDef:EVAL('Color Get Boxes    ', Sys()[SYS_COLOR][DC_GETBOX  ], ;
      SysDefault()[SYS_COLOR][DC_GETBOX  ]) + ;
    bDef:EVAL('Color Tables       ', Sys()[SYS_COLOR][DC_TABLE   ], ;
      SysDefault()[SYS_COLOR][DC_TABLE   ]) + ;
    bDef:EVAL('Color Pick Lists   ', Sys()[SYS_COLOR][DC_LIST    ], ;
      SysDefault()[SYS_COLOR][DC_LIST    ]) + ;
    bDef:EVAL('Color Status Boxes ', Sys()[SYS_COLOR][DC_STATUS  ], ;
      SysDefault()[SYS_COLOR][DC_STATUS  ]) + ;
    bDef:EVAL('Color File View    ', Sys()[SYS_COLOR][DC_FILEVIEW], ;
      SysDefault()[SYS_COLOR][DC_FILEVIEW]) + ;
    bDef:EVAL('Color Memvar Browse', Sys()[SYS_COLOR][DC_MVBROWSE], ;
      SysDefault()[SYS_COLOR][DC_MVBROWSE]) + ;
    bDef:EVAL('Color Memvar Edit  ', Sys()[SYS_COLOR][DC_MVEDIT  ], ;
      SysDefault()[SYS_COLOR][DC_MVEDIT  ]) + ;
    bDef:EVAL('Color Memvar Status', Sys()[SYS_COLOR][DC_MVSTATUS], ;
      SysDefault()[SYS_COLOR][DC_MVSTATUS]) + ;
    bDef:EVAL('Color Help Screen  ', Sys()[SYS_COLOR][DC_HELP    ], ;
      SysDefault()[SYS_COLOR][DC_HELP    ]) + ;
    bDef:EVAL('Color Quit Box     ', Sys()[SYS_COLOR][DC_QUIT    ], ;
      SysDefault()[SYS_COLOR][DC_QUIT    ])
  cText += ;
    bDef:EVAL('Cursor Insert      ', Sys()[SYS_ICURSOR], ;
      SysDefault()[SYS_ICURSOR]) + ;
    bDef:EVAL('Cursor Overstrike  ', Sys()[SYS_OCURSOR], ;
      SysDefault()[SYS_OCURSOR])
  cText += ;
    bDef:EVAL('Small Setup Length ', Sys()[SYS_SSLINES], ;
      SysDefault()[SYS_SSLINES]) + ;
    bDef:EVAL('Large Setup Length ', Sys()[SYS_LSLINES], ;
      SysDefault()[SYS_LSLINES])
  cText += ;
    bDefArr:EVAL('Print Reset        ', aReset , ;
      Sys()[SYS_PRINTCODE][PC_RESET ], ;
      SysDefault()[SYS_PRINTCODE][PC_RESET ]) + ;
    bDefArr:EVAL('Print Pitch        ', aPitch , ;
      Sys()[SYS_PRINTCODE][PC_PITCH ], ;
      SysDefault()[SYS_PRINTCODE][PC_PITCH ]) + ;
    bDefArr:EVAL('Print Lpi          ', aLPI   , ;
      Sys()[SYS_PRINTCODE][PC_LPI   ], ;
      SysDefault()[SYS_PRINTCODE][PC_LPI   ]) + ;
    bDefArr:EVAL('Print Orient       ', aOrient, ;
      Sys()[SYS_PRINTCODE][PC_ORIENT], ;
      SysDefault()[SYS_PRINTCODE][PC_ORIENT]) + ;
    bDefArr:EVAL('Print Paper        ', aPaper , ;
      Sys()[SYS_PRINTCODE][PC_PAPER ], ;
      SysDefault()[SYS_PRINTCODE][PC_PAPER ]) + ;
    bDefArr:EVAL('Print Tray         ', aTray  , ;
      Sys()[SYS_PRINTCODE][PC_TRAY  ], ;
      SysDefault()[SYS_PRINTCODE][PC_TRAY  ]) + ;
    bDefArr:EVAL('Print Bin          ', aBin   , ;
      Sys()[SYS_PRINTCODE][PC_BIN   ], ;
      SysDefault()[SYS_PRINTCODE][PC_BIN   ]) + ;
    bDefArr:EVAL('Print Duplex       ', aDuplex, ;
      Sys()[SYS_PRINTCODE][PC_DUPLEX], ;
      SysDefault()[SYS_PRINTCODE][PC_DUPLEX]) + ;
    bDefArr:EVAL('Print Symbol       ', aSymbol, ;
      Sys()[SYS_PRINTCODE][PC_SYMBOL], ;
      SysDefault()[SYS_PRINTCODE][PC_SYMBOL])
  cText += ;
    bDef:EVAL('Date Format        ', SET(_SET_DATEFORMAT), ;
      SetDefault()[_SET_DATEFORMAT]) + ;
    bDef:EVAL('Epoch              ', SET(_SET_EPOCH), ;
      SetDefault()[_SET_EPOCH]) + ;
    bDef:EVAL('Decimals           ', SET(_SET_DECIMALS), ;
      SetDefault()[_SET_DECIMALS]) + ;
    bDef:EVAL('Memo Width         ', Sys()[SYS_MEMOWIDTH], ;
      SysDefault()[SYS_MEMOWIDTH]) + ;
    bDef:EVAL('Memo Tab           ', Sys()[SYS_MEMOTAB], ;
      SysDefault()[SYS_MEMOTAB]) + ;
    bDef:EVAL('Margin             ', SET(_SET_MARGIN), ;
      SetDefault()[_SET_MARGIN]) + ;
    bDef:EVAL('Deleted            ', SET(_SET_DELETED), ;
      SetDefault()[_SET_DELETED]) + ;
    bDef:EVAL('Softseek           ', SET(_SET_SOFTSEEK), ;
      SetDefault()[_SET_SOFTSEEK]) + ;
    bDef:EVAL('Odometer           ', Sys()[SYS_ODOMETER][ODOM_ROLL], ;
      SysDefault()[SYS_ODOMETER][ODOM_ROLL])

END

IF !EMPTY(cKeyRecord)
  cText += bCon:EVAL('Play   ', cKeyRecord)
END

IF lDBEHist
  cText += bCon:EVAL('History', '')
  FOR nItem := 1 TO LEN(aHistory)
    cText += aHistory[nItem] + CRLF
  NEXT
END

RETURN cText  //  GenDBE

//***************************************************************************

PROCEDURE ReadMem(cMemFile)

LOCAL cMemPath   := PathAdj(cMemFile, .Y., '.MEM', .Y.)
LOCAL aDirectory := DIRECTORY(cMemPath, 'H')
LOCAL lFile      := !EMPTY(aDirectory)
LOCAL nHandle    := IF(!lFile, -1, FOPEN(cMemPath))
LOCAL nPos       := 0
LOCAL cVar       := ''
LOCAL cChar      := ''
LOCAL cType      := ''
LOCAL lRead      := .Y.
LOCAL lVar       := .N.
LOCAL aType      := {'C', 'N', 'D', 'L', 'B'}
LOCAL xVal       := NIL
LOCAL bError     := {|| lRead := .N., BREAK(NIL)}

BEGIN SEQUENCE

  IF !lFile
    BREAK
  END

  WHILE lRead
    cChar := FileRead(nHandle, .N., nPos, 1, bError)
    nPos  ++
    DO CASE
    CASE ISALPHA(cChar) .OR. cChar == '_' .OR. ;
      (ISDIGIT(cChar) .AND. !EMPTY(cVar))
      cVar += cChar
    CASE !EMPTY(cVar)
      lVar := .N.
      WHILE !EMPTY(cVar) .AND. !lVar
        cType := TYPE(cVar)
        IF !ISDIGIT(cVar) .AND. !EMPTY(ASCAN(aType, cType))
          lVar := .Y.
        ELSE
          cVar := SUBSTR(cVar, 2)
        END
      END
      IF lVar
        xVal := MEMVARBLOCK(cVar):EVAL()
        AADD(Sys()[SYS_MEMVAR], ;
          {cVar, cType, IF(cType == 'C', LEN(xVal), 0), xVal})
      END
      cVar := ''
    END
  END

END SEQUENCE

FCLOSE(nHandle)

ASORT(Sys()[SYS_MEMVAR],,, {|aLine1, aLine2| aLine1[1] < aLine2[1]})
FOR nPos := 2 TO LEN(Sys()[SYS_MEMVAR])
  IF Sys()[SYS_MEMVAR][nPos][1] == Sys()[SYS_MEMVAR][nPos - 1][1]
    ADEL(Sys()[SYS_MEMVAR], nPos)
    ASIZE(Sys()[SYS_MEMVAR], LEN(Sys()[SYS_MEMVAR]) - 1)
    nPos --
  END
NEXT

RETURN  //  ReadMem

//***************************************************************************

FUNCTION ReadFRM(cFRMFile, lEval)

LOCAL cFRMPath   := PathAdj(cFRMFile, .Y.)
LOCAL aDirectory := DIRECTORY(cFRMPath, 'H')
LOCAL lFile      := !EMPTY(aDirectory) .OR. lEval
LOCAL nHandle    := IF(!lFile, -1, FOPEN(cFRMPath))
LOCAL bError     := {|| IF(!lEval, NIL, ;
  CheckError(EG_READ, 'File format', cFRMPath, .Y.)), BREAK(NIL)}
LOCAL nExpSize   := 0
LOCAL cFields    := ''
LOCAL cParams    := ''
LOCAL cExprs     := ''
LOCAL cOffsets   := ''
LOCAL cLengths   := ''
LOCAL nOffset    := 12
LOCAL cBuffer    := SPACE(RBSIZ_FILE)
LOCAL cGroup     := ''
LOCAL cSubGroup  := ''
LOCAL nColumns   := 0
LOCAL nColumn    := 0
LOCAL nBytes     := 0
LOCAL nPointer   := 0
LOCAL nOption    := 0
LOCAL bExp       := {|| NIL}
LOCAL aReport    := ARRAY(RP_COUNT)
LOCAL aHeader    := {}
LOCAL nHeader    := 4

aReport[RP_HEADER]   := {'', '', '', ''}
aReport[RP_WIDTH]    := 80
aReport[RP_LMARGIN]  := 8
aReport[RP_RMARGIN]  := 0
aReport[RP_LINES]    := 58
aReport[RP_DBLSPACE] := .N.
aReport[RP_BEJECT]   := .N.
aReport[RP_AEJECT]   := .Y.
aReport[RP_PLAIN]    := .N.
aReport[RP_SUMMARY]  := .N.
aReport[RP_COLUMNS]  := {}
aReport[RP_GROUPS]   := ARRAY(2, RG_COUNT)
aReport[RP_GROUPS][1][RG_TEXT  ] := ''
aReport[RP_GROUPS][1][RG_HEADER] := ''
aReport[RP_GROUPS][1][RG_AEJECT] := .N.
aReport[RP_GROUPS][2][RG_TEXT  ] := ''
aReport[RP_GROUPS][2][RG_HEADER] := ''
aReport[RP_GROUPS][2][RG_AEJECT] := .N.

BEGIN SEQUENCE

  IF !lFile
    BREAK
  END

  CheckError(EG_OPEN, 'Open', cFRMPath, .N.)
  cBuffer := FileRead(nHandle, .N., 0, RBSIZ_FILE, bError)
  FCLOSE(nHandle)

  IF !(BIN2W(SUBSTR(cBuffer, 1, 2)) == 2 .AND.;
    BIN2W(SUBSTR(cBuffer, RBSIZ_FILE - 1, 2)) == 2)
    bError:EVAL()
  END

  nExpSize := BIN2W(SUBSTR(cBuffer, RBOFF_EXPSIZE, 2))
  cLengths := SUBSTR(cBuffer, RBOFF_LENGTHS, RBSIZ_LENGTHS)
  cOffsets := SUBSTR(cBuffer, RBOFF_OFFSETS, RBSIZ_OFFSETS)
  cExprs   := SUBSTR(cBuffer, RBOFF_EXPRS  , RBSIZ_EXPRS  )
  cFields  := SUBSTR(cBuffer, RBOFF_FIELDS , RBSIZ_FIELDS )
  cParams  := SUBSTR(cBuffer, RBOFF_PARAMS , RBSIZ_PARAMS )

  aReport[RP_WIDTH   ] := BIN2W(SUBSTR(cParams, RBOFF_PGWIDTH , 2))
  aReport[RP_LINES   ] := BIN2W(SUBSTR(cParams, RBOFF_PGLINES , 2))
  aReport[RP_LMARGIN ] := BIN2W(SUBSTR(cParams, RBOFF_LMARGIN , 2))
  aReport[RP_RMARGIN ] := BIN2W(SUBSTR(cParams, RBOFF_RMARGIN , 2))
  nColumns             := BIN2W(SUBSTR(cParams, RBOFF_COLCOUNT, 2))
  aReport[RP_DBLSPACE] := ;
    (UPPER(SUBSTR(cParams, RBOFF_DBLSPACE, 1)) $ 'YT')
  aReport[RP_SUMMARY ] := ;
    (UPPER(SUBSTR(cParams, RBOFF_SUMMARY , 1)) $ 'YT')

  nOption := ASC(SUBSTR(cParams, RBOFF_OPTION, 1))
  aReport[RP_PLAIN ] := !EMPTY(INT(nOption / 4) % 2)
  aReport[RP_AEJECT] := !EMPTY(INT(nOption / 2) % 2)
  aReport[RP_BEJECT] := !EMPTY(INT(nOption / 1) % 2)

  nPointer := BIN2W(SUBSTR(cParams, RBOFF_PGHDR, 2))
  aHeader  := PARSEHEADER( ;
    FRMExpGet(nPointer, cExprs, cOffsets, cLengths), nHeader)

  ASIZE(aHeader, nHeader)
  AEVAL(aHeader, {|cRow, nRow| IF(cRow # NIL, NIL, aHeader[nRow] := '')})
  aReport[RP_HEADER] := aHeader

  nPointer := BIN2W(SUBSTR(cParams, RBOFF_GRPEXP, 2))
  cGroup   := FRMExpGet(nPointer, cExprs, cOffsets, cLengths)

  IF EMPTY(cGroup)
    aReport[RP_GROUPS][1][RG_TEXT  ] := ''
    aReport[RP_GROUPS][1][RG_HEADER] := ''
    aReport[RP_GROUPS][1][RG_AEJECT] := .N.
  ELSE
    aReport[RP_GROUPS][1][RG_TEXT  ] := cGroup
    nPointer := BIN2W(SUBSTR(cParams, RBOFF_GRPHDR, 2))
    aReport[RP_GROUPS][1][RG_HEADER] := ;
      FRMExpGet(nPointer, cExprs, cOffsets, cLengths)
    aReport[RP_GROUPS][1][RG_AEJECT] := ;
      (UPPER(SUBSTR(cParams, RBOFF_AEJECT, 1)) $ 'YT')
  END
  IF lEval
    bExp := BlockStr(cGroup)
    aReport[RP_GROUPS][1][RG_EXP ] := bExp
    aReport[RP_GROUPS][1][RG_TYPE] := VALTYPE(bExp:EVAL())
  END

  nPointer  := BIN2W(SUBSTR(cParams, RBOFF_SUBEXP, 2))
  cSubGroup := FRMExpGet(nPointer, cExprs, cOffsets, cLengths)

  IF EMPTY(cSubGroup)
    aReport[RP_GROUPS][2][RG_TEXT  ] := ''
    aReport[RP_GROUPS][2][RG_HEADER] := ''
  ELSE
    aReport[RP_GROUPS][2][RG_TEXT  ] := cSubGroup
    nPointer := BIN2W(SUBSTR(cParams, RBOFF_SUBHDR, 2))
    aReport[RP_GROUPS][2][RG_HEADER] := ;
      FRMExpGet(nPointer, cExprs, cOffsets, cLengths)
  END
  aReport[RP_GROUPS][2][RG_AEJECT] := .N.
  IF lEval
    bExp := BlockStr(cSubGroup)
    aReport[RP_GROUPS][2][RG_EXP ] := bExp
    aReport[RP_GROUPS][2][RG_TYPE] := VALTYPE(bExp:EVAL())
  END

  FOR nColumn := 1 TO nColumns
    AADD(aReport[RP_COLUMNS], FRMColGet(cFields, cExprs, ;
      cOffsets, cLengths, @nOffset, lEval))
  NEXT

RECOVER

  IF lEval
    aReport := {}
  END
  FCLOSE(nHandle)

END SEQUENCE

RETURN aReport  //  ReadFRM

//***************************************************************************

STATIC FUNCTION FRMExpGet(nPointer, cExprs, cOffsets, cLengths)

LOCAL nExpOff := 0
LOCAL nExpLen := 0
LOCAL cString := ''
LOCAL nOffOff := nPointer * 2 + 1

IF nPointer < 65535
  nExpOff := BIN2W(SUBSTR(cOffsets, nOffOff, 2)) + 1
  nExpLen := BIN2W(SUBSTR(cLengths, nOffOff, 2)) - 1
  cString := SUBSTR(cExprs, nExpOff, nExpLen)
  IF LEFT(cString, 1) == CHR(0)
    cString := ''
  END
END

RETURN cString  //  FRMExpGet

//***************************************************************************

STATIC FUNCTION FRMColGet(cFields, cExprs, ;
  cOffsets, cLengths, nOffset, lEval)

LOCAL nPointer := 0
LOCAL nNumber  := 0
LOCAL aColumn  := ARRAY(RC_COUNT)
LOCAL nLen     := 0
LOCAL nDec     := 0
LOCAL lTotal   := .N.
LOCAL cText    := ''
LOCAL cType    := ''
LOCAL cPict    := ''
LOCAL bText    := {|| NIL}
LOCAL aText    := {}

nLen     := BIN2W(SUBSTR(cFields, nOffset + RBOFF_FWIDTH  , 2))
lTotal   := UPPER(SUBSTR(cFields, nOffset + RBOFF_FTOTAL, 1)) $ 'YT'
nDec     := BIN2W(SUBSTR(cFields, nOffset + RBOFF_FDEC    , 2))
nPointer := BIN2W(SUBSTR(cFields, nOffset + RBOFF_FCONTENT, 2))
cText    := FRMExpGet(nPointer, cExprs, cOffsets, cLengths)

aColumn[RC_TEXT    ] := cText
aColumn[RC_WIDTH   ] := nLen
aColumn[RC_DECIMALS] := nDec
aColumn[RC_TOTAL   ] := lTotal

IF lEval
  bText := BlockStr(cText)
  cType := VALTYPE(bText:EVAL())
  DO CASE
  CASE cType = 'C' .OR. cType = 'M'
    cPict := REPLICATE('X', nLen)
  CASE cType = 'D'
    cPict := '@D'
  CASE cType = 'N'
    cPict := IF(EMPTY(nDec), REPLICATE('9', nLen), ;
      REPLICATE('9', nLen - nDec - 1) + '.' + REPLICATE('9', nDec))
  CASE cType = 'L'
    cPict := '@L' + REPLICATE('X', nLen - 1)
  CASE cType = 'A'
    aText := bText:EVAL()
    bText := BlockStr(aText[1])
    cType := VALTYPE(bText:EVAL())
    cPict := aText[2]
  END
  aColumn[RC_EXP ] := bText
  aColumn[RC_TYPE] := cType
  aColumn[RC_PICT] := cPict
END

nPointer             := BIN2W(SUBSTR(cFields, nOffset + RBOFF_FHEADER , 2))
cText                := FRMExpGet(nPointer, cExprs, cOffsets, cLengths)
aColumn[RC_HEADTEXT] := cText
aColumn[RC_HEADER]   := ArrayList(cText, ';')

nOffset += 12

RETURN aColumn  //  FRMColGet

//***************************************************************************

PROCEDURE WriteFRM(cFRMFile, aReport)

LOCAL cFRMPath  := PathAdj(cFRMFile, .Y.)
LOCAL nHandle   := 0
LOCAL nPointer  := 0
LOCAL nColumn   := 1
LOCAL nMarker   := 0
LOCAL nOffset   := 12
LOCAL nExpSize  := 0
LOCAL cHeader   := ''
LOCAL cGroup    := ''
LOCAL cNull     := CHR(0)
LOCAL cBuffer   := REPLICATE(cNull, RBSIZ_FILE)
LOCAL cLengths  := REPLICATE(cNull, RBSIZ_LENGTHS)
LOCAL cOffsets  := REPLICATE(cNull, RBSIZ_OFFSETS)
LOCAL cExprs    := REPLICATE(cNull, RBSIZ_EXPRS  )
LOCAL cFields   := REPLICATE(cNull, RBSIZ_FIELDS )
LOCAL cParams   := REPLICATE(cNull, RBSIZ_PARAMS )
LOCAL nOption   := ;
  IF(aReport[RP_PLAIN ], 1, 0) * 4 + ;
  IF(aReport[RP_AEJECT], 1, 0) * 2 + ;
  IF(aReport[RP_BEJECT], 1, 0) * 1
LOCAL aHeader   := aReport[RP_HEADER]
LOCAL nHeader   := 4
LOCAL aGroups   := aReport[RP_GROUPS]
LOCAL aColumns  := aReport[RP_COLUMNS]
LOCAL nColumns  := LEN(aColumns)
LOCAL aGroup    := {}
LOCAL aColumn   := {}

cParams  := STUFF(cParams, RBOFF_PGWIDTH , 2, W2Bin(aReport[RP_WIDTH  ]))
cParams  := STUFF(cParams, RBOFF_PGLINES , 2, W2Bin(aReport[RP_LINES  ]))
cParams  := STUFF(cParams, RBOFF_LMARGIN , 2, W2Bin(aReport[RP_LMARGIN]))
cParams  := STUFF(cParams, RBOFF_RMARGIN , 2, W2Bin(aReport[RP_RMARGIN]))
nColumns := LEN(aReport[RP_COLUMNS])
cParams  := STUFF(cParams, RBOFF_COLCOUNT, 2, W2Bin(nColumns))
cParams  := STUFF(cParams, RBOFF_DBLSPACE, 1, ;
  TRANSFORM(aReport[RP_DBLSPACE], 'Y'))
cParams  := STUFF(cParams, RBOFF_SUMMARY , 1, ;
  TRANSFORM(aReport[RP_SUMMARY ], 'Y'))
cParams  := STUFF(cParams, RBOFF_OPTION, 1, CHR(nOption))

ASIZE(aHeader, nHeader)
WHILE !EMPTY(nHeader) .AND. EMPTY(aHeader[nHeader])
  nHeader --
END
ASIZE(aHeader, nHeader)
AEVAL(aHeader, {|cLine, nLine| cHeader += ;
  IF(nLine == 1, '', ';') + IF(cLine == NIL, '', ALLTRIM(cLine))})
nMarker := FRMExpPut(@nPointer, @nExpSize, ;
  @cExprs, @cOffsets, @cLengths, cHeader, .N.)
cParams := STUFF(cParams, RBOFF_PGHDR, 2, W2Bin(nMarker))

aGroup  := aGroups[1]
cGroup  := aGroup[RG_TEXT]
nMarker := FRMExpPut(@nPointer, @nExpSize, ;
  @cExprs, @cOffsets, @cLengths, cGroup, .N.)
cParams := STUFF(cParams, RBOFF_GRPEXP, 2, W2Bin(nMarker))
cGroup  := aGroup[RG_HEADER]
nMarker := FRMExpPut(@nPointer, @nExpSize, ;
  @cExprs, @cOffsets, @cLengths, cGroup, .N.)
cParams := STUFF(cParams, RBOFF_GRPHDR, 2, W2Bin(nMarker))
cParams := STUFF(cParams, RBOFF_AEJECT, 1, ;
  TRANSFORM(aGroup[RG_AEJECT], 'Y'))

aGroup  := aGroups[2]
cGroup  := aGroup[RG_TEXT]
nMarker := FRMExpPut(@nPointer, @nExpSize, ;
  @cExprs, @cOffsets, @cLengths, cGroup, .N.)
cParams := STUFF(cParams, RBOFF_SUBEXP, 2, W2Bin(nMarker))
cGroup  := aGroup[RG_HEADER]
nMarker := FRMExpPut(@nPointer, @nExpSize, ;
  @cExprs, @cOffsets, @cLengths, cGroup, .Y.)
cParams := STUFF(cParams, RBOFF_SUBHDR, 2, W2Bin(nMarker))

FOR nColumn := 1 TO nColumns
  aColumn := aColumns[nColumn]
  FRMColPut(@cFields, @cExprs, @cOffsets, @cLengths, ;
    @nOffset, @nPointer, @nExpSize, aColumn)
NEXT

cBuffer := STUFF(cBuffer, 1             , 2            , W2Bin(2))
cBuffer := STUFF(cBuffer, RBSIZ_FILE - 1, 2            , W2Bin(2))
cBuffer := STUFF(cBuffer, RBOFF_EXPSIZE , 2            , W2Bin(nExpSize))
cBuffer := STUFF(cBuffer, RBOFF_LENGTHS , RBSIZ_LENGTHS, cLengths)
cBuffer := STUFF(cBuffer, RBOFF_OFFSETS , RBSIZ_OFFSETS, cOffsets)
cBuffer := STUFF(cBuffer, RBOFF_EXPRS   , RBSIZ_EXPRS  , cExprs  )
cBuffer := STUFF(cBuffer, RBOFF_FIELDS  , RBSIZ_FIELDS , cFields )
cBuffer := STUFF(cBuffer, RBOFF_PARAMS  , RBSIZ_PARAMS , cParams )

BEGIN SEQUENCE

  nHandle := FCREATE(cFRMPath)
  CheckError(EG_CREATE, 'Create', cFRMPath, .N.)
  FWRITE(nHandle, cBuffer, RBSIZ_FILE)
  CheckError(EG_WRITE, 'Write', cFRMPath, .N.)
  FCLOSE(nHandle)

END SEQUENCE

RETURN  //  WriteFRM

//***************************************************************************

STATIC FUNCTION FRMExpPut(nPointer, nExpSize, ;
  cExprs, cOffsets, cLengths, cString, lEmptyWrite)

LOCAL nExpOff    := 1
LOCAL nExpLen    := 0
LOCAL nOffOff    := nPointer * 2 + 1
LOCAL nMarker    := 65535
LOCAL nExpOffPre := 1
LOCAL nExpLenPre := 0
LOCAL nOffOffPre := nOffOff - 2
LOCAL lWrite     := !EMPTY(cString) .OR. lEmptyWrite

IF lWrite
  IF nPointer > 0
    nExpOffPre := BIN2W(SUBSTR(cOffsets, nOffOffPre, 2)) + 1
    nExpLenPre := BIN2W(SUBSTR(cLengths, nOffOffPre, 2)) - 1
    nExpOff    := nExpOffPre + nExpLenPre + 1
  END
  cOffsets := STUFF(cOffsets, nOffOff, 2, W2Bin(nExpOff - 1))
  nExpLen  := LEN(cString)
  cLengths := STUFF(cLengths, nOffOff, 2, W2Bin(nExpLen + 1))
  cExprs   := STUFF(cExprs  , nExpOff, nExpLen, cString)
  nExpSize := nExpOff + nExpLen
  nMarker  := nPointer
  nPointer ++
END


RETURN nMarker  //  FRMExpPut

//***************************************************************************

STATIC PROCEDURE FRMColPut(cFields, cExprs, cOffsets, cLengths, ;
  nOffset, nPointer, nExpSize, aColumn)

LOCAL nNumber  := 0
LOCAL cText    := ''
LOCAL nMarker  := 0

cFields := STUFF(cFields, nOffset + RBOFF_FWIDTH, 2, ;
  W2Bin(aColumn[RC_WIDTH]))
cFields := STUFF(cFields, nOffset + RBOFF_FTOTAL, 1, ;
  TRANSFORM(aColumn[RC_TOTAL], 'Y'))
cFields := STUFF(cFields, nOffset + RBOFF_FDEC  , 2, ;
  W2Bin(aColumn[RC_DECIMALS]))

cText   := aColumn[RC_TEXT]
nMarker := FRMExpPut(@nPointer, @nExpSize, ;
  @cExprs, @cOffsets, @cLengths, cText, .Y.)
cFields := STUFF(cFields, nOffset + RBOFF_FCONTENT, 2, W2Bin(nMarker))

cText   := aColumn[RC_HEADTEXT]
nMarker := FRMExpPut(@nPointer, @nExpSize, ;
  @cExprs, @cOffsets, @cLengths, cText, .N.)
cFields := STUFF(cFields, nOffset + RBOFF_FHEADER , 2, W2Bin(nMarker))

nOffset += 12

RETURN  //  FRMColPut

//***************************************************************************

FUNCTION ListFRM(cFRMFile, aHeading, aCol, ;
  cGrpHead, cGrpExp, lGrpSum, lGrpEject, cSubHead, cSubExp, ;
  nSetWidth, nLMargin, nRMargin, nColWidth, nTotWidth, nVLines, ;
  lDouble, lBEject, lAEject, lPlain, lSummary)

LOCAL cText    := ''
LOCAL nPad     := 22
LOCAL nIndent  := nPad + 2
LOCAL cIndent  := SPACE(nIndent)
LOCAL bYesNo   := {|lYesNo| IF(lYesNo, 'Yes', 'No')}
LOCAL nCol     := 1
LOCAL nCols    := LEN(aCol)
LOCAL aColRow  := {}

cText += 'REPORT FILE ' + cFRMFile + CRLF + ;
  CRLF
cText += 'Heading' + CRLF + ;
  CRLF
cText += ;
  aHeading[1] + CRLF + ;
  aHeading[2] + CRLF + ;
  aHeading[3] + CRLF + ;
  aHeading[4] + CRLF + ;
  CRLF
cText += 'Parameters' + CRLF + ;
  CRLF
cText += ;
  PADR(PADL('Set page width:'       , nPad), nIndent) + ;
  STR(nSetWidth, 3) + CRLF + ;
  PADR(PADL('Left margin:'          , nPad), nIndent) + ;
  STR(nLMargin, 3) + CRLF + ;
  PADR(PADL('Right margin:'         , nPad), nIndent) + ;
  STR(nRMargin, 3) + CRLF + ;
  PADR(PADL('Computed column width:', nPad), nIndent) + ;
  STR(nColWidth, 3) + CRLF + ;
  PADR(PADL('Computed total width:' , nPad), nIndent) + ;
  STR(nTotWidth, 3) + CRLF + ;
  PADR(PADL('Lines per page:'       , nPad), nIndent) + ;
  STR(nVLines, 3) + CRLF + ;
  PADR(PADL('Double space:'         , nPad), nIndent) + ;
  bYesNo:EVAL(lDouble) + CRLF + ;
  PADR(PADL('Eject before print:'   , nPad), nIndent) + ;
  bYesNo:EVAL(lBEject) + CRLF + ;
  PADR(PADL('Eject after print:'    , nPad), nIndent) + ;
  bYesNo:EVAL(lAEject) + CRLF + ;
  PADR(PADL('Plain print:'          , nPad), nIndent) + ;
  bYesNo:EVAL(lPlain) + CRLF + ;
  CRLF
cText += 'Group' + CRLF + ;
  CRLF
cText += ;
  PADR(PADL('Group heading:'        , nPad), nIndent) + ;
  ALLTRIM(cGrpHead) + CRLF + ;
  PADR(PADL('Group expression:'     , nPad), nIndent) + ;
  ALLTRIM(cGrpExp) + CRLF + ;
  PADR(PADL('Summary only:'         , nPad), nIndent) + ;
  bYesNo:EVAL(lGrpSum) + CRLF + ;
  PADR(PADL('Eject after group:'    , nPad), nIndent) + ;
  bYesNo:EVAL(lGrpEject) + CRLF + ;
  PADR(PADL('Subgroup heading:'     , nPad), nIndent) + ;
  ALLTRIM(cSubHead) + CRLF + ;
  PADR(PADL('Subgroup expression:'  , nPad), nIndent) + ;
  ALLTRIM(cSubExp) + CRLF + ;
  CRLF
cText += 'Columns' + CRLF + ;
  CRLF

FOR nCol := 1 TO nCols
  aColRow := ACLONE(aCol[nCol])
  cText   += 'Column ' + STR(nCol, 3) + ':' + CRLF + ;
    CRLF
  cText   += ;
    PADR(PADL('  Heading:', nPad), nIndent) + ;
    aColRow[RC_HEADER][1] + CRLF + ;
    cIndent + aColRow[RC_HEADER][2] + CRLF + ;
    cIndent + aColRow[RC_HEADER][3] + CRLF + ;
    cIndent + aColRow[RC_HEADER][4] + CRLF + ;
    PADR(PADL('  Expression:', nPad), nIndent) + ;
    aColRow[RC_TEXT  ] + CRLF + ;
    PADR(PADL('  Length:'    , nPad), nIndent) + ;
    STR(aColRow[RC_WIDTH   ], 3) + CRLF + ;
    PADR(PADL('  Decimals:'  , nPad), nIndent) + ;
    STR(aColRow[RC_DECIMALS], 3) + CRLF + ;
    PADR(PADL('  Total:'     , nPad), nIndent) + ;
    bYesNo:EVAL(aColRow[RC_TOTAL   ]) + CRLF + ;
    CRLF
NEXT

RETURN cText  //  ListFRM

//***************************************************************************

PROCEDURE RunFRM(cFRMName, bFor, bWhile, nNext, nRecord, lRest, ;
  lPlain, cHeading, lBEject, lSummary)

LOCAL aReport    := {}
LOCAL aRepTotal  := {}
LOCAL aGrpTotal  := {}
LOCAL nPage      := 1
LOCAL nLLines    := 0
LOCAL nLMaxLines := 0
LOCAL nCol       := 1
LOCAL nGroup     := 1
LOCAL xBreak     := NIL
LOCAL lBroke     := .N.
LOCAL lTotal     := .N.
LOCAL lSubTotal  := .N.
LOCAL lFirst     := .Y.
LOCAL lFeed      := .Y.
LOCAL bEject     := {|| IF(lFeed, __Eject(), NIL)}

IF cHeading == NIL
  cHeading := ''
END

BEGIN SEQUENCE

  aReport := ReadFRM(cFRMName, .Y.)
  IF EMPTY(aReport)
    BREAK
  END
  nLMaxLines := aReport[RP_LINES]
  IF lSummary
    aReport[RP_SUMMARY] := lSummary
  END
  IF lBEject # NIL .AND. lBEject
    aReport[RP_BEJECT] := .N.
  END
  IF lPlain
    aReport[RP_PLAIN] := .Y.
    cHeading          := ''
    lFeed             := .N.
  END
  aReport[RP_HEADING] := cHeading
  IF EMPTY(aReport[RP_GROUPS][2][RG_TEXT])
    ASIZE(aReport[RP_GROUPS], 1)
  END
  IF EMPTY(aReport[RP_GROUPS][1][RG_TEXT])
    ASIZE(aReport[RP_GROUPS], 0)
  END
  nLLines := aReport[RP_LINES]
  IF aReport[RP_BEJECT]
    bEject:EVAL()
  END

  FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)

  aRepTotal := ARRAY(LEN(aReport[RP_GROUPS]) + 1, LEN(aReport[RP_COLUMNS]))

  FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
    IF aReport[RP_COLUMNS][nCol][RC_TOTAL]
      FOR nGroup := 1 TO LEN(aRepTotal)
        aRepTotal[nGroup][nCol] := 0
      NEXT
    END
  NEXT

  aGrpTotal := ARRAY(LEN(aReport[RP_GROUPS]))

  DBEVAL({|| FRMLine(aReport, @nPage, @nLLines, @nLMaxLines, ;
    @aRepTotal, @aGrpTotal, lFirst, lFeed)}, ;
    bFor, bWhile, nNext, nRecord, lRest)

  FOR nGroup := LEN(aReport[RP_GROUPS]) TO 1 STEP -1
    lSubTotal := .N.
    FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
      lSubTotal := lSubTotal .OR. aReport[RP_COLUMNS][nCol][RC_TOTAL]
    NEXT
    IF lSubTotal
      IF nLLines < 2
        bEject:EVAL()
        IF aReport[RP_PLAIN]
          nLLines := 1000
        ELSE
          FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)
        END
      END
      QLine(SPACE(aReport[RP_LMARGIN]) + ;
        IF(nGroup == 1, '** Subtotal **', '* Subsubtotal *'))
      QQOUT(SPACE(aReport[RP_LMARGIN]))
      FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
        IF nCol > 1
          QQOUT(' ')
        END
        IF aReport[RP_COLUMNS][nCol][RC_TOTAL]
          QQOUT(TRANSFORM(aRepTotal[nGroup+1][nCol], ;
            aReport[RP_COLUMNS][nCol][RC_PICT]))
        ELSE
          QQOUT(SPACE(aReport[RP_COLUMNS][nCol][RC_WIDTH]))
        END
      NEXT
      QOUT()
    END
  NEXT

  lTotal := .N.
  FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
    lTotal := lTotal .OR. aReport[RP_COLUMNS][nCol][RC_TOTAL]
  NEXT
  IF lTotal
    QLine(SPACE(aReport[RP_LMARGIN]) + '*** Total ***')
    QQOUT(SPACE(aReport[RP_LMARGIN]))
    FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
      IF nCol > 1
        QQOUT(' ')
      END
      IF aReport[RP_COLUMNS][nCol][RC_TOTAL]
        QQOUT(TRANSFORM(aRepTotal[1][nCol], ;
          aReport[RP_COLUMNS][nCol][RC_PICT]))
      ELSE
        QQOUT(SPACE(aReport[RP_COLUMNS][nCol][RC_WIDTH]))
      END
    NEXT nCol
    QOUT()
  END
  IF aReport[RP_AEJECT]
    bEject:EVAL()
  END

RECOVER USING xBreak

  lBroke := .Y.

END SEQUENCE

IF lBroke
  BREAK xBreak
END

RETURN  //  RunFRM

//***************************************************************************

STATIC PROCEDURE FRMLine(aReport, nPage, nLLines, nLMaxLines, ;
  aRepTotal, aGrpTotal, lFirst, lFeed)

LOCAL aHeader    := {}
LOCAL aRecord    := {}
LOCAL nCol       := 1
LOCAL nGroup     := 1
LOCAL nMaxLines  := 0
LOCAL nLine      := 1
LOCAL cLine      := ''
LOCAL lGrpChange := .N.
LOCAL lGrpEject  := .N.
LOCAL lSubTotal  := .N.
LOCAL bEject     := {|| IF(lFeed, __Eject(), NIL)}

FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
  IF aReport[RP_COLUMNS][nCol][RC_TOTAL]
    aRepTotal[1][nCol] += aReport[RP_COLUMNS][nCol][RC_EXP]:EVAL()
  END
NEXT

IF !lFirst
  FOR nGroup := LEN(aReport[RP_GROUPS]) TO 1 STEP -1
    lSubTotal := .N.
    FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
      lSubTotal := lSubTotal .OR. aReport[RP_COLUMNS][nCol][RC_TOTAL]
    NEXT
    IF lSubTotal
      IF PlainDisp(aReport[RP_GROUPS][1][RG_EXP]:EVAL(), .N.) # aGrpTotal[1]
        lGrpChange := .Y.
      END
      IF lGrpChange .OR. ;
        PlainDisp(aReport[RP_GROUPS][nGroup][RG_EXP]:EVAL(), .N.) # ;
        aGrpTotal[nGroup]
        AADD(aHeader, IF(nGroup == 1, '** Subtotal **', '* Subsubtotal *'))
        AADD(aHeader, '')
        FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
          IF aReport[RP_COLUMNS][nCol][RC_TOTAL]
            aHeader[LEN(aHeader)] += ;
              TRANSFORM(aRepTotal[nGroup+1][nCol], ;
              aReport[RP_COLUMNS][nCol][RC_PICT])
            aRepTotal[nGroup+1][nCol] := 0
          ELSE
            aHeader[LEN(aHeader)] += ;
              SPACE(aReport[RP_COLUMNS][nCol][RC_WIDTH])
          END
          aHeader[LEN(aHeader)] += ' '
        NEXT
        aHeader[LEN(aHeader)] := ;
          LEFT(aHeader[LEN(aHeader)], ;
          LEN(aHeader[LEN(aHeader)]) - 1)
      END
    END
  NEXT
END

IF !EMPTY(aHeader) .AND. lGrpEject .AND. lGrpChange
  IF LEN(aHeader) > nLLines
    bEject:EVAL()
    IF (aReport[RP_PLAIN])
      nLLines := 1000
    ELSE
      FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)
    END
  END
  AEVAL(aHeader, {|cHLine| QLine(SPACE(aReport[RP_LMARGIN]) + cHLine)})
  aHeader := {}
  bEject:EVAL()
  IF aReport[RP_PLAIN]
    nLLines := 1000
  ELSE
    FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)
  END
END

FOR nGroup := 1 TO LEN(aReport[RP_GROUPS])
  IF !(PlainDisp(aReport[RP_GROUPS][nGroup][RG_EXP]:EVAL(), .N.) == ;
    aGrpTotal[nGroup])
    AADD(aHeader, '')
    AADD(aHeader, IF(nGroup == 1, '** ', '* ') + ;
    aReport[RP_GROUPS][nGroup][RG_HEADER] + ' ' + ;
    PlainDisp(aReport[RP_GROUPS][nGroup][RG_EXP]:EVAL(), .N.))
  END
NEXT

IF !EMPTY(aHeader)
  IF LEN(aHeader) > nLLines
    bEject:EVAL()
    IF aReport[RP_PLAIN]
      nLLines := 1000
    ELSE
      FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)
    END
  END
  AEVAL(aHeader, {|cHLine| QLine(SPACE(aReport[RP_LMARGIN]) + cHLine)})
  nLLines -= LEN(aHeader)
  IF EMPTY(nLLines)
    bEject:EVAL()
    IF aReport[RP_PLAIN]
      nLLines := 1000
    ELSE
      FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)
    END
  END
END

FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
  IF aReport[RP_COLUMNS][nCol][RC_TOTAL]
    FOR nGroup := 1 TO LEN(aRepTotal) - 1
      aRepTotal[nGroup+1][nCol] += ;
        aReport[RP_COLUMNS][nCol][RC_EXP]:EVAL()
    NEXT
  END
NEXT

FOR nGroup := 1 TO LEN(aReport[RP_GROUPS])
  aGrpTotal[nGroup] := ;
    PlainDisp(aReport[RP_GROUPS][nGroup][RG_EXP]:EVAL(), .N.)
NEXT

IF !aReport[RP_SUMMARY]

  nMaxLines := 1

  FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
    DO CASE
    CASE aReport[RP_COLUMNS][nCol][RC_TYPE] = 'M'
      nMaxLines := MAX(XMLCount(aReport[RP_COLUMNS][nCol][RC_EXP]:EVAL(), ;
        aReport[RP_COLUMNS][nCol][RC_WIDTH]), nMaxLines)
    CASE aReport[RP_COLUMNS][nCol][RC_TYPE] = 'C'
      nMaxLines := MAX(XMLCount(STRTRAN( ;
        aReport[RP_COLUMNS][nCol][RC_EXP]:EVAL(), ';', CRLF), ;
        aReport[RP_COLUMNS][nCol][RC_WIDTH]), nMaxLines)
    END
  NEXT

  ASIZE(aRecord, nMaxLines)
  AFILL(aRecord, '')

  FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
    FOR nLine := 1 TO nMaxLines
      DO CASE
      CASE aReport[RP_COLUMNS][nCol][RC_TYPE] = 'M'
        cLine := XMemoLine(RTRIM(aReport[RP_COLUMNS][nCol][RC_EXP]:EVAL()), ;
          aReport[RP_COLUMNS][nCol][RC_WIDTH], nLine)
        cLine := PADR(cLine, aReport[RP_COLUMNS][nCol][RC_WIDTH])
      CASE aReport[RP_COLUMNS][nCol][RC_TYPE] = 'C'
        cLine := XMemoLine(RTRIM(STRTRAN( ;
          aReport[RP_COLUMNS][nCol][RC_EXP]:EVAL(), ';', CRLF)), ;
          aReport[RP_COLUMNS][nCol][RC_WIDTH], nLine)
        cLine := PADR(cLine, aReport[RP_COLUMNS][nCol][RC_WIDTH])
      CASE nLine == 1
        cLine := TRANSFORM(aReport[RP_COLUMNS][nCol][RC_EXP]:EVAL(), ;
          aReport[RP_COLUMNS][nCol][RC_PICT])
        cLine := PADR(cLine, aReport[RP_COLUMNS][nCol][RC_WIDTH])
      OTHERWISE
        cLine := SPACE(aReport[RP_COLUMNS][nCol][RC_WIDTH])
      END
      IF nCol > 1
        aRecord[nLine] += ' '
      END
      aRecord[nLine] += cLine
    NEXT
  NEXT

  IF LEN(aRecord) > nLLines
    IF LEN(aRecord) > nLMaxLines
      nLine := 1
      WHILE nLine < LEN(aRecord)
        QLine(SPACE(aReport[RP_LMARGIN]) + aRecord[nLine])
        nLine ++
        nLLines --
        IF nLLines == 0
          bEject:EVAL()
          IF aReport[RP_PLAIN]
            nLLines := 1000
          ELSE
            FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)
          END
        END
      END
    ELSE
      bEject:EVAL()
      IF aReport[RP_PLAIN]
        nLLines := 1000
      ELSE
        FRMHeader(aReport, @nPage, @nLLines, @nLMaxLines)
      END
      AEVAL(aRecord, {|cRLine| QLine(SPACE(aReport[RP_LMARGIN]) + cRLine)})
      nLLines -= LEN(aRecord)
    END
  ELSE
    AEVAL(aRecord, {|cRLine| QLine(SPACE(aReport[RP_LMARGIN]) + cRLine)})
    nLLines -= LEN(aRecord)
  END

  IF aReport[RP_DBLSPACE]
    IF nLLines >= 1
      QLine()
      nLLines--
    END
  END

END

RETURN  //  FRMLine

//***************************************************************************

STATIC PROCEDURE FRMHeader(aReport, nPage, nLLines, nLMaxLines)

LOCAL aHeader    := {}
LOCAL aTHeader   := {}
LOCAL cHeader    := ''
LOCAL nHLines    := aReport[RP_WIDTH] - aReport[RP_LMARGIN] - 30
LOCAL nTHLines   := 0
LOCAL nMaxHLines := 0
LOCAL nCol       := 1
LOCAL nLine      := 1
LOCAL nHLine     := 1
LOCAL nGroup     := 1
LOCAL nRWidth    := aReport[RP_WIDTH] - aReport[RP_RMARGIN]
LOCAL nHWidth    := MIN(nRWidth, 254)

IF !aReport[RP_PLAIN]
  IF aReport[RP_HEADING] == ''
    AADD(aHeader, 'Page No.' + STR(nPage,6))
  ELSE
    aTHeader := ParseHeader(aReport[RP_HEADING], ;
      Occurs(';', aReport[RP_HEADING]) + 1)
    FOR nLine := 1 TO LEN(aTHeader)
      nTHLines := MAX(XMLCount(LTRIM(aTHeader[nLine]), nHLines), 1)
      FOR nHLine := 1 TO nTHLines
        AADD(aHeader, SPACE(15) + ;
          PADC(TRIM(XMemoLine(LTRIM(aTHeader[nLine]), ;
          nHLines, nHLine)), nHLines))
      NEXT
    NEXT
    aHeader[1] := STUFF(aHeader[1], 1, 14, 'Page No.' + STR(nPage,6))
  END
  AADD(aHeader, DTOC(DATE()))
END

FOR nLine := 1 TO LEN(aReport[RP_HEADER])
  nTHLines := MAX(XMLCount(LTRIM(aReport[RP_HEADER][nLine]), nHWidth), 1)
  FOR nHLine := 1 TO nTHLines
    cHeader := RTRIM(XMemoLine(LTRIM(aReport[RP_HEADER][nLine]), ;
      nHWidth, nHLine))
    AADD(aHeader, SPACE((nRWidth - aReport[RP_LMARGIN] - ;
      LEN(cHeader)) / 2) + cHeader)
  NEXT
NEXT

nTHLines   := LEN(aHeader)
nMaxHLines := 0
FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
  nMaxHLines := MAX(nMaxHLines, LEN(aReport[RP_COLUMNS][nCol][RC_HEADER]))
NEXT
FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
  ASIZE(aReport[RP_COLUMNS][nCol][RC_HEADER], nMaxHLines)
NEXT
FOR nLine := 1 TO nMaxHLines
  AADD(aHeader, '')
NEXT

FOR nCol := 1 TO LEN(aReport[RP_COLUMNS])
  FOR nLine := 1 TO nMaxHLines
    IF nCol > 1
      aHeader[nTHLines + nLine] += ' '
    END
    IF aReport[RP_COLUMNS][nCol][RC_HEADER][nLine] == NIL
      aHeader[nTHLines + nLine] += ;
        SPACE(aReport[RP_COLUMNS][nCol][RC_WIDTH])
    ELSE
      IF aReport[RP_COLUMNS][nCol][RC_TYPE] == 'N'
        aHeader[nTHLines + nLine] += ;
          PADL(aReport[RP_COLUMNS][nCol][RC_HEADER][nLine], ;
          aReport[RP_COLUMNS][nCol][RC_WIDTH])
      ELSE
        aHeader[nTHLines + nLine] += ;
          PADR(aReport[RP_COLUMNS][nCol][RC_HEADER][nLine], ;
          aReport[RP_COLUMNS][nCol][RC_WIDTH])
      END
    END
  NEXT
NEXT

AADD(aHeader, '')
AADD(aHeader, '')
AEVAL(aHeader, {|cHLine| QLine(SPACE(aReport[RP_LMARGIN]) + cHLine)})
nPage      ++
nLLines    := aReport[RP_LINES] - LEN(aHeader)
nLMaxLines := aReport[RP_LINES] - LEN(aHeader)

RETURN  //  FRMHeader

//***************************************************************************

FUNCTION ReadLBL(cLBLFile, lEval)

LOCAL cLBLPath   := PathAdj(cLBLFile, .Y.)
LOCAL aDirectory := DIRECTORY(cLBLPath, 'H')
LOCAL lFile      := !EMPTY(aDirectory) .OR. lEval
LOCAL nHandle    := IF(!lFile, -1, FOPEN(cLBLPath))
LOCAL cBuffer    := SPACE(LBSIZ_FILE)
LOCAL bError     := {|| IF(!lEval, NIL, ;
  CheckError(EG_READ, 'File format', cLBLPath, .Y.)), BREAK(NIL)}
LOCAL lStatus    := .N.
LOCAL nLine      := 1
LOCAL nOffset    := LBOFF_FILE
LOCAL cField     := ''
LOCAL aLabel     := ARRAY(LB_COUNT)
LOCAL aField     := ARRAY(LF_COUNT)

aLabel[LB_REMARK ] := SPACE(60)
aLabel[LB_HEIGHT ] := 5
aLabel[LB_WIDTH  ] := 35
aLabel[LB_LMARGIN] := 0
aLabel[LB_LINES  ] := 1
aLabel[LB_SPACES ] := 0
aLabel[LB_ACROSS ] := 1
aLabel[LB_FIELDS ] := {}

BEGIN SEQUENCE

  IF !lFile
    BREAK
  END

  CheckError(EG_OPEN, 'Open', cLBLPath, .N.)
  cBuffer := FileRead(nHandle, .N., 0, LBSIZ_FILE, bError)
  FCLOSE(nHandle)

  IF !(LEFT(cBuffer, 1) == CHR(2) .AND. RIGHT(cBuffer, 1) == CHR(2))
    bError:EVAL()
  END

  aLabel[LB_REMARK ] := SUBSTR(cBuffer, LBOFF_REMARK, LBSIZ_REMARK)
  aLabel[LB_HEIGHT ] := BIN2W(SUBSTR(cBuffer, LBOFF_HEIGHT , 2))
  aLabel[LB_WIDTH  ] := BIN2W(SUBSTR(cBuffer, LBOFF_WIDTH  , 2))
  aLabel[LB_LMARGIN] := BIN2W(SUBSTR(cBuffer, LBOFF_LMARGIN, 2))
  aLabel[LB_LINES  ] := BIN2W(SUBSTR(cBuffer, LBOFF_LINES  , 2))
  aLabel[LB_SPACES ] := BIN2W(SUBSTR(cBuffer, LBOFF_SPACES , 2))
  aLabel[LB_ACROSS ] := BIN2W(SUBSTR(cBuffer, LBOFF_ACROSS , 2))

  FOR nLine := 1 TO aLabel[LB_HEIGHT]
    cField  := ALLTRIM(SUBSTR(cBuffer, nOffset, LBSIZ_FIELD))
    nOffset += 60
    aField[LF_TEXT] := cField
    IF lEval
      aField[LF_EXP  ] := BlockStr(cField)
      aField[LF_BLANK] := .Y.
    END
    AADD(aLabel[LB_FIELDS], ACLONE(aField))
  NEXT

RECOVER

  IF lEval
    aLabel := {}
  END

END SEQUENCE

RETURN aLabel  //  ReadLBL

//***************************************************************************

PROCEDURE WriteLBL(cLBLFile, aLabel)

LOCAL cLBLPath := PathAdj(cLBLFile, .Y.)
LOCAL cBuffer  := SPACE(LBSIZ_FILE)
LOCAL nHandle  := 0
LOCAL nLine    := 1
LOCAL nOffset  := LBOFF_FILE
LOCAL cLine    := ''

cBuffer := STUFF(cBuffer, 1            , 1            , CHR(2))
cBuffer := STUFF(cBuffer, LBSIZ_FILE   , 1            , CHR(2))
cBuffer := STUFF(cBuffer, LBOFF_REMARK , LBSIZ_REMARK , ;
  PADR(aLabel[LB_REMARK], LBSIZ_REMARK))
cBuffer := STUFF(cBuffer, LBOFF_HEIGHT , 2, W2Bin(aLabel[LB_HEIGHT ]))
cBuffer := STUFF(cBuffer, LBOFF_WIDTH  , 2, W2Bin(aLabel[LB_WIDTH  ]))
cBuffer := STUFF(cBuffer, LBOFF_LMARGIN, 2, W2Bin(aLabel[LB_LMARGIN]))
cBuffer := STUFF(cBuffer, LBOFF_LINES  , 2, W2Bin(aLabel[LB_LINES  ]))
cBuffer := STUFF(cBuffer, LBOFF_SPACES , 2, W2Bin(aLabel[LB_SPACES ]))
cBuffer := STUFF(cBuffer, LBOFF_ACROSS , 2, W2Bin(aLabel[LB_ACROSS ]))

FOR nLine := 1 TO aLabel[LB_HEIGHT]
  cLine := aLabel[LB_FIELDS][nLine][LF_TEXT]
  IF !EMPTY(cLine)
    cBuffer := STUFF(cBuffer, nOffset, LBSIZ_FIELD, PADR(cLine, LBSIZ_FIELD))
  END
  nOffset += 60
NEXT

BEGIN SEQUENCE

  nHandle := FCREATE(cLBLPath)
  CheckError(EG_CREATE, 'Create', cLBLPath, .N.)
  FWRITE(nHandle, cBuffer, LBSIZ_FILE)
  CheckError(EG_WRITE, 'Write', cLBLPath, .N.)
  FCLOSE(nHandle)

END SEQUENCE

RETURN  //  WriteLBL

//***************************************************************************

FUNCTION ListLBL(cLBLFile, aLine, cRemark, nWidth, nHeight, nAcross, ;
  nLMargin, nVSpace, nHSpace)

LOCAL cText     := ''
LOCAL nPad      := 41
LOCAL nIndent   := nPad + 2
LOCAL cIndent   := SPACE(nIndent)
LOCAL bYesNo    := {|lYesNo| IF(lYesNo, 'Yes', 'No')}
LOCAL nLine     := 1
LOCAL nLines    := LEN(aLine)

cText += 'LABEL FILE ' + cLBLFile + CRLF + ;
  CRLF
cText += 'Parameters' + CRLF + ;
  CRLF
cText += ;
  PADR(PADL('Label width in characters:'               , nPad), nIndent) + ;
  STR(nWidth, 3) + CRLF + ;
  PADR(PADL('Label height in lines:'                   , nPad), nIndent) + ;
  STR(nHeight, 3) + CRLF + ;
  PADR(PADL('Number of labels across page:'            , nPad), nIndent) + ;
  STR(nAcross, 3) + CRLF + ;
  PADR(PADL('Left margin:'                             , nPad), nIndent) + ;
  STR(nLMargin, 3) + CRLF + ;
  PADR(PADL('Blank lines between labels vertically:'   , nPad), nIndent) + ;
  STR(nVSpace, 3) + CRLF + ;
  PADR(PADL('Blank spaces between labels horizontally:', nPad), nIndent) + ;
  STR(nHSpace, 3) + CRLF + ;
  CRLF
cText += 'Remark' + CRLF + ;
  CRLF
cText += cRemark + CRLF + ;
  CRLF
cText += 'Expressions' + CRLF + ;
  CRLF

FOR nLine := 1 TO nLines
  cText += 'Line ' + STR(nLine, 3) + ':  ' + aLine[nLine] + CRLF
NEXT

cText += CRLF

RETURN cText  //  ListLBL

//***************************************************************************

PROCEDURE RunLBL(cLBLName, bFor, bWhile, nNext, nRecord, lRest)

LOCAL aLabel   := {}
LOCAL aBand    := {}
LOCAL cBlank   := ''
LOCAL lOneMore := .Y.
LOCAL lBroke   := .N.
LOCAL nCol     := 1
LOCAL xBreak   := NIL
LOCAL nMargin  := SET(_SET_MARGIN, 0)

BEGIN SEQUENCE

  aLabel := ReadLBL(cLBLName, .Y.)
  IF EMPTY(aLabel)
    BREAK
  END

  aLabel[LB_LMARGIN] += nMargin
  ASIZE(aBand, LEN(aLabel[LB_FIELDS]))
  AFILL(aBand, SPACE(aLabel[LB_LMARGIN]))
  cBlank := SPACE(aLabel[LB_WIDTH] + aLabel[LB_SPACES])

  DBEVAL({|| LBLLine(aLabel, @aBand, @cBlank, @lOneMore, @nCol)}, ;
    bFor, bWhile, nNext, nRecord, lRest)

  IF lOneMore
    AEVAL(aBand, {|cBLine| QLine(cBLine)})
  END

RECOVER USING xBreak

  lBroke := .Y.

END SEQUENCE

IF lBroke
  BREAK xBreak
END

RETURN  //  RunLBL

//***************************************************************************

STATIC PROCEDURE LBLLine(aLabel, aBand, cBlank, lOneMore, nCol)

LOCAL nField  := 1
LOCAL aBuffer := {}
LOCAL cBuffer := ''
LOCAL xField  := NIL

FOR nField := 1 TO LEN(aLabel[LB_FIELDS])
  IF aLabel[LB_FIELDS][nField] <> NIL
    xField  := aLabel[LB_FIELDS][nField][LF_EXP]:EVAL()
    cBuffer := PADR(xField, aLabel[LB_WIDTH])
    cBuffer += SPACE(aLabel[LB_SPACES])
    IF aLabel[LB_FIELDS][nField][LF_BLANK]
      IF !EMPTY(cBuffer)
        AADD(aBuffer, cBuffer)
      END
    ELSE
      AADD(aBuffer, cBuffer)
    END
  ELSE
    AADD(aBuffer, NIL)
  END
NEXT

ASIZE(aBuffer, LEN(aLabel[LB_FIELDS]))

FOR nField := 1 TO LEN(aLabel[LB_FIELDS])
  aBand[nField] += IF(aBuffer[nField] == NIL, cBlank, aBuffer[nField])
NEXT

IF nCol == aLabel[LB_ACROSS]
  FOR nField := 1 TO LEN(aBand)
    aBand[nField] := RTRIM(aBand[nField])
  NEXT
  lOneMore := .N.
  nCol     := 1
  AEVAL(aBand, {|cBLine| QLine(cBLine)})
  FOR nField := 1 TO aLabel[LB_HEIGHT] - LEN(aBand)
    QLine()
  NEXT
  FOR nField := 1 TO aLabel[LB_LINES]
    QLine()
  NEXT
  AFILL(aBand, SPACE(aLabel[LB_LMARGIN]))
ELSE
  lOneMore := .Y.
  nCol     ++
END

RETURN  //  LBLLine

//***************************************************************************