/*

  DBEHELP.PRG

  Help text for DBE.

  This file is part of the source code for DBE, a general purpose database
  utility.

  Copyright (C) 1998 Kevin Carmody
  kcarmody@compuserve.com

  This program is free software.  You can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.

  You should have received a copy of the GNU General Public License along
  with this program, in a file called COPYING.TXT.  If not, write to:

  Free Software Foundation, Inc.
  59 Temple Place, Suite 330,
  Boston, MA 02111-1307
  USA
  http://www.gnu.org

*/

ANNOUNCE DBEHelp

#INCLUDE 'DBE.CH'
#INCLUDE 'KEYS.CH'
#INCLUDE 'CHARS.CH'
#INCLUDE 'GETEXIT.CH'
#INCLUDE 'SETCURS.CH'
#INCLUDE 'MOUSCLOC.CH'

//***************************************************************************

FUNCTION HelpTran(aSelect, nSelect, nDMWidth)

LOCAL aText  := aSelect[nSelect]
LOCAL cText  := ALLTRIM(aText[1]) + '~==~' + HelpText(aText[2])
LOCAL cSLine := REPLICATE(C_S_HORIZ, nDMWidth)

cText := STRTRAN(cText, '~|', '|'        )
cText := STRTRAN(cText, '~' , CRLF       )
cText := STRTRAN(cText, '|' , CRLF + CRLF)
cText := STRTRAN(cText, '==', cSLine     )

RETURN cText  //  HelpTran

//***************************************************************************

FUNCTION HelpTopic()

LOCAL aTopic  := {}
LOCAL bAppend := {|aRow| AADD(aTopic, aRow)}

AEVAL({ ;
  {'Introduction to DBE'          , H_INTRO     }  }, bAppend)

AEVAL({ ;
  {'General features of DBE'      , H_GENERAL   }, ;
  {'  DBE command line parameters', H_PARAM     }, ;
  {'  DBE.INI file               ', H_DBEINI    }, ;
  {'  Dialog boxes'               , H_DIALOG    }, ;
  {'  Using a mouse'              , H_MOUSE     }, ;
  {'  Hot keys'                   , H_HOTKEYS   }, ;
  {'  Setup window'               , H_SETUP     }, ;
  {'  Data browse'                , H_DATABROWSE}, ;
  {'  Menus'                      , H_MENUS     }, ;
  {'  Drivers'                    , H_DRIVERS   }, ;
  {'  Data transfer buffer'       , H_TRANSFER  }, ;
  {'  Incremental search'         , H_INCSEARCH }, ;
  {'  Memo editor'                , H_MEMOEDIT  }, ;
  {'  Structure editor'           , H_STRUEDIT  }, ;
  {'  DBE files basic information', H_DBEFILE   }, ;
  {'  DBE files keyword reference', H_DBEKEY    }, ;
  {'  Keystroke recording'        , H_KRECORD   }, ;
  {'  Keystroke code reference'   , H_KRCODE    }  }, bAppend)

AEVAL({ ;
  {'Open Menu'                    , H_OPENMENU  }, ;
  {'  Database open'              , H_DATAOPEN  }, ;
  {'  Database and index open'    , H_BOTHOPEN  }, ;
  {'  Index open'                 , H_INDOPEN   }, ;
  {'  Index create'               , H_INDCREATE }, ;
  {'  Indexes recreate'           , H_INDRECRE  }, ;
  {'  Index destroy'              , H_INDDESTROY}, ;
  {'  Expand current setup'       , H_SETUPEXP  }, ;
  {'    List current setup'       , H_SETUPLIST }, ;
  {'  Create structure'           , H_CRESTRU   }, ;
  {'  Create database from'       , H_CREFROM   }, ;
  {'  Read DBE file'              , H_DBEREAD   }, ;
  {'  Write DBE file'             , H_DBEWRITE  }, ;
  {'  Quit DBE'                   , H_QUIT      }  }, bAppend)

AEVAL({ ;
  {'File Menu'                    , H_FILEMENU  }, ;
  {'  Replace'                    , H_REPLACE   }, ;
  {'  Pack'                       , H_PACK      }, ;
  {'  Zap'                        , H_ZAP       }, ;
  {'  Delete'                     , H_DELETE    }, ;
  {'  Recall'                     , H_RECALL    }, ;
  {'  Count'                      , H_COUNT     }, ;
  {'  Sum'                        , H_SUM       }, ;
  {'  Accumulate'                 , H_ACCUM     }, ;
  {'  Evaluate on database'       , H_DBEVAL    }, ;
  {'  Copy to'                    , H_COPYTO    }, ;
  {'  Append from'                , H_APPEND    }, ;
  {'  Modify structure'           , H_MODISTRU  }, ;
  {'  Copy structure'             , H_COPYSTRU  }, ;
  {'  Copy structure extended'    , H_COPYEXT   }, ;
  {'  List'                       , H_LISTFILE  }, ;
  {'  List structure'             , H_LISTSTRU  }, ;
  {'  Report form'                , H_FRMUSE    }, ;
  {'  Label form'                 , H_LBLUSE    }, ;
  {'  Join'                       , H_JOIN      }  }, bAppend)

AEVAL({ ;
  {'Record Menu'                  , H_RECMENU   }, ;
  {'  New record'                 , H_NEWRECORD }, ;
  {'  Memowrite'                  , H_MEMOWRITE }, ;
  {'  Memoread'                   , H_MEMOREAD  }, ;
  {'  Move record'                , H_MOVEREC   }, ;
  {'  Replace field'              , H_REPLACEFLD}, ;
  {'  Snapshot'                   , H_SNAPSHOT  }, ;
  {'  Field modify'               , H_EDITFIELD }, ;
  {'  Enter data'                 , H_ENTERDATA }  }, bAppend)

AEVAL({ ;
  {'Go Menu'                      , H_GOMENU    }, ;
  {'  Seek'                       , H_SEEK      }, ;
  {'  Incremental seek'           , H_SEEKINC   }, ;
  {'  Locate'                     , H_LOCATE    }, ;
  {'  Continue locate'            , H_LOCCONT   }, ;
  {'  Goto'                       , H_GOTO      }, ;
  {'  Records skip'               , H_RECSKIP   }  }, bAppend)

AEVAL({ ;
  {'Set Menu'                     , H_SETMENU   }, ;
  {'  Local directory'            , H_LOCAL     }, ;
  {'  Relation add'               , H_RELADD    }, ;
  {'  Relation edit'              , H_RELEDIT   }, ;
  {'  Filter add'                 , H_FILTADD   }, ;
  {'  Filter edit'                , H_FILTEDIT  }, ;
  {'  Key record'                 , H_KRTOGGLE  }, ;
  {'  Hot keys'                   , H_HOTKEYSET }, ;
  {'  Colors'                     , H_COLORS    }, ;
  {'  Cursors'                    , H_CURSORS   }, ;
  {'  Setup window length'        , H_SLINES    }, ;
  {'  Printer codes'              , H_PRINTCODE }  }, bAppend)

AEVAL({ ;
  {'View Menu'                    , H_VIEWMENU  }, ;
  {'  Add field'                  , H_FLDADD    }, ;
  {'  Edit field'                 , H_FLDEDIT   }, ;
  {'  Manage field list'          , H_FLDMANAGE }  }, bAppend)

AEVAL({ ;
  {'Util Menu'                    , H_UTILMENU  }, ;
  {'  Copy to clipboard'          , H_COPYCLIP  }, ;
  {'  Evaluate'                   , H_EVAL      }, ;
  {'  Repeat keystroke'           , H_REPEAT    }, ;
  {'  Play back key record'       , H_KRPLAY    }, ;
  {'  History'                    , H_HISTORY   }, ;
  {'    History append clipboard' , H_PASTEHIST }, ;
  {'  Structure current'          , H_STRUONE   }, ;
  {'  Structures all'             , H_STRUALL   }, ;
  {'  Directory local'            , H_DIRLOCAL  }, ;
  {'  Directory global'           , H_DIRGLOBAL }, ;
  {'  File manager local'         , H_MANLOCAL  }, ;
  {'  File manager global'        , H_MANGLOBAL }, ;
  {'    File manager tree spec'   , H_MANTREE   }, ;
  {'    File manager file spec'   , H_MANFILE   }, ;
  {'    File manager print file'  , H_MANPRINT  }, ;
  {'    File manager new file'    , H_MANNEW    }, ;
  {'    File manager copy file'   , H_MANCOPY   }, ;
  {'    File manager rename file' , H_MANRENAME }, ;
  {'    File manager list files'  , H_MANLIST   }, ;
  {'    File manager retry'       , H_MANRETRY  }, ;
  {'  ASCII table'                , H_ASCTABLE  }, ;
  {'  Color table'                , H_COLORTABLE}, ;
  {'  Calendar'                   , H_CALENDAR  }, ;
  {'    Calendar new date'        , H_CALNEW    }, ;
  {'    Calendar list'            , H_CALLIST   }, ;
  {'  Memvar editor'              , H_MEMEDIT   }, ;
  {'    Restore from MEM file'    , H_MEMREST   }, ;
  {'    Save to MEM file'         , H_MEMSAVE   }, ;
  {'    Input from text file'     , H_MEMINPUT  }, ;
  {'    Output to text file'      , H_MEMOUTPUT }, ;
  {'    Add variable'             , H_MEMADD    }, ;
  {'  Report modify'              , H_FRMEDIT   }, ;
  {'    Report Heading'           , H_FRMHEAD   }, ;
  {'    Report Parameters'        , H_FRMPARAM  }, ;
  {'    Report Group'             , H_FRMGROUP  }, ;
  {'    Report Add column'        , H_FRMCOLADD }, ;
  {'    Report Edit column'       , H_FRMCOLEDIT}, ;
  {'    Report Formats'           , H_FRMFORM   }, ;
  {'    Report List'              , H_FRMLIST   }, ;
  {'  Label modify'               , H_LBLEDIT   }, ;
  {'    Label Parameters'         , H_LBLPARAM  }, ;
  {'    Label Remark'             , H_LBLREMARK }, ;
  {'    Label Expressions'        , H_LBLLINE   }, ;
  {'    Label Formats'            , H_LBLFORM   }, ;
  {'    Label List'               , H_LBLLIST   }  }, bAppend)

AEVAL({ ;
  {'Help menu'                    , H_HELPMENU  }, ;
  {'  Near current context'       , H_CONTEXT   }, ;
  {'  Contents'                   , H_CONTENTS  }, ;
  {'  Index'                      , H_INDEX     }, ;
  {'  Find'                       , H_FIND      }, ;
  {'  Help text'                  , H_HELPTEXT  }, ;
  {'  Write help topics'          , H_HELPWRITE }  }, bAppend)

AEVAL({ ;
  {'About DBE'                    , H_ABOUT     }, ;
  {'  License'                    , H_LICENSE   }  }, bAppend)

RETURN aTopic  //  HelpTopic

//***************************************************************************

FUNCTION HelpText(nContext)

LOCAL cText := ''
LOCAL nType := INT(nContext/10000) * 10000

DO CASE
CASE nType == H_INTRO
  cText := HelpIntro(nContext)
CASE nType == H_GENERAL
  cText := HelpGeneral(nContext)
CASE nType == H_OPENMENU
  cText := HelpOpen(nContext)
CASE nType == H_FILEMENU
  cText := HelpFile(nContext)
CASE nType == H_RECMENU
  cText := HelpRecord(nContext)
CASE nType == H_GOMENU
  cText := HelpGo(nContext)
CASE nType == H_SETMENU
  cText := HelpSet(nContext)
CASE nType == H_VIEWMENU
  cText := HelpView(nContext)
CASE nType == H_UTILMENU
  cText := HelpUtil(nContext)
CASE nType == H_HELPMENU
  cText := HelpHelp(nContext)
CASE nType == H_ABOUT
  cText := HelpAbout(nContext)
END

RETURN cText  //  HelpText

//***************************************************************************

STATIC FUNCTION HelpPar(nCode)

LOCAL cPar := ''

DO CASE
CASE nCode == HP_HOTMEN

  cPar := ;
    'This is a hot menu.  That means that the menu option is selected ' + ;
    'as soon as you press the initial of the option, without waiting ' + ;
    'for the Enter key.'

CASE nCode == HP_SHORTSETUP

  cPar := ;
    '(S) in the list below indicates that a shortcut for this ' + ;
    'option is available from the data browse.  See the help topic ' + ;
    '"Data browse" for more information.'

CASE nCode == HP_SHORTDATA

  cPar := ;
    '(D) in the list below indicates that a shortcut for this ' + ;
    'option is available in the setup window.  See the help topic ' + ;
    '"Setup window" for more information.'

CASE nCode == HP_HOTKEY

  cPar := ;
    '(H) in the list below indicates that a hot key for this ' + ;
    'option is available anywhere in DBE.  See the help topic ' + ;
    '"Hot keys" for more information.'

CASE nCode == HP_ASTERISK

  cPar := ;
    '(*) in the list below indicates that more help is available ' + ;
    'under a help topic for that item.'

CASE nCode == HP_FILENAME

  cPar := ;
    'including the drive and directory if needed.  If the operating ' + ;
    'system supports long file names, you may use a long file name, ' + ;
    'with each component up to 16 characters long.  The default ' + ;
    'extension '

CASE nCode == HP_PRINTER

  cPar := ;
    'If you want to send output directly to the printer, you may enter ' + ;
    '"PRN", "LPT1", "LPT2", or "LPT3".'

CASE nCode == HP_ALIAS

  cPar := ;
    'Alias:  This item is optional.  Use it if you want to assign an ' + ;
    'alias to this database which is different from the default alias.  ' + ;
    'You need to do this if the database name is not a valid alias, or ' + ;
    'if there are duplicate database names.'

CASE nCode == HP_DRIVER

  cPar := ;
    'Driver:  Choose from the list of drivers.  The list shows the ' + ;
    'default index extension for each driver.  See the help topic ' + ;
    '"Drivers".'

CASE nCode == HP_EXREAD

  cPar := ;
    'Exclusive:  If Yes, the database is opened in exclusive mode, ' + ;
    'which means that only you can use the database.  If No, the ' + ;
    'database is opened in shared mode, which means other users can ' + ;
    'use the database.' + ;
    '|' + ;
    'Read only:  If Yes, the database in opened is read-only mode, ' + ;
    'which means that you can read but not modify the file.  If No, ' + ;
    'the database is opened in read-write mode, which means that you ' + ;
    'can make changes to the file.'

CASE nCode == HP_CONDITION

  cPar := ;
    'For:  This clause is optional.  You must either leave it blank or ' + ;
    'enter a logical expression.  If you enter a logical expression, ' + ;
    'only those records which for which the expression is true will be ' + ;
    'processed.  If you leave this field blank, all records are ' + ;
    'processed.' + ;
    '|' + ;
    'While:  This clause is optional.  You must either leave it blank ' + ;
    'or enter a logical expression.  If you enter a logical expression, ' + ;
    'processing continues as long as the expression is true.  As soon ' + ;
    'the expression is false processing stops.  If you leave this field ' + ;
    'blank, processing is performed for the entire scope.' + ;
    '|' + ;
    'Scope:  Choose from the list.  If you choose ALL, then processing ' + ;
    'starts at the top of the file and goes to the end of the file.  If ' + ;
    'you choose REST, the processing starts at the current record and ' + ;
    'continues to the end of the file.  If you choose NEXT, you must ' + ;
    'specify a number of records in the "Next" item.' + ;
    '|' + ;
    '"For", "While", and "Scope" clauses may be combined.  When they ' + ;
    'are combined, only those records which satisfy all conditions are ' + ;
    'processed.' + ;
    '|' + ;
    'Next:  This item is used only if you choose a scope of NEXT.  ' + ;
    'Processing then starts at the current record and continues for the ' + ;
    'indicated number of records.'

CASE nCode == HP_ADDITIVE

  cPar := ;
    'Additive:  If you answer Yes and the output file already exists, ' + ;
    'then the output file is appended to.  If you answer No and the ' + ;
    'output file already exists, then the output file is overwritten ' + ;
    'and the previous contents are lost.'

CASE nCode == HP_PRINTCODE

  cPar := ;
    'Printer codes:  If you answer Yes, you can send printer codes to ' + ;
    'output printer or file.  Before the codes are sent, you can set ' + ;
    'the printer codes that you want to send.  See the help topic ' + ;
    '"Printer codes".'

CASE nCode == HP_FLDMOD

  cPar := ;
    'Field kind:  Choose from one of the following:~' + ;
    '  Simple field:  a field in the current database.~' + ;
    '  Aliased field:  a field in another database.~' + ;
    '  Calculated field:  a browse field defined by any expression.~' + ;
    '|' + ;
    'Field name:  For a SIMPLE field, enter the name of the field from ' + ;
    'the current database.  For an ALIASED field, enter the name of the ' + ;
    'field from the database in the "Alias" item.  For a CALCULATED ' + ;
    'field, enter any name of your choosing.' + ;
    '|' + ;
    'Alias:  This item is only used if you choose an ALIASED field.  ' + ;
    'Enter the database from which to get the field.' + ;
    '|' + ;
    'Expression:  This item is only used if you choose a CALCULATED ' + ;
    'field.  Enter the expression to be used to define the browse field.' + ;
    '|' + ;
    'Field type:  This item is only used if you choose a CALCULATED ' + ;
    'field.  Enter the data type of the expression.  A character ' + ;
    'expression may be assigned a type Character or a type Memo.  This ' + ;
    'affects the way the data is displayed.' + ;
    '|' + ;
    'Field length:  This item is only used if you choose a CALCULATED ' + ;
    'field.  Enter the length you wish to allocate to the browse ' + ;
    'field.  This becomes the width of the browse column or line.' + ;
    '|' + ;
    'Field decimals:  This item is only used if you choose a CALCULATED ' + ;
    'field and if the field is type Numeric.  Enter the number of ' + ;
    'decimal places to display.'

CASE nCode == HP_FLDSTRU

  cPar := ;
    'This option in no way modifies the structure of any database.  It ' + ;
    'affects only the browse display in DBE.  To modify the structure ' + ;
    'of the current database, use the option File menu / Modify ' + ;
    'structure.  See the help topic "Modify structure".'

CASE nCode == HP_FLDCOLS

  cPar := ;
    'field name, field type, field length, field decimals, field ' + ;
    'number, start byte for the field, and end byte for the field.  ' + ;
    'The start and end bytes are the byte positions of the field ' + ;
    'within the record.  The first data byte in the record is byte 1, ' + ;
    'and the delete byte can be regarded as byte 0.'

CASE nCode == HP_MANLOC

  cPar := ;
    'Each line contains the following data for each file:  name, size ' + ;
    'in bytes, date and time of last modification, and attributes.  ' + ;
    'Attribute codes are as follows:' + ;
    '|' + ;
    'R = read only~' + ;
    'A = archive~' + ;
    'S = system~' + ;
    'H = hidden~' + ;
    'D = directory~' + ;
    '|' + ;
    'If the operating system supports long file names, then the first ' + ;
    'column shows the short file name, and the long file name is ' + ;
    'displayed in columns to the right.'

CASE nCode == HP_MANGLOB

  cPar := ;
    'The browse list also includes a line marked either "." or "..".' + ;
    'A line marked "." refers to the current directory and appears if ' + ;
    'the directory being shown is the highest level directory on the ' + ;
    'drive.  Expanding this directory rescans this directory.  A line ' + ;
    'marked ".." refers to the directory at the next level up, and it ' + ;
    'appears in directories lists below the highest level.  If you ' + ;
    'expand this directory, you will move one directory level up.'

CASE nCode == HP_MANDISP

  cPar := ;
    'When you are browsing the file list, you can use the following ' + ;
    'options:' + ;
    '|' + ;
    'Enter - Select file / Expand directory:  If the highlighted object ' + ;
    'is a directory, then the directory list moves to that directory.  ' + ;
    'If the highlighted object is a file, then that file is selected, ' + ;
    'and the directory list exits.  A selected file name is passed into ' + ;
    'the data transfer buffer, where it can be used by any DBE object ' + ;
    'that accepts data transfer.  See the help topic "Object data ' + ;
    'transfer".' + ;
    '|' + ;
    'Ctrl-Enter - Select directory:  If the highlighted object is a ' + ;
    'directory, then that directory is selected, and the directory list ' + ;
    'exits.  If the highlighted directory is either "." or "..", then ' + ;
    'the current directory is selected.  A selected directory name is ' + ;
    'passed into the data transfer buffer, where it can be used by any ' + ;
    'DBE object that accepts data transfer.  See the help topic "Object ' + ;
    'data transfer".' + ;
    '|' + ;
    'Esc - Cancel:  Exits the directory list without selecting a file ' + ;
    'or moving to another directory.' + ;
    '|' + ;
    'Alt-T - Tree:  Allows you to enter a new drive, directory, and ' + ;
    'file spec to browse. (*)' + ;
    '|' + ;
    'Alt-F - Files:  Allows you to enter a new file spec to browse. (*)'

CASE nCode == HP_MANEDIT

  cPar := ;
    'Alt-X - Execute:  Executes the highlighted file, assuming that it ' + ;
    'is an executable or batch file.' + ;
    '|' + ;
    'Alt-V - View:  Displays the contents of the highlighted file with ' + ;
    'the memo editor.  This option works properly only on an ASCII file ' + ;
    'that is less than 65,536 bytes in size.' + ;
    '|' + ;
    'Alt-P - Print:  Prints the contents of the highlighted file, ' + ;
    'assuming that it contains only ASCII data and printer codes.  You ' + ;
    'also have the option of adding printer codes to the output. (*)' + ;
    '|' + ;
    'Alt-N - Newfile:  Allows you to create a new file.  This option ' + ;
    'creates only an ASCII file, using the memo editor. (*)' + ;
    '|' + ;
    'Alt-E - Edit:  Allows you to edit the highlighed file using the ' + ;
    'memo editor.  USE THIS OPTION ONLY ON AN ASCII FILE THAT IS LESS ' + ;
    'THAN 65,536 BYTES IN SIZE.' + ;
    '|' + ;
    'Alt-R - Rename:  Renames the highlighed file. (*)' + ;
    '|' + ;
    'Ins - Copy:  Copies the highlighed file to a new or existing file. ' + ;
    '(*)' + ;
    '|' + ;
    'Del - Delete:  Deletes the highlighed file.' + ;
    '|' + ;
    'Alt-L - List:  Writes the list of files and subdirectories to an ' + ;
    'ASCII file.' + ;
    '|' + ;
    'Alt-S - Stats:  Displays summary statistics for the drive and ' + ;
    'directory.'

CASE nCode == HP_MANINC

  cPar := ;
    'You may perform an incremental search on the file name. See the ' + ;
    'help topic "Incremental search".'

CASE nCode == HP_CHARFIT

  cPar := ;
    'Number of characters (columns) across the page:~' + ;
    '|' + ;
    '                    Letter      Legal    ~' + ;
    'Pitch   Portrait    Landscape   Landscape~' + ;
    '|' + ;
    '  10         80         106         136~'   + ;
    '  12         96         128         164~'   + ;
    '  16        134         177         227~'   + ;
    '|' + ;
    'Number of lines (rows) down the page:~' + ;
    '|' + ;
    '                    Letter      Legal   ~' + ;
    'LPI     Landscape   Portrait    Portrait~' + ;
    '|' + ;
    '   6         45          60          78~' + ;
    '   8         60          80         104~'

CASE nCode == HP_FRMCOL

  cPar := ;
    'Heading:  Enter up to four lines of text for the column heading.  ' + ;
    'These lines are interpreted as literal data and thus do not change.' + ;
    '|' + ;
    'Expression:  Enter an expression which is to be evaluated on each ' + ;
    'record and the value output into the report column.  This ' + ;
    'expression may be a field name or any other valid expression of ' + ;
    'type character, numeric, logical, or date.' + ;
    '|' + ;
    'If you are running the report from within DBE, the column ' + ;
    'expression may also be an array of two strings.  The first string ' + ;
    'is the value expression, and the second string is the picture.  ' + ;
    'For example, instead of a column expression of FIELD1, you could ' + ;
    'use {"FIELD1", "999,999"}.  If the value expression is numeric, ' + ;
    'then the column can be totalled.' + ;
    '|' + ;
    'Length:  Enter the width of the column in characters.  If the ' + ;
    'expression has a value that is longer than the column length, ' + ;
    'it will be wrapped onto as many lines as it takes to output the ' + ;
    'data.' + ;
    '|' + ;
    'Decimals:  If the expression is type numeric, enter the number of ' + ;
    'decimal positions to output.' + ;
    '|' + ;
    'Total:  If the expression is type numeric, and you answer Yes, ' + ;
    'then the values will be added and the sum for all records will be ' + ;
    'output on a totals line at the end of the report.  If you use ' + ;
    'groups and subgroups, subtotals lines will be output for each ' + ;
    'group and subgroup also.  If you answer No, no totals lines will ' + ;
    'be output.'

CASE nCode == HP_HELPBROWSE

  cPar := ;
    'Enter - View:  Displays the help text.' + ;
    '|' + ;
    'Alt-W - Write:  Writes the list of help topics to an ASCII file.  ' + ;
    'See the help topic "Write help topics".' + ;
    '|' + ;
    'Esc - Exit:  Exits help.' + ;
    '|' + ;
    'You may perform an incremental search on the topic name. See the ' + ;
    'help topic "Incremental search".'

END

RETURN cPar  //  HelpPar

//***************************************************************************

STATIC FUNCTION HelpIntro(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_INTRO

  cText := ;
    'DBE is a general purpose program for database browsing.  ' + ;
    'Features include support for RDDs, editing MEM files, editing ' + ;
    'FRM files, editing LBL files, and other utilities.' + ;
    'DBE runs in 16-bit protected mode.  It therefore uses all ' + ;
    'available memory and runs faster than real mode programs.' + ;
    'DBE supports a mouse.  This is discussed further in the help ' + ;
    'topic "Mouse support".' + ;
    '|' + ;
    'The top part of the screen, framed by double lines, is the setup ' + ;
    'window, where you enter databases, indexes, relations, filters, ' + ;
    'and fields.  The bottom part of the screen, framed by single ' + ;
    'lines, is the data browse area, where you view and edit data in ' + ;
    'the databases.' + ;
    '|' + ;
    'On the top line are the menus, which can be used to control all ' + ;
    'operations of DBE.  To access each menu, press the letter of the ' + ;
    'menu name.  Do not press Alt plus the letter.' + ;
    '|' + ;
    'Some menu entries have shortcuts.  This allows you to press one ' + ;
    'key to access common menu functions.  For example, the Tab key ' + ;
    'jumps back and forth between the setup window and the data ' + ;
    'browse.  This is a shortcut for the Open Menu / Jump options.' + ;
    '|' + ;
    'Another shortcut is the Ins key.  In the Database setup pane, Ins ' + ;
    'adds a new database.  In the Index setup pane, Ins adds a new ' + ;
    'index.  In the data pane, Ins adds a new record.  For further ' + ;
    'information on shortcuts, see the help topics "Setup window" and ' + ;
    '"Data browse".' + ;
    '|' + ;
    'DBE has seven menus.  All DBE features can be accessed through ' + ;
    'these menus.  These menus are described in the help topic ' + ;
    '"Menus".'

END

RETURN cText  //  HelpIntro

//***************************************************************************

STATIC FUNCTION HelpGeneral(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_GENERAL

  cText := ;
    'There are several help topics that describe general features of ' + ;
    'DBE, i.e. features of DBE that are not found on just one menu.' + ;
    'The following is a list of these help topics, with a brief ' + ;
    'description.  For more information, see the help topic described.' + ;
    '|' + ;
    'DBE command line parameters:  How to use parameters on the DOS ' + ;
    'command line that starts DBE.' + ;
    '|' + ;
    'Dialog boxes:  How to use the dialog boxes in DBE.' + ;
    '|' + ;
    'Using a mouse:  How to use a mouse in DBE.' + ;
    '|' + ;
    'Hot keys:  About keys that invoke special utilites that can be ' + ;
    'used almost anywhere in DBE.' + ;
    '|' + ;
    'Setup window:  How to use the window with the five small panes at ' + ;
    'the top of the DBE screen.  The panes control the databases that ' + ;
    'you open in a DBE session.' + ;
    '|' + ;
    'Data browse:  How to use the large window at the bottom of the ' + ;
    'screen, where you browse databases.' + ;
    '|' + ;
    'Menus:  Basic information on the options in the menus in DBE.' + ;
    '|' + ;
    'Drivers:  How to use the four database drivers in DBE.' + ;
    '|' + ;
    'Data transfer buffer:  How DBE transfers data between various ' + ;
    'sets of its objects, such as a dialog box and a file list.' + ;
    '|' + ;
    'Incremental search:  How to search for items in a non-database ' + ;
    'browse.' + ;
    '|' + ;
    'Memo editor:  Special keys in the memo editor.' + ;
    '|' + ;
    'Structure editor:  How to use the utility that allows you to ' + ;
    'create a new structure or modify an existing one.' + ;
    '|' + ;
    'DBE files basic information:  How to save DBE session settings ' + ;
    'to a file.  The following topic, "DBE files keyword reference", is ' + ;
    'a detailed reference for these files.' + ;
    '|' + ;
    'Keystroke recording:  How to record and play back keystrokes in ' + ;
    'DBE.  The following topic, "Keystroke code reference", is a ' + ;
    'detailed reference for keystroke record strings.'

CASE nContext == H_PARAM

  cText := ;
    'DBE accepts up to 16 parameters on the DOS command line.   ' + ;
    'These parameters allow you to specify operations to be performed ' + ;
    'automatically when DBE starts up.  Command line parameters can ' + ;
    'take the following forms:' + ;
    '|' + ;
    'Name starting with "@" or ending with ".DBE": Name of DBE file.~' + ;
    'Expression containing "=": DBE file command.~' + ;
    'Simple name: The first simple name is a database.~' + ;
    '             Subsequent simple names are index names.' + ;
    '|' + ;
    'These types may be mixed in any order.  They will be executed ' + ;
    'in the order they are placed on the command line.' + ;
    '|' + ;
    'Examples:' + ;
    '|' + ;
    'DBE FRUIT APPLES ORANGES~' + ;
    '    Opens database FRUIT and indexes APPLES and ORANGES.~' + ;
    'DBE @FOOD~' + ;
    '    Reads and processes FOOD.DBE.~' + ;
    'DBE FRUIT APPLES EXCLUSIVE=Y~' + ;
    '    Opens database FRUIT and index APPLES in exclusive mode.' + ;
    '|' + ;
    'For more information on performing operations automatically when ' + ;
    'DBE starts up, see the help topic "DBE.INI file".' + ;
    '|' + ;
    'For information on DBE files, see the help topic "DBE files ' + ;
    'basic information".'

CASE nContext == H_DBEINI

  cText := ;
    'When DBE starts up, it looks for the file DBE.INI in the same ' + ;
    'location as the DBE executable.  If it finds this file, it ' + ;
    'executes the commands in this file.  This file has the same format ' + ;
    'as a DBE file.' + ;
    '|' + ;
    'DBE.INI need not be specified on the command line.  The commands ' + ;
    'in DBE.INI are always executed, regardless of parameters on the ' + ;
    'command line.' + ;
    '|' + ;
    'For more information on performing operations automatically when ' + ;
    'DBE starts up, see the help topic "DBE command line parameters".' + ;
    '|' + ;
    'For information on DBE files, see the help topic "DBE files ' + ;
    'basic information".'


CASE nContext == H_DIALOG

  cText := ;
    'DBE uses its own style of dialog box to get information.  These ' + ;
    'dialog boxes are similar to those in Windows.' + ;
    '|' + ;
    'DBE dialog boxes all have "OK" and "Cancel" buttons:' + ;
    '|' + ;
    'The "OK" button exits the dialog box and acts on the data you ' + ;
    'have just entered.' + ;
    '|' + ;
    'The "Cancel" button exits the dialog box and does not act on any ' + ;
    'of the data you entered.' + ;
    '|' + ;
    'Use the following keys to navigate dialog boxes:' + ;
    '|' + ;
    'Tab or Down arrow: moves to the next item.' + ;
    '|' + ;
    'Shift-Tab or Up arrow: moves to the previous item.' + ;
    '|' + ;
    'Enter: From any item except the "Cancel" button, the Enter key ' + ;
    'accepts all items and exits the dialog box.  Equivalent to ' + ;
    'pressing the "OK" button.' + ;
    '|' + ;
    'Esc: Exits the dialog box without performing any action.  ' + ;
    'Equivalent to pressing the "Cancel" button.'

CASE nContext == H_MOUSE

  nPad  := 36
  cText := ;
    'DBE fully supports a mouse.  The mouse buttons work as follows:' + ;
    '|' + ;
    'Left button: select~' + ;
    'Right button: equivalent to pressing ESC key~' + ;
    'Middle button, if present: equivalent to pressing ENTER key~' + ;
    '|' + ;
    'A single click of the left mouse button on a command prompt or a ' + ;
    'special mouse area executes that the command associated with ' + ;
    'that area.  A double click of the left mouse button on a menu ' + ;
    'item or a browse field moves the cursor to that position.' + ;
    '|' + ;
    'Scroll bars: The data browse and most other browse areas have ' + ;
    'vertical and horizontal scroll bars.  The vertical scroll bars are ' + ;
    'on the right border of the box around the browse, and the ' + ;
    'horizontal scroll bars are on the bottom border.  These areas have ' + ;
    'special marks, which are equivalent to the following keys:' + ;
    '|' + ;
    PADR('"x" in top left corner', nPad) + 'Esc~' + ;
    PADR('"' + C_UP_TRIANGLE + ;
    '" in top right corner'   , nPad) + 'Ctrl-PageUp~' + ;
    PADR('"' + C_DOWN_TRIANGLE + ;
    '" in bottom right corner', nPad) + 'Ctrl-PageDown~' + ;
    PADR('Between "' + C_UP_TRIANGLE + '" and "' + C_UPDOWN_ARROW + ;
    '" on right side'         , nPad) + 'PageUp~' + ;
    PADR('Between "' + C_UPDOWN_ARROW + '" and "' + C_DOWN_TRIANGLE + ;
    '" on right side'         , nPad) + 'PageDown~' + ;
    PADR('"' + C_LEFT_TRIANGLE + ;
    '" in bottom left corner' , nPad) + 'Ctrl-Home~' + ;
    PADR('"' + C_RIGHT_TRIANGLE + ;
    '" in bottom right corner', nPad) + 'Ctrl-End~' + ;
    PADR('Between "' + C_LEFT_TRIANGLE + '" and "' + C_LEFT_RIGHT_ARROW + ;
    '" on bottom side'        , nPad) + 'Ctrl-Left~' + ;
    PADR('Between "' + C_LEFT_RIGHT_ARROW + '" and "' + C_RIGHT_TRIANGLE + ;
    '" on bottom side'        , nPad) + 'Ctrl-Right' + ;
    '|' + ;
    'On a vertical scroll bar, the up-down arrow (' + C_UPDOWN_ARROW + ;
    ') changes to a down arrow (' + C_DOWN_ARROW + ') when the browse ' + ;
    'is on the first item, and it changes to an up arrow (' + C_UP_ARROW + ;
    ') when the browse is on the last item.  Similarly, on a horizontal ' + ;
    'scroll bar, the left-right arrow (' + C_LEFT_RIGHT_ARROW + ') ' + ;
    'changes to a right arrow (' + C_RIGHT_ARROW + ') when the browse ' + ;
    'is on the first column, and it changes to a left arrow (' + ;
    C_LEFT_ARROW + ') when the browse is on the last column.' + ;
    '|' + ;
    'The vertical scroll bars for a database displays an unmoving block ' + ;
    '(' + C_CENTER_BLOCK + ') instead of a moving arrow (' + ;
    C_UPDOWN_ARROW + ') if a database driver does not report index ' + ;
    'positions.  Although the block does not move, you can click on ' + ;
    'these static scroll bars to page up and down through the database.'

CASE nContext == H_HOTKEYS

  nPad  := 20
  cText := ;
    'The following keys can be used almost anywhere in DBE.' + ;
    '|' + ;
    PADR('F1 or Alt-H'     , nPad) + 'Help menu~'             + ;
    PADR('F2'              , nPad) + 'Utility Menu~'          + ;
    PADR('F3'              , nPad) + 'Evaluate~'              + ;
    PADR('F4'              , nPad) + 'History~'               + ;
    PADR('F5'              , nPad) + ;
    'Database structure of current database~' + ;
    PADR('F6'              , nPad) + ;
    'Database structure of all databases~'    + ;
    PADR('F8'              , nPad) + 'Local directory~'       + ;
    PADR('F9'              , nPad) + 'Global directory~'      + ;
    PADR('F10'             , nPad) + 'ASCII chart~'           + ;
    PADR('F11 or Alt-F1'   , nPad) + 'Color chart~'           + ;
    PADR('F12 or Alt-F2'   , nPad) + 'Calendar~'              + ;
    PADR('Alt-F3'          , nPad) + 'Repeat keystroke~'      + ;
    PADR('Alt-F4'          , nPad) + ;
    'Start/Stop keystroke recording~' + ;
    PADR('Alt-F5'          , nPad) + 'Play back last record~' + ;
    PADR('Alt-F6'          , nPad) + 'Play back keystrokes~'  + ;
    PADR('Alt-F7'          , nPad) + 'Quick quit to DOS~'     + ;
    PADR('Alt-F8'          , nPad) + 'Local file manager~'    + ;
    PADR('Alt-F9'          , nPad) + 'Global file manager~'   + ;
    PADR('Alt-F10'         , nPad) + 'Set hot keys~'          + ;
    PADR('Alt-Del or Alt-O', nPad) + 'Copy to clipboard~'     + ;
    PADR('Alt-Ins or Alt-I', nPad) + 'Paste from clipboard~'

CASE nContext == H_SETUP

  nPad  := 27
  cText := ;
    'The setup window is the window with the five small panes at the ' + ;
    'top of the DBE screen.  It controls the databases that you open in ' + ;
    'a DBE session.  The highlight bar in one of these panes indicates ' + ;
    'that you are navigating that pane.' + ;
    '|' + ;
    'To open a database, go to the "Database" pane and press the Ins ' + ;
    'key, or use the Open menu.  To move between the setup panes, press ' + ;
    'the right and left arrow keys.  To jump to the data browse, press ' + ;
    'press the Tab key.' + ;
    '|' + ;
    'The following keys can be used as shortcuts when you are in the ' + ;
    'setup window.' + ;
    '|' + ;
    PADR('H'                     , nPad) + 'Help~'                            + ;
    PADR('Tab'                   , nPad) + ;
    'Open menu / Jump to data browse~' + ;
    PADR('Space'                 , nPad) + ;
    'Open menu / View current setup entry~' + ;
    PADR('Enter'                 , nPad) + ;
    'Open menu / Select current setup entry~'  + ;
    PADR('Ins on Database'       , nPad) + 'Open menu / Database open~'   + ;
    PADR('Del on Database'       , nPad) + 'Open menu / Database close~'  + ;
    PADR('Ins on Index'          , nPad) + 'Open menu / Index open~'      + ;
    PADR('Del on Index'          , nPad) + 'Open menu / Index close~'     + ;
    PADR('Ins on Relation'       , nPad) + 'Set  menu / Relation add~'    + ;
    PADR('Ctrl-Enter on Relation', nPad) + 'Set  menu / Relation edit~'   + ;
    PADR('Del on Relation'       , nPad) + 'Set  menu / Relation delete~' + ;
    PADR('Ins on Filter'         , nPad) + 'Set  menu / Filter add~'      + ;
    PADR('Ctrl-Enter on Filter'  , nPad) + 'Set  menu / Filter edit~'     + ;
    PADR('Del on Filter'         , nPad) + 'Set  menu / Filter delete~'   + ;
    PADR('Ins on Field'          , nPad) + 'View menu / Add field~'       + ;
    PADR('Ctrl-Enter on Field'   , nPad) + 'View menu / Edit field~'      + ;
    PADR('Del on Field'          , nPad) + 'View menu / Delete field~'    + ;
    PADR('Ctrl-' + C_UP_ARROW   + ' or Alt-U on Field', nPad)          + ;
    'View menu / < move field up~'    + ;
    PADR('Ctrl-' + C_DOWN_ARROW + ' or Alt-B on Field', nPad)          + ;
    'View menu / > move field down~' + ;
    PADR('Esc'            , nPad) + 'Open menu / Quit to DOS~'         + ;
    '|' + ;
    'For more information on the setup window, see the help topic ' + ;
    '"Introduction to DBE".'

CASE nContext == H_DATABROWSE

  nPad  := 15
  cText := ;
    'The data browse is the large window at the bottom of the screen.  ' + ;
    'Once you have opened one or more databases, you browse the ' + ;
    'database in the data browse window.  The following keys can be ' + ;
    'used as shortcuts when you are in the data browse.' + ;
    '|' + ;
    'The columns in the data browse are controlled by the Field pane of ' + ;
    'the setup window.  Initially, all the fields in the database are ' + ;
    'in the Field pane.' + ;
    '|' + ;
    'The data browse can be set in two different modes, the ' + ;
    '"horizontal" mode, in which each record is on one line, and the ' + ;
    '"vertical" mode, which displays only one record and lists each ' + ;
    'field on one line.  Use the View menu to switch between these two ' + ;
    'modes.' + ;
    '|' + ;
    'In the horizontal mode, there are two columns at the extreme left ' + ;
    'side of the browse.  The first is marked "*".  This indicates the ' + ;
    'deleted status of the record: blank means not deleted, "*" means ' + ;
    'deleted.  The second column is labelled "#".  This is the record ' + ;
    'number.' + ;
    '|' + ;
    PADR('H'         , nPad) + 'Help~'                           + ;
    PADR('Tab'       , nPad) + 'Open   menu / Jump to setup~'    + ;
    PADR('I'         , nPad) + 'Go     menu / Incremental seek~' + ;
    PADR('P'         , nPad) + 'Go     menu / Previous~'         + ;
    PADR('N'         , nPad) + 'Go     menu / Next~'             + ;
    PADR('Space'     , nPad) + 'Record menu / Field look~'       + ;
    PADR('Enter or E', nPad) + 'Record menu / Field modify~'     + ;
    PADR('Ctrl-Enter', nPad) + 'Record menu / Enter data~'       + ;
    PADR('Ins'       , nPad) + 'Record menu / New record~'       + ;
    PADR('Del'       , nPad) + 'Record menu / Delete record~'    + ;
    PADR('Esc'       , nPad) + 'Open   menu / Quit to DOS~'      + ;
    '|'  + ;
    PADR('F7'        , nPad) + 'While data is being generated on ' + ;
    'the screen, F7 halts the data generation and returns you to ' + ;
    'the setup window.'                                          + ;
    '|'  + ;
    'For more information on the data browse, see the help topic ' + ;
    '"Introduction to DBE".'

CASE nContext == H_MENUS

  cText := ;
    'Open menu: opens and creates databases and indexes; expands setup ' + ;
    'browse to full screen; reads and writes DBE files; quits DBE.' + ;
    '|' + ;
    'File menu: performs operations on the database that involve more ' + ;
    'than one record, for example, replace, pack, list.' + ;
    '|' + ;
    'Record menu: performs operations on a single record, for example, ' + ;
    'delete a record, store a record into memory, store a field into '  + ;
    'memory.' + ;
    '|' + ;
    'Go menu: moves to a new record, for example, seek, locate, skip.' + ;
    '|' + ;
    'Set menu: performs setup functions, for example, set colors, add ' + ;
    'a relation, add a filter, and record or play back keystrokes.' + ;
    '|' + ;
    'View menu: performs functions that affect the display, for ' + ;
    'example, change the number of lines on the screen, add and ' + ;
    'delete a field to the data browse, switch between horizontal and ' + ;
    'vertical field display.' + ;
    '|' + ;
    'Util menu: miscellaneous functions, such as MEM file editor, ' + ;
    'directory display, copy file, ASCII table.' + ;
    '|' + ;
    'For further information on each menu, see the help topic for ' + ;
    'that menu.'

CASE nContext == H_DRIVERS

  cText := ;
    'DBE supports four database drivers.  These drivers control the ' + ;
    'format of both database files and index files.  When you open a ' + ;
    'database, you must specify the driver you wish to use.  Although ' + ;
    'the default extension of the database file for all drivers is ' + ;
    '.DBF, the exact format of the database file may vary from one ' + ;
    'driver to another.  The default extension of the index files ' + ;
    'varies with each driver.  In the list below, the three capital ' + ;
    'letters are the default extension of the index files for that ' + ;
    'driver.' + ;
    '|' + ;
    'Driver                  NTX        NDX         CDX        MDX~' + ;
    'Attribute~' + ;
    '|' + ;
    'Compatible with         Clipper    dBASE III   FoxPro     dBASE IV~' + ;
    'Database extension      .DBF       .DBF        .DBF       .DBF~' + ;
    'Memo file extension     .DBT       .DBT        .FPT       .DBT~' + ;
    'Index file extension    .NTX       .NDX        .CDX       .MDX~' + ;
    'Multiple tags in~' + ;
    '  one index file        No         No          Yes        Yes~' + ;
    'Maximum number of~' + ;
    '  tags in index file    1          1           50         47~' + ;
    'FOR clause allowed      Yes        No          Yes        Yes~' + ;
    'DESCENDING allowed      Yes        No          Yes        Yes~' + ;
    'UNIQUE allowed          Yes        Yes         Yes        Yes~' + ;
    'Reports index position  Yes        No          No         No~' + ;
    '|' + ;
    'All drivers can read and modify database files that can be read ' + ;
    'by dBASE III.  However, a database file that is created with CDX ' + ;
    'driver cannot be read by dBASE IV or dBASE III, or by the MDX, ' + ;
    'NDX, or NTX drivers in Clipper.  Similarly, a database file ' + ;
    'created with the MDX driver cannot be read by FoxPro or dBASE ' + ;
    'III, or by the CDX, NDX, or NTX drivers in Clipper.  Databases ' + ;
    'created with NDX or NTX drivers are in identical format, which  ' + ;
    'can be read by any dBASE-compatible software and by all Clipper ' + ;
    'drivers.'

CASE nContext == H_TRANSFER

  cText := ;
    'DBE uses a data transfer buffer to transfer data between its ' + ;
    'various objects.  For example, if you are opening a database, you ' + ;
    'will enter a dialog box in which you will be prompted for a ' + ;
    'database name.  Instead of typing the name, you can press F8 to ' + ;
    'bring up the local directory listing of .DBF files.  When you ' + ;
    'press Enter on a file in this list, the name of the database is ' + ;
    'transferred from the directory list into the database name line in ' + ;
    'the dialog box, and appears on that line.' + ;
    '|' + ;
    'DBE dialog boxes have the names of a few hot keys that you can ' + ;
    'press to retrieve data in this way.  You can press any hot key to ' + ;
    'retrieve data, not just the ones printed in the box.' + ;
    '|' + ;
    'The following DBE objects save a selected item into the data ' + ;
    'transfer buffer:' + ;
    '|' + ;
    'Record menu / Scatter~' + ;
    'Record menu / Store field~' + ;
    'Util menu / Paste from clipboard~' + ;
    'Util menu / History~' + ;
    'Util menu / Structures current and all~' + ;
    'Util menu / Directory local and global~' + ;
    'Util menu / File manager local and global~' + ;
    'Util menu / ASCII table~' + ;
    'Util menu / Color table~' + ;
    'Util menu / Calendar~' + ;
    '|' + ;
    'The following DBE objects retrieve a selected item from the data ' + ;
    'transfer:' + ;
    '|' + ;
    'Dialog box lines, except logical fields and pick lists~' + ;
    'Memo editor~' + ;
    'Record menu / Field modify~' + ;
    'Record menu / Gather~' + ;
    'Record menu / Retrieve field replace~' + ;
    'Record menu / Retrieve field append~' + ;
    'Structure editor / Edit field name~' + ;
    'Util menu / Copy to clipboard~' + ;
    'Memvar editor / Edit memvar value~' + ;
    '|' + ;
    'A wide variety of information is saved into the history and can ' + ;
    'thus be used in a data transfer.  For more information, see the ' + ;
    'help topic "History".'

CASE nContext == H_INCSEARCH

  cText := ;
    'Most browses other than databases alllow a direct incremental ' + ;
    'search on the main data item.  This means that you can search for ' + ;
    'a specific main data item by typing in its value one character at ' + ;
    'a time.  Immediately after you press each character, DBE searches ' + ;
    'for the next occurrence of the data item starting with the letters ' + ;
    'you have entered so far.  If it does not find the letters you have ' + ;
    'entered, it moves to the next value.  As you press each character, ' + ;
    'DBE underscores the latest letter it has found.' + ;
    '|' + ;
    'Within the browses that support it, any printing character enters ' + ;
    'DBE into incremental search mode.  Any non-printing character ' + ;
    'exits the incremental search, and any subsequent printing ' + ;
    'character starts the incremental search over from the first ' + ;
    'character position.' + ;
    '|' + ;
    'DBE also supports an incremental seek on indexed databases, but it ' + ;
    'works somewhat differently.  See the help topic "Incremental seek".' + ;
    '|' + ;
    'The following DBE objects support incremental search:' + ;
    '|' + ;
    'Open menu / Expand current setup~' + ;
    'Util menu / History~' + ;
    'Util menu / Structure~' + ;
    'Util menu / Directory~' + ;
    'Util menu / File manager~' + ;
    'Util menu / Memvar edit~' + ;
    'Structure editor~' + ;
    'Help topic lists~'

CASE nContext == H_MEMOEDIT

  nPad  := 17
  cText := ;
    'Special keys in memo editor:' + ;
    '|' + ;
    PADR('Ins'           , nPad) + ;
    'Switches between insert and overstrike mode~' + ;
    PADR('Ctrl-Left'     , nPad) + 'Moves one word left~'               + ;
    PADR('Ctrl-Right'    , nPad) + 'Moves one word right~'              + ;
    PADR('Ctrl-Backspace', nPad) + 'Deletes word backwards~'            + ;
    PADR('Ctrl-T'        , nPad) + 'Deletes word forwards~'             + ;
    PADR('Ctrl-Y'        , nPad) + 'Deletes rest of line~'              + ;
    PADR('Ctrl-P'        , nPad) + 'Reformats rest of text~'            + ;
    PADR('Alt-Del'       , nPad) + 'Appends rest of line to clipboard~' + ;
    PADR('Alt-Ins'       , nPad) + 'Pasted clipboard to text~'          + ;
    PADR('F4'            , nPad) + 'Access history and clipboard~'      + ;
    PADR('F7'            , nPad) + 'Saves and exits~'                   + ;
    PADR('Esc'           , nPad) + 'Exits without saving'

CASE nContext == H_STRUEDIT

  cText := ;
    'The structure editor allows you to create a new structure or to ' + ;
    'modify an existing one.' + ;
    '|' + ;
    'Each line contains the following data for each field:  ' + ;
    HelpPar(HP_FLDCOLS) + ;
    '|' + ;
    'To modify the structure, use the following keys.  The changes you ' + ;
    'make within the structure editor are not saved until you press the ' + ;
    'End key to exit the structure editor.' + ;
    '|' + ;
    'Ins - Add:  Adds a new field to the end of the structure.  The new ' + ;
    'field is assigned the following initial values:  a field name of ' + ;
    '"FIELDx", where x is the field number of the new field; a type of ' + ;
    '"C" (character); and a length of 10.  Use the Enter key to change ' + ;
    'these values.' + ;
    '|' + ;
    'Del - Delete:  Deletes the highlighted field from the structure.' + ;
    '|' + ;
    'Enter - Edit:  Changes the values of the highlighted fields.  You ' + ;
    'can change only the name, type, length, and decimals.  For the ' + ;
    'type, enter "C" (character), "N" (numeric), "D" (date), "L" ' + ;
    '(logical), or "M" (memo).  The length of a logical field is always ' + ;
    '1; the length of a date field is always 8; and the "length" of a ' + ;
    'memo field is always 10.  (The "length" of the memo field in the ' + ;
    'structure array does not indicate the actual length of the memo ' + ;
    'data, but of the pointer to the data in .DBF file.)  The decimals ' + ;
    'value applies only to numeric fields; it indicates how many ' + ;
    'positions out of the length are to be used for decimal places.  If ' + ;
    'the decimals is nonzero, then the decimal point uses up one ' + ;
    'position out of the length.' + ;
    '|' + ;
    'Ctrl-' + C_UP_ARROW + ' - Move up:  Moves the highlighted field up ' + ;
    'one position within the structure.' + ;
    '|' + ;
    'Ctrl-' + C_DOWN_ARROW + ' - Move down:  Moves the highlighted ' + ;
    'field down one position within the structure.' + ;
    '|' + ;
    'End - Save:  Saves the changes you have made and exits the ' + ;
    'structure editor.' + ;
    '|' + ;
    'Esc - Cancel:  Throws away the changes you have made and exits the ' + ;
    'structure editor.' + ;
    '|' + ;
    'You may perform an incremental search on the field name. See the ' + ;
    'help topic "Incremental search".'

CASE nContext == H_DBEFILE

  cText := ;
    'All aspects of a DBE session may be saved into a DBE file.  ' + ;
    'This includes databases, indexes, relations, filters, fields, ' + ;
    'everything in the Set menu, the number of display lines, the ' + ;
    'current record for each database, the history, and a keystroke ' + ;
    'playback string.  A DBE file is an ASCII file, so you can edit ' + ;
    'it inside or outside of DBE.' + ;
    '|' + ;
    'Each line of a DBE file has the form <keyword> = <value>.  The ' + ;
    'settings for a given database, including database name and ' + ;
    'attributes, index names, relations, filters, and fields, are in ' + ;
    'a work area section that begins with "Database =".  Keyword ' + ;
    'lines for non-database settings may be put anywhere in the file.' + ;
    '|' + ;
    'Keywords are not case-significant.  If a keyword contains more ' + ;
    'than one word, the spaces may be omitted.  All spaces to the ' + ;
    'left of the equal sign are optional.' + ;
    '|' + ;
    'Values are case-significant only if they are used that way in ' + ;
    'the program.  For example, database names are not ' + ;
    'case-significant, but filter expressions are.' + ;
    '|' + ;
    'DBE files can be read and written via the Open menu.  They can ' + ;
    'also be read by passing in a reference to them on the command ' + ;
    'line.  For more information on this second technique, see the ' + ;
    'help topic "DBE command line parameters".' + ;
    'For detailed information on DBE keywords, see the help topic' + ;
    '"DBE files keyword reference".'

CASE nContext == H_DBEKEY

  nPad  := 20
  cText := ;
    'The following keywords are used in DBE files.  See the help topic ' + ;
    '"DBE files basic information.' + ;
    '|' + ;
    'Database keywords:  The following keywords can be used only in a ' + ;
    'database section.  A database section starts with the keyword ' + ;
    '"Database =".  After this keyword, other database keywords can be ' + ;
    'in any order.' + ;
    '|' + ;
    PADR('Database'           , nPad) + '= <file name>~'            + ;
    PADR('Alias'              , nPad) + '= <alias>~'                + ;
    PADR('Exclusive'          , nPad) + '= Y/N~'                    + ;
    PADR('Readonly'           , nPad) + '= Y/N~'                    + ;
    PADR('Driver'             , nPad) + '= <driver name>~'          + ;
    PADR('Index'              , nPad) + '= <file name>~'            + ;
    PADR('Relation'           , nPad) + '= <alias>, <expression>~'  + ;
    PADR('Filter'             , nPad) + '= <expression>~'           + ;
    PADR('Filter Name'        , nPad) + '= <name>~'                 + ;
    PADR('Field'              , nPad) + '= <field name>~'           + ;
    PADR('Field Aliased'      , nPad) + '= <alias>, <field name>~'  + ;
    PADR('Field Calculated'   , nPad) + '= <name>, <expression>~'   + ;
    PADR('Horizontal'         , nPad) + '= Y/N~'                    + ;
    '  set to Y if "View / Horizontal" is in effect~' + ;
    PADR('Field Sort'         , nPad) + '= Y/N~'                    + ;
    '  set to Y if "View / Field order for fields" is in effect~'   + ;
    PADR('Record No'          , nPad) + '= <number>~'               + ;
    '  set to current record number~' + ;
    PADR('Index No'           , nPad) + '= <number>~'               + ;
    '  set to current index number~' + ;
    PADR('Relation No'        , nPad) + '= <number>~'               + ;
    '  set to current relation number~' + ;
    PADR('Filter No'          , nPad) + '= <number>~'               + ;
    '  set to current filter number~' + ;
    PADR('Field No'           , nPad) + '= <number>~'               + ;
    '  set to current field number~' + ;
    '|'
  cText += ;
    'Settings keywords:  The following keywords can be used anywhere ' + ;
    'in the file.' + ;
    '|' + ;
    PADR('Database No'        , nPad) + '= <number>~'               + ;
    '  set to current database number~' + ;
    PADR('Max Lines'          , nPad) + '= Y/N~'                    + ;
    '  set to Y if "View / Large number of lines" is in effect~' + ;
    PADR('Local'              , nPad) + '= <directory name>~'       + ;
    PADR('Keystroke Recording', nPad) + '= Y/N~'                    + ;
    '|'
  cText += ;
    PADR('Hot Key Shift F1'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F2'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F3'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F4'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F5'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F6'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F7'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F8'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F9'   , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F10'  , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F11'  , nPad) + '= <keystroke string>~'     + ;
    PADR('Hot Key Shift F12'  , nPad) + '= <keystroke string>~'     + ;
    '|'
  cText += ;
    PADR('Color Main Screen'  , nPad) + '= <color string>~'         + ;
    PADR('Color Edit Cell'    , nPad) + '= <color string>~'         + ;
    PADR('Color Memo View'    , nPad) + '= <color string>~'         + ;
    PADR('Color Menus'        , nPad) + '= <color string>~'         + ;
    PADR('Color Hotkey Menus' , nPad) + '= <color string>~'         + ;
    PADR('Color Get Boxes'    , nPad) + '= <color string>~'         + ;
    PADR('Color Tables'       , nPad) + '= <color string>~'         + ;
    PADR('Color Pick Lists'   , nPad) + '= <color string>~'         + ;
    PADR('Color Status Boxes' , nPad) + '= <color string>~'         + ;
    PADR('Color File View'    , nPad) + '= <color string>~'         + ;
    PADR('Color Memvar Browse', nPad) + '= <color string>~'         + ;
    PADR('Color Memvar Edit'  , nPad) + '= <color string>~'         + ;
    PADR('Color Memvar Status', nPad) + '= <color string>~'         + ;
    PADR('Color Help Screen'  , nPad) + '= <color string>~'         + ;
    PADR('Color Quit Box'     , nPad) + '= <color string>~'         + ;
    '|'
  cText += ;
    PADR('Cursor Insert'      , nPad) + '= <cursor code>~'          + ;
    PADR('Cursor Overstrike'  , nPad) + '= <cursor code>~'          + ;
    '|'
  cText += ;
    PADR('Small Setup Lines'  , nPad) + '= <number of lines>~'      + ;
    PADR('Large Setup Lines'  , nPad) + '= <number of lines>~'      + ;
    '|'
  cText += ;
    PADR('Print Reset'        , nPad) + '= Y/N~'                    + ;
    PADR('Print Pitch'        , nPad) + '= None/10/12/16~'          + ;
    PADR('Print Lpi'          , nPad) + '= None/6/8~'               + ;
    PADR('Print Orient'       , nPad) + '= None/Portrait/Landscape~'+ ;
    PADR('Print Paper'        , nPad) + '= None/Letter/Legal~'      + ;
    PADR('Print Tray'         , nPad) + '= None/1/2/3/4/Manual~'    + ;
    PADR('Print Bin'          , nPad) + '= None/Top/Rear~'          + ;
    PADR('Print Duplex'       , nPad) + ;
      '= N/Simplex/Duplex Long/Duplex Short~' + ;
    PADR('Print Symbol'       , nPad) + '= None/PC/ASCII~'          + ;
    '|'
  cText += ;
    PADR('Date Format'        , nPad) + '= <date format string>~'   + ;
    PADR('Epoch'              , nPad) + '= <number>~'               + ;
    PADR('Decimals'           , nPad) + '= <number>~'               + ;
    PADR('Memo Width'         , nPad) + '= <number>~'               + ;
    PADR('Memo Tab'           , nPad) + '= <number>~'               + ;
    PADR('Margin'             , nPad) + '= <number>~'               + ;
    PADR('Bell'               , nPad) + '= Y/N~'                    + ;
    PADR('Deleted'            , nPad) + '= Y/N~'                    + ;
    PADR('Softseek'           , nPad) + '= Y/N~'                    + ;
    PADR('Cancel'             , nPad) + '= Y/N~'                    + ;
    PADR('Odometer'           , nPad) + '= <number>~'               + ;
    '|'
  cText += ;
    'Play keyword:  The keyword "Play =" may be set to a keystroke ' + ;
    'string.  This keystroke string will be played back when all other ' + ;
    'keywords have been loaded.' + ;
    '|' + ;
    PADR('Play'               , nPad) + '= <keystroke string>~'    + ;
    '|' + ;
    'History section:  This section must be at the end of the file.  ' + ;
    'It consists of the keyword "History =" on a line by itself, ' + ;
    'followed by history items.  All lines in the file after this ' + ;
    'keyword are interpreted as history items.' + ;
    '|' + ;
    PADR('History'            , nPad) + '=~' + ;
    '|'                      + ;
    'For an example of all these keywords, generate a DBE file ' + ;
    'through the Open menu.'

CASE nContext == H_KRECORD

  cText := ;
    'A keystroke record is a series of keystrokes that is stored and ' + ;
    'used later.  The keystrokes are stored in record mode, and are ' + ;
    'used in playback mode.' + ;
    '|' + ;
    'To make a keystroke record, enter the keystroke record mode in ' + ;
    'Set menu / Key record.  While DBE is in keystroke record mode, a ' + ;
    'red bar at the bottom of the screen says "Alt-F4 Exit keystroke ' + ;
    'recording".  You may then press any series of keys.  When you ' + ;
    'press Alt-F4, keystroke recording stops.  The keystroke record ' + ;
    'is then stored for later use.  To play back the keystrokes, use ' + ;
    'Util menu / Play back last record (hot key Alt-F5), or Util menu / ' + ;
    'Play back keystrokes (hot key Alt-F6), or Set Menu / Set hot key ' + ;
    '(hot key Alt-F10).' + ;
    '|' + ;
    'A keystroke record is stored as an ASCII string.  These ' + ;
    'ASCII strings are stored in the history.  Keys that do not ' + ;
    'correspond to displayable ASCII characters are converted to a ' + ;
    'series of displayable characters in the keystroke string.  For a ' + ;
    'list of these conversions, see the help topic "Keystroke code ' + ;
    'reference".'

CASE nContext == H_KRCODE

  nPad  := 24
  cText := ;
    'The following codes are used in keystroke record strings to ' + ;
    'represent characters that do not have a displayable ' + ;
    'representation.  See the help topic "Keystroke recording".' + ;
    '|' + ;
    PADR('Left bracket ['       , nPad) + '[LB]~'            + ;
    PADR('Right bracket ]'      , nPad) + '[RB]~'            + ;
    PADR('Enter'                , nPad) + '[EN]~'            + ;
    PADR('Escape'               , nPad) + '[ESC]~'           + ;
    PADR('Up arrow'             , nPad) + '[UP]~'            + ;
    PADR('Down arrow'           , nPad) + '[DN]~'            + ;
    PADR('Left arrow'           , nPad) + '[LF]~'            + ;
    PADR('Right arrow'          , nPad) + '[RT]~'            + ;
    PADR('Home'                 , nPad) + '[HOME]~'          + ;
    PADR('End'                  , nPad) + '[END]~'           + ;
    PADR('Page Up'              , nPad) + '[PU]~'            + ;
    PADR('Page Down'            , nPad) + '[PD]~'            + ;
    PADR('Insert'               , nPad) + '[INS]~'           + ;
    PADR('Delete'               , nPad) + '[DEL]~'           + ;
    PADR('Backspace'            , nPad) + '[BS]~'            + ;
    PADR('Tab'                  , nPad) + '[TAB]~'           + ;
    PADR('Shift Tab'            , nPad) + '[/TAB]~'          + ;
    PADR('F1 to F12'            , nPad) + '[F1] to [F12]~'   + ;
    PADR('Shift F1 to Shift F12', nPad) + '[/F1] to [/F12]~' + ;
    PADR('Ctrl Print Scrn'      , nPad) + '[^PS]~'           + ;
    PADR('Ctrl keypad /'        , nPad) + '[^K/]~'           + ;
    PADR('Ctrl keypad *'        , nPad) + '[^K*]~'           + ;
    PADR('Ctrl keypad -'        , nPad) + '[^K-]~'           + ;
    PADR('Ctrl keypad +'        , nPad) + '[^K+]~'           + ;
    PADR('Alt keypad /'         , nPad) + '[*K/]~'           + ;
    PADR('Alt keypad *'         , nPad) + '[*K*]~'           + ;
    PADR('Alt keypad -'         , nPad) + '[*K-]~'           + ;
    PADR('Alt keypad +'         , nPad) + '[*K+]~'           + ;
    PADR('Alt keypad Enter'     , nPad) + '[*KEN]~'          + ;
    PADR('Keypad 5'             , nPad) + '[K5]~'            + ;
    '|' + ;
    'Three prefix characters are used with keys where Shift, Ctrl, or ' + ;
    'Alt used with the key is different from the key alone.  For ' + ;
    'Shift, the prefix is /; for Ctrl, the prefix is ^, and for Alt ' + ;
    'the prefix is *.  Examples:' + ;
    '|' + ;
    PADR('Ctrl Up Arrow'        , nPad) + '[^UP]~'           + ;
    PADR('Ctrl Z'               , nPad) + '[^Z]~'            + ;
    PADR('Alt Up Arrow'         , nPad) + '[*UP]~'           + ;
    PADR('Alt A'                , nPad) + '[*A]~'            + ;
    PADR('Shift Tab'            , nPad) + '[/TAB]~'          + ;
    PADR('Shift F1'             , nPad) + '[/F1]~'           + ;
    '|' + ;
    'A prefix of # followed by a number means a numerical ASCII ' + ;
    'value.  Examples:' + ;
    '|' + ;
    PADR('A'                    , nPad) + '[#65]~'           + ;
    PADR('Enter'                , nPad) + '[#13]~'

END

RETURN cText  //  HelpGeneral

//***************************************************************************

STATIC FUNCTION HelpOpen(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_OPENMENU

  cText := ;
    'Opens and creatse databases and indexes; expands setup browse to ' + ;
    'full screen; reads and writes DBE files; quits DBE.' + ;
    '|' + ;
    HelpPar(HP_SHORTSETUP) + ;
    '|' + ;
    HelpPar(HP_SHORTDATA) + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'Database open:  Opens a new database and adds it to the list of ' + ;
    'databases. (S) (*)' + ;
    '|' + ;
    'Database and index open:  Opens a new database and an index of ' + ;
    'the same name, and adds them to the lists of databases and ' + ;
    'indexes. (*)' + ;
    '|' + ;
    'Database close:  Closes the highlighted database and remove it ' + ;
    'from the list of databases. (S)' + ;
    '|' + ;
    'Databases all close:  Closes all databases and clear all the ' + ;
    'setup lists.' + ;
    '|' + ;
    'Index open:  Opens a new index file for the current database and ' + ;
    'adds its orders to the list of indexes. (S) (*)' + ;
    '|' + ;
    'Index create:  Creates a new order for the current database and ' + ;
    'adds it to the list of indexes. (*)' + ;
    '|' + ;
    'Indexes recreate:  Regenerates all the indexes for the current ' + ;
    'database (*)' + ;
    '|' + ;
    'Index destroy:  For an index file that has more than one order, ' + ;
    'removes one of those orders from the index file. (*)' + ;
    '|' + ;
    'Index close:  Closes the highlighted index and removes it from ' + ;
    'the list of indexes. (S)' + ;
    '|' + ;
    'Indexes all close:  Closes all indexes for the current database ' + ;
    'and clears the list of indexes.' + ;
    '|' + ;
    'Jump to setup:  Goes to the setup lists. (D)' + ;
    '|' + ;
    'Jump to data browse:  Goes to the data browse. (S)' + ;
    '|' + ;
    'Select current setup entry:  In the current setup pane, makes the ' + ;
    'highlighted setup entry the current setup entry. (S)' + ;
    '|' + ;
    'Expand current setup:  For the current setup pane, creates an ' + ;
    'expanded browse which shows more information. (S) (*)' + ;
    '|' + ;
    'Create structure:  Creates a new database by creating a new ' + ;
    'structure on screen, and then opens it. (*)' + ;
    '|' + ;
    'Create database from:  Creates a new database from a structure ' + ;
    'extended database, and then opens it. (*)' + ;
    '|' + ;
    'Read DBE file:  Reads settings from a DBE file into the current ' + ;
    'DBE session.  See the help topic "DBE files basic information". ' + ;
    '(*)' + ;
    '|' + ;
    'Write DBE file:  Writes settings for the current DBE session to a ' + ;
    'DBE file.  See the help topic "DBE files basic information". (*)' + ;
    '|' + ;
    'Quit: Exit DBE. (S) (D) (*)'

CASE nContext == H_DATAOPEN

  cText := ;
    'Location: Open menu~' + ;
    'Shortcut from Database setup pane: Insert key~' + ;
    '|' + ;
    'Opens a new database and adds it to the list of Databases.' + ;
    '|' + ;
    'Name:  Enter the name of the database file, ' + HelpPar(HP_FILENAME) + ;
    'is .DBF.' + ;
    '|' + ;
    HelpPar(HP_ALIAS) + ;
    '|' + ;
    HelpPar(HP_DRIVER) + ;
    '|' + ;
    HelpPar(HP_EXREAD)

CASE nContext == H_BOTHOPEN

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Opens two new files:  a database and an index, with the same ' + ;
    'name, except for the extension.  A new entry is added to both the ' + ;
    'list of databases and the list of indexes.' + ;
    '|' + ;
    'Name:  Enter the common name of both the database and index ' + ;
    'files, ' + HelpPar(HP_FILENAME) + 'is .DBF, and the default ' + ;
    'extension for the index is the index extension of the driver.  If ' + ;
    'you use a driver that allows multiple tags (orders) in a single ' + ;
    'index file, all the tags are opened when you open the index file, ' + ;
    'and the tag names are added to the list of indexes.' + ;
    '|' + ;
    HelpPar(HP_ALIAS) + ;
    '|' + ;
    HelpPar(HP_DRIVER) + ;
    '|' + ;
    HelpPar(HP_EXREAD)

CASE nContext == H_INDOPEN

  cText := ;
    'Location: Open menu~' + ;
    'Shortcut from Index setup pane: Insert key~' + ;
    '|' + ;
    'Opens a new index file for the current database and adds it to ' + ;
    'the list of indexes.' + ;
    '|' + ;
    'Name:  Enter the name of the index file, including the drive ' + ;
    'and directory if needed.  The default extension depends on the ' + ;
    'driver that the database was opened with.  If you used a driver ' + ;
    'that allows multiple tags (orders) in a single index file, all ' + ;
    'the tags are opened when you open the file, and the tag names are ' + ;
    'added to the list of indexes.'

CASE nContext == H_INDCREATE

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'For the current database, this option either creates a new index ' + ;
    'file, or, in the case of a driver that allows multiple tags ' + ;
    '(orders) within a single index file, adds a new tag to an ' + ;
    'existing index file.  After the new index is created, the file is ' + ;
    'opened and is added to the list of indexes.' + ;
    '|' + ;
    'File name:  Enter the name of the index file, including the drive ' + ;
    'and directory if needed.  The default extension depends on the ' + ;
    'driver that the database was opened with.' + ;
    '|' + ;
    'Tag name:  For a driver that allows you to have multiple orders ' + ;
    'in a single file, enter the name of the order.  When the new ' + ;
    'index file is opened, the tag names are added to the list of ' + ;
    'indexes.  For a driver that allows only a single order in an ' + ;
    'index file, this option does not appear.' + ;
    '|' + ;
    'On:  Enter the expression that controls the order of records in ' + ;
    'this index.' + ;
    '|' + ;
    'For:  This field is optional.  If used, it is an expression which ' + ;
    'describes which records are to be included in the index.  Records ' + ;
    'which do not satisfy the condition are not accessible through ' + ;
    'this index.  This item is available only for drivers that support ' + ;
    'this feature.' + ;
    '|' + ;
    'Descending:  If set to Yes, the records are indexed in reverse ' + ;
    'order, starting with the highest key value.  You can make part ' + ;
    'of the index descending by using the DESCEND function in the ' + ;
    'key expression.  This item is available only for drivers that ' + ;
    'support this feature.' + ;
    '|' + ;
    'Unique:  If set to Yes, only the first record satisfying a given ' + ;
    'key value is put into the index; subsequent records with the same ' + ;
    'key value are ignored.'

CASE nContext == H_INDRECRE

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Recreates all indexes for the current database.  This option uses ' + ;
    'the index expressions that DBE has stored rather than the index ' + ;
    'expressions that are stored in the index files.'

CASE nContext == H_INDDESTROY

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Destroys the tag that is highlighted in the Indexes list.  The ' + ;
    'tag is permenently removed from the index file.  The Indexes list ' + ;
    'is updated to reflect this change.  This option applies only to ' + ;
    'databases that have been opened with a driver that permits ' + ;
    'multiple tags (orders) in an index file.'

CASE nContext == H_SETUPEXP

  nPad  := 15
  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Expands the current setup pane so that it displays more rows and ' + ;
    'more columns.  You may add, delete, edit, and select from the ' + ;
    'expanded setup the same as you can from the regular setup panes ' + ;
    'at the top of the screen.  You may also list the information in ' + ;
    'the expanded setup to a file.' + ;
    '|' + ;
    'You may perform an incremental search on the following fields in ' + ;
    'an expanded setup.  See the help topic "Incremental search".' + ;
    '|' + ;
    PADR('Pane'    , 15) + 'Incremental search on~' + ;
    '|' + ;
    PADR('Database', 15) + 'Alias~' + ;
    PADR('Index'   , 15) + 'Index name~' + ;
    PADR('Relation', 15) + 'To alias~' + ;
    PADR('Filter'  , 15) + 'Filter name~' + ;
    PADR('Field'   , 15) + 'Field name~'

CASE nContext == H_SETUPLIST

  cText := ;
    'Location: Open menu / Expand current setup~' + ;
    '|' + ;
    'Writes the contents of the expanded setup to a file.  Each line ' + ;
    'contains the data for each column of the expanded setup.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_CRESTRU

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Creates a new database by having you create a new structure on ' + ;
    'screen, and then opens it.' + ;
    '|' + ;
    'To:  Enter the name of the database file to be created, including ' + ;
    'the drive and directory if needed.  The default extension is ' + ;
    '.DBF.' + ;
    '|' + ;
    HelpPar(HP_ALIAS) + '  This option applies to the created database.' + ;
    '|' + ;
    HelpPar(HP_DRIVER)

CASE nContext == H_CREFROM

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Creates a new database from an existing structure extended ' + ;
    'database, and then opens it.' + ;
    '|' + ;
    'From:  Enter the name of the existing structure extended ' + ;
    'database, ' + HelpPar(HP_FILENAME) + 'is .DBF.' + ;
    '|' + ;
    'To:  Enter the name of the database file to be created, ' + ;
    HelpPar(HP_FILENAME) + 'is .DBF.' + ;
    '|' + ;
    HelpPar(HP_ALIAS) + '  This option applies to the created database.' + ;
    '|' + ;
    HelpPar(HP_DRIVER)

CASE nContext == H_DBEREAD

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Reads settings from a DBE file into the current DBE session.  See ' + ;
    'the help topic "DBE files basic information".' + ;
    '|' + ;
    'Name:  Enter the name of the DBE file, ' + HelpPar(HP_FILENAME) + ;
    'is .DBE.'

CASE nContext == H_DBEWRITE

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Writes settings for the current DBE session to a DBE file.  See ' + ;
    'the help topic "DBE files basic information".' + ;
    '|' + ;
    'Name:  Enter the name of the DBE file, ' + HelpPar(HP_FILENAME) + ;
    'is .DBE.' + ;
    '|' + ;
    'Include setup window:  If Yes, then settings from the Databases, ' + ;
    'Indexes, Relations, Filters, and Fields panes are written to the ' + ;
    'DBE file.' + ;
    '|' + ;
    'Include data positions:  If Yes, and if the setup window is ' + ;
    'included, then the following settings are written to the DBE file ' + ;
    'for each database:  horizontal/vertical display mode, ' + ;
    'wandering/center highlight mode, field sort/name sort mode, record ' + ;
    'number, and the positions within the Index, Relation, Filter, and ' + ;
    'Field setup panes.' + ;
    '|' + ;
    'Include set values:  If Yes, then the following settings are ' + ;
    'written to the DBE file:  the position within the Database setup ' + ;
    'pane, the small/large number of lines mode, and the Set menu ' + ;
    'settings from the keystroke recording mode to the odometer setting ' + ;
    'mode.' + ;
    '|' + ;
    'Include default values:  If Yes, settings are written to the DBE ' + ;
    'file regardless of whether they are the default.  If No, settings ' + ;
    'are written to the DBE file only if they differ from the default.' + ;
    '|' + ;
    'Include history:  If Yes, then all the history lines are written ' + ;
    'to the DBE file.' + ;
    '|' + ;
    'Optional keystrokes to play back:  Enter a keystroke record ' + ;
    'string.  When a future DBE session reads the DBE file, the ' + ;
    'keystrokes recorded in this string will be played back after the ' + ;
    'rest of the DBE file has been read in.  This field is optional.' + ;
    '|' + ;
    'Edit before saving:  If Yes, the file contents are displayed for ' + ;
    'you to edit before the DBE file is written.  After you have ' + ;
    'finished editing, you are given the option of not writing the DBE ' + ;
    'file.'

CASE nContext == H_QUIT

  cText := ;
    'Location: Open menu~' + ;
    '|' + ;
    'Exits DBE.  Session settings are not automatically saved.  If you ' + ;
    'want to save information about the current DBE session before you ' + ;
    'exit DBE, use the option "Write DBE file".  For an introduction ' + ;
    'to DBE files, see the help topic "DBE files basic information".'

END

RETURN cText  //  HelpOpen

//***************************************************************************

STATIC FUNCTION HelpFile(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_FILEMENU

  cText := ;
    'Performs operations on the database that involve more than one ' + ;
    'record, for example, replace, pack, list.' + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'Replace:  Replaces the contents of a field with the value of an ' + ;
    'expression. (*)' + ;
    '|' + ;
    'Pack:  Permanently removes records marked for deletion from the ' + ;
    'current database. (*)' + ;
    '|' + ;
    'Zap:  Permanently removes all records from the current database. ' + ;
    '(*)' + ;
    '|' + ;
    'Delete:  Marks a group of records for deletion. (*)' + ;
    '|' + ;
    'Recall:  Removes deletion mark from a group of records. (*)' + ;
    '|' + ;
    'Count:  Counts the number of records in a group of records. (*)' + ;
    '|' + ;
    'Sum:  Computes a sum in a group of records. (*)' + ;
    '|' + ;
    'Accumulate:  Makes a calculation across a group of records. (*)' + ;
    '|' + ;
    'Evaluate on database:  Evaluates an expression on a group of ' + ;
    'records. (*)' + ;
    '|' + ;
    'Copy to:  Copies a group of records to another file. (*)' + ;
    '|' + ;
    'Append from:  Adds another file to the current database. (*)' + ;
    '|' + ;
    'Modify structure:  Changes the field structure of the current ' + ;
    'database. (*)' + ;
    '|' + ;
    'Copy structure:  Makes another database file with the same ' + ;
    'structure as the current database. (*)' + ;
    '|' + ;
    'Copy structure extended:  Copies the structure of the current ' + ;
    'database to a structure extended database. (*)' + ;
    '|' + ;
    'List:  Writes the field values of a group of records to an ASCII ' + ;
    'file. (*)' + ;
    '|' + ;
    'List structure:  Writes the structure of the current database to ' + ;
    'an ASCII file. (*)' + ;
    '|' + ;
    'Report form:  Generates a report from a report file (an .FRM ' + ;
    'file). (*)' + ;
    '|' + ;
    'Label form:  Generates labels from a label file (an .LBL file). ' + ;
    '(*)' + ;
    '|' + ;
    'Join:  Creates a database from the current database by merging it ' + ;
    'with another database. (*)'

CASE nContext == H_REPLACE

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Replaces the contents of a field with the value of an ' + ;
    'expression for a group of records.' + ;
    '|' + ;
    'Field:  Enter the name of the field in the current database.' + ;
    '|' + ;
    'Expression:  Enter an expression to be evaluated on each record in ' + ;
    'the group.  The value of this expression will be put into the ' + ;
    'field.' + ;
    '|' + ;
    HelpPar(HP_CONDITION)

CASE nContext == H_PACK

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Permanently removes records marked for deletion from the ' + ;
    'current database.  USE WITH CAUTION, as the deleted records ' + ;
    'cannot be recovered after you have packed the database.'

CASE nContext == H_ZAP

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Permanently removes ALL records from the current database.  USE ' + ;
    'WITH EXTREME CAUTION, as records cannot be recovered after you ' + ;
    'have zapped the database.'

CASE nContext == H_DELETE

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Marks a group of records for deletion.' + ;
    '|' + ;
    HelpPar(HP_CONDITION)

CASE nContext == H_RECALL

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Removes the deletion mark from a group of records, thereby ' + ;
    'undeleting them.' + ;
    '|' + ;
    HelpPar(HP_CONDITION)

CASE nContext == H_COUNT

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Counts the number of records in a group of records.  After they ' + ;
    'are counted, DBE displays the result and puts it into the history.' + ;
    '|' + ;
    HelpPar(HP_CONDITION)

CASE nContext == H_SUM

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Computes a sum in a group of records.  After the sum is computed, ' + ;
    'DBE displays the result and puts it into the history.' + ;
    '|' + ;
    'On:  Enter a numeric expression to be evaluated on each record in ' + ;
    'the group.' + ;
    '|' + ;
    HelpPar(HP_CONDITION)

CASE nContext == H_ACCUM

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Makes a calculation across a group of records.  The result of the ' + ;
    'calculation is stored in a variable called X.  You must specify an ' + ;
    'initial value for X, and then a rule for modifying X from record ' + ;
    'to record.  After the sum is computed, DBE displays the result and ' + ;
    'puts it into the history.' + ;
    '|' + ;
    'Initialization expression for X:  Enter an expression for the ' + ;
    'of X before the accumulation begins.' + ;
    '|' + ;
    'Accumulation expression for X:  Enter an assignment expression ' + ;
    'which modifies the value of X as it is evaluated at each record.' + ;
    '|' + ;
    HelpPar(HP_CONDITION) + ;
    '|' + ;
    'Example:  If the current database contains a logical field ' + ;
    'LOGFIELD, and you wish to determine the result of ANDing this ' + ;
    'field across all records, enter ".Y." for the initial value of X, ' + ;
    'and enter "X := X .AND. LOGFIELD" for the accumulation expression.' + ;
    '|' + ;
    'Example:  If the current database contains a character field ' + ;
    'CHARFIELD, and you wish to concatenate the contents this field ' + ;
    'in all records, enter "[]" for the initial value of X, and enter ' + ;
    '"X := X + CHARFIELD" for the accumulation expression.' + ;
    '|' + ;
    'Example:  If the current database contains a numeric field ' + ;
    'NUMFIELD, and you wish to compute the maximum value of this field ' + ;
    'in all records, enter "NUMFIELD" for the initial value of X, and ' + ;
    'enter "X := MAX(X, NUMFIELD)" for the accumulation expression.'

CASE nContext == H_DBEVAL

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Evaluates an expression on a group of records.' + ;
    '|' + ;
    'Expression:  Enter an expression to be evaluated on each record in ' + ;
    'the group.' + ;
    '|' + ;
    HelpPar(HP_CONDITION)

CASE nContext == H_COPYTO

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Copies a group of records to another file.' + ;
    '|' + ;
    'Type:  Choose from the list of types.  If you choose DBF, the ' + ;
    'selected records are written to another database with the same ' + ;
    'structure as the current database.  If you choose SDF, the ' + ;
    'selected records are written to an ASCII file in which each data ' + ;
    'position occupies one column.  If you choose BLANK, the selected ' + ;
    'records are written to an ASCII file in which fields are separated ' + ;
    'with blanks.  If you choose DELIMITED, you must specifiy a ' + ;
    'character in the "Delimiter" item.' + ;
    '|' + ;
    'To:  Enter the name of the file to be written to, ' + ;
    HelpPar(HP_FILENAME) + 'for a DBF type is .DBF, and for all other ' + ;
    'types is .TXT.' + ;
    '|' + ;
    HelpPar(HP_DRIVER) + '  This item is used on the To file only for ' + ;
    'type DBF.  For other types, it is ignored.' + ;
    '|' + ;
    HelpPar(HP_CONDITION) + ;
    '|' + ;
    'Delimiter:  This item is used only if you choose a type of ' + ;
    'DELIMITED.  The selected records are then written to an ASCII file ' + ;
    'in which the fields are separated by commas, and the character ' + ;
    'fields are delimited with the indicated character.'

CASE nContext == H_APPEND

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Adds another file to the current database.  The file that you are ' + ;
    'adding to the current database is called the append file.' + ;
    '|' + ;
    'Type:  Choose from the list of types.  If you choose DBF, only ' + ;
    'those fields in the append file that match the current database ' + ;
    'will be read in.  If you choose SDF, the append file is assumed to ' + ;
    'be an an ASCII file in which each column corresponds a data ' + ;
    'position in the current database.  If you choose BLANK, the append ' + ;
    'file is assumed to be an ASCII file in which fields are separated ' + ;
    'with blanks.  If you choose DELIMITED, you must specifiy a ' + ;
    'character in the "Delimiter" item.' + ;
    '|' + ;
    'From:  Enter the name of the file to be read into the current ' + ;
    'database, ' + HelpPar(HP_FILENAME) + 'for a DBF type is .DBF, and ' + ;
    'for all other types is .TXT.' + ;
    '|' + ;
    HelpPar(HP_DRIVER) + '  This item is used on the From file only for ' + ;
    'type DBF.  For other types, it is ignored.' + ;
    '|' + ;
    HelpPar(HP_CONDITION) + ;
    '|' + ;
    'Delimiter:  This item is used only if you choose a type of ' + ;
    'DELIMITED.  The append file is assumed to be an ASCII file in ' + ;
    'which the fields are separated by commas, and the character fields ' + ;
    'are delimited with the indicated character.'

CASE nContext == H_MODISTRU

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Allows you to change the field structure of the current database.' + ;
    '|' + ;
    'You first come to the structure editor.  For help on this feature, ' + ;
    'see the help topic "Structure editor".  When you have finished ' + ;
    'making changes to the structure in the structure editor, you have ' + ;
    'the option of saving your changes or throwing them away.  If you ' + ;
    'save your changes, the old file will be replaced with a file with ' + ;
    'the new structure.  If you throw your changes away, you will ' + ;
    'return to the current file with no changes made.' + ;
    '|' + ;
    'You are then asked whether you wish to back up data from the old ' + ;
    'structure.  If you answer Yes, you will be prompted for a file ' + ;
    'name, and the current database will be copied into this file.  If ' + ;
    'you answer No, the current file will be overwritten by the file ' + ;
    'with the new structure.'

CASE nContext == H_COPYSTRU

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Makes another database file with the same structure as the ' + ;
    'current database.  This second file has no records.' + ;
    '|' + ;
    'To:  Enter the name of the file to copy the strucrure into, ' + ;
    HelpPar(HP_FILENAME) + 'is .DBF.'

CASE nContext == H_COPYEXT

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Copies the structure of the current database to a structure ' + ;
    'extended database.  A structure extended database has one record ' + ;
    'for every field in the current database.' + ;
    '|' + ;
    'To:  Enter the name of the file to copy the strucrure into, ' + ;
    HelpPar(HP_FILENAME) + 'is .DBF.'

CASE nContext == H_LISTFILE

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Writes the field values of a group of records to an ASCII file.  ' + ;
    'Each column has a fixed width which is the width of the field up ' + ;
    'to a maximum of 78, and there is one space between columns.  There ' + ;
    'are no column headers.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_CONDITION) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    'Include deleted status:  If you answer Yes, then the deleted ' + ;
    'status is written to a column at the beginning of each line.  This ' + ;
    'column is blank if the record is not deleted, and it is "*" if the ' + ;
    'record is deleted.' + ;
    '|' + ;
    'Include record number:  If you answer Yes, then the record number ' + ;
    'is written to a column at the beginning of each line.' + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_LISTSTRU

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Writes the structure of the current database to an ASCII file.  ' + ;
    'Each line contains the following data for each field:  field ' + ;
    'number, field type, field length, field decimals, field name, ' + ;
    'start byte for the field, and end byte for the field.  The start ' + ;
    'and end bytes the byte positions of the field within the record.  ' + ;
    'The first data byte in the record is byte 1, and the delete byte ' + ;
    'can be regarded as byte 0.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_FRMUSE

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Generates a report from a report form (an .FRM file).  For ' + ;
    'information on creating or modifying a report form, see the help ' + ;
    'topic "Report modify".' + ;
    '|' + ;
    'Form:  Enter the name of the report form, ' + HelpPar(HP_FILENAME) + ;
    'is .FRM.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .PRN.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_CONDITION) + ;
    '|' + ;
    'Heading:  This item is optional.  The text you enter here will be ' + ;
    'printed at the top of each page of the report.' + ;
    '|' + ;
    'Plain:  If you answer Yes, the report is output without date, page ' + ;
    'numbers, or page breaks, and the report header and column headers ' + ;
    'are output only once at the top of the output.' + ;
    '|' + ;
    'Summary only:  If you answer Yes, the report is output with only ' + ;
    'group, subgroup, and grand total lines.' + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_LBLUSE

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Generates labels from a label form (an .LBL file).  For ' + ;
    'information on creating or modifying a label form, see the help ' + ;
    'topic "Label modify".' + ;
    '|' + ;
    'Form:  Enter the name of the label form, ' + HelpPar(HP_FILENAME) + ;
    'is .LBL.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .PRN.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_CONDITION) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_JOIN

  cText := ;
    'Location: File menu~' + ;
    '|' + ;
    'Creates a database from the current database by merging it with ' + ;
    'another database.  The general effect is to multiply records.  ' + ;
    'For each record in the current database, all the records in the ' + ;
    'second database are examined.  When the "For" condition is true, ' + ;
    'a record is written to the output file.  The fields that are ' + ;
    'written to the output are those that are in a combined field ' + ;
    'list.  This combined field list is a concatenation of the field ' + ;
    'list of the current database and the field list of the second ' + ;
    'database.' + ;
    '|' + ;
    'With:  Enter the alias of a database that is currently open but ' + ;
    'not selected.' + ;
    '|' + ;
    'To:  Enter the name of the output database, ' + HelpPar(HP_FILENAME) + ;
    'is .DBF.' + ;
    '|' + ;
    HelpPar(HP_ALIAS) + '  This option applies to the created database.' + ;
    '|' + ;
    'For:  Enter a logical expression which describes the records to be ' + ;
    'created in the output file.  Unaliased fields refer to the current ' + ;
    'database.  To refer to the second database, use an aliased ' + ;
    'expression.  If you leave this field blank, all records in both ' + ;
    'files are output.'

END

RETURN cText  //  HelpFile

//***************************************************************************

STATIC FUNCTION HelpRecord(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_RECMENU

  cText := ;
    'Performs operations on a single record, for example, deleting a ' + ;
    'record, storing a record or a field into memory.' + ;
    '|' + ;
    HelpPar(HP_SHORTDATA) + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'Delete record:  Marks the current record for deletion, or removes ' + ;
    'the deletion mark. (D)' + ;
    '|' + ;
    'New record:  Creates a temporary blank record and allows you to ' + ;
    'edit and save it or throw it away. (D) (*)' + ;
    '|' + ;
    'Append blank:  Adds a new, blank record to the current database.' + ;
    '|' + ;
    'Field look:  Looks at the current field using the memo viewer. ' + ;
    '(D)' + ;
    '|' + ;
    'Memowrite:  Writes the current field into an ASCII file. (*)' + ;
    '|' + ;
    'Memoread:  Reads an ASCII file into the current field. (*)' + ;
    '|' + ;
    'Scatter:  Copies all the field values for the current record into ' + ;
    'the data transfer buffer.' + ;
    '|' + ;
    'Gather:  Replaces all the fields for the current record with ' + ;
    'values from the data transfer buffer.' + ;
    '|' + ;
    'Move record:  Changes the record number of the current record. ' + ;
    '(*)' + ;
    '|' + ;
    'Store field:  Copies the value of the current field into the data ' + ;
    'transfer buffer.' + ;
    '|' + ;
    'Retrieve field replace:  Replaces the current field with the ' + ;
    'contents of the data transfer buffer.' + ;
    '|' + ;
    'Retrieve field append:  Appends the contents of the data transfer ' + ;
    'buffer to the current field.' + ;
    '|' + ;
    'Replace field:  Replaces the current field with the result of an ' + ;
    'expression. (*)' + ;
    '|' + ;
    'Snapshot:  Writes the field values of the current record into an ' + ;
    'ASCII file. (*)' + ;
    '|' + ;
    'Field modify:  Edits the contents of the current field. (D) (*)' + ;
    '|' + ;
    'Field modify as memo:  Edits the contents of the current field ' + ;
    'using the memo editor.' + ;
    '|' + ;
    'Enter data:  Enters a mode for editing multiple field values ' + ;
    'and/or multiple records. (D) (*)'

CASE nContext == H_NEWRECORD

  cText := ;
    'Location: Record menu~' + ;
    'Shortcut from data browse: Insert key~' + ;
    '|' + ;
    'Adds a new record to the database, conditional on your entering ' + ;
    'data into the current field.  Until the new record is added, the ' + ;
    'record number is one greater than the number of records.  To add ' + ;
    'the new record, enter data into the field and press Enter.  To ' + ;
    'abort adding the new reocrd, press Esc.'

CASE nContext == H_MEMOWRITE

  cText := ;
    'Location: Record menu~' + ;
    '|' + ;
    'Writes the contents of the current field to a file.  The current ' + ;
    'field must be a memo field or a character field.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  If the file already exists, it is overwritten, and the ' + ;
    'previous contents are lost.  ' + HelpPar(HP_PRINTER)

CASE nContext == H_MEMOREAD

  cText := ;
    'Location: Record menu~' + ;
    '|' + ;
    'Reads the contents of a file to the current field.  The current ' + ;
    'field must be a memo field or a character field.' + ;
    '|' + ;
    'From:  Enter the name of the input file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.' + ;
    '|' + ;
    'Append:  If you answer Yes, the contents of the file are appended ' + ;
    'to the end of the contents of the current field.  If you answer ' + ;
    'No, the field is replaced with the contents of the file, and the ' + ;
    'previous contents of the field are lost.'

CASE nContext == H_MOVEREC

  cText := ;
    'Location: Record menu~' + ;
    '|' + ;
    'Moves the contents of the current record to another record ' + ;
    'number.  The records in between the current record and the new ' + ;
    'record are moved up or down one record number.' + ;
    '|' + ;
    'To record number:  Enter the record number to which the current ' + ;
    'record should be moved.'

CASE nContext == H_REPLACEFLD

  cText := ;
    'Location: Record menu~' + ;
    '|' + ;
    'Replaces the current field with the value of an expression.' + ;
    '|' + ;
    'Expression:  Enter the expression to evaluate and put into the ' + ;
    'current field.'

CASE nContext == H_SNAPSHOT

  cText := ;
    'Location: Record menu~' + ;
    '|' + ;
    'Writes the field values of the current records to an ASCII file.  ' + ;
    'Each field is written onto a separate line, and the line has a ' + ;
    'maximum length of 78.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    'Expand memo and long character fields:  If you answer No, then ' + ;
    'memo and long character fields are written on a single line.  If ' + ;
    'you answer Yes, then these fields are written out in full on as ' + ;
    'many lines as necessary.' + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_EDITFIELD

  cText := ;
    'Location: Record menu~' + ;
    'Shortcut from data browse: Enter key, or doubleclick on field~' + ;
    '|' + ;
    'Allows you to edit the current field.  To save the data, press ' + ;
    'Enter.  To abort the change, press Esc.' + ;
    '|' + ;
    'To make changes in many fields, you may wish to use data entry ' + ;
    'mode.  See the help topic "Enter data".'

CASE nContext == H_ENTERDATA

  nPad  := 11
  cText := ;
    'Location: Record menu~' + ;
    'Shortcut from data browse: Ctrl-Enter key~' + ;
    '|' + ;
    'Puts DBE into a data entry mode, which allows you to edit many ' + ;
    'fields records quickly.  As you exit one field, DBE immediately ' + ;
    'moves to another field and allows you to edit that field.  To exit ' + ;
    'data entry mode, press PageDown.' + ;
    '|' + ;
    'In data entry mode, all keys are used to enter data.  Menus and ' + ;
    'shortcut keys are not available.  Hot keys, however, are ' + ;
    'available.' + ;
    '|' + ;
    'The following keys can be used to save changes to the current ' + ;
    'field and move to another field.' + ;
    '|' + ;
    'In horizontal mode:' + ;
    '|' + ;
    PADR('Enter'    , nPad) + ;
    'Moves to the following field in the same record~' + ;
    PADR('Tab'      , nPad) + ;
    'Moves to the following field in the same record~' + ;
    PADR('Shift-Tab', nPad) + ;
    'Moves to the previous field in the same record~' + ;
    PADR('Up'       , nPad) + ;
    'Moves to the same field in the previous record~' + ;
    PADR('Down'     , nPad) + ;
    'Moves to the same field in the following record~' + ;
    PADR('PageUp'   , nPad) + ;
    'Exits data entry mode~' + ;
    PADR('PageDown' , nPad) + ;
    'Exits data entry mode~' + ;
    PADR('Esc'      , nPad) + ;
    'Throws away changes to the current field,~' + ;
    SPACE(nPad)             + ;
    'remains in data entry mode in the same field~' + ;
    SPACE(nPad)             + ;
    'in the same record~' + ;
    '|' + ;
    'In vertical mode:' + ;
    '|' + ;
    PADR('Enter'    , nPad) + ;
    'Moves to the following field in the same record~' + ;
    PADR('Tab'      , nPad) + ;
    'Moves to the following field in the same record~' + ;
    PADR('Shift-Tab', nPad) + ;
    'Moves to the previous field in the same record~' + ;
    PADR('Up'       , nPad) + ;
    'Moves to the previous field in the same record~' + ;
    PADR('Down'     , nPad) + ;
    'Moves to the following field in the same record~' + ;
    PADR('PageUp'   , nPad) + ;
    'Exits data entry mode~' + ;
    PADR('PageDown' , nPad) + ;
    'Exits data entry mode~' + ;
    PADR('Esc'      , nPad) + ;
    'Throws away changes to the current field,~' + ;
    SPACE(nPad)             + ;
    'remains in data entry mode in the same field~' + ;
    SPACE(nPad)             + ;
    'in the same record'

END

RETURN cText  //  HelpRecord

//***************************************************************************

STATIC FUNCTION HelpGo(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_GOMENU

  cText := ;
    'Moves to a new record, for example, seek, locate, skip.' + ;
    '|' + ;
    HelpPar(HP_HOTMEN) + ;
    '|' + ;
    HelpPar(HP_SHORTDATA) + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'Seek:  Looks for a value matching the expression of the current ' + ;
    'index, by entering the value all at once. (*)' + ;
    '|' + ;
    'Incremental seek:  Looks for a value matching the expression of ' + ;
    'the current index, by entering the value one character at a time. ' + ;
    '(*)' + ;
    '|' + ;
    'Locate:  Looks for the first value satisfying a condition, ' + ;
    'without using indexes. (*)' + ;
    '|' + ;
    'Continue locate:  Looks for the next value that satisfies the ' + ;
    'condition specified in the Locate option. (*)' + ;
    '|' + ;
    'Goto:  Moves to a given record number. (*)' + ;
    '|' + ;
    'Records skip:  Moves forwards or backwards a given number of ' + ;
    'records. (*)' + ;
    '|' + ;
    'Previous:  Moves to the previous record. (D)' + ;
    '|' + ;
    'Next:  Moves to the next record. (N)' + ;
    '|' + ;
    'Top:  Moves to the first record.' + ;
    '|' + ;
    'Bottom:  Moves to the last record.'

CASE nContext == H_SEEK

  cText := ;
    'Location: Go menu~' + ;
    '|' + ;
    'Allows you to look for a value matching the expression of the ' + ;
    'current index.  This value is called a key value.' + ;
    '|' + ;
    'If the current database has no index, or if it has one or more ' + ;
    'indexes but you are browsing the database in natural order, you ' + ;
    'cannot use this option.' + ;
    '|' + ;
    'An alternative to this option is the incremental seek.  See the ' + ;
    'help topic "Incremental Seek".' + ;
    '|' + ;
    'Value:  Enter the key value to look for.  Blanks are not trimmed ' + ;
    'trimmed from the left side of this value.' + ;
    '|' + ;
    'Number of trailing blanks:  Normally, blanks are trimmed from the ' + ;
    'right side of the seek value.  If you wish to put trailing blanks ' + ;
    'blanks at the end of the seek value, enter the number of blanks ' + ;
    'here.'

CASE nContext == H_SEEKINC

  nPad  := 17
  cText := ;
    'Location: Go menu~' + ;
    '|' + ;
    'Allows you to look for a value matching the expression of the ' + ;
    'current index.  This value is called the key value.' + ;
    '|' + ;
    'Using this option, DBE enters incremental seek mode.  In this ' + ;
    'mode, you enter the key value one character at a time.  ' + ;
    'Immediately after you press each character, DBE seeks for the key ' + ;
    'value you have entered so far and refreshes the data browse.  The ' + ;
    'key value you have entered is displayed at the top of the screen.  ' + ;
    'To exit incremental seek mode, press Esc.' + ;
    '|' + ;
    'In incremental seek mode, all printing characters are used to ' + ;
    'enter the key value.  Menus and shortcuts are not available.  When ' + ;
    'you press Esc, a hot key, or any other non-printing character, DBE ' + ;
    'exits incremental seek mode.' + ;
    '|' + ;
    'The following editing keys can be used in incremental seek mode:' + ;
    '|' + ;
    PADR('Esc'           , nPad) + ;
    'Exit incremental seek mode~' + ;
    PADR('BackSpace'     , nPad) + ;
    'Erase last character, remain in incremental seek~' + ;
    PADR('Ctrl-BackSpace', nPad) + ;
    'Erase all characters, remain in incremental seek~' + ;
    '|' + ;
    'An alternative to this option is the full seek.  See the help ' + ;
    'topic "Seek".'

CASE nContext == H_LOCATE

  cText := ;
    'Location: Go menu~' + ;
    '|' + ;
    'Allows you to look for a record that satisfies a given criterion.  ' + ;
    'DBE moves to the first record that satisfies the criterion.  This ' + ;
    'option does not require an index.  However, if there is a current ' + ;
    'index, the database is searched in the order of the current index.' + ;
    '|' + ;
    'This option finds only the first record which satifies the ' + ;
    'conditions.  To find subsequent records, use the "Continue locate"' + ;
    'option.' + ;
    '|' + ;
    'For:  Enter a logical expression describing the record you wish to ' + ;
    'locate.' + ;
    '|' + ;
    'While:  This clause is optional.  You must either leave it blank ' + ;
    'or enter a logical expression.  If you enter a logical expression, ' + ;
    'searching continues as long as the expression is true.  As soon ' + ;
    'the expression is false, searching stops.  If you leave this ' + ;
    'field blank, searching is performed for the entire scope.' + ;
    '|' + ;
    'Scope:  Choose from the list.  If you choose ALL, then the whole ' + ;
    'file is searched.  If you choose REST, the searching starts at the ' + ;
    'current record.  If you choose NEXT, you must specify a number of ' + ;
    'records in the "Next" item.' + ;
    '|' + ;
    '"While", and "Scope" clauses may be combined.  When they are ' + ;
    'combined, searching is performed for the first record which ' + ;
    'satisfies all conditions.' + ;
    '|' + ;
    'Next:  This field is used only if you choose a scope of NEXT.  ' + ;
    'Searching starts at the current record and continues only for the ' + ;
    'indicated number of records.'

CASE nContext == H_LOCCONT

  cText := ;
    'Location: Go menu~' + ;
    '|' + ;
    'If you have already used the "Locate" option, you can use this ' + ;
    'to locate the succeeding records.  The "Continue locate" option ' + ;
    'searches for the next record after the current record that ' + ;
    'satisfies the conditions that you specified in your last use of ' + ;
    'the "Locate" option.'

CASE nContext == H_GOTO

  cText := ;
    'Location: Go menu~' + ;
    '|' + ;
    'Moves to a given record number.  This move is carried out ' + ;
    'regardless of the current index, if any.'

CASE nContext == H_RECSKIP

  cText := ;
    'Location: Go menu~' + ;
    '|' + ;
    'Skips a given number of record forward or backward through the ' + ;
    'database.  If there is a current index, skipping is carried out ' + ;
    'using the current index.' + ;
    '|' + ;
    'Number of records:  Enter the number of records to skip.  If the ' + ;
    'number is positive, DBE will skip forward.  If the number is ' + ;
    'negative, DBE will skip backward.'

END

RETURN cText  //  HelpGo

//***************************************************************************

STATIC FUNCTION HelpSet(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_SETMENU

  cText := ;
    'Performs setup functions, for example, setting colors, adding a ' + ;
    'relation, adding a filter, setting keystroke record mode.' + ;
    '|' + ;
    HelpPar(HP_SHORTSETUP) + ;
    '|' + ;
    HelpPar(HP_HOTKEY) + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'Local directory:  Defines a local directory.  This is the default ' + ;
    'directory for all file name references.' + ;
    '|' + ;
    'Relation add:  Defines a new relation from the current database ' + ;
    'into another database that has been opened, and adds the new ' + ;
    'relation to the list of relations. (S) (*)' + ;
    '|' + ;
    'Relation edit:  Modifies the highlighted relation. (S) (*)' + ;
    '|' + ;
    'Relation delete:  Closes the highlighted relation and remove it ' + ;
    'from the list of relations. (S)' + ;
    '|' + ;
    'Filter add:  Defines a filter on the current database, and adds it ' + ;
    'to the list of filters. (S) (*)' + ;
    '|' + ;
    'Filter edit:  Modifies the highlighted filter. (S) (*)' + ;
    '|' + ;
    'Filter delete:  Closes the highighted filter and remove it from ' + ;
    'the list of filters. (S)' + ;
    '|' + ;
    'Keystroke record:  Records a set of keystrokes for playback at a ' + ;
    'later time. (H) (*)' + ;
    '|' + ;
    'Hot keys:  Allows you to set up a hot key so that it plays back a ' + ;
    'set of keystrokes. (H) (*)' + ;
    '|' + ;
    'Colors:  Sets the colors used in DBE. (*)' + ;
    '|' + ;
    'Cursors:  Sets the shape of the cursors used in DBE. (*)' + ;
    '|' + ;
    'Printer codes:  Sets the printer control codes that are optionally ' + ;
    'sent to a printer or print file. (*)' + ;
    '|' + ;
    'Date format:  Sets the format for date fields and date values.' + ;
    '|' + ;
    'Epoch:  Sets the first year of the century spanned by two-digit ' + ;
    'years in date values.' + ;
    '|' + ;
    'Decimals:  Sets the number of decimal places to display for ' + ;
    'numeric values.' + ;
    '|' + ;
    'Memo width:  Sets the width of lines to be used in the memo ' + ;
    'editor.' + ;
    '|' + ;
    'Memo tab:  Sets the number of spaces that a tab character expands ' + ;
    'to in the memo editor.' + ;
    '|' + ;
    'Margin:  Sets the number spaces to output before every line in ' + ;
    'ASCII files.' + ;
    '|' + ;
    'Deleted:  Sets whether to display deleted fields.' + ;
    '|' + ;
    'Softseek:  Sets whether to look for the next record if a seek ' + ;
    'does not find an exact match.' + ;
    '|' + ;
    'Cancel:  Sets whether to allow Ctrl-Break in DBE.' + ;
    '|' + ;
    'Odometer:  Sets the interval in the on-screen update for ' + ;
    'multi-record operations.'

CASE nContext == H_LOCAL

  cText := ;
    'Location: Set menu~' + ;
    '|' + ;
    'Defines a local directory.  This is the default directory for all ' + ;
    'file name references.' + ;
    '|' + ;
    'A file name that does not include a drive or directory ' + ;
    'specification is assumed to be in the local directory.  If you do ' + ;
    'not use this option to set the local directory, then the local ' + ;
    'directory is the directory in which you stared DBE.' + ;
    '|' + ;
    'Directory:  Enter the name of the local directory.  The directory ' + ;
    'name may be a complete path name, or it may be relative to the ' + ;
    'current directory or the current drive, or it may be the current ' + ;
    'directory on another drive.  On a network, in lieu of a drive ' + ;
    'letter, you may use a server/volume name instead.  A server/volume ' + ;
    'type of drive reference does not allow a relative path, and so the ' + ;
    'lack of a leading slash is interpreted as starting from the ' + ;
    'highest level directory.  To set the local directory to the ' + ;
    'directory from which DBE is being run, enter ".".'

CASE nContext == H_RELADD

  cText := ;
    'Location: Set menu~' + ;
    'Shortcut from Relation setup pane: Insert key~' + ;
    '|' + ;
    'Defines a new relation from the current database into another ' + ;
    'database that has been opened, and adds the new relation to the ' + ;
    'list of relations.' + ;
    '|' + ;
    'Into:  Enter the alias of a database that is currently open but is ' + ;
    'not selected.' + ;
    '|' + ;
    'Expression:  Enter the expression to be used to control the ' + ;
    'relation.  This expression is evaluated and the result used to ' + ;
    'seek or goto into the dependent database.'

CASE nContext == H_RELEDIT

  cText := ;
    'Location: Set menu~' + ;
    'Shortcut from Relation setup pane: Ctrl-Enter~' + ;
    '|' + ;
    'Modifies the highlighted relation.' + ;
    '|' + ;
    'Into:  Enter the alias of a database that is currently open but is ' + ;
    'not selected.' + ;
    '|' + ;
    'Expression:  Enter the expression to be used to control the ' + ;
    'relation.  This expression is evaluated and the result used to ' + ;
    'seek or goto into the dependent database.'

CASE nContext == H_FILTADD

  cText := ;
    'Location: Set menu~' + ;
    'Shortcut from Filter setup pane: Insert key~' + ;
    '|' + ;
    'Defines a filter on the current database, and adds it to the list ' + ;
    'of filters.' + ;
    '|' + ;
    'Expression:  Enter a logical expression to be used to determine ' + ;
    'which records to display.' + ;
    '|' + ;
    'Optional name:  This field is optional.  If you use it, this name ' + ;
    'is displayed in the Filters pane instead of the expression.'

CASE nContext == H_FILTEDIT

  cText := ;
    'Location: Set menu~' + ;
    'Shortcut from Filter setup pane: Ctrl-Enter~' + ;
    '|' + ;
    'Modifies the highlighted filter.' + ;
    '|' + ;
    'Expression:  Enter a logical expression to be used to determine ' + ;
    'which records to display.' + ;
    '|' + ;
    'Optional name:  This field is optional.  If you use it, this name ' + ;
    'is displayed in the Filters pane instead of the expression.'

CASE nContext == H_KRTOGGLE

  cText := ;
    'Location: Set menu~' + ;
    'Hot key: Alt-F4~' + ;
    '|' + ;
    'Records a set of keystrokes for playback at a later time.' + ;
    '|' + ;
    'This option allows you to enter and exit keystroke recording ' + ;
    'mode.  In this mode, keystrokes and mouse clicks are recorded ' + ;
    'internally.  When you exit the mode, the keystrokes are converted ' + ;
    'to a string, which you can then retrieve.' + ;
    '|' + ;
    'For more information, see the help topics "Keystroke recording" ' + ;
    'and "Keystroke code reference".'

CASE nContext == H_HOTKEYSET

  cText := ;
    'Location: Set menu~' + ;
    'Hot key: Alt-F10~' + ;
    '|' + ;
    'Allows you to set up a hot key so that it plays back a set of ' + ;
    'keystrokes.' + ;
    '|' + ;
    'This option allows you to perform two tasks: (1) assign the last ' + ;
    'recorded set of keystrokes to a hot key, or (2) edit the set of ' + ;
    'keystrokes that has already been assigned to a hot key.  The hot ' + ;
    'keys that you can set up are Shift-F1 to Shift-F12.' + ;
    '|' + ;
    'First record a set of keystrokes.  Then use this option to assign ' + ;
    'that set of keystrokes to a hot key.  Pick the hot key from the ' + ;
    'in the top part of the box.' + ;
    '|' + ;
    'When you have already used this option to assign a set of ' + ;
    'keystrokes to a hot key, you can view and edit the set of ' + ;
    'keystrokes by calling up this option, and pressing Enter.  You can ' + ;
    'pick the hot key to edit, and lastly edit the keystroke assignment.'

CASE nContext == H_COLORS

  cText := ;
    'Location: Set menu~' + ;
    '|' + ;
    'Sets the colors used in DBE.' + ;
    '|' + ;
    'First choose the DBE context from the menu.  Then set the color ' + ;
    'specification for the chosen context by choosing a color pair for ' + ;
    'the five types of screen output shown.  The colors that are shown ' + ;
    'when you first enter the color pair box are the colors that are ' + ;
    'currently in effect.' + ;
    '|' + ;
    'The first color of the pair controls the foreground, and the second ' + ;
    'color of the pair controls the background.  The foreground refers ' + ;
    'to the glyphs, i.e. the written part of the character cells.  The ' + ;
    'background refers to the rest of the character cells.' + ;
    '|' + ;
    'Normal:  The color of all areas of the context except those listed ' + ;
    'below.' + ;
    '|' + ;
    'Highlighted:  The color of the main highlight cell or bar in the ' + ;
    'center of a browse or pick list, indicating the current item.' + ;
    '|' + ;
    'Screen border:  The color of the border area around the entire ' + ;
    'screen.  The border area cannot have data written to it.  The ' + ;
    'foreground color controls the border color; the background color ' + ;
    'is ignored.' + ;
    '|' + ;
    'Secondary highlight:  The color of items related to the current ' + ;
    'item, such as the cross-shaped area of the data browse.  Not all ' + ;
    'contexts use this color pair.' + ;
    '|' + ;
    'Unselected items:  The color of items which cannot be selected.  ' + ;
    'Not all contexts use this color pair.'

CASE nContext == H_CURSORS

  cText := ;
    'Location: Set menu~' + ;
    '|' + ;
    'Sets the shape of the cursors used in DBE.' + ;
    '|' + ;
    'A cursor appears only when you are entering data into a text ' + ;
    'area.  When you press the Ins key, DBE switches between insert ' + ;
    'and overstrike mode.  The cursor changes shape to indicate which ' + ;
    'mode you are in.  This set option allows you to change the cursor ' + ;
    'shape that is used in these modes.'

CASE nContext == H_SLINES

  cText := ;
    'Location: Set menu~' + ;
    '|' + ;
    'Sets the number of screen lines used by the setup window.  The ' + ;
    'data browse uses the rest of the screen.' + ;
    '|' + ;
    'Length of setup window using small number of lines:  The number of ' + ;
    'lines used by the setup window when View / Small number of lines ' + ;
    'is in effect.  The minimum is 1; the default is 4; the maximum is ' + ;
    '16.' + ;
    '|' + ;
    'Length of setup window using large number of lines:  The number of ' + ;
    'lines used by the setup window when View / Large number of lines ' + ;
    'is in effect.  The minimum is 1; the default is 8; the maximum is ' + ;
    '41.'

CASE nContext == H_PRINTCODE

  cText := ;
    'Location: Set menu~' + ;
    '|' + ;
    'Sets the printer control codes that are optionally sent to a ' + ;
    'printer or print file.' + ;
    '|' + ;
    'When you request text output, you have the option of sending ' + ;
    'printer codes.  These codes can be sent to a printer or to a ' + ;
    'file.  This set option allows you to specify which codes are ' + ;
    'sent.' + ;
    '|' + ;
    'If you are setting printer codes at the time you are requesting ' + ;
    'text output, you can abort printing by pressing Esc.' + ;
    '|' + ;
    'Not all features described below are available on all printers.' + ;
    '|' + ;
    'Reset code:  If set to Yes, the printer reset code will be sent.' + ;
    '|' + ;
    'Pitch code:  The number of characters (columns) per inch, ' + ;
    'horizontally.' + ;
    '|' + ;
    'Lines per inch code:  The number of lines (rows) per inch, ' + ;
    'vertically.' + ;
    '|' + ;
    'Orientation code:  Which way the paper is turned.  Portrait mode ' + ;
    'means that the long edge of the paper is up and down.  ' + ;
    'Landscape mode means that the long edge of the paper is left and ' + ;
    'right.' + ;
    '|' + ;
    'Paper size code:  The size of paper to use.  Letter size is 8.5" x ' + ;
    '11", and legal size is 8.5" x 14".  If the size of paper you ' + ;
    'request does not match what is currently in the printer, the ' + ;
    'printer may request you to insert it.' + ;
    '|' + ;
    'Input tray code:  The area from which to feed blank paper into the ' + ;
    'printer.' + ;
    '|' + ;
    'Output bin code:  The area to which to feed printed paper out of ' +;
    'the printer.' + ;
    '|' + ;
    'Duplex code:  Whether to print on both sides of the paper.' + ;
    '|' + ;
    'Symbol set code:  How to interpret high ASCII codes, i.e. in the ' + ;
    'range 128 to 255.  The "PC" symbol set prints these codes as they ' + ;
    'are displayed on screen.  The "ASCII" symbol set is the printer ' + ;
    'standard.' + ;
    '|' + ;
    'The following charts show how many characters and lines fit on a ' + ;
    'physical sheet of paper in various output modes.' + ;
    '|' + ;
    HelpPar(HP_CHARFIT)

END

RETURN cText  //  HelpSet

//***************************************************************************

STATIC FUNCTION HelpView(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_VIEWMENU

  cText := ;
    'Performs functions that affect the display, for example, changing ' + ;
    'number of lines on the screen, adding and deleting a field to the ' + ;
    'data browse, switching between horizontal and vertical field ' + ;
    'display.' + ;
    '|' + ;
    HelpPar(HP_HOTMEN) + ;
    '|' + ;
    HelpPar(HP_SHORTSETUP) + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'Small number of lines:  Displays 25 lines on the screen.' + ;
    '|' + ;
    'Large number of lines:  Displays 50 lines on the screen.' + ;
    '|' + ;
    'Horizontal display:  In the data browse, displays fields so that ' + ;
    'multiple records are displayed, with each record on one ' + ;
    'horizontal line.' + ;
    '|' + ;
    'Vertical display:  In the data browse, displays fields so that ' + ;
    'one record is displayed at a time, with the fields displayed in ' + ;
    'a vertical column.' + ;
    '|' + ;
    'Wandering highlight:  In the data browse, the vertical position of ' + ;
    'the current record on the screen is allowed to wander.' + ;
    '|' + ;
    'Centered highlight:  In the data browse, displays the current ' + ;
    'record vertically centered on the screen, except at the beginning ' + ;
    'and end of the database.' + ;
    '|' + ;
    'Add field:  Adds a browse field to the Fields list and to the data ' + ;
    'browse.  A browse field means a display column or line in the data ' + ;
    'browse, not a structure field in a database.  A browse field is ' + ;
    'displayed as a column in horizontal mode, and a line in vertical ' + ;
    'mode.  A browse field may be a field in the current database, a ' + ;
    'field in another database, or a calculated field defined by any ' + ;
    'expression. (S) (*)' + ;
    '|' + ;
    'Edit field:  Modifies the definition of the highlighted browse ' + ;
    'field in the Fields list. (S) (*)' + ;
    '|' + ;
    'Delete field:  Removes the highlighted field from the data browse ' + ;
    'and the list of fields. (S)' + ;
    '|' + ;
    'Manage field list:  Brings up the field manager, which allows you ' + ;
    'to add and delete many fields at once to and from the Fields list.' + ;
    '|' + ;
    '< move field up:  In the field list, moves the highlighted field ' + ;
    'up by one position, and changes the corresponding position of the ' + ;
    'field in the data browse.' + ;
    '|' + ;
    '> move field down:  In the field list, moves the highlighted ' + ;
    'field down by one position, and changes the corresponding ' + ;
    'position of the field in the data browse.' + ;
    '|' + ;
    'Original fields:  Restores the field list to its original state, ' + ;
    'the list of fields as defined in the database.' + ;
    '|' + ;
    'Field order for fields:  Sorts the field list in the order of ' + ;
    'field definition.' + ;
    '|' + ;
    'Name order for fields:  Sorts the field list in the order of ' + ;
    'field name.' + ;
    '|' + ;
    'Refresh browses:  Redisplays all data on the screen from ' + ;
    'underlying sources.' + ;
    '|' + ;
    'Initialize browse:  Redefines the data browse from the field ' + ;
    'list.'

CASE nContext == H_FLDADD

  cText := ;
    'Location: View menu~' + ;
    'Shortcut from Field setup pane: Insert key~' + ;
    '|' + ;
    'Adds a browse field to the Fields list and to the data browse.  A ' + ;
    'browse field means a display column or line in the data browse, ' + ;
    'not a structure field in a database.  A browse field is displayed ' + ;
    'as a column in horizontal mode, and a line in vertical mode.' + ;
    '|' + ;
    HelpPar(HP_FLDSTRU) + ;
    '|' + ;
    'To add or delete a group of fields, you can also use the field ' + ;
    'manager.  See the help topic "Manage field list".' + ;
    '|' + ;
    HelpPar(HP_FLDMOD)

CASE nContext == H_FLDEDIT

  cText := ;
    'Location: View menu~' + ;
    'Shortcut from Field setup pane: Ctrl-Enter~' + ;
    '|' + ;
    'Modifies the definition of the highlighted browse field in the ' + ;
    'Fields list.' + ;
    '|' + ;
    HelpPar(HP_FLDSTRU) + ;
    '|' + ;
    'To add or delete a group of fields, you can also use the field ' + ;
    'manager.  See the help topic "Manage field list".' + ;
    '|' + ;
    HelpPar(HP_FLDMOD)

CASE nContext == H_FLDMANAGE

  cText := ;
    'Location: View menu~' + ;
    '|' + ;
    'This option brings up the field manager, which allows you to add ' + ;
    'and delete many fields at once to and from the Fields list.' + ;
    '|' + ;
    HelpPar(HP_FLDSTRU) + ;
    '|' + ;
    'The field manager shows all the fields in the structure of the ' + ;
    'current database.  If the current database has any relations into ' + ;
    'other databases, the fields in those databases are also shown.' + ;
    '|' + ;
    'Each line contains the following data for each field:  selection ' + ;
    'status, alias if there are any relations, ' + ;
    HelpPar(HP_FLDCOLS) + ;
    '|' + ;
    'The selection status shows a diamond if the field is currently in ' + ;
    'the Fields list.  To change the selection status, press the space ' + ;
    'bar or double click on the field.  This will add or delete the ' + ;
    'field from the Fields list.  When you done, press the End key, and ' + ;
    'the Fields list and the browse will be updated.' + ;
    '|' + ;
    'A field that has been removed shows a space instead of a diamond, ' + ;
    'followed by a minus sign.  A field that has been added shows a ' + ;
    'diamond followed by a plus sign.' + ;
    '|' + ;
    'If you add fields with the file manager, i.e. if the selection ' + ;
    'status shows a diamond and a plus sign, those fields are added to ' + ;
    'the end of the Fields list.  The file manager does not allow you ' + ;
    'to edit calcualted fields or to move items up or down within the ' + ;
    'field list.  To do this, use the setup or extended setup browses.  ' + ;
    'See the help topic "Expand current setup".'

END

RETURN cText  //  HelpView

//***************************************************************************

STATIC FUNCTION HelpUtil(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_UTILMENU

  cText := ;
    'Hot key: F2~' + ;
    '|' + ;
    'Miscellaneous functions, such as MEM file editor, directory ' + ;
    'display, copy file, ASCII table.' + ;
    '|' + ;
    HelpPar(HP_HOTKEY) + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'Copy to clipboard:  Writes the current data transfer buffer to the ' + ;
    'Windows clipboard. (H) (*)' + ;
    '|' + ;
    'Paste from clipboard:  Reads the current contents of the Windows ' + ;
    'clipboard into the data transfer buffer. (H)' + ;
    '|' + ;
    'Evaluate:  Evaluates an expression. (H) (*)' + ;
    '|' + ;
    'Repeat keystroke:  Allows you to repeat a keystroke a specified ' + ;
    'number of times. (H) (*)' + ;
    '|' + ;
    'Play back last record:  Plays back the last recorded set of ' + ;
    'keystrokes. (H)' + ;
    '|' + ;
    'Play back keystrokes:  Plays back a set of keystrokes that you ' + ;
    'enter as a string. (H) (*)' + ;
    '|' + ;
    'History:  Displays the history buffer and allows you to choose ' + ;
    'from it. (H) (*)' + ;
    '|' + ;
    'Structure current:  Displays the list of fields of the current ' + ;
    'database and allows you to select from it. (H) (*)' + ;
    '|' + ;
    'Structures all:  Displays the list of fields for all open ' + ;
    'databases and allows you to select from it. (H) (*)' + ;
    '|' + ;
    'Directory local:  Displays a list of files in the current ' + ;
    'directory and allows you to select from it. (H) (*)' + ;
    '|' + ;
    'Directory global:  Displays a list of files in any directory on ' + ;
    'any drive, and allows you to select from it. (H) (*)' + ;
    '|' + ;
    'File manager local:  Displays a list of files in the current ' + ;
    'directory.  You may view, edit, copy, delete, rename, and execute ' + ;
    'files. (H) (*)' + ;
    '|' + ;
    'File manager global:  Display a list of files in any directory ' + ;
    'on any drive.  You may view, edit, copy, delete, rename, and ' + ;
    'execute files. (H) (*)' + ;
    '|' + ;
    'ASCII table:  Displays a table of ASCII characters. (H) (*)' + ;
    '|' + ;
    'Color table:  Displays a table of colors and color strings. (H) ' + ;
    '(*)' + ;
    '|' + ;
    'Calendar:  Displays a calendar for several weeks before and after ' + ;
    'a given date. (H) (*)' + ;
    '|' + ;
    'Memvar edit:  Starts the memvar editor, which allows you to manage ' + ;
    'memory variables.  It allows you to read in memory variable files ' + ;
    '(.MEM files), write them out, and to display, add, edit, and ' + ;
    'delete individual variables.' + ;
    '|' + ;
    'Report modify:  Starts the report editor, which allows you to ' + ;
    'create or modify a report file (.FRM file). (*)' + ;
    '|' + ;
    'Label modify:  Starts the label editor, which allows you to ' + ;
    'create or modify a label file (.LBL file). (*)' + ;
    '|' + ;
    'Data screen:  Displays the data browse, temporarily removing ' + ;
    'overlying windows.' + ;
    '|' + ;
    'Shell to DOS:  Suspends DBE and goes temporarilty to DOS, where ' + ;
    'you can enter DOS commands.  Type EXIT to return to DBE.'

CASE nContext == H_COPYCLIP

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: Alt-Del~' + ;
    '|' + ;
    'Writes the current data transfer buffer to the Windows clipboard.' + ;
    '|' + ;
    'This option asks if you want to append the transfer buffer to the ' + ;
    'clipboard.  If you answer No, the transfer buffer overwrites the ' + ;
    'clipboard.  If you answer Yes, the transfer buffer is appended to ' + ;
    'the clipboard, and the appended text is written to the clipboard.'

CASE nContext == H_EVAL

  nPad  := 22
  cText := ;
    'Location: Util menu~' + ;
    'Hot key: F3~' + ;
    '|' + ;
    'Determines the value of any valid expression.  The resulting value ' + ;
    'is displayed and saved into the history.' + ;
    '|' + ;
    'Besides Clipper functions, the following functions can be used in ' + ;
    'an expression:' + ;
    '|' + ;
    PADR('TRANSFER()'              , nPad) + ;
    'Contents of data transfer buffer~' + ;
    PADR('TRANSFER(<Item>)'        , nPad) + ;
    'Stuff data transfer buffer~' + ;
    PADR('FREESPACE()'             , nPad) + ;
    'Available bytes on local drive~' + ;
    PADR('FREESPACE(<DriveLetter>)', nPad) + ;
    'Available bytes on drive letter~' + ;
    PADR('PRINTCODES(.Y., .Y.)'    , nPad) + ;
    'Printer codes at file start~' + ;
    PADR('PRINTCODES(.Y., .N.)'    , nPad) + ;
    'Printer codes at file end~' + ;
    PADR('EXEPATH()'               , nPad) + ;
    'Full path name of DBE executable~' + ;
    PADR('EXEARGS()'               , nPad) + ;
    'Command line arguments of DBE executable~' + ;
    PADR('ORDRECNO()'              , nPad) + ;
    'Logical record number via current index~' + ;
    PADR('ORDRECNO(<OrderNumber>)' , nPad) + ;
    'Logical record number via order number~'

CASE nContext == H_REPEAT

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: Alt-F3~' + ;
    '|' + ;
    'Allows you to repeat a keystroke a specified number of times.  At ' + ;
    'the first box, enter the number of times you wish to repeat the ' + ;
    'keystroke.  At the second box, simply press the key you wish to ' + ;
    'repeat.  The keystroke will be immediately repeated.'

CASE nContext == H_KRPLAY

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: Alt-F6~' + ;
    '|' + ;
    'Plays back a set of keystrokes that you enter as a string.' + ;
    '|' + ;
    'Keystrokes to play back:' + ;
    ''

CASE nContext == H_HISTORY

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: F4~' + ;
    '|' + ;
    'Displays the history buffer and allows you to choose from it.' + ;
    '|' + ;
    'The history buffer contains one-line data items that you have ' + ;
    'entered or generated in the current DBE session.' + ;
    '|' + ;
    'When you select an item from the history, it is passed into the ' + ;
    'data transfer buffer, where it can be used by any DBE object that ' + ;
    'accepts data through this buffer.  See the help topic "Object data ' + ;
    'transfer".' + ;
    '|' + ;
    'The history includes data you have entered through from the ' + ;
    'following DBE objects:' + ;
    '|' + ;
    'Dialog box lines (except logical and list box lines)~' + ;
    'Record menu / Field change (except logical and memo fields)~' + ;
    'Set menu / Keystroke record~' + ;
    '|' + ;
    'The history also includes data generated by the following DBE ' + ;
    'objects:' + ;
    '|' + ;
    'Util menu / Evaluate result (except logical)~' + ;
    'Record menu / Store field~' + ;
    'File menu / Sum result~' + ;
    'File menu / Count result~' + ;
    'File menu / Accumulate result (except logical)~' + ;
    'Open menu / Read DBE file~' + ;
    '|' + ;
    'The history browse also allows you to exchange data through the ' + ;
    'Windows clipboard.  See the commands below.|' + ;
    '|' + ;
    'When you are browsing the history, you can use the following ' + ;
    'options:' + ;
    '|' + ;
    'Alt-Del - Copy to clipboard:  Copy the highlighted item to the ' + ;
    'Windows clipboard, where it can be pasted by other applications.' + ;
    '|' + ;
    'Alt-Ins - Paste from clipboard:  Paste from the Windows clipboard ' + ;
    'to a new history item.' + ;
    '|' + ;
    'Space - View:  Displays the item with the memo editor.' + ;
    '|' + ;
    'Ctrl-Enter - Edit:  Allows you to edit the item with the memo ' + ;
    'editor.' + ;
    '|' + ;
    'Enter - Select:  Selects the item and exits the browse.  The item ' + ;
    'is passed into the data transfer buffer, where it can be used by ' + ;
    'any DBE object that accepts data transfer.  See the help topic ' + ;
    '"Object data transfer".' + ;
    '|' + ;
    'Esc - Cancel:  Exits the browse without selecting an item.' + ;
    '|' + ;
    'You may perform an incremental search on the contents of the ' + ;
    'history item.  See the help topic "Incremental search".'

CASE nContext == H_PASTEHIST

  cText := ;
    'Location: Util menu / Copy to clipboard~' + ;
    '|' + ;
    'Pastes from the Windows clipboard to a new history item.' + ;
    '|' + ;
    'This option asks if you want to append the clipboard to the ' + ;
    'current item.  If you answer No, the clipboard is written to a new ' + ;
    'history item, ignoring the current history item.  If you answer ' + ;
    'Yes, the clipboard is appended to the current history item, and ' + ;
    'the appended text is written to a new history item.'

CASE nContext == H_STRUONE

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: F5~' + ;
    '|' + ;
    'Displays the list of fields of the current database and allows you ' + ;
    'to select from it.' + ;
    '|' + ;
    'Each line contains the following data for each field:  ' + ;
    HelpPar(HP_FLDCOLS) + ;
    '|' + ;
    'When you are browsing the structure, you can use the following ' + ;
    'options:' + ;
    '|' + ;
    'Enter - Select:  Selects the field and exits the structure ' + ;
    'browse.  The field name is passed into the data transfer buffer, ' + ;
    'where it can be used by any DBE object that accepts data ' + ;
    'transfer.  See the help topic "Object data transfer".' + ;
    '|' + ;
    'Esc - Cancel:  Exit the structure browse without selecting a ' + ;
    'field.' + ;
    '|' + ;
    'Alt-F - Field order:  Sort the field list in field number order, ' + ;
    'i.e. in the order in which they appear in the database.' + ;
    '|' + ;
    'Alt-N - Name order:  Sort the field list in field name order.' + ;
    '|' + ;
    'You may perform an incremental search on the field name.  See the ' + ;
    'help topic "Incremental search".' + ;
    '|' + ;
    'To modify the structure of the database, use File menu / Modify ' + ;
    'structure.  See the help topic "Structure editor".'

CASE nContext == H_STRUALL

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: F6~' + ;
    '|' + ;
    'Displays the list of fields for all open databases and allows you ' + ;
    'to select from it.' + ;
    '|' + ;
    'Each line contains the following data for each field:  alias, ' + ;
    HelpPar(HP_FLDCOLS) + ;
    '|' + ;
    'When you are browsing the structure, you can use the following ' + ;
    'options:' + ;
    '|' + ;
    'Enter - Select:  Select the aliased field and exit the structure ' + ;
    'browse.  The field name is passed into the data transfer buffer, ' + ;
    'where it can be used by any DBE object that accepts data ' + ;
    'transfer.  See the help topic "Object data transfer".' + ;
    '|' + ;
    'Esc - Cancel:  Exit the structure browse without selecting a ' + ;
    'field.' + ;
    '|' + ;
    'Alt-F - Field order:  Sort the field list in alias and field ' + ;
    'number order, i.e. in the order in which they appear in the ' + ;
    'database.' + ;
    '|' + ;
    'Alt-N - Name order:  Sort the field list in field name order,  ' + ;
    'without regard to alias.' + ;
    '|' + ;
    'You may perform an incremental search in this table.  See the help ' + ;
    'topic "Incremental search".  When you are browsing in field order, ' + ;
    'the incremental search works on the alias name only.  When you are ' + ;
    'browsing in name order, the incremental search works on field name ' + ;
    'only.' + ;
    '|' + ;
    'To modify the structure of the database, use File menu / Modify ' + ;
    'structure.  See the help topic "Structure editor".'

CASE nContext == H_DIRLOCAL

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: F8~' + ;
    '|' + ;
    'Displays a list of files in the current directory and allows you ' + ;
    'to select from it.' + ;
    '|' + ;
    'The list may include only a subset of files in the current ' + ;
    'directory--the current file spec is displayed as the title of the ' + ;
    'box.' + ;
    '|' + ;
    HelpPar(HP_MANLOC) + ;
    '|' + ;
    HelpPar(HP_MANDISP) + ;
    '|' + ;
    HelpPar(HP_MANINC)

CASE nContext == H_DIRGLOBAL

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: F9~' + ;
    '|' + ;
    'Displays a list of files in any directory on any drive, and allows ' + ;
    'you to select from it.' + ;
    '|' + ;
    'You must first specify the drive, directory, and file spec to ' + ;
    'browse.  See the help topic "Tree spec".  The matching files are ' + ;
    'then displayed with the drive, directory, and file spec as the ' + ;
    'title of the box.' + ;
    '|' + ;
    HelpPar(HP_MANLOC) + ;
    '|' + ;
    HelpPar(HP_MANGLOB) + ;
    '|' + ;
    HelpPar(HP_MANDISP) + ;
    '|' + ;
    HelpPar(HP_MANINC)

CASE nContext == H_MANLOCAL

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: Alt-F8~' + ;
    '|' + ;
    'Displays a list of files in the current directory.  You may view, ' + ;
    'edit, copy, delete, rename, and execute files.' + ;
    '|' + ;
    HelpPar(HP_MANLOC) + ;
    '|' + ;
    HelpPar(HP_MANDISP) + ;
    '|' + ;
    HelpPar(HP_MANEDIT) + ;
    '|' + ;
    HelpPar(HP_MANINC)

CASE nContext == H_MANGLOBAL

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: Alt-F9~' + ;
    '|' + ;
    'Display a list of files in any directory on any drive.  You may ' + ;
    'view, edit, copy, delete, rename, and execute files.' + ;
    '|' + ;
    HelpPar(HP_MANLOC) + ;
    '|' + ;
    HelpPar(HP_MANGLOB) + ;
    '|' + ;
    HelpPar(HP_MANDISP) + ;
    '|' + ;
    HelpPar(HP_MANEDIT) + ;
    '|' + ;
    HelpPar(HP_MANINC)

CASE nContext == H_MANTREE

  cText := ;
    'Location: Util menu / Directory or File manager~' + ;
    '|' + ;
    'Allows you to enter a new drive, directory, and file spec to ' + ;
    'browse.  Only subdirectories and files matching the file spec in ' + ;
    'the indicated drive and directory will be listed.' + ;
    '|' + ;
    'Drive:  Enter a drive letter, or a network server/volume name.' + ;
    '|' + ;
    'Directory:  Enter a directory path.  If you use a drive letter to ' + ;
    'specify the drive, then you may use a relative path in the ' + ;
    'directory.  This means that a path name that does not start with ' + ;
    '"\" is interpreted as starting from the current directory.  A ' + ;
    'blank entry indicates the current directory itself.  Network ' + ;
    'server/volume names do not allow relative paths and interpret the ' + ;
    'lack of a leading slash as starting from the highest level ' + ;
    'directory.' + ;
    '|' + ;
    'Files:  Enter a file name pattern.  You may use the wildcard ' + ;
    'characters "*" and "?".' + ;
    '|' + ;
    'Include subdirectories:  Whether to include subdirectories in the ' + ;
    'browse list.' + ;
    '|' + ;
    'Include files:  Whether to include files in the browse list.' + ;
    '|' + ;
    'Include hidden and system files:  Whether to include hidden and ' + ;
    'system files in the browse list.'

CASE nContext == H_MANFILE

  cText := ;
    'Location: Util menu / Directory or File manager~' + ;
    '|' + ;
    'Allows you to enter a new file spec to browse, while remaining in ' + ;
    'the current directory.' + ;
    '|' + ;
    'Files:  Enter a file name pattern.  You may use the wildcard ' + ;
    'characters "*" and "?".' + ;
    '|' + ;
    'Include hidden and system files:  Whether to include hidden and ' + ;
    'system files in the browse list.'

CASE nContext == H_MANPRINT

  cText := ;
    'Location: Util menu / File manager~' + ;
    '|' + ;
    'Prints the contents of the highlighted file, assuming that it ' + ;
    'contains only ASCII data and printer codes.  You also have the ' + ;
    'option of adding printer codes to the output.' + ;
    '|' + ;
    'To:  Pick a printer name from the list.' + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_MANNEW

  nPad  := 18
  cText := ;
    'Location: Util menu / File manager~' + ;
    '|' + ;
    'Allows you to create a new ASCII file using the memo editor.' + ;
    '|' + ;
    'Name:  Enter the name of the new file.' + ;
    '|' + ;
    'This option is only for creating new ASCII files.  Other file ' + ;
    'creation options in DBE include the following:' + ;
    '|' + ;
    PADR('Database files', nPad) + ;
    'Open menu / Create from or Create structure~' + ;
    PADR('Index files'   , nPad) + 'Open menu / Index create~  ' + ;
    PADR('DBE files'     , nPad) + 'Open menu / Write DBE file~' + ;
    PADR('Memvar files'  , nPad) + 'Util menu / Memvar edit~'    + ;
    PADR('Report files'  , nPad) + 'Util menu / Report modify~'  + ;
    PADR('Label files'   , nPad) + 'Util menu / Label modify~'

CASE nContext == H_MANCOPY

  cText := ;
    'Location: Util menu / File manager~' + ;
    '|' + ;
    'Copies the highlighed file to a new or existing file.  If the file ' + ;
    'being copied to already exists, it will be overwritten.' + ;
    '|' + ;
    'To:  Enter the file name, including the drive and directory if ' + ;
    'needed.  Do not use wildcards, and do not enter just a directory ' + ;
    'name.'

CASE nContext == H_MANRENAME

  cText := ;
    'Location: Util menu / File manager~' + ;
    '|' + ;
    'Renames the highlighed file.' + ;
    '|' + ;
    'To:  Enter the file name, possibly including a drive and ' + ;
    'directory.  Do not use wildcards, and do not enter just a ' + ;
    'directory name.  If you use a drive and directory, the file is ' + ;
    'moved to that location.'

CASE nContext == H_MANLIST

  cText := ;
    'Location: Util menu / File manager~' + ;
    '|' + ;
    'Writes the list of files and subdirectories to an ASCII file.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_MANRETRY

  cText := ;
    'Location: Util menu / Directory or File manager~' + ;
    '|' + ;
    'On a failed search, allows you to retry with a new tree or file ' + ;
    'spec.' + ;
    '|' + ;
    'If the directory or file manager does not find the directory or ' + ;
    'files you were looking for, it asks you if you want to retry with ' + ;
    'a new tree or file spec.  Choose "Tree" to enter a new tree spec; ' + ;
    'choose "Files" to enter a new file spec; or choose "Cancel" to ' + ;
    'exit the directory or file mananger.'

CASE nContext == H_ASCTABLE

  cText := ;
    'Location: Util menu~' + ;
    'Hot key: F10~' + ;
    '|' + ;
    'Displays a table of ASCII characters.  Each line contains the ' + ;
    'following data for each character:  numeric code in decimal, ' + ;
    'numeric code in hexadecimal, and the character glyph.' + ;
    '|' + ;
    'The Enter key selects the highlighted character and exits the ' + ;
    'table.  A selected character is passed into the data transfer ' + ;
    'buffer, where it can be used by any DBE object that accepts data ' + ;
    'transfer.  If used on a data entry line, the character is appended ' + ;
    'to the text currently on the data entry line. See the help topic ' + ;
    '"Object data transfer".'

CASE nContext == H_COLORTABLE

  cText := ;
    'Location: Util menu~' + ;
    'Hot keys: F11 or Alt-F1~' + ;
    '|' + ;
    'Displays a table of all possible colors and color strings.  Each ' + ;
    'line contains the following data for each color combination:  in ' + ;
    'the first four columns, the foreground color: presence or absence ' + ;
    'of blue, green, red, and intensity attributes; in the second four ' + ;
    'columns, the same attributes for the background color; and in the ' + ;
    'last column, sample output in that color.' + ;
    '|' + ;
    'The Enter key selects the highlighted color combination and exits ' + ;
    'the table.  A selected color combination is passed as a color ' + ;
    'string into the data transfer buffer, where it can be used by any ' + ;
    'DBE object that accepts data transfer.  See the help topic "Object ' + ;
    'data transfer".'

CASE nContext == H_CALENDAR

  cText := ;
    'Location: Util menu~' + ;
    'Hot keys: F12 or Alt-F2~' + ;
    '|' + ;
    'Displays a calendar for several weeks before and after a given ' + ;
    'date.  The calendar is displayed in a "flow format".  In this ' + ;
    'format, days of the week are indicated at the top, months are ' + ;
    'indicated in the left margin, and years are indicated in the right ' + ;
    'margin.  There are no line breaks between months or years.' + ;
    '|' + ;
    'The calendar defaults to today' + "'" + 's date.  To move to a new ' + ;
    'date, you may press the cursor keys, or you may press "N" to enter ' + ;
    'the new date.  When you cursor up or down in the calendar, the ' + ;
    'whole browse is scrolled, and the highlight bar remains in the ' + ;
    'same screen position.' + ;
    '|' + ;
    'To write a calendar to a file, press "L".' + ;
    '|' + ;
    'The Enter key selects the highlighted date and exits the ' + ;
    'calendar.  A selected date is passed as a date value into the data ' + ;
    'transfer buffer, where it can be used by any DBE object that ' + ;
    'accepts data transfer.  See the help topic "Object data transfer".'

CASE nContext == H_CALNEW

  cText := ;
    'Location: Util menu / Calendar~' + ;
    '|' + ;
    'Moves the calendar to a new date.' + ;
    '|' + ;
    'Start date:  Enter the new date.  The calendar display will then ' + ;
    'use this date as the new starting point for the calendar.'

CASE nContext == H_CALLIST

  cText := ;
    'Location: Util menu / Calendar~' + ;
    '|' + ;
    'Writes a calendar to a file.  You may pick any starting and ending ' + ;
    'dates for this calendar.  The calendar display is not affected by ' + ;
    'the start and end dates you pick for the file.' + ;
    '|' + ;
    'Starting date:  Enter the date for the first line on the calendar ' + ;
    'in the file.' + ;
    '|' + ;
    'Ending date:  Enter the date for last line on the calendar in the ' + ;
    'file.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_MEMEDIT

  cText := ;
    'Location: Util menu~' + ;
    '|' + ;
    'Starts the memvar editor, which allows you to manage memory ' + ;
    'variables.  It allows you to read in memory variable files (.MEM ' + ;
    'files), write them out, and to display, add, edit, and delete ' + ;
    'individual variables.' + ;
    '|' + ;
    'While the memvar editor is active, the memvars remain private ' + ;
    'within the memvar editor.  When you return to the main screen of ' + ;
    'DBE, the memvars are made public and can then be used anywhere in ' + ;
    'DBE.' + ;
    '|' + ;
    'The memvar editor consists of a browse of variables and options ' + ;
    'for managing them.  In the table, each line contains the following ' + ;
    'items:  the name of the variable, its type, its length if it is a ' + ;
    'character type, and an expression which is a representation of the ' + ;
    'value of the variable.' + ;
    '|' + ;
    'The following options can be used from the memvar editor:' + ;
    '|' + ;
    'Home - Restore:  Reads memvars from a memvar file (.MEM file) and ' + ;
    'adds them to the memvar browse. (*)' + ;
    '|' + ;
    'End - Save:  Writes the memvars currently on the memvar browse to ' + ;
    'a memvar file (.MEM file). (*)' + ;
    '|' + ;
    'Ins - Add:  Adds a new memvar. (*)' + ;
    '|' + ;
    'Del - Release:  Removes the highlighted memvar from the browse ' + ;
    'and releases it from memory.' + ;
    '|' + ;
    'Enter - Edit:  Edits the expression for the highlighted memvar.' + ;
    '|' + ;
    'Alt-Eval:  If the highlighted memvar is a code block, this option ' + ;
    'evaluates it and displays the result.' + ;
    '|' + ;
    'Ctrl-Del - Release all:  Removes all memvars from the browse and ' + ;
    'releases them from memory.  The Ctrl-End key also performs this ' + ;
    'operation.' + ;
    '|' + ;
    'Ctrl-Enter - Edit memo:  Edits the expression for the highlighted ' + ;
    'memvar through the memo editor.  This can only be used on ' + ;
    'character variables.' + ;
    '|' + ;
    'Alt-Input text:  Reads memvars from a special-format ASCII file ' + ;
    'and adds them to the memvar browse. (*)' + ;
    '|' + ;
    'Alt-Output text:  Writes the memvars currently on the memvar ' + ;
    'browse to a special-format ASCII file. (*)' + ;
    '|' + ;
    'Alt-Refresh:  Redisplays all data on the memvar browse from ' + ;
    'underlying sources.' + ;
    '|' + ;
    'Esc - Exit:  Exits the memvar editor.  The memvars currently in ' + ;
    'the browse remain in memory for use elsewhere in DBE.'

CASE nContext == H_MEMREST

  cText := ;
    'Location: Util menu / Memvar edit / Menu~' + ;
    '|' + ;
    'Reads memvars from a memvar file (.MEM file) and adds them to the ' + ;
    'memvar browse.' + ;
    '|' + ;
    'Name:  Enter the name of the memvar file, ' + HelpPar(HP_FILENAME) + ;
    'is .MEM.' + ;
    '|' + ;
    'Additive:  If you answer Yes, then the memvars in the file are ' + ;
    'added to the memvars already in the browse.  If you answer No, ' + ;
    'then the memvars in the browse are replaced with the memvars in ' + ;
    'the file.'

CASE nContext == H_MEMSAVE

  cText := ;
    'Location: Util menu / Memvar edit / Menu~' + ;
    '|' + ;
    'Writes the memvars currently on the memvar browse to a memvar ' + ;
    'file (.MEM file).' + ;
    '|' + ;
    'Name:  Enter the name of the memvar file, ' + HelpPar(HP_FILENAME) + ;
    'is .MEM.' + ;
    '|' + ;
    'Like or Except:  Enter "L" for Like or "E" for Except.  If you ' + ;
    'answer Like, then all the memvars in the browse whose names match ' + ;
    'the pattern in the skeleton are written to the file.  If you ' + ;
    'answer Except, then all memvars that do not match the pattern in ' + ;
    'the skeleton are written to the file.' + ;
    '|' + ;
    'Skeleton:  Enter a pattern for controlling which memvars in the ' + ;
    'browse are written to the file, as described under "Like or ' + ;
    'Except".  This pattern may include the wildcards "?" and "*".'

CASE nContext == H_MEMINPUT

  cText := ;
    'Location: Util menu / Memvar edit / Menu~' + ;
    '|' + ;
    'Reads memvars from a special-format ASCII file and adds them to ' + ;
    'the memvar browse.  This ASCII file is similar in format to the ' + ;
    'memvar browse screen.' + ;
    '|' + ;
    'Name:  Enter the name of the ASCII file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.'

CASE nContext == H_MEMOUTPUT

  cText := ;
    'Location: Util menu / Memvar edit / Menu~' + ;
    '|' + ;
    'Writes the memvars currently on the memvar browse to a ' + ;
    'special-format ASCII file.  This ASCII file is similar in format ' + ;
    'to the memvar browse screen.' + ;
    '|' + ;
    'Name:  Enter the name of the ASCII file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.' + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_MEMADD

  cText := ;
    'Location: Util menu / Memvar edit / Menu~' + ;
    'Shortcut from the memvar editor: Insert key~' + ;
    '|' + ;
    'Adds a new memvar to the browse and defines it in memory.' + ;
    '|' + ;
    'Name:  Enter the name of the new memvar.' + ;
    '|' + ;
    'Expression:  Enter an expression which when evaluated is the value ' + ;
    'of the memvar.  The type and length of the memvar are computed by ' + ;
    'the memvar editor from its value.'

CASE nContext == H_FRMEDIT

  cText := ;
    'Location: Util menu~' + ;
    '|' + ;
    'Starts the report editor, which allows you to create or modify a ' + ;
    'report file (.FRM file).  You can use a report file to produce a ' + ;
    'report within or without DBE.  See the help topic "Report form".' + ;
    '|' + ;
    'A report file consists of specifications for report columns and ' + ;
    'additional specifications to control the report header, the width ' + ;
    'of the report lines, etc.' + ;
    '|' + ;
    'File: Enter the name of the report file, ' + HelpPar(HP_FILENAME) + ;
    'is .FRM.' + ;
    '|' + ;
    'After you enter the file name, the report editor screen comes up.  ' + ;
    'The cursor movement keys allow you to browse the list of columns.  ' + ;
    'In this screen, you can use the commands listed below.' + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'H - Heading:  Modifies the report heading lines. (*)' + ;
    '|' + ;
    'P - Parameters:  Modifies the report parameters listed in the ' + ;
    'Parameters section of the screen. (*)' + ;
    '|' + ;
    'G - Group:  Modifies the group and subgroup characteristics. (*)' + ;
    '|' + ;
    'C - Columns:  Refreshes the column list from underlying data.' + ;
    '|' + ;
    'Ins - Add column:  Adds a new column to the report file. (*)' + ;
    '|' + ;
    'Del - Delete column:  Deletes the highlighted column from the ' + ;
    'report file.' + ;
    '|' + ;
    'Enter - Edit column:  Modifies the highlighted column. (*)' + ;
    '|' + ;
    'F - Formats:  Sets the report page width and lines per page from a ' + ;
    'menu of common formats. (*)' + ;
    '|' + ;
    'L - List:  Writes a description of the report file to an ASCII ' + ;
    'file.' + ;
    '|' + ;
    'Ctrl-' + C_UP_ARROW + ' - Move column up:  Moves the highlighted ' + ;
    'columm up one position in the column list.' + ;
    '|' + ;
    'Ctrl-' + C_DOWN_ARROW + ' - Move column down:  Moves the ' + ;
    'highlighted column down one position in the column list.' + ;
    '|' + ;
    'End - Save:  Writes the revised report file to disk and exits the ' + ;
    'report editor.  If the report file already exists, it is replaced ' + ;
    'with the new one.' + ;
    '|' + ;
    'Esc - Cancel:  Exits the report editor without saving changes to ' + ;
    'disk.'

CASE nContext == H_FRMHEAD

  cText := ;
    'Location: Util menu / Report modify~' + ;
    '|' + ;
    'Modifies the report heading lines.  These lines are printed at the ' + ;
    'top of every report page.  The lines that you enter are output as ' + ;
    'literal text; they are not interpreted in any way and therefore do ' + ;
    'not change.' + ;
    '|' + ;
    'Heading lines:  Enter the text to appear at the top of the report.'

CASE nContext == H_FRMPARAM

  cText := ;
    'Location: Util menu / Report modify~' + ;
    '|' + ;
    'Modifies the report parameters listed in the Parameters section of ' + ;
    'the screen.' + ;
    '|' + ;
    'There are two report parameters that are calculated from other ' + ;
    'objects in the report file:  computed column width, and computed ' + ;
    'total width.  Computed column width is the sum of widths of all the ' + ;
    'columns, plus intercolumn spacing.  Computed total width is the sum ' + ;
    'of left margin, right margin, and computed column width.  The set ' + ;
    'page width setting should be normally be set to be equal or greater ' + ;
    'than the computed total width.' + ;
    '|' + ;
    'Set page width:  Enter the total width of the report in ' + ;
    'characters, including margins.  This value is used to compute the ' + ;
    'horizontal positions of the report headers.  It should normally be ' + ;
    'set to be equal to or greater than the computed total width.' + ;
    '|' + ;
    'Left margin:  Enter the number of characters to leave blank at the ' + ;
    'beginning of each line of the report.' + ;
    '|' + ;
    'Right margin:  Enter the number of characters intended to be blank ' + ;
    'at the end of each line of the report.  This value is used to ' + ;
    'adjust the horizontal positions of the report headers.' + ;
    '|' + ;
    'Lines per page:  Enter the number of lines to print on each page ' + ;
    'of the report.' + ;
    '|' + ;
    'Double space:  Enter Yes to skip a line between outputs for each ' + ;
    'record.  If a record is output onto multiple lines, this setting ' + ;
    'not affect the spacing between lines of a record line group, only ' + ;
    'the spacing between record line groups.' + ;
    '|' + ;
    'Eject before print:  Whether to output a form feed before the rest ' + ;
    'of the report output.' + ;
    '|' + ;
    'Eject after print:  Whether to output a form feed after the rest ' + ;
    'of the report output.' + ;
    '|' + ;
    'Plain print:  If you answer Yes, then the report is output without ' + ;
    'page numbers, dates, and page breaks, and the report header and ' + ;
    'column headers are output only once at the beginning of the ' + ;
    'report.  If you answer No, then the report is output with page ' + ;
    'numbers, dates, and page breaks, and the report header and column ' + ;
    'headers are output at the top of each page of the report.  This ' + ;
    'setting can also be specified when you run the report.' + ;
    '|' + ;
    'The following charts show how many characters and lines fit on a ' + ;
    'physical sheet of paper in various output modes.  You can use ' + ;
    'formats to automatically set page width and lines per page.  See ' + ;
    'the help topic "Report formats".' + ;
    '|' + ;
    HelpPar(HP_CHARFIT)

CASE nContext == H_FRMGROUP

  cText := ;
    'Location: Util menu / Report modify~' + ;
    '|' + ;
    'Modifies the group and subgroup characteristics.' + ;
    '|' + ;
    'Groups are an optional feature used to output headings and ' + ;
    'subtotals for subsets of records.  Subgroups are used for subsets ' + ;
    'of groups.  The report file format does not allow for grouping ' + ;
    'levels deeper than these two.' + ;
    '|' + ;
    'When groups are used, the report generator evaluates a group ' + ;
    'expression on each record.  When the value of the group expression ' + ;
    'changes, a new group is started.  At the beginning of each group, ' + ;
    'a group header line is output, which consists of the group heading ' + ;
    'followed by the value of the group expression.  Subgroups work ' + ;
    'similarly within groups.' + ;
    '|' + ;
    'The use of groups and subgroups is optional.  Subgroups can be ' + ;
    'used only if groups are also used.' + ;
    '|' + ;
    'Group heading:  Enter the text to be printed on the group header ' + ;
    'line at the beginning of a group.  This text is output as literal ' + ;
    'text; it is not interpreted in any way and therefore does not ' + ;
    'change.' + ;
    '|' + ;
    'Group expression:  Enter an expression which is to be evaluated on ' + ;
    'each record and which is to stay constant within a group.  This ' + ;
    'expression is often the anticipated index expression or the first ' + ;
    'part of it.' + ;
    '|' + ;
    'Summary only:  If you answer Yes, only total lines are printed for ' + ;
    'each group and subgroup; no record data is output.  If you answer ' + ;
    'No, then record data is output, along with group header lines and ' + ;
    'total lines.' + ;
    '|' + ;
    'Eject after group:  If you answer Yes, then a form feed is output ' + ;
    'when the end of a group is reached.  If you answer No, then no ' + ;
    'form feeds are output after a group.  A form feed is never output ' + ;
    'after a subgroup.' + ;
    '|' + ;
    'Subgroup heading:  Enter the text to be printed on the subgroup ' + ;
    'header line at the beginning of a subgroup.' + ;
    '|' + ;
    'Subgroup expression:  Enter an expression which is to be evaluated ' + ;
    'on each record and which is to stay constant within a subgroup.  ' + ;
    'Normally this expression is the group expression plus an ' + ;
    'additional field.'

CASE nContext == H_FRMCOLADD

  cText := ;
    'Location: Util menu / Report modify~' + ;
    '|' + ;
    'Adds a new column to the report file.' + ;
    '|' + ;
    HelpPar(HP_FRMCOL)

CASE nContext == H_FRMCOLEDIT

  cText := ;
    'Location: Util menu / Report modify~' + ;
    '|' + ;
    'Modifies the highlighted column.' + ;
    '|' + ;
    HelpPar(HP_FRMCOL)

CASE nContext == H_FRMFORM

  cText := ;
    'Location: Util menu / Report modify~' + ;
    '|' + ;
    'Sets the report page width and lines per page from based on common ' + ;
    'format settings.' + ;
    '|' + ;
    'When you pick a set of format settings, the Set page width and ' + ;
    'Lines per page parameters are set according to the tables below.' + ;
    '|' + ;
    HelpPar(HP_CHARFIT) + ;
    '|' + ;
    'The format settings are defined as follows:' + ;
    '|' + ;
    'Pitch:  The number of characters (columns) per inch, horizontally.' + ;
    '|' + ;
    'Lines per inch:  The number of lines (rows) per inch, vertically.' + ;
    '|' + ;
    'Orientation:  Which way the paper is turned.  Portrait mode means ' + ;
    'means that the long edge of the paper runs up and down.  ' + ;
    'Landscape mode means that the long edge of the paper runs left and ' + ;
    'right.' + ;
    '|' + ;
    'Paper size:  The size of paper to use.  Letter size is 8.5" x 11", ' + ;
    'and legal size is 8.5" x 14".  If the size of paper you request ' + ;
    'does not match what is currently in the printer, the printer may ' + ;
    'request you to insert it.'

CASE nContext == H_FRMLIST

  cText := ;
    'Location: Util menu / Report modify~' + ;
    '|' + ;
    'Writes a description of the report file to an ASCII file.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

CASE nContext == H_LBLEDIT

  cText := ;
    'Location: Util menu~' + ;
    '|' + ;
    'Starts the label editor, which allows you to create or modify a ' + ;
    'label file (.LBL file).  You can use a label file to produce ' + ;
    'labels within or without DBE.  See the help topic "Label form".' + ;
    '|' + ;
    'A label file consists of specifications for label lines and ' + ;
    'additional specifications to control the width of the labels, the ' + ;
    'number of lines in each label, etc.' + ;
    '|' + ;
    'File: Enter the name of the label file, ' + HelpPar(HP_FILENAME) + ;
    'is .LBL.' + ;
    '|' + ;
    'After you enter the file name, the label editor screen comes up.  ' + ;
    'The cursor movement keys allow you to browse the list of line ' + ;
    'expressions.  In this screen, you can use the commands listed ' + ;
    'below.' + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'P - Parameters:  Modifies the label parameters listed in the ' + ;
    'Parameters section of the screen. (*)' + ;
    '|' + ;
    'R - Remark:  Modifies the remark inserted into the label file. (*)' + ;
    '|' + ;
    'E - Expressions:  Edits the label line expressions. (*)' + ;
    '|' + ;
    'F - Formats:  Sets the label parameters from a menu of common ' + ;
    'formats. (*)' + ;
    '|' + ;
    'L - List:  Writes a description of the label file to an ASCII ' + ;
    'file.' + ;
    '|' + ;
    'End - Save:  Writes the revised label file to disk and exits the ' + ;
    'label editor.  If the label file already exists, it is replaced ' + ;
    'with the new one.' + ;
    '|' + ;
    'Esc - Cancel:  Exits the label editor without saving changes to ' + ;
    'disk.'

CASE nContext == H_LBLPARAM

  cText := ;
    'Location: Util menu / Label modify~' + ;
    '|' + ;
    'Modifies the label parameters listed in the Parameters section of ' + ;
    'the screen.' + ;
    '|' + ;
    'Label width in characters:  The number of characters to print ' + ;
    'horizontally on each label.' + ;
    '|' + ;
    'Label height in lines:  The number of lines to print vertically on ' + ;
    'each label.' + ;
    '|' + ;
    'Number of labels across page:  The number of labels horizontally ' + ;
    'on the physical page.' + ;
    '|' + ;
    'Left margin:  The number of blank spaces to output at the ' + ;
    'beginning of each line.' + ;
    '|' + ;
    'Vert. lines between labels:  The number of blank lines to output ' + ;
    'vertically between adjacent labels.' + ;
    '|' + ;
    'Horiz. spaces between labels:  The number of blank spaces to ' + ;
    'output horizontally between adjacent labels.'

CASE nContext == H_LBLREMARK

  cText := ;
    'Location: Util menu / Label modify~' + ;
    '|' + ;
    'Modifies the remark inserted into the label file.  This remark ' + ;
    'does not affect the output in any way.'

CASE nContext == H_LBLLINE

  cText := ;
    'Location: Util menu / Label modify~' + ;
    '|' + ;
    'Edits the label line expressions.' + ;
    '|' + ;
    'Expressions:  Enter the expressions to output on each label line.  ' + ;
    'Put an expression on each line.  The number of line expressions in ' + ;
    'in this box is equal to the Label height in lines defined in the ' + ;
    'Parameters section.  Each expression willl be evaluated on each ' + ;
    'record and the value output into the corresponding label line.  ' + ;
    'An expression may be a field name or any other valid expression of ' + ;
    'type character, numeric, logical, or date.'

CASE nContext == H_LBLFORM

  cText := ;
    'Location: Util menu / Label modify~' + ;
    '|' + ;
    'Sets the label parameters from a menu of common formats.' + ;
    '|' + ;
    'When you pick a format from the list, the parameters are set as ' + ;
    'shown in the table below.' + ;
    '|' + ;
    '                                           Left    Blank  Blank~' + ;
    'Format              Width  Height  Across  margin  lines  spaces~' + ;
    '|' + ;
    '3.5 x 0.9375 x 1      35      5       1      0       1      0~' + ;
    '3.5 x 0.9375 x 2      35      5       2      0       1      2~' + ;
    '3.5 x 0.9375 x 3      35      5       3      0       1      2~' + ;
    '4   x 1.0625 x 1      40      8       1      0       1      0~' + ;
    '3.2 x 0.9167 x 3      32      5       3      0       1      2~'

CASE nContext == H_LBLLIST

  cText := ;
    'Location: Util menu / Label modify~' + ;
    '|' + ;
    'Writes a description of the label file to an ASCII file.' + ;
    '|' + ;
    'To:  Enter the name of the output file, ' + HelpPar(HP_FILENAME) + ;
    'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

END

RETURN cText  //  HelpUtil

//***************************************************************************

STATIC FUNCTION HelpHelp(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_HELPMENU

  cText := ;
    'Hot key: F1~' + ;
    '|' + ;
    HelpPar(HP_HOTMEN) + ;
    '|' + ;
    HelpPar(HP_ASTERISK) + ;
    '|' + ;
    'The Help menu is available anywhere in DBE by pressing F1 or by ' + ;
    'clicking the "Help" button at the top of the screen.' + ;
    '|' + ;
    'DBE help is context sensitive.  That means that when you use the ' + ;
    'help menu, you have the option of displaying help text that ' + ;
    'corresponds to the feature that you are currently using.' + ;
    '|' + ;
    'On current context:  Displays the help text for the feature that ' + ;
    'you are currently using.' + ;
    '|' + ;
    'Near current context:  Displays the list of help topics in subject ' + ;
    'order, with the highlight on the topic for the feature that you ' + ;
    'are currently using. (*)' + ;
    '|' + ;
    'Contents:  Displays the list of help topics in subject order, ' + ;
    'starting with the first subject, "Introduction to DBE". (*)' + ;
    '|' + ;
    'Index:  Displays the list of help topics in alphabetical order. (*)' + ;
    '|' + ;
    'Find:  Searches for a word or phrase in all the DBE help texts. (*)' + ;
    '|' + ;
    'Hot keys:  Displays the help text for the hot keys topic.' + ;
    '|' + ;
    'Using help:  Displays this message.' + ;
    '|' + ;
    'About DBE:  Displays the help text for the about topic.'

CASE nContext == H_CONTEXT

  cText := ;
    'Location: Help menu~' + ;
    '|' + ;
    'Displays the list of help topics in subject order, with the ' + ;
    'highlight on the topic for the feature that you are currently ' + ;
    'using.' + ;
    '|' + ;
    HelpPar(HP_HELPBROWSE)

CASE nContext == H_CONTENTS

  cText := ;
    'Location: Help menu~' + ;
    '|' + ;
    'Displays the list of help topics in subject order, starting with ' + ;
    'the first subject, "Introduction to DBE".' + ;
    '|' + ;
    HelpPar(HP_HELPBROWSE)

CASE nContext == H_INDEX

  cText := ;
    'Location: Help menu~' + ;
    '|' + ;
    'Displays the list of help topics in alphabetical order.' + ;
    '|' + ;
    HelpPar(HP_HELPBROWSE)

CASE nContext == H_FIND

  cText := ;
    'Location: Help menu~' + ;
    '|' + ;
    'Searches for a word or phrase in all the DBE help texts.' + ;
    '|' + ;
    'You enter text that you wish to find.  The help texts are then ' + ;
    'searched for that text.  Those topics that match are displayed in ' + ;
    'a list.' + ;
    '|' + ;
    'Word or phrase to find:  Enter the text you wish to find.' + ;
    '|' + ;
    'Topic order:  If you choose "Contents", the topic list will be ' + ;
    'displayed in subject order (contents order).  If you choose ' + ;
    '"Index", the topic list will be displayed in alphabetical order ' + ;
    '(index order).'

CASE nContext == H_HELPTEXT

  cText := ;
    'Location: Help menu / Any option~' + ;
    '|' + ;
    'When you are viewing help text, you can use the following ' + ;
    'options:' + ;
    '|' + ;
    'Alt-P - Previous:  Moves to the help topic preceding this one in ' + ;
    'the list.' + ;
    '|' + ;
    'Alt-N - Next:  Moves to the help topic following this one in the ' + ;
    'list.' + ;
    '|' + ;
    'Alt-W - Write:  Writes the help message to a file or printer.  See ' + ;
    'the help topic "Write help topics".'

CASE nContext == H_HELPWRITE

  cText := ;
    'Location: Help menu / Any option~' + ;
    '|' + ;
    'Writes the text of one help topic or the texts of a list of help ' + ;
    'topics to an ASCII file.' + ;
    '|' + ;
    'If you use this option when you are viewing the text of a help ' + ;
    'topic, it writes only the text you are viewing.  If you use this ' + ;
    'option from a list of help topics, it writes the texts of all the ' + ;
    'topics in the list to a single file.' + ;
    '|' + ;
    'File or printer name:  Enter the name of the output file, ' + ;
    HelpPar(HP_FILENAME) + 'is .TXT.  ' + HelpPar(HP_PRINTER) + ;
    '|' + ;
    HelpPar(HP_ADDITIVE) + ;
    '|' + ;
    HelpPar(HP_PRINTCODE)

END

RETURN cText  //  HelpHelp

//***************************************************************************

STATIC FUNCTION HelpAbout(nContext)

LOCAL nPad  := 0
LOCAL cText := ''

DO CASE
CASE nContext == H_ABOUT

  cText := ;
    '|' + ;
    'DBE~' + ;
    '|' + ;
    'Version ' + Sys()[SYS_VERSION] + ;
    '|' + ;
    'A general purpose database utility' + ;
    '|' + ;
    'Copyright (C) 1998 Kevin Carmody~' + ;
    'kcarmody@compuserve.com~' + ;
    '|' + ;
    'DBE comes with ABSOLUTELY NO WARRANTY; for details see paragraphs ' + ;
    '11 and 12 in the license.  To see the license, press Alt-N, or see ' + ;
    'Help/License.' + ;
    '|' + ;
    'This is free, open source software.  You are welcome to modify ' + ;
    'and/or redistribute it under certain conditions; see paragraphs 1 ' + ;
    'through 10 in the license.  To see the license, press Alt-N, or ' + ;
    'see Help/License.'

CASE nContext == H_LICENSE

  cText := ;
    '|' + ;
    'The following license governs the distribution of DBE, in both ' + ;
    'executable and source form.' + ;
    '|==|'
  cText += ;
    'GNU GENERAL PUBLIC LICENSE' + ;
    '|' + ;
    'TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION' + ;
    '|'
  cText += ;
    '  0. This License applies to any program or other work which ' + ;
    'contains a notice placed by the copyright holder saying it may be ' + ;
    'distributed under the terms of this General Public License.  The ' + ;
    '"Program", below, refers to any such program or work, and a "work ' + ;
    'based on the Program" means either the Program or any derivative ' + ;
    'work under copyright law:  that is to say, a work containing the ' + ;
    'Program or a portion of it, either verbatim or with modifications ' + ;
    'and/or translated into another language.  (Hereinafter, ' + ;
    'translation is included without limitation in the term ' + ;
    '"modification".)  Each licensee is addressed as "you".' + ;
    '|' + ;
    'Activities other than copying, distribution and modification are ' + ;
    'not covered by this License; they are outside its scope.  The act ' + ;
    'of running the Program is not restricted, and the output from the ' + ;
    'Program is covered only if its contents constitute a work based ' + ;
    'on the Program (independent of having been made by running the ' + ;
    'Program).  Whether that is true depends on what the Program does.' + ;
    '|'
  cText += ;
    '  1. You may copy and distribute verbatim copies of the Program' + ;
    "'" + 's source code as you receive it, in any medium, provided ' + ;
    'that you conspicuously and appropriately publish on each copy an ' + ;
    'appropriate copyright notice and disclaimer of warranty; keep ' + ;
    'intact all the notices that refer to this License and to the ' + ;
    'absence of any warranty; and give any other recipients of the ' + ;
    'Program a copy of this License along with the Program.' + ;
    '|' + ;
    'You may charge a fee for the physical act of transferring a copy, ' + ;
    'and you may at your option offer warranty protection in exchange ' + ;
    'for a fee.' + ;
    '|'
  cText += ;
    '  2. You may modify your copy or copies of the Program or any ' + ;
    'portion of it, thus forming a work based on the Program, and copy ' + ;
    'and distribute such modifications or work under the terms of ' + ;
    'Section 1 above, provided that you also meet all of these ' + ;
    'conditions:' + ;
    '|' + ;
    '  a) You must cause the modified files to carry prominent notices ' + ;
    'stating that you changed the files and the date of any change.' + ;
    '|' + ;
    '  b) You must cause any work that you distribute or publish, that ' + ;
    'in whole or in part contains or is derived from the Program or ' + ;
    'any part thereof, to be licensed as a whole at no charge to all ' + ;
    'third parties under the terms of this License.' + ;
    '|' + ;
    '  c) If the modified program normally reads commands interactively ' + ;
    'when run, you must cause it, when started running for such ' + ;
    'interactive use in the most ordinary way, to print or display an ' + ;
    'announcement including an appropriate copyright notice and a ' + ;
    'notice that there is no warranty (or else, saying that you ' + ;
    'provide a warranty) and that users may redistribute the program ' + ;
    'under these conditions, and telling the user how to view a copy ' + ;
    'of this License.  (Exception: if the Program itself is ' + ;
    'interactive but does not normally print such an announcement, ' + ;
    'your work based on the Program is not required to print an ' + ;
    'announcement.)' + ;
    '|' + ;
    'These requirements apply to the modified work as a whole.  If ' + ;
    'identifiable sections of that work are not derived from the ' + ;
    'Program, and can be reasonably considered independent and ' + ;
    'separate works in themselves, then this License, and its terms, ' + ;
    'do not apply to those sections when you distribute them as ' + ;
    'separate works.  But when you distribute the same sections as ' + ;
    'part of a whole which is a work based on the Program, the ' + ;
    'distribution of the whole must be on the terms of this License, ' + ;
    'whose permissions for other licensees extend to the entire whole, ' + ;
    'and thus to each and every part regardless of who wrote it.' + ;
    '|' + ;
    'Thus, it is not the intent of this section to claim rights or ' + ;
    'contest your rights to work written entirely by you; rather, the ' + ;
    'intent is to exercise the right to control the distribution of ' + ;
    'derivative or collective works based on the Program.' + ;
    '|' + ;
    'In addition, mere aggregation of another work not based on the ' + ;
    'Program with the Program (or with a work based on the Program) on ' + ;
    'a volume of a storage or distribution medium does not bring the ' + ;
    'other work under the scope of this License.' + ;
    '|'
  cText += ;
    '  3. You may copy and distribute the Program (or a work based on ' + ;
    'it, under Section 2) in object code or executable form under the ' + ;
    'terms of Sections 1 and 2 above provided that you also do one of ' + ;
    'the following:' + ;
    '|' + ;
    '  a) Accompany it with the complete corresponding machine-readable ' + ;
    'source code, which must be distributed under the terms of ' + ;
    'Sections 1 and 2 above on a medium customarily used for software ' + ;
    'interchange; or,' + ;
    '|' + ;
    '  b) Accompany it with a written offer, valid for at least three ' + ;
    'years, to give any third party, for a charge no more than your ' + ;
    'cost of physically performing source distribution, a complete ' + ;
    'machine-readable copy of the corresponding source code, to be ' + ;
    'distributed under the terms of Sections 1 and 2 above on a medium ' + ;
    'customarily used for software interchange; or,' + ;
    '|' + ;
    '  c) Accompany it with the information you received as to the ' + ;
    'offer to distribute corresponding source code.  (This alternative ' + ;
    'is allowed only for noncommercial distribution and only if you ' + ;
    'received the program in object code or executable form with such ' + ;
    'an offer, in accord with Subsection b above.)' + ;
    '|' + ;
    'The source code for a work means the preferred form of the work ' + ;
    'for making modifications to it.  For an executable work, complete ' + ;
    'source code means all the source code for all modules it ' + ;
    'contains, plus any associated interface definition files, plus ' + ;
    'the scripts used to control compilation and installation of the ' + ;
    'executable.  However, as a special exception, the source code ' + ;
    'distributed need not include anything that is normally ' + ;
    'distributed (in either source or binary form) with the major ' + ;
    'components (compiler, kernel, and so on) of the operating system ' + ;
    'on which the executable runs, unless that component itself ' + ;
    'accompanies the executable.' + ;
    '|' + ;
    'If distribution of executable or object code is made by offering ' + ;
    'access to copy from a designated place, then offering equivalent ' + ;
    'access to copy the source code from the same place counts as ' + ;
    'distribution of the source code, even though third parties are ' + ;
    'not compelled to copy the source along with the object code.' + ;
    '|'
  cText += ;
    '  4. You may not copy, modify, sublicense, or distribute the ' + ;
    'Program except as expressly provided under this License.  Any ' + ;
    'attempt otherwise to copy, modify, sublicense or distribute the ' + ;
    'Program is void, and will automatically terminate your rights ' + ;
    'under this License.  However, parties who have received copies, ' + ;
    'or rights, from you under this License will not have their ' + ;
    'licenses terminated so long as such parties remain in full ' + ;
    'compliance.' + ;
    '|'
  cText += ;
    '  5. You are not required to accept this License, since you have ' + ;
    'not signed it.  However, nothing else grants you permission to ' + ;
    'modify or distribute the Program or its derivative works.  These ' + ;
    'actions are prohibited by law if you do not accept this License.  ' + ;
    'Therefore, by modifying or distributing the Program (or any work ' + ;
    'based on the Program), you indicate your acceptance of this ' + ;
    'License to do so, and all its terms and conditions for copying, ' + ;
    'distributing or modifying the Program or works based on it.' + ;
    '|'
  cText += ;
    '  6. Each time you redistribute the Program (or any work based on ' + ;
    'the Program), the recipient automatically receives a license from ' + ;
    'the original licensor to copy, distribute or modify the Program ' + ;
    'subject to these terms and conditions.  You may not impose any ' + ;
    'further restrictions on the recipients' + "'" + ' exercise of the ' + ;
    'rights granted herein.  You are not responsible for enforcing ' + ;
    'compliance by third parties to this License.' + ;
    '|'
  cText += ;
    '  7. If, as a consequence of a court judgment or allegation of ' + ;
    'patent infringement or for any other reason (not limited to ' + ;
    'patent issues), conditions are imposed on you (whether by court ' + ;
    'order, agreement or otherwise) that contradict the conditions of ' + ;
    'this License, they do not excuse you from the conditions of this ' + ;
    'License.  If you cannot distribute so as to satisfy ' + ;
    'simultaneously your obligations under this License and any other ' + ;
    'pertinent obligations, then as a consequence you may not ' + ;
    'distribute the Program at all.  For example, if a patent license ' + ;
    'would not permit royalty-free redistribution of the Program by ' + ;
    'all those who receive copies directly or indirectly through you, ' + ;
    'then the only way you could satisfy both it and this License ' + ;
    'would be to refrain entirely from distribution of the Program.' + ;
    '|' + ;
    'If any portion of this section is held invalid or unenforceable ' + ;
    'under any particular circumstance, the balance of the section is ' + ;
    'intended to apply and the section as a whole is intended to apply ' + ;
    'in other circumstances.' + ;
    '|' + ;
    'It is not the purpose of this section to induce you to infringe ' + ;
    'any patents or other property right claims or to contest validity ' + ;
    'of any such claims; this section has the sole purpose of ' + ;
    'protecting the integrity of the free software distribution ' + ;
    'system, which is implemented by public license practices.  Many ' + ;
    'people have made generous contributions to the wide range of ' + ;
    'software distributed through that system in reliance on ' + ;
    'consistent application of that system; it is up to the ' + ;
    'author/donor to decide if he or she is willing to distribute ' + ;
    'software through any other system and a licensee cannot impose ' + ;
    'that choice.' + ;
    '|' + ;
    'This section is intended to make thoroughly clear what is ' + ;
    'believed to be a consequence of the rest of this License.' + ;
    '|'
  cText += ;
    '  8. If the distribution and/or use of the Program is restricted ' + ;
    'in certain countries either by patents or by copyrighted ' + ;
    'interfaces, the original copyright holder who places the Program ' + ;
    'under this License may add an explicit geographical distribution ' + ;
    'limitation excluding those countries, so that distribution is ' + ;
    'permitted only in or among countries not thus excluded.  In such ' + ;
    'case, this License incorporates the limitation as if written in ' + ;
    'the body of this License.' + ;
    '|'
  cText += ;
    '  9. The Free Software Foundation may publish revised and/or new ' + ;
    'versions of the General Public License from time to time.  Such ' + ;
    'new versions will be similar in spirit to the present version, ' + ;
    'but may differ in detail to address new problems or concerns.' + ;
    '|' + ;
    'Each version is given a distinguishing version number.  If the ' + ;
    'Program specifies a version number of this License which applies ' + ;
    'to it and "any later version", you have the option of following ' + ;
    'the terms and conditions either of that version or of any later ' + ;
    'version published by the Free Software Foundation.  If the ' + ;
    'Program does not specify a version number of this License, you ' + ;
    'may choose any version ever published by the Free Software ' + ;
    'Foundation.' + ;
    '|'
  cText += ;
    '  10. If you wish to incorporate parts of the Program into other ' + ;
    'free programs whose distribution conditions are different, write ' + ;
    'to the author to ask for permission.  For software which is ' + ;
    'copyrighted by the Free Software Foundation, write to the Free ' + ;
    'Software Foundation; we sometimes make exceptions for this.  Our ' + ;
    'decision will be guided by the two goals of preserving the free ' + ;
    'status of all derivatives of our free software and of promoting ' + ;
    'the sharing and reuse of software generally.' + ;
    '|'
  cText += ;
    'NO WARRANTY' + ;
    '|' + ;
    '  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO ' + ;
    'WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE ' + ;
    'LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT ' + ;
    'HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT ' + ;
    'WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT ' + ;
    'NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND ' + ;
    'FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE ' + ;
    'QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE ' + ;
    'PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY ' + ;
    'SERVICING, REPAIR OR CORRECTION.' + ;
    '|'
  cText += ;
    '  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO ' + ;
    'IN WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY ' + ;
    'MODIFY AND/OR REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE ' + ;
    'LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, ' + ;
    'INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR ' + ;
    'INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS ' + ;
    'OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY ' + ;
    'YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ' + ;
    'ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ' + ;
    'ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.' + ;
    '|'
  cText += ;
    'END OF TERMS AND CONDITIONS'

END

RETURN cText  //  HelpAbout

//***************************************************************************