/*

    MOUSCLOC.PRG

    MouseClock Library

    Mouse and clock routines for Clipper programs.

    Version 3.1

    Kevin Carmody, NSI

*/

//---------------------------------------------------------------------------

/*

Procedure summary

FUNCTION  McVersion()

FUNCTION  McInKey(nSec)
PROCEDURE McClock()
FUNCTION  McLastKey(nSetKey)

FUNCTION  McKeyRecord(aSetKeyRecord)

FUNCTION  McSet(nItem, xNewSet)
FUNCTION  McSetArea(n1, n2, n3, n4, xSet)
FUNCTION  McTransBlock(nKey, aSetKey, aSetArea)
FUNCTION  McTransPad(aKeyPad, nKey)
PROCEDURE McSetPad(aKeyPad)
PROCEDURE McClearPad(aKeyPad)

PROCEDURE MSetPos(nMRow, nMCol)
FUNCTION  MRow(nSetRow)
FUNCTION  MCol(nSetCol)

FUNCTION  McInBox(nRow, nCol, nTop, nLeft, nBottom, nRight)
FUNCTION  McMouseIn(nKey, nTop, nLeft, nBottom, nRight)
FUNCTION  McMouseOut(nKey, nTop, nLeft, nBottom, nRight)

Old style
FUNCTION  McSetBox(lBoxDefined, cBoxScreen, ;
  nBoxTop, nBoxLeft, nBoxBottom, nBoxRight, ;
  lBoxClose, lBoxResize, lVertScroll, lHorizScroll)
New style
FUNCTION  McSetBox(aSetBox)
FUNCTION  McRestBox(aSetBox)
Old style
FUNCTION  McBox(nBoxTop, nBoxLeft, nBoxBottom, nBoxRight, ;
  cBoxTitle, lCenter, cBoxColor, xLineType, cControls, ;
  lBoxClose, lBoxResize, lVertScroll, lHorizScroll)
New style
FUNCTION  McBox(aSetBox, cBoxTitle, lCenter, cBoxColor, xLineType, lSave, ;
  cControls)
PROCEDURE McScrollBox(nVertPart, nVertTot, nHorizPart, nHorizTot, ;
  xLineType, cControls)
FUNCTION  McBoxPad(aSetBox, oBrowse)

FUNCTION  McTBApplyKey(oBrowse, nKey, aSetKey, aSetArea)
PROCEDURE McTBCross(oBrowse, nRowIndex, nColIndex, nLowIndex, nCellIndex, ;
  lHiLite)

FUNCTION  McAChoice(nTop, nLeft, nBottom, nRight, ;
  aItem, xSelect, cFunction, nDefault, nTopItem, lConfirm, lRehilite, ;
  aSetKey, aSetArea)

FUNCTION  McHAWidth(aItem, nSeparate, nBorder)
FUNCTION  McHAPad(nRow, nCol, aItem, nSeparate, nBorder, lCenter)
FUNCTION  McHAChoice(nRow, nCol, aItem, xSelect, nDefault, ;
  nSeparate, nBorder, lCenter, lConfirm, lRehilite, lVertExit)
FUNCTION  McAlert(cMessage, aChoices, cColor, nDefault)

FUNCTION  McMemoEdit(cString, nTop, nLeft, nBottom, nRight, ;
  lMode, lEnter, nLength, nTab, nBufRow, nBufCol, nWinRow, nWinCol, ;
  aSetKey, aSetArea)
FUNCTION  McMeApplyKey(nKey, oBrowse, oGet, aLine, nLine, aMemoSys, ;
  aSetKey, aSetArea)
PROCEDURE McMeMove(oBrowse, oGet, aLine, nLine, aMemoSys)
PROCEDURE McMeBreak(cString, aLine, aMemoSys)
FUNCTION  McMeJoin(aLine, nStart)
PROCEDURE McMeDoSetKey(bKey, oGet, oBrowse, aLine, nLine, aMemoSys, lEdit)
FUNCTION  McMemoArray(cString, nWidth, nTab)

FUNCTION  McReadModal(aGetList, nPos)
PROCEDURE McGetReader(oGet, aGetList, nPos, aGetSys)
PROCEDURE McGetApplyKey(oGet, nKey, aGetList, nPos, aGetSys)
FUNCTION  McGetPreValidate(oGet, aGetSys)
FUNCTION  McGetPostValidate(oGet, aGetSys)
PROCEDURE McGetDoSetKey(bKey, oGet, aGetSys)
FUNCTION  McSettle(aGetList, nPos, aGetSys)
FUNCTION  McGetReadVar(oGet)
FUNCTION  MReadExit(lSetExit)
PROCEDURE McShowScoreboard()
FUNCTION  McUpdated(lSetUpdated)
FUNCTION  McReadKill(lSetReadKill)

FUNCTION  McSaveItem(xSetExpr, lClear)

*/

//---------------------------------------------------------------------------

#INCLUDE 'KEYS.CH'
#INCLUDE 'CHARS.CH'
#INCLUDE 'GETEXIT.CH'
#INCLUDE 'SET.CH'
#INCLUDE 'SETCURS.CH'

#DEFINE  K_CLICK         -50
#DEFINE  K_DOUBLECLICK   -51

#DEFINE  MCSET_DSHOW       1
#DEFINE  MCSET_DROW        2
#DEFINE  MCSET_DCOL        3
#DEFINE  MCSET_DCOLOR      4
#DEFINE  MCSET_DFORM       5
#DEFINE  MCSET_TSHOW       6
#DEFINE  MCSET_TROW        7
#DEFINE  MCSET_TCOL        8
#DEFINE  MCSET_TCOLOR      9
#DEFINE  MCSET_TFORM      10
#DEFINE  MCSET_MACTIVE    11
#DEFINE  MCSET_MCURSOR    12
#DEFINE  MCSET_MSPEED     13
#DEFINE  MCSET_EVERCHECK  14
#DEFINE  MCSET_RCURSOR    15
#DEFINE  MCSET_MECURSOR   16
#DEFINE  MCSET_MEREAD     17
#DEFINE  MCSET_SAVER      18
#DEFINE  MCSET_SAVEDELAY  19
#DEFINE  MCSET_KEYRECORD  20

#DEFINE  TA_UNKNOWN        0
#DEFINE  TA_STD            1
#DEFINE  TA_DOUBLECLICK    2
#DEFINE  TA_OUTBOX         3
#DEFINE  TA_RESIZE         4
#DEFINE  TA_CLOSE          5

#DEFINE  SCORE_ROW         0
#DEFINE  SCORE_COL        60

#DEFINE  KPV_TOP           1
#DEFINE  KPV_LEFT          2
#DEFINE  KPV_BOTTOM        3
#DEFINE  KPV_RIGHT         4
#DEFINE  KPV_KEY           5
#DEFINE  KPV_SELECT        6
#DEFINE  KPV_COUNT         6

#DEFINE  MSV_TOP           1
#DEFINE  MSV_LEFT          2
#DEFINE  MSV_BOTTOM        3
#DEFINE  MSV_RIGHT         4
#DEFINE  MSV_MODE          5
#DEFINE  MSV_ENTER         6
#DEFINE  MSV_WIDTH         7
#DEFINE  MSV_TAB           8
#DEFINE  MSV_BUFROW        9
#DEFINE  MSV_BUFCOL       10
#DEFINE  MSV_WINROW       11
#DEFINE  MSV_WINCOL       12
#DEFINE  MSV_GETPOS       13
#DEFINE  MSV_SETPOS       14
#DEFINE  MSV_TYPEOUT      15
#DEFINE  MSV_REFORM       16
#DEFINE  MSV_INSERT       17
#DEFINE  MSV_BOXTOP       18
#DEFINE  MSV_BOXLEFT      19
#DEFINE  MSV_BOXBOTTOM    20
#DEFINE  MSV_BOXRIGHT     21
#DEFINE  MSV_ROWMID       22
#DEFINE  MSV_COLMID       23
#DEFINE  MSV_CLOSE        24
#DEFINE  MSV_RESIZE       25
#DEFINE  MSV_VSCROLL      26
#DEFINE  MSV_HSCROLL      27
#DEFINE  MSV_COUNT        27

#DEFINE  GSV_UPDATED       1
#DEFINE  GSV_LENGTH        2
#DEFINE  GSV_BOXTOP        3
#DEFINE  GSV_BOXLEFT       4
#DEFINE  GSV_BOXBOTTOM     5
#DEFINE  GSV_BOXRIGHT      6
#DEFINE  GSV_BOXCLOSE      7
#DEFINE  GSV_BOXDEFINED    8
#DEFINE  GSV_BUMPTOP       9
#DEFINE  GSV_BUMPBOT      10
#DEFINE  GSV_LASTEXIT     11
#DEFINE  GSV_LASTPOS      12
#DEFINE  GSV_ACTIVEGET    13
#DEFINE  GSV_READVAR      14
#DEFINE  GSV_READPROCNAME 15
#DEFINE  GSV_READPROCLINE 16
#DEFINE  GSV_COUNT        16

#DEFINE  SB_TOP            1
#DEFINE  SB_LEFT           2
#DEFINE  SB_BOTTOM         3
#DEFINE  SB_RIGHT          4
#DEFINE  SB_VERTSCROLL     5
#DEFINE  SB_HORIZSCROLL    6
#DEFINE  SB_CLOSE          7
#DEFINE  SB_RESIZE         8
#DEFINE  SB_DEFINED        9
#DEFINE  SB_SCREEN        10
#DEFINE  SB_VERTMOVE      11
#DEFINE  SB_HORIZMOVE     12
#DEFINE  SB_VSCROLLPOS    13
#DEFINE  SB_VHITTOP       14
#DEFINE  SB_VHITBOTTOM    15
#DEFINE  SB_HSCROLLPOS    16
#DEFINE  SB_HHITLEFT      17
#DEFINE  SB_HHITRIGHT     18
#DEFINE  SB_COUNT         18

#DEFINE  KR_OFF            0
#DEFINE  KR_RECORD         1
#DEFINE  KR_PLAY           2

#COMMAND DEFAULT <var> TO <val> => ;
  IF <var> == NIL; ;
    <var> := <val>; ;
  END

#COMMAND CONDSET <var> TO <val> => ;
  IF <val> # NIL; ;
    <var> := <val>; ;
  END

//---------------------------------------------------------------------------

FUNCTION McVersion()

LOCAL cVersion := 'MouseClock Library version 3.1'

RETURN cVersion  //  McVersion

//---------------------------------------------------------------------------

FUNCTION McInKey(nSec)

STATIC nMRow := 0
STATIC nMCol := 0

LOCAL nKey          := 0
LOCAL lWait         := .Y.
LOCAL lMouseActive  := McSet(MCSET_MACTIVE)
LOCAL lMouseCursor  := McSet(MCSET_MCURSOR)
LOCAL nSecStart     := SECONDS()
LOCAL nSecKey       := nSecStart - 1
LOCAL nSecClock     := nSecStart
LOCAL nSecDouble    := 0
LOCAL nSecDelay     := nSecStart
LOCAL nLeft         := 0
LOCAL nRight        := 0
LOCAL nMid          := 0
LOCAL nLeftWait     := 1
LOCAL nRightWait    := 1
LOCAL nMidWait      := 1
LOCAL bEverCheck    := McSet(MCSET_EVERCHECK)
LOCAL lEverCheck    := !EMPTY(bEverCheck)
LOCAL bSaver        := McSet(MCSET_SAVER)
LOCAL nSaveDelay    := McSet(MCSET_SAVEDELAY)
LOCAL lSaver        := !EMPTY(bSaver) .AND. !EMPTY(nSaveDelay)
LOCAL nKeyRecord    := McSet(MCSET_KEYRECORD)
LOCAL lRecord       := (nKeyRecord == KR_RECORD)
LOCAL lPlay         := (nKeyRecord == KR_PLAY)
LOCAL aKeyRecord    := {}

IF lEverCheck
  bEverCheck:EVAL()
END
IF lMouseActive
  MousePut(nMRow, nMCol)
  IF lMouseCursor
    MouseShow()
  ELSE
    MouseHide()
  END
END

IF lPlay
  aKeyRecord := ACLONE(McKeyRecord())
  IF EMPTY(aKeyRecord)
    McSet(MCSET_KEYRECORD, KR_OFF)
  ELSE
    nKey := aKeyRecord[1]
    IF nKey == K_CLICK .OR. nKey == K_DOUBLECLICK .AND. LEN(aKeyRecord) >= 3
      nMRow := aKeyRecord[2]
      nMCol := aKeyRecord[3]
      MSetPos(nMRow, nMCol)
      ADEL(aKeyRecord, 1)
      ADEL(aKeyRecord, 2)
      ADEL(aKeyRecord, 3)
      ASIZE(aKeyRecord, LEN(aKeyRecord) - 3)
    ELSE
      ADEL(aKeyRecord, 1)
      ASIZE(aKeyRecord, LEN(aKeyRecord) - 1)
    END
    McKeyRecord(aKeyRecord)
    lWait := .N.
  END
END

WHILE lWait
  IF INT(nSecKey) # INT(nSecClock)
    McClock()
    nSecClock := SECONDS()
    IF lEverCheck
      bEverCheck:EVAL()
    END
  END
  IF lMouseActive
    MouseState(@nLeft, @nRight, @nMid, @nMRow, @nMCol)
  END
  IF !EMPTY(nLeft) .OR. !EMPTY(nRight) .OR. !EMPTY(nMid)
    WHILE !EMPTY(nLeftWait) .OR. !EMPTY(nRightWait) .OR. !EMPTY(nMidWait)
      MouseState(@nLeftWait, @nRightWait, @nMidWait)
    END
  END
  DO CASE
  CASE !EMPTY(nLeft)
    nKey       := K_CLICK
    MSetPos(nMRow, nMCol)
    nSecDouble := SECONDS() + McSet(MCSET_MSPEED)
    nLeft      := 0
    nLeftWait  := 1
    WHILE SECONDS() < nSecDouble .AND. nLeft # 1
      MouseState(@nLeft)
      IF !EMPTY(nLeft)
        WHILE !EMPTY(nLeftWait)
          MouseState(@nLeftWait)
        END
        nKey := K_DOUBLECLICK
      END
    END
  CASE !EMPTY(nRight)
    nKey := K_ESC
  CASE !EMPTY(nMid)
    nKey := K_ENTER
  OTHERWISE
    nKey := INKEY()
  END
  DO CASE
  CASE nSec == NIL
    lWait := .N.
  CASE nSec == 0
    lWait := EMPTY(nKey)
  OTHERWISE
    lWait := (EMPTY(nKey) .AND. nSecStart + nSec > nSecKey)
  END
  IF lSaver .AND. nSecKey - nSecDelay > nSaveDelay
    bSaver:EVAL()
    nSecKey   := SECONDS()
    nSecDelay := nSecKey
  ELSE
    nSecKey   := SECONDS()
  END
END

McLastKey(nKey)
IF lRecord
  AADD(McKeyRecord(), nKey)
END
IF lMouseActive
  MouseHide()
END

RETURN nKey  //  McInKey

//---------------------------------------------------------------------------

PROCEDURE McClock()

LOCAL dDate        := DATE()
LOCAL cTime        := IF('9' $ McSet(MCSET_TFORM), TIME(), AMPM(TIME()))
LOCAL nOldRow      := ROW()
LOCAL nOldCol      := COL()
LOCAL cOldDevice   := SET(_SET_DEVICE, 'SCREEN')
LOCAL nOldCursor   := SETCURSOR(0)
LOCAL cOldDateForm := SET(_SET_DATEFORMAT, McSet(MCSET_DFORM))

IF McSet(MCSET_DSHOW)
  @ McSet(MCSET_DROW), McSet(MCSET_DCOL) ;
    SAY LEFT(CDOW(dDate),3) + ' ' + DTOC(dDate) ;
    COLOR McSet(MCSET_DCOLOR)
END
IF McSet(MCSET_TSHOW)
  @ McSet(MCSET_TROW), McSet(MCSET_TCOL) ;
    SAY STR(VAL(LEFT(cTime,2)),2) + SUBSTR(cTime,3) ;
    COLOR McSet(MCSET_TCOLOR) PICTURE McSet(MCSET_TFORM)
END

SET(_SET_DATEFORMAT, cOldDateForm)
SET(_SET_DEVICE, cOldDevice)
SETPOS(nOldRow, nOldCol)
SETCURSOR(nOldCursor)

RETURN  //  McClock

//---------------------------------------------------------------------------

FUNCTION McLastKey(nSetKey)

STATIC nCurKey := 0

LOCAL  nOldKey := nCurKey

IF nSetKey # NIL
  nCurKey := nSetKey
END

RETURN nOldKey  //  McLastKey

//---------------------------------------------------------------------------

FUNCTION McKeyRecord(aSetKeyRecord)

STATIC aKeyRecord := {}

IF aSetKeyRecord # NIL
  aKeyRecord := aSetKeyRecord
END

RETURN aKeyRecord  //  McKeyRecord

//---------------------------------------------------------------------------

FUNCTION McSet(nItem, xNewSet)

STATIC lDate         := .N.
STATIC nDateRow      := 0
STATIC nDateCol      := 0
STATIC cDateColor    := ''
STATIC cDateForm     := ''
STATIC lTime         := .N.
STATIC nTimeRow      := 0
STATIC nTimeCol      := 69
STATIC cTimeColor    := ''
STATIC cTimeForm     := '##:##:## XX'
STATIC lMouseActive  := .Y.
STATIC lMouseCursor  := .Y.
STATIC cMouseSpeed   := 0.2
STATIC bEverCheck    := NIL
STATIC bRCursor      := NIL
STATIC bMeCursor     := NIL
STATIC bMeRead       := NIL
STATIC bSaver        := NIL
STATIC nSaveDelay    := 300
STATIC nKeyRecord    := KR_OFF

LOCAL xOldSet := NIL

IF EMPTY(cDateColor)
  cDateColor := SETCOLOR()
END
IF EMPTY(cDateForm)
  cDateForm := SET(_SET_DATEFORMAT)
END
IF EMPTY(cTimeColor)
  cTimeColor := SETCOLOR()
END

DO CASE
CASE nItem == MCSET_DSHOW
  xOldSet := lDate
CASE nItem == MCSET_DROW
  xOldSet := nDateRow
CASE nItem == MCSET_DCOL
  xOldSet := nDateCol
CASE nItem == MCSET_DCOLOR
  xOldSet := cDateColor
CASE nItem == MCSET_DFORM
  xOldSet := cDateForm
CASE nItem == MCSET_TSHOW
  xOldSet := lTime
CASE nItem == MCSET_TROW
  xOldSet := nTimeRow
CASE nItem == MCSET_TCOL
  xOldSet := nTimeCol
CASE nItem == MCSET_TCOLOR
  xOldSet := cTimeColor
CASE nItem == MCSET_TFORM
  xOldSet := cTimeForm
CASE nItem == MCSET_MACTIVE
  xOldSet := lMouseActive
CASE nItem == MCSET_MCURSOR
  xOldSet := lMouseCursor
CASE nItem == MCSET_MSPEED
  xOldSet := cMouseSpeed
CASE nItem == MCSET_EVERCHECK
  xOldSet := bEverCheck
CASE nItem == MCSET_RCURSOR
  xOldSet := bRCursor
CASE nItem == MCSET_MECURSOR
  xOldSet := bMeCursor
CASE nItem == MCSET_MEREAD
  xOldSet := bMeRead
CASE nItem == MCSET_SAVER
  xOldSet := bSaver
CASE nItem == MCSET_SAVEDELAY
  xOldSet := nSaveDelay
CASE nItem == MCSET_KEYRECORD
  xOldSet := nKeyRecord
END

IF PCOUNT() > 1
  DO CASE
  CASE nItem == MCSET_DSHOW
    lDate := xNewSet
  CASE nItem == MCSET_DROW
    nDateRow := xNewSet
  CASE nItem == MCSET_DCOL
    nDateCol := xNewSet
  CASE nItem == MCSET_DCOLOR
    cDateColor := xNewSet
  CASE nItem == MCSET_DFORM
    cDateForm := xNewSet
  CASE nItem == MCSET_TSHOW
    lTime := xNewSet
  CASE nItem == MCSET_TROW
    nTimeRow := xNewSet
  CASE nItem == MCSET_TCOL
    nTimeCol := xNewSet
  CASE nItem == MCSET_TCOLOR
    cTimeColor := xNewSet
  CASE nItem == MCSET_TFORM
    cTimeForm := xNewSet
  CASE nItem == MCSET_MACTIVE
    lMouseActive := xNewSet
  CASE nItem == MCSET_MCURSOR
    lMouseCursor := xNewSet
  CASE nItem == MCSET_MSPEED
    cMouseSpeed := xNewSet
  CASE nItem == MCSET_EVERCHECK
    bEverCheck := xNewSet
  CASE nItem == MCSET_RCURSOR
    bRCursor := xNewSet
  CASE nItem == MCSET_MECURSOR
    bMeCursor := xNewSet
  CASE nItem == MCSET_MEREAD
    bMeRead := xNewSet
  CASE nItem == MCSET_SAVER
    bSaver := xNewSet
  CASE nItem == MCSET_SAVEDELAY
    nSaveDelay := xNewSet
  CASE nItem == MCSET_KEYRECORD
    nKeyRecord := xNewSet
  END
END

RETURN xOldSet  //  McSet

//---------------------------------------------------------------------------

FUNCTION McSetArea(n1, n2, n3, n4, xSet)

STATIC aArea := {}

LOCAL xOld  := NIL
LOCAL nArea := 0

DO CASE
CASE PCOUNT() == 5
  nArea := ASCAN(aArea, {|aRow| aRow[1] == n1 .AND. aRow[2] == n2 .AND. ;
    aRow[3] == n3 .AND. aRow[4] == n4})
  DO CASE
  CASE EMPTY(nArea) .AND. !EMPTY(xSet)
    ASIZE(aArea, LEN(aArea) + 1)
    AINS(aArea, 1)
    aArea[1] := {n1, n2, n3, n4, xSet}
  CASE !EMPTY(nArea) .AND. !EMPTY(xSet)
    xOld := aArea[nArea][5]
    ADEL(aArea, nArea)
    AINS(aArea, 1)
    aArea[1] := {n1, n2, n3, n4, xSet}
  CASE !EMPTY(nArea) .AND. EMPTY(xSet)
    xOld := aArea[nArea][5]
    ADEL(aArea, nArea)
    ASIZE(aArea, LEN(aArea) - 1)
  END
CASE n2 == NIL
  nArea := ASCAN(aArea, {|aRow| McMouseIn(n1, aRow[1], aRow[2], ;
    aRow[3], aRow[4])})
  IF !EMPTY(nArea)
    xOld := aArea[nArea][5]
    n1   := aArea[nArea][1]
    n2   := aArea[nArea][2]
    n3   := aArea[nArea][3]
    n4   := aArea[nArea][4]
  END
CASE n3 == NIL
  nArea := ASCAN(aArea, {|aRow| McInBox(n1, n2, aRow[1], aRow[2], ;
    aRow[3], aRow[4])})
  IF !EMPTY(nArea)
    xOld := aArea[nArea][5]
    n1   := aArea[nArea][1]
    n2   := aArea[nArea][2]
    n3   := aArea[nArea][3]
    n4   := aArea[nArea][4]
  END
OTHERWISE
  nArea := ASCAN(aArea, {|aRow| aRow[1] == n1 .AND. aRow[2] == n2 .AND. ;
    aRow[3] == n3 .AND. aRow[4] == n4})
  IF !EMPTY(nArea)
    xOld := aArea[nArea][5]
  END
END

RETURN xOld  //  McSetArea

//---------------------------------------------------------------------------

FUNCTION McTransBlock(nKey, aSetKey, aSetArea)

LOCAL nRow := 0
LOCAL bKey := NIL

IF !(nKey == K_CLICK .OR. nKey == K_DOUBLECLICK)
  IF EMPTY(aSetKey)
    bKey := SETKEY(nKey)
  ELSE
    nRow := ASCAN(aSetKey, {|aRow| aRow[1] == nKey})
    IF !EMPTY(nRow)
      bKey := aSetKey[nRow][2]
    END
  END
ELSE
  IF EMPTY(aSetArea)
    bKey := McSetArea(nKey)
  ELSE
    nRow := ASCAN(aSetArea, ;
      {|aRow| McMouseIn(nKey, aRow[1], aRow[2], aRow[3], aRow[4])})
    IF !EMPTY(nRow)
      bKey := aSetArea[nRow][5]
    END
  END
END

RETURN bKey  //  McTransBlock

//---------------------------------------------------------------------------

FUNCTION McTransPad(aKeyPad, nKey)

LOCAL nMRow  := MRow()
LOCAL nMCol  := MCol()
LOCAL nScan  := 0
LOCAL cType  := 'N'
LOCAL xMatch := NIL
LOCAL nMatch := nKey

IF nKey == K_CLICK .OR. nKey == K_DOUBLECLICK
  nScan := ASCAN(aKeyPad, {|aPad| aPad[KPV_SELECT] .AND. ;
    McInBox(nMRow, nMCol, ;
    aPad[KPV_TOP], aPad[KPV_LEFT], aPad[KPV_BOTTOM], aPad[KPV_RIGHT])})
END
IF !EMPTY(nScan)
  xMatch := aKeyPad[nScan][KPV_KEY]
  cType  := VALTYPE(xMatch)
  DO CASE
  CASE cType == 'N'
    nMatch := xMatch
  CASE cType == 'C'
    nMatch := ASC(xMatch)
  END
  McLastKey(nMatch)
END

RETURN nMatch  //  McTransPad

//---------------------------------------------------------------------------

PROCEDURE McSetPad(aKeyPad)

LOCAL bPad := NIL

AEVAL(aKeyPad, {|aPad| bPad := SETKEY(aPad[5]), IF(EMPTY(bPad), NIL, ;
  McSetArea(;
  aPad[KPV_TOP], aPad[KPV_LEFT], aPad[KPV_BOTTOM], aPad[KPV_RIGHT], bPad))})

RETURN  //  McSetPad

//---------------------------------------------------------------------------

PROCEDURE McClearPad(aKeyPad)

AEVAL(aKeyPad, {|aPad| McSetArea(;
  aPad[KPV_TOP], aPad[KPV_LEFT], aPad[KPV_BOTTOM], aPad[KPV_RIGHT])})

RETURN  //  McClearPad

//---------------------------------------------------------------------------

PROCEDURE MSetPos(nMRow, nMCol)

MRow(nMRow)
MCol(nMCol)
MousePut(nMRow, nMCol)

RETURN  //  MSetPos

//---------------------------------------------------------------------------

FUNCTION MRow(nSetRow)

STATIC nCurRow := 0

LOCAL  nOldRow := nCurRow

IF nSetRow # NIL
  nCurRow := nSetRow
END

RETURN nOldRow  //  MRow

//---------------------------------------------------------------------------

FUNCTION MCol(nSetCol)

STATIC nCurCol := 0

LOCAL  nOldCol := nCurCol

IF nSetCol # NIL
  nCurCol := nSetCol
END

RETURN nOldCol  //  MCol

//---------------------------------------------------------------------------

FUNCTION McInBox(nRow, nCol, nTop, nLeft, nBottom, nRight)

LOCAL lRLand  := (nTop  <= nRow .AND. nRow <= nBottom)
LOCAL lCLand  := (nLeft <= nCol .AND. nCol <= nRight)
LOCAL lLand   := lRLand .AND. lCLand

RETURN lLand  //  McInBox

//---------------------------------------------------------------------------

FUNCTION McMouseIn(nKey, nTop, nLeft, nBottom, nRight)

LOCAL lMouse  := (nKey == K_CLICK .OR. nKey == K_DOUBLECLICK)
LOCAL nRow    := MRow()
LOCAL nCol    := MCol()
LOCAL lRLand  := (nTop  <= nRow .AND. nRow <= nBottom)
LOCAL lCLand  := (nLeft <= nCol .AND. nCol <= nRight)
LOCAL lLand   := lMouse .AND. lRLand .AND. lCLand

RETURN lLand  //  McMouseIn

//---------------------------------------------------------------------------

FUNCTION McMouseOut(nKey, nTop, nLeft, nBottom, nRight)

LOCAL lMouse  := (nKey == K_CLICK .OR. nKey == K_DOUBLECLICK)
LOCAL nRow    := MRow()
LOCAL nCol    := MCol()
LOCAL lRLand  := (nTop  <= nRow .AND. nRow <= nBottom)
LOCAL lCLand  := (nLeft <= nCol .AND. nCol <= nRight)
LOCAL lLand   := lMouse .AND. !(lRLand .AND. lCLand)

RETURN lLand  //  McMouseOut

//---------------------------------------------------------------------------

FUNCTION McSetBox(xSetBox, cBoxScreen, ;
  nBoxTop, nBoxLeft, nBoxBottom, nBoxRight, ;
  lBoxClose, lBoxResize, lVertScroll, lHorizScroll)

STATIC aCurSetBox := { ;
  0  , ; // SB_TOP
  0  , ; // SB_LEFT
  0  , ; // SB_BOTTOM
  0  , ; // SB_RIGHT
  .N., ; // SB_VERTSCROLL
  .N., ; // SB_HORIZSCROLL
  .Y., ; // SB_CLOSE
  .N., ; // SB_RESIZE
  .N., ; // SB_DEFINED
  '' , ; // SB_SCREEN
  .N., ; // SB_VERTMOVE
  .N., ; // SB_HORIZMOVE
  0  , ; // SB_VSCROLLPOS
  .N., ; // SB_VHITTOP
  .N., ; // SB_VHITBOTTOM
  0  , ; // SB_HSCROLLPOS
  .N., ; // SB_HHITLEFT
  .N.  } // SB_HHITRIGHT

LOCAL lOldStyle   := VALTYPE(xSetBox) == 'L' .OR. PCOUNT() >= 2
LOCAL aOldSetBox  := ACLONE(aCurSetBox)
LOCAL aSetBox     := {}
LOCAL lBoxDefined := .Y.
LOCAL xReturn     := NIL

IF lOldStyle
  lBoxDefined := xSetBox
  IF lBoxDefined # NIL
    DEFAULT lVertScroll  TO .N.
    DEFAULT lHorizScroll TO .N.
    DEFAULT lBoxClose    TO .Y.
    DEFAULT lBoxResize   TO .N.
    DEFAULT lBoxDefined  TO .Y.
    DEFAULT cBoxScreen   TO ;
      SAVESCREEN(nBoxTop, nBoxLeft, nBoxBottom, nBoxRight)
    aCurSetBox[SB_TOP        ] := nBoxTop
    aCurSetBox[SB_LEFT       ] := nBoxLeft
    aCurSetBox[SB_BOTTOM     ] := nBoxBottom
    aCurSetBox[SB_RIGHT      ] := nBoxRight
    aCurSetBox[SB_VERTSCROLL ] := lVertScroll
    aCurSetBox[SB_HORIZSCROLL] := lHorizScroll
    aCurSetBox[SB_CLOSE      ] := lBoxClose
    aCurSetBox[SB_RESIZE     ] := lBoxResize
    aCurSetBox[SB_DEFINED    ] := lBoxDefined
    aCurSetBox[SB_SCREEN     ] := cBoxScreen
    aCurSetBox[SB_VERTMOVE   ] := .N.
    aCurSetBox[SB_HORIZMOVE  ] := .N.
    aCurSetBox[SB_VSCROLLPOS ] := INT((nBoxTop  + nBoxBottom)/2)
    aCurSetBox[SB_VHITTOP    ] := .N.
    aCurSetBox[SB_VHITBOTTOM ] := .N.
    aCurSetBox[SB_HSCROLLPOS ] := INT((nBoxLeft + nBoxRight )/2)
    aCurSetBox[SB_HHITLEFT   ] := .N.
    aCurSetBox[SB_HHITRIGHT  ] := .N.
  END
  nBoxTop      := aOldSetBox[SB_TOP        ]
  nBoxLeft     := aOldSetBox[SB_LEFT       ]
  nBoxBottom   := aOldSetBox[SB_BOTTOM     ]
  nBoxRight    := aOldSetBox[SB_RIGHT      ]
  lVertScroll  := aOldSetBox[SB_VERTSCROLL ]
  lHorizScroll := aOldSetBox[SB_HORIZSCROLL]
  lBoxClose    := aOldSetBox[SB_CLOSE      ]
  lBoxResize   := aOldSetBox[SB_RESIZE     ]
  lBoxDefined  := aOldSetBox[SB_DEFINED    ]
  cBoxScreen   := aOldSetBox[SB_SCREEN     ]
  xReturn      := lBoxDefined
ELSE
  aSetBox := xSetBox
  IF aSetBox # NIL
    ASIZE(aSetBox, SB_COUNT)
    nBoxTop    := aSetBox[SB_TOP   ]
    nBoxLeft   := aSetBox[SB_LEFT  ]
    nBoxBottom := aSetBox[SB_BOTTOM]
    nBoxRight  := aSetBox[SB_RIGHT ]
    DEFAULT aSetBox[SB_VERTSCROLL ] TO .N.
    DEFAULT aSetBox[SB_HORIZSCROLL] TO .N.
    DEFAULT aSetBox[SB_CLOSE      ] TO .Y.
    DEFAULT aSetBox[SB_RESIZE     ] TO .N.
    DEFAULT aSetBox[SB_DEFINED    ] TO .Y.
    DEFAULT aSetBox[SB_SCREEN     ] TO ;
      SAVESCREEN(nBoxTop, nBoxLeft, nBoxBottom, nBoxRight)
    DEFAULT aSetBox[SB_VERTMOVE   ] TO .N.
    DEFAULT aSetBox[SB_HORIZMOVE  ] TO .N.
    IF !aSetBox[SB_VERTMOVE]
      DEFAULT aSetBox[SB_VSCROLLPOS ] TO INT((nBoxTop + nBoxBottom)/2)
      DEFAULT aSetBox[SB_VHITTOP    ] TO .N.
      DEFAULT aSetBox[SB_VHITBOTTOM ] TO .N.
    ELSE
      DEFAULT aSetBox[SB_VSCROLLPOS ] TO nBoxTop + 2
      DEFAULT aSetBox[SB_VHITTOP    ] TO .Y.
      DEFAULT aSetBox[SB_VHITBOTTOM ] TO .N.
    END
    IF !aSetBox[SB_HORIZMOVE]
      DEFAULT aSetBox[SB_HSCROLLPOS ] TO INT((nBoxLeft + nBoxRight)/2)
      DEFAULT aSetBox[SB_HHITLEFT   ] TO .N.
      DEFAULT aSetBox[SB_HHITRIGHT  ] TO .N.
    ELSE
      DEFAULT aSetBox[SB_HSCROLLPOS ] TO nBoxLeft + 2
      DEFAULT aSetBox[SB_HHITLEFT   ] TO .Y.
      DEFAULT aSetBox[SB_HHITRIGHT  ] TO .N.
    END
    aCurSetBox := ACLONE(aSetBox)
  END
  IF aCurSetBox[SB_DEFINED] == NIL
    aCurSetBox[SB_DEFINED] := .Y.
  END
  xReturn := aOldSetBox
END

RETURN xReturn  //  McSetBox

//---------------------------------------------------------------------------

PROCEDURE McRestBox(aSetBox)

LOCAL aNewSetBox := McSetBox(aSetBox)
LOCAL nBoxTop    := aNewSetBox[SB_TOP   ]
LOCAL nBoxLeft   := aNewSetBox[SB_LEFT  ]
LOCAL nBoxBottom := aNewSetBox[SB_BOTTOM]
LOCAL nBoxRight  := aNewSetBox[SB_RIGHT ]
LOCAL cBoxScreen := aNewSetBox[SB_SCREEN]

RESTSCREEN(nBoxTop, nBoxLeft, nBoxBottom, nBoxRight, cBoxScreen)

RETURN  //  McRestBox

//---------------------------------------------------------------------------

FUNCTION McBox(xArg1, xArg2, xArg3, xArg4, xArg5, xArg6, xArg7, xArg8, ;
  xArg9, xArg10, xArg11, xArg12, xArg13)

LOCAL lOldStyle    := VALTYPE(xArg1) == 'N'
LOCAL aSetBox      := {}
LOCAL aFullSetBox  := {}
LOCAL nBoxTop      := 0
LOCAL nBoxLeft     := 0
LOCAL nBoxBottom   := 0
LOCAL nBoxRight    := 0
LOCAL lVertScroll  := .N.
LOCAL lHorizScroll := .N.
LOCAL lBoxClose    := .Y.
LOCAL lBoxResize   := .N.
LOCAL lBoxDefined  := .Y.
LOCAL cBoxScreen   := ''
LOCAL lVertMove    := .N.
LOCAL lHorizMove   := .N.
LOCAL nVScrollPos  := 0
LOCAL lVHitTop     := .Y.
LOCAL lVHitBottom  := .N.
LOCAL nHScrollPos  := 0
LOCAL lHHitLeft    := .Y.
LOCAL lHHitRight   := .N.
LOCAL cBoxTitle    := ''
LOCAL lCenter      := .Y.
LOCAL cBoxColor    := SETCOLOR()
LOCAL xLineType    := 1
LOCAL lSave        := .Y.
LOCAL cControls    := ''
LOCAL nControl     := ''
LOCAL aOldSetBox   := {}
LOCAL aNewSetBox   := {}
LOCAL nVertTop     := 0
LOCAL nVertMid     := 0
LOCAL nVertBottom  := 0
LOCAL nHorizLeft   := 0
LOCAL nHorizMid    := 0
LOCAL nHorizRight  := 0
LOCAL cDispTitle   := ''
LOCAL cOldColor    := SETCOLOR()
LOCAL xReturn      := NIL

IF lOldStyle
  nBoxTop      := xArg1
  nBoxLeft     := xArg2
  nBoxBottom   := xArg3
  nBoxRight    := xArg4
  lVertScroll  := xArg12
  lHorizScroll := xArg13
  lBoxClose    := xArg10
  lBoxResize   := xArg11
  lBoxDefined  := .Y.
  lVertMove    := NIL
  lHorizMove   := NIL
  nVScrollPos  := NIL
  nHScrollPos  := NIL
  nVScrollPos  := NIL
  lVHitTop     := NIL
  lVHitBottom  := NIL
  nHScrollPos  := NIL
  lHHitLeft    := NIL
  lHHitRight   := NIL
  cBoxTitle    := xArg5
  lCenter      := xArg6
  cBoxColor    := xArg7
  xLineType    := xArg8
  lSave        := .Y.
  cControls    := xArg9
ELSE
  aSetBox      := xArg1
  aFullSetBox  := ASIZE(aSetBox, SB_COUNT)
  nBoxTop      := aFullSetBox[SB_TOP        ]
  nBoxLeft     := aFullSetBox[SB_LEFT       ]
  nBoxBottom   := aFullSetBox[SB_BOTTOM     ]
  nBoxRight    := aFullSetBox[SB_RIGHT      ]
  lVertScroll  := aFullSetBox[SB_VERTSCROLL ]
  lHorizScroll := aFullSetBox[SB_HORIZSCROLL]
  lBoxClose    := aFullSetBox[SB_CLOSE      ]
  lBoxResize   := aFullSetBox[SB_RESIZE     ]
  lBoxDefined  := aFullSetBox[SB_DEFINED    ]
  lVertMove    := aFullSetBox[SB_VERTMOVE   ]
  lHorizMove   := aFullSetBox[SB_HORIZMOVE  ]
  nVScrollPos  := aFullSetBox[SB_VSCROLLPOS ]
  lVHitTop     := aFullSetBox[SB_VHITTOP    ]
  lVHitBottom  := aFullSetBox[SB_VHITBOTTOM ]
  nHScrollPos  := aFullSetBox[SB_HSCROLLPOS ]
  lHHitLeft    := aFullSetBox[SB_HHITLEFT   ]
  lHHitRight   := aFullSetBox[SB_HHITRIGHT  ]
  cBoxTitle    := xArg2
  lCenter      := xArg3
  cBoxColor    := xArg4
  xLineType    := xArg5
  lSave        := xArg6
  cControls    := xArg7
END

nVertTop    := nBoxTop + 2
nVertMid    := INT((nBoxTop + nBoxBottom)/2)
nVertBottom := nBoxBottom - 2
nHorizLeft  := nBoxLeft + 2
nHorizMid   := INT((nBoxLeft + nBoxRight )/2)
nHorizRight := nBoxRight - 2

DEFAULT lVertScroll  TO .N.
DEFAULT lHorizScroll TO .N.
DEFAULT lBoxClose    TO .Y.
DEFAULT lBoxResize   TO .N.
DEFAULT lBoxDefined  TO .Y.
DEFAULT lVertMove    TO .N.
DEFAULT lHorizMove   TO .N.
DEFAULT nVScrollPos  TO ;
  IF(!(lVertScroll .AND. lVertMove), nVertMid, nVertTop)
DEFAULT lVHitTop     TO lVertMove
DEFAULT lVHitBottom  TO .N.
DEFAULT nHScrollPos  TO ;
  IF(!(lHorizScroll .AND. lHorizMove), nHorizMid, nHorizLeft)
DEFAULT lHHitLeft    TO lHorizMove
DEFAULT lHHitRight   TO .N.
DEFAULT cBoxTitle    TO ''
DEFAULT lCenter      TO .Y.
DEFAULT xLineType    TO 1
DEFAULT lSave        TO .Y.
DEFAULT cControls    TO ;
  'x'                                 + ; //  1
  C_EQUIVALENCE                       + ; //  2
  C_UP_TRIANGLE                       + ; //  3
  IF(!(lVertScroll .AND. lVertMove)   , ;
  C_CENTER_BLOCK, C_UPDOWN_ARROW    ) + ; //  4
  C_DOWN_TRIANGLE                     + ; //  5
  C_LEFT_TRIANGLE                     + ; //  6
  IF(!(lHorizScroll .AND. lHorizMove) , ;
  C_CENTER_BLOCK, C_LEFT_RIGHT_ARROW) + ; //  7
  C_RIGHT_TRIANGLE                    + ; //  8
  C_DOWN_ARROW                        + ; //  9
  C_UP_ARROW                          + ; // 10
  C_RIGHT_ARROW                       + ; // 11
  C_LEFT_ARROW                            // 12

DO CASE
CASE EMPTY(cBoxTitle)
  cDispTitle := ''
CASE LEN(ALLTRIM(cBoxTitle)) + 2 > nBoxRight - nBoxLeft - 1
  cDispTitle := LEFT(ALLTRIM(cBoxTitle), nBoxRight - nBoxLeft - 1)
OTHERWISE
  cDispTitle := ' ' + ALLTRIM(cBoxTitle) + ' '
END
cBoxScreen := SAVESCREEN(nBoxTop, nBoxLeft, nBoxBottom, nBoxRight)
aNewSetBox := { ;
  nBoxTop     , ; // SB_TOP
  nBoxLeft    , ; // SB_LEFT
  nBoxBottom  , ; // SB_BOTTOM
  nBoxRight   , ; // SB_RIGHT
  lVertScroll , ; // SB_VERTSCROLL
  lHorizScroll, ; // SB_HORIZSCROLL
  lBoxClose   , ; // SB_CLOSE
  lBoxResize  , ; // SB_RESIZE
  lBoxDefined , ; // SB_DEFINED
  cBoxScreen  , ; // SB_SCREEN
  lVertMove   , ; // SB_VERTMOVE
  lHorizMove  , ; // SB_HORIZMOVE
  nVScrollPos , ; // SB_VSCROLLPOS
  lVHitTop    , ; // SB_VHITTOP
  lVHitBottom , ; // SB_VHITBOTTOM
  nHScrollPos , ; // SB_HSCROLLPOS
  lHHitLeft   , ; // SB_HHITLEFT
  lHHitRight    } // SB_HHITRIGHT
IF lSave
  aOldSetBox := McSetBox(aNewSetBox)
END

SETCOLOR(cBoxColor)
@ nBoxTop, nBoxLeft CLEAR TO nBoxBottom, nBoxRight
DISPBOX(nBoxTop, nBoxLeft, nBoxBottom, nBoxRight, xLineType)
IF lBoxClose
  @ nBoxTop, nBoxLeft  SAY SUBSTR(cControls, 1, 1)
END
IF lBoxResize
  @ nBoxTop, nBoxRight SAY SUBSTR(cControls, 2, 1)
END
IF lVertScroll
  @ nBoxTop + 1, nBoxRight SAY SUBSTR(cControls, 3, 1)
  @ nVScrollPos, nBoxRight SAY SUBSTR(cControls, ;
    IF(!(lVertScroll .AND. lVertMove), 4, ;
    IF(lVHitTop, 9, IF(lVHitBottom, 10, 4))), 1)
  @ nBoxBottom - 1, nBoxRight SAY SUBSTR(cControls, 5, 1)
END
IF lHorizScroll
  @ nBoxBottom, nBoxLeft + 1 SAY SUBSTR(cControls, 6, 1)
  @ nBoxBottom, nHScrollPos  SAY SUBSTR(cControls, ;
    IF(!(lHorizScroll .AND. lHorizMove), 7, ;
    IF(lHHitLeft, 11, IF(lHHitRight, 12, 7))), 1)
  @ nBoxBottom, nBoxRight - 1 SAY SUBSTR(cControls, 8, 1)
END
IF !EMPTY(cBoxTitle)
  @ nBoxTop, IF(lCenter, ;
    INT((nBoxLeft + nBoxRight + 1 - LEN(cDispTitle))/2), nBoxLeft + 2) ;
    SAY cDispTitle
END

SETCOLOR(cOldColor)
xReturn := IF(lOldStyle, cBoxScreen, aOldSetBox)

RETURN xReturn  //  McBox

//---------------------------------------------------------------------------

PROCEDURE McScrollBox(nVertPart, nVertTot, nHorizPart, nHorizTot, ;
  xLineType, cControls)

LOCAL aSetBox      := McSetBox()
LOCAL nBoxTop      := 0
LOCAL nBoxLeft     := 0
LOCAL nBoxBot      := 0
LOCAL nBoxRight    := 0
LOCAL lVertScroll  := .N.
LOCAL lHorizScroll := .N.
LOCAL lBoxDefined  := .Y.
LOCAL lVertMove    := .N.
LOCAL lHorizMove   := .N.
LOCAL nVScrollPos  := 0
LOCAL lVHitTop     := .Y.
LOCAL lVHitBottom  := .N.
LOCAL nHScrollPos  := 0
LOCAL lHHitLeft    := .Y.
LOCAL lHHitRight   := .N.

LOCAL nOldPos      := 0
LOCAL lOldMin      := .Y.
LOCAL lOldMax      := .N.
LOCAL nVPageTop    := 0
LOCAL nVPageMid    := 0
LOCAL nVPageBottom := 0
LOCAL nVPageRange  := 0
LOCAL nVertQuot    := 0
LOCAL nVertExt     := 0
LOCAL nHPageLeft   := 0
LOCAL nHPageMid    := 0
LOCAL nHPageRight  := 0
LOCAL nHPageRange  := 0
LOCAL nHorizQuot   := 0
LOCAL nHorizExt    := 0
LOCAL nRow         := ROW()
LOCAL nCol         := COL()

ASIZE(aSetBox, SB_COUNT)
CONDSET lBoxDefined TO aSetBox[SB_DEFINED]

IF lBoxDefined

  nBoxTop      := aSetBox[SB_TOP        ]
  nBoxLeft     := aSetBox[SB_LEFT       ]
  nBoxBot      := aSetBox[SB_BOTTOM     ]
  nBoxRight    := aSetBox[SB_RIGHT      ]
  lVertScroll  := aSetBox[SB_VERTSCROLL ]
  lHorizScroll := aSetBox[SB_HORIZSCROLL]
  lVertMove    := aSetBox[SB_VERTMOVE   ]
  lHorizMove   := aSetBox[SB_HORIZMOVE  ]

  nVPageTop    := nBoxTop + 2
  nVPageBottom := nBoxBot - 2
  nVPageRange  := nVPageBottom - nVPageTop + 1
  nHPageLeft   := nBoxLeft + 2
  nHPageRight  := nBoxRight - 2
  nHPageRange  := nHPageRight - nHPageLeft + 1

  DEFAULT xLineType TO 1
  DEFAULT cControls TO ;
    'x'                                 + ; //  1
    C_EQUIVALENCE                       + ; //  2
    C_UP_TRIANGLE                       + ; //  3
    IF(!(lVertScroll .AND. lVertMove)   , ;
    C_CENTER_BLOCK, C_UPDOWN_ARROW    ) + ; //  4
    C_DOWN_TRIANGLE                     + ; //  5
    C_LEFT_TRIANGLE                     + ; //  6
    IF(!(lHorizScroll .AND. lHorizMove) , ;
    C_CENTER_BLOCK, C_LEFT_RIGHT_ARROW) + ; //  7
    C_RIGHT_TRIANGLE                    + ; //  8
    C_DOWN_ARROW                        + ; //  9
    C_UP_ARROW                          + ; // 10
    C_RIGHT_ARROW                       + ; // 11
    C_LEFT_ARROW                            // 12

  IF lVertScroll .AND. lVertMove
    nVScrollPos  := aSetBox[SB_VSCROLLPOS]
    lVHitTop     := aSetBox[SB_VHITTOP   ]
    lVHitBottom  := aSetBox[SB_VHITBOTTOM]
    nOldPos      := nVScrollPos
    lOldMin      := lVHitTop
    lOldMax      := lVHitBottom
    lVHitTop     := nVertPart <= 1
    lVHitBottom  := nVertPart >= nVertTot
    IF !EMPTY(nVertTot)
      nVertQuot := (nVertPart-1) / (nVertTot-1)
    END
    nVertExt    := ROUND(nVPageTop + nVertQuot * nVPageRange, 0)
    nVScrollPos := MAX(nVPageTop, MIN(nVPageBottom, nVertExt))
    IF nVScrollPos # nOldPos .OR. ;
      lVHitTop # lOldMin .OR. lVHitBottom # lOldMax
      DISPBOX(nVPageTop, nBoxRight, nVPageBottom, nBoxRight, xLineType)
      @ nVScrollPos, nBoxRight SAY ;
        SUBSTR(cControls, IF(lVHitTop, 9, IF(lVHitBottom, 10, 4)), 1)
    END
    aSetBox[SB_VSCROLLPOS] := nVScrollPos
    aSetBox[SB_VHITTOP   ] := lVHitTop
    aSetBox[SB_VHITBOTTOM] := lVHitBottom
  END

  IF lHorizScroll .AND. lHorizMove
    nHScrollPos := aSetBox[SB_HSCROLLPOS]
    lHHitLeft   := aSetBox[SB_HHITLEFT  ]
    lHHitRight  := aSetBox[SB_HHITRIGHT ]
    nOldPos     := nHScrollPos
    lOldMin     := lHHitLeft
    lOldMax     := lHHitRight
    lHHitLeft   := nHorizPart <= 1
    lHHitRight  := nHorizPart >= nHorizTot
    IF !EMPTY(nHorizTot)
      nHorizQuot := (nHorizPart-1) / (nHorizTot-1)
    END
    nHorizExt   := ROUND(nHPageLeft + nHorizQuot * nHPageRange, 0)
    nHScrollPos := MAX(nHPageLeft, MIN(nHPageRight, nHorizExt))
    IF nHScrollPos # nOldPos .OR. ;
      lHHitLeft # lOldMin .OR. lHHitRight # lOldMax
      DISPBOX(nBoxBot, nHPageLeft, nBoxBot, nHPageRight, xLineType)
      @ nBoxBot, nHScrollPos SAY ;
        SUBSTR(cControls, IF(lHHitLeft, 11, IF(lHHitRight, 12, 7)), 1)
    END
    aSetBox[SB_HSCROLLPOS] := nHScrollPos
    aSetBox[SB_HHITLEFT  ] := lHHitLeft
    aSetBox[SB_HHITRIGHT ] := lHHitRight
  END

  McSetBox(aSetBox)

END

SETPOS(nRow, nCol)

RETURN  //  McScrollBox

//---------------------------------------------------------------------------

FUNCTION McBoxPad(aSetBox, oBrowse)

STATIC aKeyPad := {}

LOCAL nBoxTop      := 0
LOCAL nBoxLeft     := 0
LOCAL nBoxBot      := 0
LOCAL nBoxRight    := 0
LOCAL lVertScroll  := .N.
LOCAL lHorizScroll := .N.
LOCAL lBoxClose    := .Y.
LOCAL lBoxResize   := .N.
LOCAL lBoxDefined  := .Y.
LOCAL lVertMove    := .N.
LOCAL lHorizMove   := .N.
LOCAL nVScrollPos  := 0
LOCAL lVHitTop     := lVertMove
LOCAL lVHitBottom  := .N.
LOCAL nHScrollPos  := 0
LOCAL lHHitLeft    := lHorizMove
LOCAL lHHitRight   := .N.

LOCAL nVPageTop    := 0
LOCAL nVPageMid    := 0
LOCAL nVPageBottom := 0
LOCAL nHPageLeft   := 0
LOCAL nHPageMid    := 0
LOCAL nHPageRight  := 0
LOCAL nVExtrTop    := 0
LOCAL nVExtrBottom := 0
LOCAL nHExtrLeft   := 0
LOCAL nHExtrRight  := 0
LOCAL nVCursTop    := 0
LOCAL nVCursBottom := 0
LOCAL nHCursLeft   := 0
LOCAL nHCursRight  := 0

IF aSetBox # NIL

  ASIZE(aSetBox, SB_COUNT)
  CONDSET lBoxDefined TO aSetBox[SB_DEFINED]

  IF lBoxDefined
    nBoxTop   := aSetBox[SB_TOP        ]
    nBoxLeft  := aSetBox[SB_LEFT       ]
    nBoxBot   := aSetBox[SB_BOTTOM     ]
    nBoxRight := aSetBox[SB_RIGHT      ]
    CONDSET lVertScroll  TO aSetBox[SB_VERTSCROLL ]
    CONDSET lHorizScroll TO aSetBox[SB_HORIZSCROLL]
    CONDSET lBoxClose    TO aSetBox[SB_CLOSE      ]
    CONDSET lBoxResize   TO aSetBox[SB_RESIZE     ]
    CONDSET lVertMove    TO aSetBox[SB_VERTMOVE   ]
    CONDSET lHorizMove   TO aSetBox[SB_HORIZMOVE  ]
    CONDSET nVScrollPos  TO aSetBox[SB_VSCROLLPOS ]
    CONDSET lVHitTop     TO aSetBox[SB_VHITTOP    ]
    CONDSET lVHitBottom  TO aSetBox[SB_VHITBOTTOM ]
    CONDSET nHScrollPos  TO aSetBox[SB_HSCROLLPOS ]
    CONDSET lHHitLeft    TO aSetBox[SB_HHITLEFT   ]
    CONDSET lHHitRight   TO aSetBox[SB_HHITRIGHT  ]
  ELSE
    nBoxTop   := oBrowse:NTOP    - 1
    nBoxLeft  := oBrowse:NLEFT   - 1
    nBoxBot   := oBrowse:NBOTTOM + 1
    nBoxRight := oBrowse:NRIGHT  + 1
  END

  nVPageTop    := nBoxTop + 2
  nVPageMid    := INT((nBoxTop + nBoxBot)/2)
  nVPageBottom := nBoxBot - 2
  nHPageLeft   := nBoxLeft + 2
  nHPageMid    := INT((nBoxLeft + nBoxRight)/2)
  nHPageRight  := nBoxRight - 2

  IF EMPTY(nVScrollPos)
    nVScrollPos := ;
      IF(!(lVertScroll .AND. lVertMove), nVPageMid, nVPageTop)
  END
  IF EMPTY(nHScrollPos)
    nHScrollPos := ;
      IF(!(lHorizScroll .AND. lHorizMove), nHPageMid, nHPageLeft)
  END

  nVExtrTop    := nBoxTop   + 1
  nVExtrBottom := nBoxBot   - 1
  nHExtrLeft   := nBoxLeft  + 1
  nHExtrRight  := nBoxRight - 1
  nVCursTop    := nVScrollPos - IF(nVScrollPos == nVPageTop   , 0, 1)
  nVCursBottom := nVScrollPos + IF(nVScrollPos == nVPageBottom, 0, 1)
  nHCursLeft   := nHScrollPos - IF(nHScrollPos == nHPageLeft  , 0, 1)
  nHCursRight  := nHScrollPos + IF(nHScrollPos == nHPageRight , 0, 1)

  ASIZE(aKeyPad, 0)
  IF lBoxClose
    AADD(aKeyPad, ;
      {nBoxTop  , nBoxLeft   , nBoxTop  , nBoxLeft   , K_ESC       , .Y.})
  END
  IF lBoxResize
    AADD(aKeyPad, ;
      {nBoxTop  , nBoxRight  , nBoxTop  , nBoxRight  , KP_ALT_ENTER, .Y.})
  END
  IF lVertScroll
    AADD(aKeyPad, ;
      {nVExtrTop   , nBoxLeft , nVPageTop   , nBoxLeft , K_UP        , .Y.})
    AADD(aKeyPad, ;
      {nVPageBottom, nBoxLeft , nVExtrBottom, nBoxLeft , K_DOWN      , .Y.})
    AADD(aKeyPad, ;
      {nVPageTop   , nBoxRight, nVCursTop   , nBoxRight, K_PGUP      , .Y.})
    AADD(aKeyPad, ;
      {nVCursBottom, nBoxRight, nVPageBottom, nBoxRight, K_PGDN      , .Y.})
    AADD(aKeyPad, ;
      {nVExtrTop   , nBoxRight, nVExtrTop   , nBoxRight, K_CTRL_PGUP , .Y.})
    AADD(aKeyPad, ;
      {nVExtrBottom, nBoxRight, nVExtrBottom, nBoxRight, K_CTRL_PGDN , .Y.})
  END
  IF lHorizScroll
    AADD(aKeyPad, ;
      {nBoxTop, nHExtrLeft , nBoxTop, nHPageLeft , K_LEFT      , .Y.})
    AADD(aKeyPad, ;
      {nBoxTop, nHPageRight, nBoxTop, nHExtrRight, K_RIGHT     , .Y.})
    AADD(aKeyPad, ;
      {nBoxBot, nHPageLeft , nBoxBot, nHCursLeft , K_CTRL_LEFT , .Y.})
    AADD(aKeyPad, ;
      {nBoxBot, nHCursRight, nBoxBot, nHPageRight, K_CTRL_RIGHT, .Y.})
    AADD(aKeyPad, ;
      {nBoxBot, nHExtrLeft , nBoxBot, nHExtrLeft , K_CTRL_HOME , .Y.})
    AADD(aKeyPad, ;
      {nBoxBot, nHExtrRight, nBoxBot, nHExtrRight, K_CTRL_END  , .Y.})
  END

END

RETURN aKeyPad  //  McBoxPad

//---------------------------------------------------------------------------

FUNCTION McTBApplyKey(oBrowse, nKey, aSetKey, aSetArea)

STATIC lSavMouse := .N.
STATIC nSavMRow  := 0
STATIC nSavMCol  := 0

LOCAL nApply      := TA_STD
LOCAL nMRow       := MRow()
LOCAL nMCol       := MCol()
LOCAL nRowDif     := nMRow - ROW()
LOCAL nRow        := 0
LOCAL nTCol       := oBrowse:COLPOS
LOCAL nLCol       := COL()
LOCAL nRCol       := nLCol + oBrowse:GETCOLUMN(nTCol):WIDTH - 1
LOCAL lAssign     := .N.
LOCAL lLook       := .Y.
LOCAL bKey        := NIL
LOCAL aSetBox     := McSetBox()
LOCAL aKeyPad     := McBoxPad(aSetBox, oBrowse)
LOCAL lBoxDefined := aSetBox[SB_DEFINED]
LOCAL nBoxTop     := aSetBox[SB_TOP    ]
LOCAL nBoxLeft    := aSetBox[SB_LEFT   ]
LOCAL nBoxBot     := aSetBox[SB_BOTTOM ]
LOCAL nBoxRight   := aSetBox[SB_RIGHT  ]

nKey := McTransPad(aKeyPad, nKey)
bKey := McTransBlock(nKey, aSetKey, aSetArea)

DO CASE
CASE !EMPTY(bKey)
  bKey:EVAL(PROCNAME(1), PROCLINE(1), '', @oBrowse)
CASE nKey == K_ESC
  nApply := TA_CLOSE
CASE nKey == KP_ALT_ENTER
  nApply := TA_RESIZE
CASE nKey == K_UP
  oBrowse:UP()
CASE nKey == K_DOWN
  oBrowse:DOWN()
CASE nKey == K_LEFT .OR. nKey == K_SH_TAB
  oBrowse:LEFT()
CASE nKey == K_RIGHT .OR. nKey == K_TAB
  oBrowse:RIGHT()
CASE nKey == K_CTRL_PGUP
  oBrowse:GOTOP()
CASE nKey == K_CTRL_PGDN
  oBrowse:GOBOTTOM()
CASE nKey == K_PGUP
  oBrowse:PAGEUP()
CASE nKey == K_PGDN
  oBrowse:PAGEDOWN()
CASE nKey == K_HOME
  oBrowse:HOME()
CASE nKey == K_END
  oBrowse:END()
CASE nKey == K_CTRL_HOME
  oBrowse:PANHOME()
CASE nKey == K_CTRL_END
  oBrowse:PANEND()
CASE nKey == K_CTRL_LEFT
  oBrowse:PANLEFT()
CASE nKey == K_CTRL_RIGHT
  oBrowse:PANRIGHT()

CASE nKey == K_CLICK .OR. nKey == K_DOUBLECLICK
  DO CASE
  CASE McInBox(nMRow, nMCol, ;
    oBrowse:NTOP, oBrowse:NLEFT, oBrowse:NBOTTOM, oBrowse:NRIGHT)

    DO CASE
    CASE nRowDif < 0
      FOR nRow := -1 TO nRowDif STEP -1
        oBrowse:UP()
      NEXT
    CASE nRowDif > 0
      FOR nRow := 1 TO nRowDif
        oBrowse:DOWN()
      NEXT
    END

    IF nMCol < nLCol
      WHILE !lAssign .AND. lLook
        DO CASE
        CASE nTCol > oBrowse:LEFTVISIBLE .OR. ;
          (nTCol <= oBrowse:FREEZE .AND. nTCol > 0)
          nTCol --
        CASE nTCol == oBrowse:LEFTVISIBLE .AND. ;
          !EMPTY(oBrowse:FREEZE)
          nTCol := oBrowse:FREEZE
        OTHERWISE
          lLook := .N.
        END
        IF lLook
          nLCol   -= oBrowse:GETCOLUMN(nTCol):WIDTH + 1
          nRCol   := nLCol + oBrowse:GETCOLUMN(nTCol):WIDTH - 1
          lAssign := (nLCol <= nMCol .AND. nMCol <= nRCol)
        END
      END
    ELSE
      WHILE !lAssign .AND. lLook
        lAssign := (nLCol <= nMCol .AND. nMCol <= nRCol)
        IF !lAssign
          DO CASE
          CASE (nTCol < oBrowse:RIGHTVISIBLE .AND. ;
            nTCol >= oBrowse:LEFTVISIBLE) .OR. ;
            nTCol < oBrowse:FREEZE
            nLCol += oBrowse:GETCOLUMN(nTCol):WIDTH + 1
            nTCol ++
          CASE nTCol == oBrowse:FREEZE .AND. ;
            !EMPTY(oBrowse:LEFTVISIBLE)
            nLCol += oBrowse:GETCOLUMN(nTCol):WIDTH + 1
            nTCol := oBrowse:LEFTVISIBLE
          OTHERWISE
            lLook := .N.
          END
          nRCol := nLCol + oBrowse:GETCOLUMN(nTCol):WIDTH - 1
        END
      END
    END
    IF lAssign
      oBrowse:COLPOS := nTCol
    END
    IF nKey == K_DOUBLECLICK
      nApply := TA_DOUBLECLICK
    END

  CASE lBoxDefined .AND. ;
    McMouseOut(nKey, nBoxTop, nBoxLeft, nBoxBot, nBoxRight)
    nApply := TA_OUTBOX
  END

OTHERWISE
  nApply := TA_UNKNOWN
END

RETURN nApply  //  McTBApplyKey

//---------------------------------------------------------------------------

PROCEDURE McTBCross(oBrowse, nRowIndex, nColIndex, nLowIndex, nCellIndex, ;
  lHiLite)

LOCAL nRowPos   := oBrowse:ROWPOS
LOCAL nColPos   := oBrowse:COLPOS
LOCAL nRowCount := oBrowse:ROWCOUNT
LOCAL nColCount := oBrowse:COLCOUNT
LOCAL aRow      := {nRowIndex,  nCellIndex}
LOCAL aCol      := {nColIndex,  nCellIndex}
LOCAL aLow      := {nLowIndex,  nCellIndex}
LOCAL aCell     := {nCellIndex, nCellIndex}

IF lHiLite == NIL
  lHiLite := .Y.
END
oBrowse:COLORRECT({1, 1, nRowPos - 1, nColPos - 1}, aLow)
oBrowse:COLORRECT({nRowPos + 1, 1, nRowCount, nColPos - 1}, aLow)
oBrowse:COLORRECT({1, nColPos + 1, nRowPos - 1, nColCount}, aLow)
oBrowse:COLORRECT({nRowPos + 1, nColPos + 1, nRowCount, nColCount}, aLow)
oBrowse:COLORRECT({nRowPos, 1, nRowPos, nColPos - 1}, aRow)
oBrowse:COLORRECT({nRowPos, nColPos + 1, nRowPos, nColCount}, aRow)
oBrowse:COLORRECT({1, nColPos, nRowPos - 1, nColPos}, aCol)
oBrowse:COLORRECT({nRowPos + 1, nColPos, nRowCount, nColPos}, aCol)
IF lHiLite
  oBrowse:HILITE()
ELSE
  oBrowse:DEHILITE()
  oBrowse:COLORRECT({nRowPos, nColPos, nRowPos, nColPos}, aCell)
  oBrowse:HILITE()
END

RETURN  //  McTBCross

//---------------------------------------------------------------------------

FUNCTION McAChoice(nTop, nLeft, nBottom, nRight, ;
  aItem, xSelect, cFunction, nDefault, nTopItem, lConfirm, lRehilite, ;
  aSetKey, aSetArea)

LOCAL nChoice     := 0
LOCAL nChoices    := LEN(aItem)
LOCAL nItem       := 1
LOCAL nRItem      := 1
LOCAL nFirst      := 1
LOCAL nRFirst     := 1
LOCAL nNewItem    := 1
LOCAL nNewRItem   := 1
LOCAL nItemDif    := 0
LOCAL oCol        := ;
  TBCOLUMNNEW('', {|| PADR(aItem[nItem], nRight - nLeft + 1)})
LOCAL oChoice     := ;
  TBROWSENEW(nTop, nLeft, MIN(nBottom, nTop + nChoices - 1), nRight)
LOCAL lBrowse     := .Y.
LOCAL lExit       := .Y.
LOCAL nKey        := 0
LOCAL bKey        := NIL
LOCAL cKey        := ''
LOCAL nLook       := 0
LOCAL lLook       := .Y.
LOCAL lSelect     := IF(VALTYPE(xSelect) == 'L', xSelect, .Y.)
LOCAL aSelect     := IF(VALTYPE(xSelect) == 'A', xSelect, ARRAY(nChoices))
LOCAL aRSelect    := ARRAY(nChoices)
LOCAL nSRow       := ROW()
LOCAL nPageLen    := nBottom - nTop + 1
LOCAL nRow        := 0
LOCAL xBreak      := NIL
LOCAL aSetBox     := McSetBox()
LOCAL aKeyPad     := McBoxPad(aSetBox, oChoice)
LOCAL lBoxDefined := aSetBox[SB_DEFINED]
LOCAL nBoxTop     := aSetBox[SB_TOP   ]
LOCAL nBoxLeft    := aSetBox[SB_LEFT  ]
LOCAL nBoxBot     := aSetBox[SB_BOTTOM]
LOCAL nBoxRight   := aSetBox[SB_RIGHT ]

IF aSelect[1] == NIL
  AFILL(aSelect, lSelect)
END
AEVAL(aRSelect, ;
  {|lEval, nEval| aRSelect[nEval] := aSelect[nChoices - nEval + 1]})
nFirst  := ASCAN(aSelect, .Y.)
nRFirst := ASCAN(aRSelect, .Y.)
DEFAULT lConfirm  TO .Y.
DEFAULT lRehilite TO .N.

oCol:WIDTH := nRight - nLeft + 1
oChoice:ADDCOLUMN(oCol)
oChoice:AUTOLITE      := .N.
oChoice:GOTOPBLOCK    := {|| nItem := 1}
oChoice:GOBOTTOMBLOCK := {|| nItem := nChoices}
oChoice:SKIPBLOCK     := {|nSkip, nItemOld| nItemOld := nItem,;
  (nItem := MIN(MAX(nItem + nSkip, 1), nChoices)) - nItemOld}

DO CASE
CASE EMPTY(nDefault)
  nDefault := 1
CASE nDefault > nChoices
  nDefault := nChoices
END
IF EMPTY(nTopItem)
  nTopItem := 1
END
nItem    := nTopItem
oChoice:FORCESTABLE()
nLook    := ASCAN(aSelect, .Y.)
lExit    := EMPTY(nLook)
nNewItem := IF(aSelect[nDefault], nDefault, nLook)
FOR nLook := nTopItem + 1 TO nNewItem
  oChoice:DOWN()
NEXT

WHILE lBrowse

  oChoice:FORCESTABLE()
  oChoice:HILITE()
  nSRow := ROW()
  nLook := nItem - oChoice:ROWPOS
  FOR nRow := 1 TO oChoice:ROWCOUNT
    IF lSelect .AND. !aSelect[nRow + nLook]
      oChoice:COLORRECT({nRow, 1, nRow, 1}, {5,4})
    END
  NEXT

  BEGIN SEQUENCE

    IF lExit
      lBrowse := .N.
      BREAK
    END
    nKey      := McTransPad(aKeyPad, McInKey(0))
    bKey      := McTransBlock(nKey, aSetKey, aSetArea)
    cKey      := UPPER(CHR(nKey))
    nRItem    := nChoices - nItem + 1
    nNewItem  := nItem
    nNewRItem := nRItem
    DO CASE
    CASE !EMPTY(bKey)
      bKey:EVAL(PROCNAME(1), PROCLINE(1), '', ;
        @oChoice, aItem, @nItem, @lBrowse)
    CASE nKey == K_UP
      nLook     := ASCAN(aRSelect, .Y., nRItem + 1)
      nNewRItem := IF(!EMPTY(nLook), nLook, nRItem)
    CASE nKey == K_DOWN
      nLook    := ASCAN(aSelect, .Y., nItem + 1)
      nNewItem := IF(!EMPTY(nLook), nLook, nItem)
    CASE nKey == K_HOME .OR. nKey == K_CTRL_PGUP
      nNewItem := nFirst
    CASE nKey == K_END .OR. nKey == K_CTRL_PGDN
      nNewRItem := nRFirst
    CASE nKey == K_PGUP
      nLook    := ASCAN(aRSelect, .Y., nRItem + nPageLen)
      IF !EMPTY(nLook)
        nNewRItem := nLook
      ELSE
        nNewItem  := nFirst
      END
    CASE nKey == K_PGDN
      nLook     := ASCAN(aSelect, .Y., nItem + nPageLen)
      IF !EMPTY(nLook)
        nNewItem  := nLook
      ELSE
        nNewRItem := nRFirst
      END
    CASE McMouseIn(nKey, ;
      oChoice:NTOP, oChoice:NLEFT, oChoice:NBOTTOM, oChoice:NRIGHT)
      nNewItem += MRow() - nSRow
      DO CASE
      CASE !aSelect[nNewItem]
        nNewItem := nItem
      CASE nKey == K_DOUBLECLICK .OR. !lConfirm
        nChoice := nNewItem
        lExit   := .Y.
      END
    CASE nKey == K_ESC .OR. (lBoxDefined .AND. ;
      McMouseIn(nKey, nBoxTop, nBoxLeft, nBoxTop, nBoxLeft))
      nChoice := 0
      lBrowse := .N.
    CASE nKey == K_ENTER .OR. nKey == K_TAB .OR. nKey == K_SH_TAB .OR. ;
      (lBoxDefined .AND. ;
      McMouseOut(nKey, nBoxTop, nBoxLeft, nBoxBot, nBoxRight))
      nChoice := nItem
      lBrowse := .N.
    CASE ISALPHA(cKey) .OR. ISDIGIT(cKey)
      nLook := nItem
      lLook := .Y.
      WHILE lLook
        nLook    := IF(nLook == nChoices, 1, nLook + 1)
        nNewItem := IF(LEFT(UPPER(aItem[nLook]),1) == cKey .AND. ;
          aSelect[nLook], nLook, 0)
        lLook    := EMPTY(nNewItem) .AND. nLook # nItem
      END
      DO CASE
      CASE EMPTY(nNewItem)
        nNewItem := nItem
      CASE !EMPTY(nNewItem) .AND. !lConfirm
        nChoice := nNewItem
        lExit   := .Y.
      END
    END
    DO CASE
    CASE nNewItem # nItem
      nItemDif := nNewItem - nItem
    CASE nNewRItem # nRItem
      nItemDif := nRItem - nNewRItem
    OTHERWISE
      nItemDif := 0
    END
    DO CASE
    CASE nItemDif < 0
      DISPBEGIN()
      AEVAL(ARRAY(ABS(nItemDif)), {|| oChoice:UP(), oChoice:FORCESTABLE()})
      DISPEND()
    CASE nItemDif > 0
      DISPBEGIN()
      AEVAL(ARRAY(ABS(nItemDif)), {|| oChoice:DOWN(), oChoice:FORCESTABLE()})
      DISPEND()
    END
  RECOVER USING xBreak
    IF !EMPTY(xBreak)
      BREAK xBreak
    END
  END SEQUENCE

END

IF lRehilite
  oChoice:DEHILITE()
  oChoice:COLORRECT({oChoice:ROWPOS, 1, oChoice:ROWPOS, 1}, {1,4})
  oChoice:HILITE()
END

RETURN nChoice  //  McAChoice

//---------------------------------------------------------------------------

FUNCTION McHAWidth(aItem, nSeparate, nBorder)

LOCAL nWidth := 0

DEFAULT nSeparate TO 2
DEFAULT nBorder   TO 1

AEVAL(aItem, {|cRow| nWidth := nWidth + ;
  nBorder + LEN(cRow) + nBorder + nSeparate})
nWidth -= nSeparate

RETURN nWidth  //  McHAWidth

//---------------------------------------------------------------------------

FUNCTION McHAPad(nRow, nCol, aItem, nSeparate, nBorder, lCenter)

LOCAL nChoices := LEN(aItem)
LOCAL nChoice  := 1
LOCAL nWidth   := 0
LOCAL aPad     := ARRAY(nChoices, 5)

DEFAULT nSeparate TO 2
DEFAULT nBorder   TO 1
DEFAULT lCenter   TO .N.

nWidth     := McHAWidth(aItem, nSeparate, nBorder)
aPad[1][5] := SPACE(nBorder) + aItem[1] + SPACE(nBorder)
aPad[1][1] := nRow
aPad[1][2] := IF(lCenter, nCol - INT(nWidth/2), nCol)
aPad[1][3] := nRow
aPad[1][4] := aPad[1][2] + LEN(aPad[1][5]) - 1

FOR nChoice := 2 TO nChoices
  aPad[nChoice][5] := SPACE(nBorder) + aItem[nChoice] + SPACE(nBorder)
  aPad[nChoice][1] := nRow
  aPad[nChoice][2] := aPad[nChoice - 1][4] + nSeparate + 1
  aPad[nChoice][3] := nRow
  aPad[nChoice][4] := aPad[nChoice][2] + LEN(aPad[nChoice][5]) - 1
NEXT

RETURN aPad  //  McHAPad

//---------------------------------------------------------------------------

FUNCTION McHAChoice(nRow, nCol, aItem, xSelect, nDefault, ;
  nSeparate, nBorder, lCenter, lConfirm, lRehilite, lVertExit, ;
  aSetKey, aSetArea)

LOCAL nChoices := LEN(aItem)
LOCAL nChoice  := 1
LOCAL nCursor  := SETCURSOR(0)
LOCAL aPad     := McHAPad(nRow, nCol, aItem, nSeparate, nBorder, lCenter)
LOCAL nLeft    := aPad[1][2]
LOCAL nRight   := ATAIL(aPad)[4]
LOCAL nKey     := 0
LOCAL cKey     := ''
LOCAL bKey     := NIL
LOCAL nPos     := 0
LOCAL nFirst   := 1
LOCAL nLast    := nChoices
LOCAL lSelect  := IF(VALTYPE(xSelect) == 'L', xSelect, .Y.)
LOCAL aSelect  := IF(VALTYPE(xSelect) == 'A', xSelect, ARRAY(nChoices))
LOCAL aRSelect := ARRAY(nChoices)
LOCAL lDisplay := .Y.
LOCAL lBrowse  := lSelect
LOCAL lSearch  := .Y.
LOCAL nSearch  := 0
LOCAL nMRow    := 0
LOCAL nMCol    := 0
LOCAL aVertKey := {K_TAB, K_SH_TAB, K_UP, K_DOWN, K_PGUP, K_PGDN, ;
  K_CTRL_PGUP, K_CTRL_PGDN, K_CTRL_HOME, K_CTRL_END}

IF aSelect[1] == NIL
  AFILL(aSelect, lSelect)
END
AEVAL(aRSelect, ;
  {|lEval, nEval| aRSelect[nEval] := aSelect[nChoices - nEval + 1]})
nFirst  := ASCAN(aSelect, .Y.)
nLast   := nChoices - ASCAN(aRSelect, .Y.) + 1
nChoice := IF(EMPTY(nDefault), nFirst, nDefault)
DEFAULT lConfirm  TO .N.
DEFAULT lRehilite TO .N.
DEFAULT lVertExit TO .N.

WHILE lDisplay
  FOR nPos := 1 TO nChoices
    COLORSELECT(IF(aSelect[nPos] .OR. !lSelect, ;
      IF(nPos == nChoice, IF(!lBrowse .AND. lRehilite, 4, 2), 1), 5)-1)
    @ aPad[nPos][1], aPad[nPos][2] SAY aPad[nPos][5]
  NEXT
  IF lBrowse
    nKey := McInKey(0)
    bKey := McTransBlock(nKey, aSetKey, aSetArea)
    cKey := UPPER(CHR(nKey))
  ELSE
    lDisplay := .N.
  END
  DO CASE
  CASE !EMPTY(bKey)
    bKey:EVAL(PROCNAME(1), PROCLINE(1), '', aItem, @nChoice, @lBrowse)
  CASE nKey == K_ESC
    lBrowse := .N.
    nChoice := 0
  CASE nKey == K_ENTER .OR. nKey == K_TAB .OR. nKey == K_SH_TAB .OR. ;
    (lVertExit .AND. !EMPTY(ASCAN(aVertKey, nKey))) .OR. ;
    McMouseOut(nKey, nRow, nLeft, nRow, nRight)
    lBrowse := .N.
  CASE nKey == K_HOME
    nChoice := nFirst
  CASE nKey == K_END
    nChoice := nLast
  CASE nKey == K_LEFT
    nSearch := ASCAN(aRSelect, {|lEval| lEval}, nChoices - nChoice + 2)
    nChoice := IF(EMPTY(nSearch), nLast, nChoices - nSearch + 1)
  CASE nKey == K_RIGHT
    nSearch := ASCAN(aSelect, {|lEval| lEval}, nChoice + 1)
    nChoice := IF(EMPTY(nSearch), nFirst, nSearch)
  CASE ISALPHA(cKey)
    nPos    := nChoice
    lSearch := .Y.
    nSearch := 0
    WHILE lSearch
      nPos    := IF(nPos == nLast, nFirst, nPos + 1)
      nSearch := IF(LEFT(UPPER(aItem[nPos]),1) == cKey .AND. ;
        aSelect[nPos], nPos, 0)
      lSearch := EMPTY(nSearch) .AND. nPos # nChoice
    END
    IF !EMPTY(nSearch)
      nChoice := nSearch
      lBrowse := lConfirm
    END
  CASE nKey == K_CLICK .OR. nKey == K_DOUBLECLICK
    nMRow   := MRow()
    nMCol   := MCol()
    nSearch := 0
    FOR nPos := 1 TO nChoices
      IF McInBox(nMRow, nMCol, aPad[nPos][1], aPad[nPos][2], ;
        aPad[nPos][3], aPad[nPos][4]) .AND. aSelect[nPos]
        nSearch := nPos
      END
    NEXT
    IF !EMPTY(nSearch)
      nChoice := nSearch
      lBrowse := lConfirm .AND. nKey # K_DOUBLECLICK
    END
  END
END

COLORSELECT(0)
SETCURSOR(nCursor)

RETURN nChoice  //  McHAChoice

//---------------------------------------------------------------------------

FUNCTION McAlert(cMessage, aChoices, cColor, nDefault)

LOCAL nChoices  := LEN(IF(!EMPTY(aChoices), aChoices, aChoices := {'Ok'}))
LOCAL cTMessage := STRTRAN(cMessage, ';', C_HARD_NEW_LINE)
LOCAL nMaxWidth := 58
LOCAL aMessage  := McMemoArray(cTMessage, nMaxWidth)
LOCAL nLines    := LEN(aMessage) + 4
LOCAL nChoice   := nDefault
LOCAL cOldColor := SETCOLOR(IF(EMPTY(cColor), 'W+/R, W+/B', cColor))
LOCAL nCursor   := SETCURSOR(0)
LOCAL cScreen   := SAVESCREEN()
LOCAL nOldRow   := ROW()
LOCAL nOldCol   := COL()
LOCAL nOldMRow  := MRow()
LOCAL nOldMCol  := MRow()
LOCAL nChWidth  := 0
LOCAL nWidth    := 0
LOCAL nTop      := 0
LOCAL nBottom   := 0
LOCAL nLeft     := 0
LOCAL nRight    := 0
LOCAL nPos      := 0

AEVAL(aMessage, {|aLine| nWidth := MAX(nWidth, LEN(aLine[1]))})
nTop    := INT((MAXROW() - nLines)/2)
nBottom := nTop + nLines - 1
AEVAL(aChoices, {|cRow| nChWidth := nChWidth + LEN(cRow) + 4})
nWidth  := MAX(nWidth, nChWidth)
nWidth  += 8
nLeft   := 40 - INT(nWidth/2)
nRight  := nLeft + nWidth - 1

@ nTop, nLeft CLEAR TO nBottom, nRight
@ nTop, nLeft TO nBottom, nRight
FOR nPos := 1 TO LEN(aMessage)
   @ nTop + nPos, nLeft + 4 SAY PADC(aMessage[nPos][1], nWidth - 8)
NEXT
nChoice := McHAChoice(nBottom - 1, 40, aChoices,, nDefault,,, .Y.)

SETCOLOR(cOldColor)
SETPOS(nOldRow, nOldCol)
MSetPos(nOldMRow, nOldMCol)
SETCURSOR(nCursor)
RESTSCREEN(,,,, cScreen)

RETURN nChoice  //  McAlert

//---------------------------------------------------------------------------

FUNCTION McMemoEdit(cString, nTop, nLeft, nBottom, nRight, ;
  lMode, lEnter, nLength, nTab, nBufRow, nBufCol, nWinRow, nWinCol, ;
  aSetKey, aSetArea)

LOCAL cOrig        := IF(cString # NIL, cString, '')
LOCAL cBuffer      := ''
LOCAL lEdit        := .Y.
LOCAL nLine        := 1
LOCAL aLine        := {}
LOCAL oBrowse      := TBROWSENEW(0, 0, MAXROW(), 79)
LOCAL oCol         := TBCOLUMNNEW('', {|| aLine[nLine][1]})
LOCAL nPos         := 0
LOCAL nSearch      := 0
LOCAL cBoxColor    := ''
LOCAL aSetBox      := McSetBox()
LOCAL nBoxTop      := aSetBox[SB_TOP        ]
LOCAL nBoxLeft     := aSetBox[SB_LEFT       ]
LOCAL nBoxBot      := aSetBox[SB_BOTTOM     ]
LOCAL nBoxRight    := aSetBox[SB_RIGHT      ]
LOCAL lVertScroll  := aSetBox[SB_VERTSCROLL ]
LOCAL lHorizScroll := aSetBox[SB_HORIZSCROLL]
LOCAL lBoxClose    := aSetBox[SB_CLOSE      ]
LOCAL lBoxResize   := aSetBox[SB_RESIZE     ]
LOCAL lBoxDefined  := aSetBox[SB_DEFINED    ]
LOCAL oGet         := GETNEW(ROW(), COL(), {|| aLine})
LOCAL nKey         := 0
LOCAL xBreak       := NIL
LOCAL aMemoSys     := ARRAY(MSV_COUNT)
LOCAL cGetColor    := LEFT(SETCOLOR(), AT(',', SETCOLOR())-1)
LOCAL bMeCursor    := McSet(MCSET_MECURSOR)
LOCAL lMeCursor    := !EMPTY(bMeCursor)

DEFAULT cString TO ''

aMemoSys[MSV_TOP]       := IF(nTop    == NIL, 0, nTop)
aMemoSys[MSV_LEFT]      := IF(nLeft   == NIL, 0, nLeft)
aMemoSys[MSV_BOTTOM]    := IF(nBottom == NIL, MAXROW(), nBottom)
aMemoSys[MSV_RIGHT]     := IF(nRight  == NIL, 79, nRight)
aMemoSys[MSV_MODE]      := IF(lMode   == NIL, .Y., lMode)
aMemoSys[MSV_ENTER]     := ;
  IF(lEnter  == NIL .OR. VALTYPE(lEnter) # 'L', .Y., lEnter)
aMemoSys[MSV_WIDTH]     := ;
  IF(nLength == NIL, aMemoSys[MSV_RIGHT] - aMemoSys[MSV_LEFT], ;
  MIN(nLength, aMemoSys[MSV_RIGHT] - aMemoSys[MSV_LEFT]))
aMemoSys[MSV_TAB]       := IF(nTab    == NIL, 4, nTab)
aMemoSys[MSV_BUFROW]    := IF(nBufRow == NIL, 1, nBufRow)
aMemoSys[MSV_BUFCOL]    := IF(nBufCol == NIL, 0, nBufCol)
aMemoSys[MSV_WINROW]    := IF(nWinRow == NIL, 0, nWinRow)
aMemoSys[MSV_WINCOL]    := IF(nWinCol == NIL, 0, nWinCol)
aMemoSys[MSV_GETPOS]    := 0
aMemoSys[MSV_SETPOS]    := aMemoSys[MSV_BUFCOL] + 1
aMemoSys[MSV_TYPEOUT]   := ''
aMemoSys[MSV_REFORM]    := .N.
aMemoSys[MSV_INSERT]    := SET(_SET_INSERT)
aMemoSys[MSV_BOXTOP]    := ;
  IF(lBoxDefined, nBoxTop, aMemoSys[MSV_TOP] - 1)
aMemoSys[MSV_BOXLEFT]   := ;
  IF(lBoxDefined, nBoxLeft, aMemoSys[MSV_LEFT] - 1)
aMemoSys[MSV_BOXBOTTOM] := ;
  IF(lBoxDefined, nBoxBot, aMemoSys[MSV_BOTTOM] + 1)
aMemoSys[MSV_BOXRIGHT]  := ;
  IF(lBoxDefined, nBoxRight, aMemoSys[MSV_RIGHT]  + 1)
aMemoSys[MSV_CLOSE]     := IF(lBoxDefined, lBoxClose   , .Y.)
aMemoSys[MSV_RESIZE]    := IF(lBoxDefined, lBoxResize  , .N.)
aMemoSys[MSV_VSCROLL]   := IF(lBoxDefined, lVertScroll , .Y.)
aMemoSys[MSV_HSCROLL]   := IF(lBoxDefined, lHorizScroll, .N.)
aMemoSys[MSV_ROWMID]    := ;
  INT((aMemoSys[MSV_BOXTOP] + aMemoSys[MSV_BOXBOTTOM])/2)
aMemoSys[MSV_COLMID]    := ;
  INT((aMemoSys[MSV_BOXLEFT] + aMemoSys[MSV_BOXRIGHT])/2)

oCol:WIDTH := aMemoSys[MSV_WIDTH]
oBrowse:ADDCOLUMN(oCol)
oBrowse:NLEFT         := aMemoSys[MSV_LEFT]
oBrowse:NTOP          := aMemoSys[MSV_TOP]
oBrowse:NBOTTOM       := aMemoSys[MSV_BOTTOM]
oBrowse:NRIGHT        := aMemoSys[MSV_LEFT] + aMemoSys[MSV_WIDTH] - 1
oBrowse:GOTOPBLOCK    := {|| nLine := 1}
oBrowse:GOBOTTOMBLOCK := {|| nLine := LEN(aLine)}
oBrowse:SKIPBLOCK     := {|nSkip, nLineOld| nLineOld := nLine,;
  (nLine := MIN(MAX(nLine + nSkip, 1), LEN(aLine))) - nLineOld}
oBrowse:COLORSPEC     := cGetColor + ',' + cGetColor
oGet:ROW       := aMemoSys[MSV_TOP]
oGet:COL       := aMemoSys[MSV_LEFT]
oGet:BLOCK     := {|| aLine}
oGet:EXITSTATE := GE_ESCAPE
oGet:COLORSPEC := cGetColor + ',' + cGetColor

BEGIN SEQUENCE

  McMeBreak(cOrig, @aLine, aMemoSys)
  AADD(aLine, {SPACE(aMemoSys[MSV_WIDTH]), .N.})
  nLine := aMemoSys[MSV_WINROW] + 1
  oBrowse:FORCESTABLE()
  FOR nPos := aMemoSys[MSV_WINROW] + 2 TO aMemoSys[MSV_BUFROW]
    oBrowse:DOWN()
  NEXT
  oBrowse:FORCESTABLE()
  IF lMeCursor
    bMeCursor:EVAL(1, nKey, oBrowse, oGet, aLine, nLine)
  END

  WHILE lEdit
    McShowScoreboard()
    aMemoSys[MSV_GETPOS] := ;
      IF(!EMPTY(aMemoSys[MSV_SETPOS]), aMemoSys[MSV_SETPOS], oGet:POS)
    aMemoSys[MSV_INSERT] := SET(_SET_INSERT)
    IF !EMPTY(aMemoSys[MSV_SETPOS]) .OR. oGet:EXITSTATE # GE_NOEXIT
      McMeMove(@oBrowse, @oGet, @aLine, @nLine, @aMemoSys)
    END
    IF lMeCursor
      bMeCursor:EVAL(2, nKey, oBrowse, oGet, aLine, nLine)
    END
    IF aMemoSys[MSV_ENTER]
      nKey  := McInKey(0)
      lEdit := McMeApplyKey(nKey, @oBrowse, @oGet, @aLine, @nLine, ;
        @aMemoSys, aSetKey, aSetArea)
    ELSE
      lEdit := .N.
    END
  END
RECOVER USING xBreak
  IF !EMPTY(xBreak)
    BREAK xBreak
  END
END SEQUENCE

IF lMeCursor
  bMeCursor:EVAL(3, nKey, oBrowse, oGet, aLine, nLine)
END
IF oGet:CHANGED
  oGet:ASSIGN()
END
IF oGet:EXITSTATE == GE_ESCAPE
  cBuffer := cOrig
ELSE
  cBuffer := McMeJoin(aLine, 1)
END
IF !EMPTY(nKey)
  McLastKey(nKey)
END

RETURN cBuffer  //  McMemoEdit

//---------------------------------------------------------------------------

FUNCTION McMeApplyKey(nKey, oBrowse, oGet, aLine, nLine, aMemoSys, ;
  aSetKey, aSetArea)

LOCAL lEdit   := .Y.
LOCAL lMode   := aMemoSys[MSV_MODE]
LOCAL lBOA    := (nLine <= 1)
LOCAL lEOA    := (nLine >= LEN(aLine))
LOCAL lBOG    := (aMemoSys[MSV_GETPOS] <= 1)
LOCAL lEOG    := (aMemoSys[MSV_GETPOS] >= aMemoSys[MSV_WIDTH])
LOCAL lEOC    := (EMPTY(SUBSTR(oGet:BUFFER, aMemoSys[MSV_GETPOS])))
LOCAL lEOL    := (lEOG .OR. lEOC)
LOCAL lBOT    := (!lBOA .AND. lBOG)
LOCAL lEOT    := (!lEOA .AND. lEOC)
LOCAL lEOB    := ( lEOA .AND. lEOL)
LOCAL lDLine  := (!lBOA .AND. (lBOG .OR. EMPTY(oGet:BUFFER)))
LOCAL cKey    := CHR(nKey)
LOCAL bKey    := NIL
LOCAL nRow    := ROW()
LOCAL nCol    := COL()
LOCAL nMRow   := MRow()
LOCAL nMCol   := MCol()
LOCAL cBuffer := ''
LOCAL lReturn := .N.
LOCAL nPos    := 0
LOCAL lPrompt := .N.
LOCAL aKeyPad := {}

IF aMemoSys[MSV_CLOSE]
  AADD(aKeyPad, ;
    {aMemoSys[MSV_BOXTOP]     , aMemoSys[MSV_BOXLEFT] , ;
     aMemoSys[MSV_BOXTOP]     , aMemoSys[MSV_BOXLEFT] , K_ESC       , .Y.})
END
IF aMemoSys[MSV_RESIZE]
  AADD(aKeyPad, ;
    {aMemoSys[MSV_BOXTOP]     , aMemoSys[MSV_BOXRIGHT], ;
     aMemoSys[MSV_BOXTOP]     , aMemoSys[MSV_BOXRIGHT], KP_ALT_ENTER, .Y.})
END
IF aMemoSys[MSV_VSCROLL]
  AADD(aKeyPad, ;
    {aMemoSys[MSV_BOXTOP]+ 1  , aMemoSys[MSV_BOXLEFT] , ;
     aMemoSys[MSV_ROWMID]     , aMemoSys[MSV_BOXLEFT] , K_UP        , .Y.})
  AADD(aKeyPad, ;
    {aMemoSys[MSV_ROWMID]+ 1  , aMemoSys[MSV_BOXLEFT] , ;
     aMemoSys[MSV_BOXBOTTOM]-1, aMemoSys[MSV_BOXLEFT] , K_DOWN      , .Y.})
  AADD(aKeyPad, ;
    {aMemoSys[MSV_BOXTOP]+ 2  , aMemoSys[MSV_BOXRIGHT], ;
     aMemoSys[MSV_ROWMID]     , aMemoSys[MSV_BOXRIGHT], K_PGUP      , .Y.})
  AADD(aKeyPad, ;
    {aMemoSys[MSV_ROWMID]+ 1  , aMemoSys[MSV_BOXRIGHT], ;
     aMemoSys[MSV_BOXBOTTOM]-2, aMemoSys[MSV_BOXRIGHT], K_PGDN      , .Y.})
  AADD(aKeyPad, ;
    {aMemoSys[MSV_BOXTOP]+ 1  , aMemoSys[MSV_BOXRIGHT], ;
     aMemoSys[MSV_BOXTOP]+ 1  , aMemoSys[MSV_BOXRIGHT], K_CTRL_PGUP , .Y.})
  AADD(aKeyPad, ;
    {aMemoSys[MSV_BOXBOTTOM]-1, aMemoSys[MSV_BOXRIGHT], ;
     aMemoSys[MSV_BOXBOTTOM]-1, aMemoSys[MSV_BOXRIGHT], K_CTRL_PGDN , .Y.})
END

aMemoSys[MSV_SETPOS]  := 0
aMemoSys[MSV_TYPEOUT] := ''
aMemoSys[MSV_REFORM]  := .N.
nKey := McTransPad(aKeyPad, nKey)
bKey := McTransBlock(nKey, aSetKey, aSetArea)
DO CASE
CASE !EMPTY(bKey)
  McMeDoSetKey(bKey, @oGet, @oBrowse, @lEdit, @aLine, @nLine, @aMemoSys)
CASE nKey == K_ESC
  lPrompt := (SET(_SET_SCOREBOARD) .AND. aMemoSys[MSV_MODE])
  IF !lPrompt .OR. ;
    (lPrompt .AND. McAlert('Abort edit?', {'Yes', 'No'},, 2) == 1)
    oGet:EXITSTATE := GE_ESCAPE
    lEdit := .N.
  END
CASE nKey == K_CTRL_W .AND. lMode
  oGet:EXITSTATE := GE_WRITE
  IF oGet:CHANGED
    oGet:ASSIGN()
  END
  lEdit := .N.
CASE nKey == K_UP
  oGet:EXITSTATE := GE_WRITE
  oBrowse:UP()
CASE nKey == K_DOWN
  oGet:EXITSTATE := GE_WRITE
  oBrowse:DOWN()
CASE nKey == K_PGUP
  oGet:EXITSTATE := GE_WRITE
  oBrowse:PAGEUP()
CASE nKey == K_PGDN
  oGet:EXITSTATE := GE_WRITE
  oBrowse:PAGEDOWN()
CASE nKey == K_CTRL_PGUP
  oGet:EXITSTATE := GE_WRITE
  oBrowse:GOTOP()
CASE nKey == K_CTRL_PGDN
  oGet:EXITSTATE := GE_WRITE
  oBrowse:GOBOTTOM()
CASE nKey == K_INS
  SET(_SET_INSERT, !SET(_SET_INSERT))
CASE nKey == K_CTRL_U
  oGet:UNDO()
CASE nKey == K_HOME
  oGet:HOME()
CASE nKey == K_END
  oGet:END()
CASE nKey == K_RIGHT .AND. !lEOL
  oGet:RIGHT()
CASE nKey == K_LEFT .AND. !lBOG
  oGet:LEFT()
CASE nKey == K_CTRL_RIGHT .AND. !lEOL
  oGet:WORDRIGHT()
CASE nKey == K_CTRL_LEFT .AND. !lBOG
  oGet:WORDLEFT()
CASE (nKey == K_RIGHT .OR. nKey == K_CTRL_RIGHT) .AND. lEOL .AND. !lEOA
  oBrowse:DOWN()
  aMemoSys[MSV_SETPOS] := 1
CASE (nKey == K_LEFT .OR. nKey == K_CTRL_LEFT) .AND. lBOG .AND. !lBOA
  oBrowse:UP()
  aMemoSys[MSV_SETPOS] := aMemoSys[MSV_WIDTH]
CASE nKey == K_BS .AND. !lBOT .AND. lMode
  oGet:BACKSPACE()
CASE nKey == K_CTRL_BS .AND. !lBOT .AND. lMode
  oGet:DELWORDLEFT()
CASE nKey == K_DEL .AND. !lEOT .AND. lMode
  oGet:DELETE()
CASE (nKey == K_CTRL_DEL .OR. nKey == K_CTRL_T) .AND. !lEOT .AND. lMode
  oGet:DELWORDRIGHT()
CASE (nKey == K_BS .OR. nKey == K_CTRL_BS) .AND. lBOT .AND. lMode
  oGet:EXITSTATE        := GE_WRITE
  oGet:SUBSCRIPT        := {nLine-1, 1}
  cBuffer               := RTRIM(aLine[nLine-1][1]) + ' '
  aMemoSys[MSV_SETPOS]  := LEN(cBuffer) + 1
  cBuffer               += oGet:BUFFER
  oGet:BUFFER           := LEFT(cBuffer, aMemoSys[MSV_WIDTH])
  oGet:CHANGED          := .Y.
  aMemoSys[MSV_TYPEOUT] := SUBSTR(cBuffer, aMemoSys[MSV_WIDTH]+1)
  aMemoSys[MSV_REFORM]  := .Y.
  ADEL(aLine, nLine-1)
  ASIZE(aLine, LEN(aLine) - 1)
  nLine --
  oBrowse:UP()
CASE (nKey == K_DEL .OR. nKey == K_CTRL_DEL .OR. nKey == K_CTRL_T) .AND. ;
  lEOT .AND. lMode
  oGet:EXITSTATE        := GE_WRITE
  cBuffer               := LEFT(oGet:BUFFER, aMemoSys[MSV_GETPOS]-1) + ;
    aLine[nLine+1][1]
  oGet:BUFFER           := LEFT(cBuffer, aMemoSys[MSV_WIDTH])
  oGet:CHANGED          := .Y.
  aMemoSys[MSV_TYPEOUT] := SUBSTR(cBuffer, aMemoSys[MSV_WIDTH]+1)
  aMemoSys[MSV_REFORM]  := .Y.
  aLine[nLine][2]       := aLine[nLine+1][2]
  ADEL(aLine, nLine+1)
  ASIZE(aLine, LEN(aLine) - 1)
CASE nKey == K_CTRL_Y .AND. !lDLine .AND. lMode
  oGet:DELEND()
CASE nKey == K_CTRL_Y .AND. lDLine .AND. lMode
  oGet:EXITSTATE := GE_WRITE
  ADEL(aLine, nLine)
  ASIZE(aLine, LEN(aLine) - 1)
CASE nKey == K_SH_TAB
  aMemoSys[MSV_SETPOS] := ;
    (INT( (aMemoSys[MSV_GETPOS]-2)/aMemoSys[MSV_TAB] )-1) * ;
    aMemoSys[MSV_TAB] + 1
CASE nKey == K_TAB
  aMemoSys[MSV_SETPOS] := ;
    (INT( (aMemoSys[MSV_GETPOS]-1)/aMemoSys[MSV_TAB] )+1) * ;
    aMemoSys[MSV_TAB] + 1
  IF aMemoSys[MSV_INSERT] .AND. lMode
    cBuffer               := STUFF(oGet:BUFFER, aMemoSys[MSV_GETPOS], 0, ;
      SPACE(aMemoSys[MSV_SETPOS] - aMemoSys[MSV_GETPOS]))
    oGet:BUFFER           := LEFT(cBuffer, aMemoSys[MSV_WIDTH])
    oGet:CHANGED          := .Y.
    aMemoSys[MSV_TYPEOUT] := SUBSTR(cBuffer, aMemoSys[MSV_WIDTH]+1)
  END
CASE nKey == K_ENTER .AND. lMode
  IF aMemoSys[MSV_INSERT] .OR. lEOB
    ASIZE(aLine, LEN(aLine) + 1)
    AINS(aLine, nLine + 1)
    cBuffer := aLine[nLine][1]
    lReturn := aLine[nLine][2]
    aLine[nLine  ][1] :=  PADR(LEFT(cBuffer, aMemoSys[MSV_GETPOS]-1), ;
      aMemoSys[MSV_WIDTH])
    aLine[nLine+1]    := ARRAY(2)
    aLine[nLine+1][1] := PADR(SUBSTR(cBuffer, aMemoSys[MSV_GETPOS]), ;
      aMemoSys[MSV_WIDTH])
    aLine[nLine+1][2] := lReturn
  END
  aLine[nLine][2] := .Y.
  oBrowse:DOWN()
  aMemoSys[MSV_SETPOS] := 1
CASE nKey >= 32 .AND. nKey <= 255 .AND. lMode
  IF aMemoSys[MSV_INSERT] .OR. lEOB
    DO CASE
    CASE !lEOG .OR. (EMPTY(RIGHT(oGet:BUFFER, 1)) .AND. !EMPTY(cKey))
      aMemoSys[MSV_TYPEOUT] := RIGHT(oGet:BUFFER, 1)
      oGet:INSERT(cKey)
    CASE EMPTY(cKey)
      IF lEOB
        ASIZE(aLine, LEN(aLine) + 1)
        AINS(aLine, nLine + 1)
        lReturn := aLine[nLine][2]
        aLine[nLine  ][2] := .N.
        aLine[nLine+1]    := ARRAY(2)
        aLine[nLine+1][1] := SPACE(aMemoSys[MSV_WIDTH])
        aLine[nLine+1][2] := lReturn
      END
      oBrowse:DOWN()
      aMemoSys[MSV_SETPOS]  := 1
    OTHERWISE
      oGet:EXITSTATE        := GE_WRITE
      aMemoSys[MSV_TYPEOUT] := cKey
      aMemoSys[MSV_SETPOS]  := oGet:POS + 2
      aMemoSys[MSV_REFORM]  := .Y.
    END
  ELSE
    oGet:OVERSTRIKE(cKey)
    IF lEOG .AND. !lEOA
      oGet:EXITSTATE := GE_WRITE
      oBrowse:DOWN()
      aMemoSys[MSV_SETPOS] := 1
    END
  END
CASE nKey == K_CLICK .OR. nKey == K_DOUBLECLICK
  DO CASE
  CASE !McInBox(nMRow, nMCol, ;
    aMemoSys[MSV_BOXTOP], aMemoSys[MSV_BOXLEFT], ;
    aMemoSys[MSV_BOXBOTTOM], aMemoSys[MSV_BOXRIGHT])
    oGet:EXITSTATE := GE_WRITE
    lEdit := .N.
  CASE McInBox(nMRow, nMCol, ;
    nRow, aMemoSys[MSV_LEFT], nRow, aMemoSys[MSV_RIGHT])
    DO CASE
    CASE nMCol < nCol
      FOR nPos := nCol - 1 TO nMCol STEP -1
        oGet:LEFT()
      NEXT
    CASE nMCol > nCol
      FOR nPos := nCol + 1 TO nMCol
        oGet:RIGHT()
      NEXT
    END
  CASE McInBox(nMRow, nMCol, aMemoSys[MSV_TOP], aMemoSys[MSV_LEFT], ;
    aMemoSys[MSV_BOTTOM], aMemoSys[MSV_RIGHT])
    aMemoSys[MSV_SETPOS] := nMCol - aMemoSys[MSV_LEFT] + 1
    DO CASE
    CASE nMRow < nRow
      FOR nPos := nRow - 1 TO nMRow STEP -1
        oBrowse:UP()
      NEXT
    CASE nMRow > nRow
      FOR nPos := nRow + 1 TO nMRow
        oBrowse:DOWN()
      NEXT
    END
  END
END
IF (nKey == K_CTRL_P .OR. !EMPTY(aMemoSys[MSV_TYPEOUT])) .AND. lMode
  oGet:EXITSTATE := GE_WRITE
  aMemoSys[MSV_REFORM] := .Y.
END

RETURN lEdit  //  McMeApplyKey

//---------------------------------------------------------------------------

PROCEDURE McMeMove(oBrowse, oGet, aLine, nLine, aMemoSys)

LOCAL aPos    := {}
LOCAL nPos    := 1
LOCAL cBuffer := ''

IF oGet:CHANGED
  oGet:ASSIGN()
END
oGet:KILLFOCUS()
IF aMemoSys[MSV_REFORM]
  aLine[nLine][1] += aMemoSys[MSV_TYPEOUT]
  cBuffer := McMeJoin(aLine, nLine)
  ASIZE(aLine, nLine-1)
  McMeBreak(cBuffer, @aLine, aMemoSys)
  aPos := MPOSTOLC(cBuffer, aMemoSys[MSV_WIDTH], ;
    aMemoSys[MSV_GETPOS], aMemoSys[MSV_TAB])
  FOR nPos := 2 TO aPos[1]
    oBrowse:DOWN()
  NEXT
  aMemoSys[MSV_GETPOS] := aPos[2] + 1
END
oBrowse:REFRESHALL()
oBrowse:FORCESTABLE()
McScrollBox(nLine, LEN(aLine))
oGet:ROW       := ROW()
oGet:SUBSCRIPT := {nLine, 1}
oGet:EXITSTATE := GE_NOEXIT
oGet:SETFOCUS()
FOR nPos := 2 TO aMemoSys[MSV_GETPOS]
  oGet:RIGHT()
NEXT

RETURN  //  McMeMove

//---------------------------------------------------------------------------

PROCEDURE McMeBreak(cString, aLine, aMemoSys)

LOCAL nWidth  := aMemoSys[MSV_WIDTH]
LOCAL nTab    := aMemoSys[MSV_TAB]
LOCAL nLine   := 1
LOCAL nLines  := MLCOUNT(cString, nWidth, nTab)
LOCAL cLine   := ''
LOCAL cReturn := CRLF
LOCAL lReturn := .N.
LOCAL nSearch := 0
LOCAL nRow    := ROW()
LOCAL nCol    := COL()
LOCAL bMeRead := McSet(MCSET_MEREAD)

IF SET(_SET_SCOREBOARD)
  SETPOS(SCORE_ROW, SCORE_COL)
  DISPOUT('Reading')
  SETPOS(nRow, nCol)
END
McMeReadShow(bMeRead, nLine, nLines, 1, aMemoSys)
WHILE !EMPTY(LEN(cString))
  nSearch := MLCTOPOS(cString, nWidth, 2, 0, nTab)
  nLine   ++
  McMeReadShow(bMeRead, nLine, nLines, 2, aMemoSys)
  cLine   := LEFT(cString, nSearch - 1)
  lReturn := cReturn $ cLine
  cLine   := STRTRAN(cLine, cReturn)
  AADD(aLine, {PADR(cLine, nWidth), lReturn})
  cString := SUBSTR(cString, nSearch)
END
nLines := nLines
IF SET(_SET_SCOREBOARD)
  SETPOS(SCORE_ROW, SCORE_COL)
  DISPOUT('       ')
  SETPOS(nRow, nCol)
END
McMeReadShow(bMeRead, nLine, nLines, 3, aMemoSys)

RETURN  //  McMeBreak

//---------------------------------------------------------------------------

PROCEDURE McMeReadShow(bMeRead, nLine, nLines, nMode, aMemoSys)

LOCAL lPrompt := (SET(_SET_SCOREBOARD) .AND. aMemoSys[MSV_MODE])

IF !EMPTY(bMeRead)
  bMeRead:EVAL(nLine, nLines, nMode)
END
IF INKEY() == K_ESC .AND. (!lPrompt .OR. ;
  (lPrompt .AND. McAlert('Abort edit?', {'Yes', 'No'},, 2) == 1))
  BREAK
END

RETURN  //  McMeReadShow

//---------------------------------------------------------------------------

FUNCTION McMeJoin(aLine, nStart)

LOCAL cBuffer := ''
LOCAL nPos    := nStart
LOCAL nStop   := IF(EMPTY(ATAIL(aLine)[1]), LEN(aLine)-1, LEN(aLine))

FOR nPos := nStart TO nStop
  cBuffer += RTRIM(aLine[nPos][1]) + ;
    IF(aLine[nPos][2], C_HARD_CR, ' ' + C_SOFT_CR) + C_LINE_FEED
NEXT

RETURN cBuffer  //  McMeJoin

//---------------------------------------------------------------------------

PROCEDURE McMeDoSetKey(bKey, oGet, oBrowse, lEdit, aLine, nLine, aMemoSys)

LOCAL lUpdated := .N.
LOCAL xExpr    := NIL
LOCAL cType    := ''
LOCAL cBuffer  := ''

IF oGet:CHANGED
  oGet:ASSIGN()
ENDIF

bKey:EVAL(PROCNAME(1), PROCLINE(1), '', ;
  @oBrowse, @oGet, @lEdit, @aLine, @nLine, @aMemoSys)
xExpr := McSaveItem(, .N.)
cType := VALTYPE(xExpr)
IF xExpr # NIL
  DO CASE
  CASE cType == 'B'
    xExpr:EVAL(oGet, @bKey, @oBrowse, @aLine, @nLine, @aMemoSys, @lEdit)
    oGet:UPDATEBUFFER()
  CASE cType == 'C' .AND. oGet:TYPE == 'C'
    oGet:EXITSTATE        := GE_WRITE
    cBuffer               := LEFT(oGet:BUFFER, oGet:POS-1) + ;
      xExpr + SUBSTR(oGet:BUFFER, oGet:POS)
    oGet:BUFFER           := LEFT(cBuffer, aMemoSys[MSV_WIDTH])
    oGet:CHANGED          := .Y.
    aMemoSys[MSV_TYPEOUT] := SUBSTR(cBuffer, aMemoSys[MSV_WIDTH]+1)
    aMemoSys[MSV_REFORM]  := .Y.
  END
END
McShowScoreboard()

RETURN  // McMeDoSetKey

//---------------------------------------------------------------------------

FUNCTION McMemoArray(cString, nWidth, nTab)

LOCAL cMemo   := cString
LOCAL nPos    := 0
LOCAL cLine   := ''
LOCAL cReturn := CRLF
LOCAL lReturn := .N.
LOCAL aMemo   := {}

WHILE !EMPTY(LEN(cMemo))
  nPos    := MLCTOPOS(cMemo, nWidth, 2, 0, nTab)
  cLine   := LEFT(cMemo, nPos - 1)
  lReturn := cReturn $ cLine
  cLine   := STRTRAN(cLine, cReturn)
  AADD(aMemo, {cLine, lReturn})
  cMemo   := SUBSTR(cMemo, nPos)
END

RETURN aMemo  //  McMemoArray

//---------------------------------------------------------------------------

FUNCTION McReadModal(aGetList, nPos, aSetKey, aSetArea)

LOCAL oGet         := NIL
LOCAL aGetSys      := ARRAY(GSV_COUNT)
LOCAL lUpdated     := .N.
LOCAL xBreak       := NIL
LOCAL nGet         := 1
LOCAL aSetBox      := McSetBox()
LOCAL nBoxTop      := aSetBox[SB_TOP        ]
LOCAL nBoxLeft     := aSetBox[SB_LEFT       ]
LOCAL nBoxBot      := aSetBox[SB_BOTTOM     ]
LOCAL nBoxRight    := aSetBox[SB_RIGHT      ]
LOCAL lBoxClose    := aSetBox[SB_CLOSE      ]
LOCAL lBoxDefined  := aSetBox[SB_DEFINED    ]

aGetSys[GSV_UPDATED     ] := .N.
aGetSys[GSV_LENGTH      ] := {}
aGetSys[GSV_BOXTOP      ] := nBoxTop
aGetSys[GSV_BOXLEFT     ] := nBoxLeft
aGetSys[GSV_BOXBOTTOM   ] := nBoxBot
aGetSys[GSV_BOXRIGHT    ] := nBoxRight
aGetSys[GSV_BOXCLOSE    ] := lBoxClose
aGetSys[GSV_BOXDEFINED  ] := lBoxDefined
aGetSys[GSV_BUMPTOP     ] := .N.
aGetSys[GSV_BUMPBOT     ] := .N.
aGetSys[GSV_LASTEXIT    ] := 0
aGetSys[GSV_LASTPOS     ] := 0
aGetSys[GSV_ACTIVEGET   ] := NIL
aGetSys[GSV_READVAR     ] := READVAR('')
aGetSys[GSV_READPROCNAME] := PROCNAME(1)
aGetSys[GSV_READPROCLINE] := PROCLINE(1)
McReadKill(.N.)

BEGIN SEQUENCE

  IF EMPTY(aGetList)
    BREAK
  END

  FOR nGet := 1 TO LEN(aGetList)
    oGet := aGetList[nGet]
    oGet:SETFOCUS()
    AADD(aGetSys[GSV_LENGTH], LEN(oGet:BUFFER))
    oGet:KILLFOCUS()
  NEXT

  IF !(VALTYPE(nPos) == 'N' .AND. nPos > 0)
    nPos := McSettle(aGetList, 0, @aGetSys)
  END

  WHILE !EMPTY(nPos)
    GETACTIVE(oGet := aGetList[nPos])
    READVAR(McGetReadVar(oGet))
    McShowScoreboard()
    IF !EMPTY(oGet:READER)
      oGet:READER:EVAL(oGet, aGetList, nPos, @aGetSys, aSetKey, aSetArea)
    ELSE
      McGetReader(oGet, aGetList, nPos, @aGetSys, aSetKey, aSetArea)
    END
    nPos := McSettle(aGetList, nPos, @aGetSys)
  END

RECOVER USING xBreak

  IF !EMPTY(xBreak)
    BREAK xBreak
  END

END SEQUENCE

SETPOS(MAXROW() - 1, 0)
lUpdated := aGetSys[GSV_UPDATED]
McUpdated(lUpdated)

RETURN lUpdated  //  McReadModal

//---------------------------------------------------------------------------

PROCEDURE McGetReader(oGet, aGetList, nPos, aGetSys, aSetKey, aSetArea)

LOCAL bRCursor := McSet(MCSET_RCURSOR)
LOCAL lRCursor := !EMPTY(bRCursor)

IF McGetPreValidate(oGet, @aGetSys)
  oGet:SETFOCUS()
  WHILE oGet:EXITSTATE == GE_NOEXIT
    IF oGet:TYPEOUT
      oGet:EXITSTATE := GE_ENTER
    END
    IF lRCursor
      bRCursor:EVAL(1, oGet)
    END
    WHILE oGet:EXITSTATE == GE_NOEXIT
      IF lRCursor
        bRCursor:EVAL(2, oGet)
      END
      McGetApplyKey(oGet, McInKey(0), aGetList, nPos, @aGetSys, ;
        aSetKey, aSetArea)
    END
    IF lRCursor
      bRCursor:EVAL(3, oGet)
    END
    IF !McGetPostValidate(oGet, @aGetSys)
      oGet:EXITSTATE := GE_NOEXIT
    END
  END
  oGet:KILLFOCUS()
END

RETURN  //  McGetReader

//---------------------------------------------------------------------------

PROCEDURE McGetApplyKey(oGet, nKey, aGetList, nPos, aGetSys, ;
  aSetKey, aSetArea)

LOCAL bKey        := McTransBlock(nKey, aSetKey, aSetArea)
LOCAL cKey        := CHR(nKey)
LOCAL nRow        := ROW()
LOCAL nCol        := COL()
LOCAL nMRow       := MRow()
LOCAL nMCol       := MCol()
LOCAL nCurPos     := 0
LOCAL nSetPos     := 0
LOCAL oSetGet     := oGet
LOCAL lBoxDefined := aGetSys[GSV_BOXDEFINED]
LOCAL nBoxTop     := IF(lBoxDefined, aGetSys[GSV_BOXTOP   ], oGet:ROW)
LOCAL nBoxLeft    := IF(lBoxDefined, aGetSys[GSV_BOXLEFT  ], oGet:COL)
LOCAL nBoxBottom  := IF(lBoxDefined, aGetSys[GSV_BOXBOTTOM], oGet:ROW)
LOCAL nBoxRight   := IF(lBoxDefined, aGetSys[GSV_BOXRIGHT ], ;
  oGet:COL + LEN(oGet:BUFFER) - 1)
LOCAL lBoxClose   := IF(lBoxDefined, aGetSys[GSV_BOXCLOSE ], .N.)

DO CASE
CASE !EMPTY(bKey)
  McGetDoSetKey(bKey, @oGet, @aGetSys)
CASE nKey == K_UP
  oGet:EXITSTATE := GE_UP
CASE nKey == K_SH_TAB
  oGet:EXITSTATE := GE_UP
CASE nKey == K_DOWN
  oGet:EXITSTATE := GE_DOWN
CASE nKey == K_TAB
  oGet:EXITSTATE := GE_DOWN
CASE nKey == K_ENTER
  oGet:EXITSTATE := GE_ENTER
CASE nKey == K_ESC .AND. SET(_SET_ESCAPE)
  oGet:UNDO()
  oGet:EXITSTATE := GE_ESCAPE
CASE nKey == K_PGUP
  oGet:EXITSTATE := GE_WRITE
CASE nKey == K_PGDN
  oGet:EXITSTATE := GE_WRITE
CASE nKey == K_CTRL_HOME
  oGet:EXITSTATE := GE_WRITE
CASE nKey == K_CTRL_END
  oGet:EXITSTATE := GE_WRITE
CASE nKey == K_CTRL_PGUP
  oGet:EXITSTATE := GE_TOP
CASE nKey == K_CTRL_PGDN
  oGet:EXITSTATE := GE_BOTTOM
CASE nKey == K_INS
  SET(_SET_INSERT, !SET(_SET_INSERT))
  McShowScoreboard()
CASE nKey == K_CTRL_U
  oGet:UNDO()
CASE nKey == K_HOME
  oGet:HOME()
CASE nKey == K_END
  oGet:END()
CASE nKey == K_RIGHT
  oGet:RIGHT()
CASE nKey == K_LEFT
  oGet:LEFT()
CASE nKey == K_CTRL_RIGHT
  oGet:WORDRIGHT()
CASE nKey == K_CTRL_LEFT
  oGet:WORDLEFT()
CASE nKey == K_BS
  oGet:BACKSPACE()
CASE nKey == K_DEL
  oGet:DELETE()
CASE nKey == K_CTRL_T
  oGet:DELWORDRIGHT()
CASE nKey == K_CTRL_Y
  oGet:DELEND()
CASE nKey == K_CTRL_BS
  oGet:DELWORDLEFT()
CASE nKey >= 32 .AND. nKey <= 255
  DO CASE
  CASE oGet:TYPE == 'N' .AND. (cKey == '.' .OR. cKey == ',')
    oGet:TODECPOS()
  CASE SET(_SET_INSERT)
    oGet:INSERT(cKey)
  OTHERWISE
    oGet:OVERSTRIKE(cKey)
  END
  IF oGet:TYPEOUT
    IF SET(_SET_BELL)
      ?? CHR(7)
    ENDIF
    IF !SET(_SET_CONFIRM)
      oGet:EXITSTATE := GE_ENTER
    END
  END
CASE nKey == K_CLICK .OR. nKey == K_DOUBLECLICK
  DO CASE
  CASE McInBox(nMRow, nMCol, ;
    oGet:ROW, oGet:COL, oGet:ROW, oGet:COL + LEN(oGet:BUFFER) - 1)
    DO CASE
    CASE nMCol < nCol
      FOR nCurPos := nCol - 1 TO nMCol STEP -1
        oGet:LEFT()
      NEXT
    CASE nMCol > nCol
      FOR nCurPos := nCol + 1 TO nMCol
        oGet:RIGHT()
      NEXT
    END
  CASE !lBoxDefined
    oGet:EXITSTATE := GE_ENTER
  CASE lBoxClose .AND. nMRow == nBoxTop .AND. nMCol == nBoxLeft
    oGet:UNDO()
    oGet:EXITSTATE := GE_ESCAPE
    McLastKey(K_ESC)
  CASE !McInBox(nMRow, nMCol, nBoxTop, nBoxLeft, nBoxBottom, nBoxRight)
    oGet:EXITSTATE := GE_WRITE
  OTHERWISE
    FOR nSetPos := 1 TO LEN(aGetList)
      oSetGet := aGetList[nSetPos]
      IF McInBox(nMRow, nMCol, ;
        oSetGet:ROW, oSetGet:COL, oSetGet:ROW, ;
        oSetGet:COL + aGetSys[GSV_LENGTH][nSetPos] - 1)
        nCurPos := nSetPos
      END
    NEXT
    DO CASE
    CASE !EMPTY(nCurPos)
      oGet:EXITSTATE := - nCurPos
    CASE MReadExit()
      oGet:EXITSTATE := GE_WRITE
    END
  END
END

RETURN  //  McGetApplyKey

//---------------------------------------------------------------------------

FUNCTION McGetPreValidate(oGet, aGetSys)

LOCAL lUpdated := aGetSys[GSV_UPDATED]
LOCAL lWhen    := .Y.

IF !EMPTY(oGet:PREBLOCK)
  lWhen := oGet:PREBLOCK:EVAL(oGet)
  oGet:DISPLAY()
  McShowScoreBoard()
  aGetSys[GSV_UPDATED] := lUpdated
END

DO CASE
CASE McReadKill()
  lWhen := .N.
  oGet:EXITSTATE := GE_ESCAPE
CASE !lWhen
  oGet:EXITSTATE := GE_WHEN
OTHERWISE
  oGet:EXITSTATE := GE_NOEXIT
END

RETURN lWhen  //  McGetPreValidate

//---------------------------------------------------------------------------

FUNCTION McGetPostValidate(oGet, aGetSys)

LOCAL lUpdated := aGetSys[GSV_UPDATED]
LOCAL lValid   := .Y.
LOCAL xBreak   := NIL
LOCAL nRow     := ROW()
LOCAL nCol     := COL()

BEGIN SEQUENCE

  IF oGet:EXITSTATE == GE_ESCAPE
    BREAK
  END
  IF oGet:BADDATE()
    oGet:HOME()
    IF SET(_SET_SCOREBOARD)
      SETPOS(SCORE_ROW, SCORE_COL)
      DISPOUT("Invalid Date")
      SETPOS(nRow, nCol)
      WHILE NEXTKEY() == 0
      END
      SETPOS(SCORE_ROW, SCORE_COL)
      DISPOUT(SPACE(12))
      SETPOS(nRow, nCol)
    END
    McShowScoreboard()
    lValid := .N.
    BREAK
  END
  IF oGet:CHANGED
    oGet:ASSIGN()
    aGetSys[GSV_UPDATED] := .Y.
  END
  oGet:RESET()
  IF !EMPTY(oGet:POSTBLOCK)
    lUpdated := aGetSys[GSV_UPDATED]
    SETPOS(oGet:ROW, oGet:COL + LEN(oGet:BUFFER))
    lValid := oGet:POSTBLOCK:EVAL(oGet)
    SETPOS(oGet:ROW, oGet:COL)
    McShowScoreBoard()
    oGet:UPDATEBUFFER()
    aGetSys[GSV_UPDATED] := lUpdated
    IF McReadKill()
      oGet:EXITSTATE := GE_ESCAPE
      lValid := .Y.
    END
  END

RECOVER USING xBreak

  IF !EMPTY(xBreak)
    BREAK xBreak
  END

END SEQUENCE

RETURN lValid  //  McGetPostValidate

//---------------------------------------------------------------------------

PROCEDURE McGetDoSetKey(bKey, oGet, aGetSys)

LOCAL lUpdated := .N.
LOCAL xExpr    := NIL
LOCAL cType    := ''

IF oGet:CHANGED
  oGet:ASSIGN()
  aGetSys[GSV_UPDATED] := .Y.
ENDIF

lUpdated := aGetSys[GSV_UPDATED]
bKey:EVAL(aGetSys[GSV_READPROCNAME], aGetSys[GSV_READPROCLINE], READVAR(), ;
  @oGet, @aGetSys)
xExpr := McSaveItem(, .N.)
cType := VALTYPE(xExpr)
IF xExpr # NIL
  DO CASE
  CASE cType == 'B'
    xExpr:EVAL(oGet, @bKey, @aGetSys)
  CASE cType == 'C' .AND. oGet:TYPE == 'C'
    oGet:VARPUT(PADR(xExpr, LEN(oGet:BUFFER)))
  CASE cType == oGet:TYPE
    oGet:VARPUT(xExpr)
  END
END
McShowScoreboard()
oGet:UPDATEBUFFER()
aGetSys[GSV_UPDATED] := lUpdated

IF McReadKill()
  oGet:EXITSTATE := GE_ESCAPE
ENDIF

RETURN  // McGetDoSetKey

//---------------------------------------------------------------------------

FUNCTION McSettle(aGetList, nPos, aGetSys)

LOCAL nExitState := IF(EMPTY(nPos), GE_DOWN, aGetList[nPos]:EXITSTATE)
LOCAL nExitPos   := nPos
LOCAL xBreak     := NIL

BEGIN SEQUENCE

  IF nExitState == GE_ESCAPE .OR. nExitState == GE_WRITE
    nExitPos := 0
    BREAK
  ENDIF

  IF nExitState # GE_WHEN
    aGetSys[GSV_LASTPOS] := nPos
    aGetSys[GSV_BUMPTOP] := .N.
    aGetSys[GSV_BUMPBOT] := .N.
  ELSE
    nExitState := aGetSys[GSV_LASTEXIT]
  ENDIF

  DO CASE
  CASE nExitState == GE_UP
    nPos--
  CASE nExitState == GE_DOWN
    nPos++
  CASE nExitState == GE_TOP
    nPos := 1
    aGetSys[GSV_BUMPTOP] := .Y.
    nExitState := GE_DOWN
  CASE nExitState == GE_BOTTOM
    nPos := LEN(aGetList)
    aGetSys[GSV_BUMPBOT] := .Y.
    nExitState := GE_UP
  CASE nExitState == GE_ENTER
    nPos++
  CASE nExitState < 0
    nPos := - nExitState
  ENDCASE

  DO CASE
  CASE EMPTY(nPos)
    IF !READEXIT() .AND. !aGetSys[GSV_BUMPBOT]
      aGetSys[GSV_BUMPTOP] := .Y.
      nPos := aGetSys[GSV_LASTPOS]
      nExitState := GE_DOWN
    END
  CASE nPos > LEN(aGetList)
    IF !READEXIT() .AND. nExitState # GE_ENTER .AND. !aGetSys[GSV_BUMPTOP]
      aGetSys[GSV_BUMPBOT] := .Y.
      nPos := aGetSys[GSV_LASTPOS]
      nExitState := GE_UP
    ELSE
      nPos := 0
    END
  END

  aGetSys[GSV_LASTEXIT] := nExitState
  IF !EMPTY(nPos)
    aGetList[nPos]:EXITSTATE := nExitState
  ENDIF
  nExitPos := nPos

RECOVER USING xBreak

  IF !EMPTY(xBreak)
    BREAK xBreak
  END

END SEQUENCE

RETURN nExitPos  //  McSettle

//---------------------------------------------------------------------------

FUNCTION McGetReadVar(oGet)

LOCAL cName := UPPER(oGet:NAME)
LOCAL nPos  := 1

IF !EMPTY(oGet:SUBSCRIPT)
  FOR nPos := 1 TO LEN(oGet:SUBSCRIPT)
    cName += '[' + LTRIM(STR(oGet:SUBSCRIPT[nPos])) + ']'
  NEXT
END

RETURN cName  //  McGetReadVar

//---------------------------------------------------------------------------

FUNCTION  MReadExit(lSetExit)

STATIC lCurExit := .N.

LOCAL  lOldExit := lCurExit

IF lSetExit # NIL
  lCurExit := lSetExit
END

RETURN lOldExit  //  MReadExit

//---------------------------------------------------------------------------

PROCEDURE McShowScoreboard()

LOCAL nRow := ROW()
LOCAL nCol := COL()

IF SET(_SET_SCOREBOARD)
  SETPOS(SCORE_ROW, SCORE_COL)
  DISPOUT(IF(SET(_SET_INSERT), 'Ins', '   '))
  SETPOS(nRow, nCol)
END

RETURN  //  McShowScoreboard

//---------------------------------------------------------------------------

FUNCTION McUpdated(lSetUpdated)

STATIC lCurUpdated := .N.

LOCAL  lOldUpdated := lCurUpdated

IF lSetUpdated # NIL
  lCurUpdated := lSetUpdated
END

RETURN lOldUpdated  //  McUpdated

//---------------------------------------------------------------------------

FUNCTION McReadKill(lSetReadKill)

STATIC lCurReadKill := .N.

LOCAL lOldReadKill := lCurReadKill

IF lSetReadKill # NIL
  lCurReadKill := lSetReadKill
ENDIF

RETURN lCurReadKill  //  McReadKill

//---------------------------------------------------------------------------

FUNCTION McSaveItem(xSetExpr, lClear)

STATIC xCurExpr := NIL

LOCAL xOldExpr := xCurExpr

DO CASE
CASE xSetExpr # NIL
  xCurExpr := xSetExpr
CASE lClear # NIL
  xCurExpr := NIL
END

RETURN xOldExpr  //  McSaveItem

//---------------------------------------------------------------------------

//  End of MOUSCLOC.PRG